package com.nationrel.modules.weibull.common.excel.app.impl;

import com.nationrel.modules.weibull.common.excel.app.ExcelApp;
import com.nationrel.modules.weibull.common.excel.bean.ExcelAppBean;
import com.nationrel.modules.weibull.common.excel.util.ExcelAppUtil;
import com.nationrel.modules.weibull.util.StringUtil;
import com.nationrel.modules.weibull.util.ValidateUtil;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.CellValue;
import org.apache.poi.ss.usermodel.FormulaEvaluator;
import org.apache.poi.ss.util.CellRangeAddress;

import java.io.*;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.*;

/**
 * @version V1.0
 * @ClassName XLSApp
 * @Description 2003Excel导入
 */
public class XLSApp implements ExcelApp {
    /**
     * 公式求值器
     */
    private FormulaEvaluator evaluator;
    private String cofSheetName = "flagSheet";

    @Override
    public List<Object> readAllRow(ExcelAppBean bean) throws Exception {
        List<Object> result = new ArrayList<Object>();
        HSSFWorkbook book = null;
        try {
            // 获取实体对象类模板
            Class<?> c = Class.forName(bean.getClassName());
            // 获取Excel对象
            book = getBook(bean);
            // 判断是否需要事先通过宏来验证Excel
            if (ValidateUtil.isNotEmpty(bean.getCofSheetName())) {
                cofSheetName = bean.getCofSheetName();
            }
            validation(book.getSheet(cofSheetName), bean);
            HSSFSheet sheet = getSheet(book, bean);

            // 获取Excel映射信息
            LinkedHashMap<String, String> map = bean.getFields();
            // 列名，列号
            List<String> titles = new ArrayList<String>();
            List<Integer> colNums = new ArrayList<Integer>();
            // 通过配置信息，获取列名，列号信息
            ExcelAppUtil.getCofInfo(map, titles, null, colNums);

            int tailRow = bean.getTailRow();
            Object obj = null;
            HSSFRow row = null;
            HSSFCell cell = null;
            int colNum;
            // 读取行, 实际情况：有可能有多行表头，因此开始行号根据配置所得
            for (int rowNum = bean.getStartRow() - 1; rowNum < sheet.getLastRowNum() - tailRow + 1; rowNum++) {
                obj = c.newInstance();
                row = sheet.getRow(rowNum);
                if (row == null) {
                    return result;
                }
                // 读取列
                for (int j = 0; j < titles.size(); j++) {
                    colNum = colNums.get(j) - 1;
                    // 获取指定列
                    cell = row.getCell(colNum);
                    // 如果第一个要读取的单元格为空，则读取下一行
                    if (cell == null) {
                        continue;
                    }

//                    boolean isMerge = isMergedRegion(sheet, rowNum, cell.getColumnIndex());
//                    //判断是否具有合并单元格
//                    if (isMerge) {
//                        // 读取单元格值并映射到实体对象
//                        readMergeCell(c, obj, cell, titles.get(j), sheet, row);
//                    } else {
                    // 读取单元格值并映射到实体对象
                    readCell(c, obj, cell, titles.get(j));
//                    }
                }
                // 当前行成功读取
                result.add(obj);
            }
        } catch (ReflectiveOperationException e) {
            throw e;
        } finally {
            if (book != null) {
                book = null;
            }
        }
        return result;
    }

    /**
     * 判断指定的单元格是否是合并单元格
     *
     * @param sheet  数据sheet
     * @param row    行下标
     * @param column 列下标
     * @return boolean
     */
    private boolean isMergedRegion(HSSFSheet sheet, int row, int column) {
        int sheetMergeCount = sheet.getNumMergedRegions();
        CellRangeAddress range = null;
        for (int i = 0; i < sheetMergeCount; i++) {
            range = sheet.getMergedRegion(i);
            int firstColumn = range.getFirstColumn();
            int lastColumn = range.getLastColumn();
            int firstRow = range.getFirstRow();
            int lastRow = range.getLastRow();
            if (row >= firstRow && row <= lastRow) {
                if (column >= firstColumn && column <= lastColumn) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public List<Object> readByRowNum(ExcelAppBean bean) throws Exception {
        List<Object> result = new ArrayList<Object>();
        HSSFWorkbook book = null;
        try {
            // 获得实体对象类模板
            Class<?> c = Class.forName(bean.getClassName());
            // 获取Excel对象
            book = getBook(bean);
            // 判断是否需要事先通过宏来验证Excel
            if (ValidateUtil.isNotEmpty(bean.getCofSheetName())) {
                cofSheetName = bean.getCofSheetName();
            }
            validation(book.getSheet(cofSheetName), bean);
            // 获取要读取的Excel的Sheet页
            HSSFSheet sheet = getSheet(book, bean);
            // 获取Excel映射信息
            LinkedHashMap<String, String> map = bean.getFields();

            // 列名，行号，列号
            List<String> titles = new ArrayList<String>();
            List<Integer> rowNums = new ArrayList<Integer>();
            List<Integer> colNums = new ArrayList<Integer>();
            // 通过配置信息，获取列名，行号，列号信息
            ExcelAppUtil.getCofInfo(map, titles, rowNums, colNums);
            LinkedHashMap<Integer, List<Integer>> rowCols = ExcelAppUtil.getRowAndCol(rowNums, colNums);

            // 读取指定的行和列
            Set<Integer> keySet = rowCols.keySet();
            Iterator<Integer> it = keySet.iterator();
            Integer rowNum = null;
            List<Integer> cols = null;
            Object obj = null;
            HSSFRow row = null;
            HSSFCell cell = null;
            int idx;
            while (it.hasNext()) {
                obj = c.newInstance();
                rowNum = it.next();
                cols = rowCols.get(rowNum);
                row = sheet.getRow(rowNum - 1);
                if (row == null) {
                    return result;
                }
                // 迭代每行对应的列号
                for (int i = 0; i < cols.size(); i++) {
                    idx = cols.get(i);
                    cell = row.getCell(colNums.get(idx) - 1);
                    if (cell == null || cell.getCellType() == CellType.BLANK) {
                        continue;
                    }
                    readCell(c, obj, cell, titles.get(idx));
                }
                result.add(obj);
            }
        } catch (ReflectiveOperationException e) {
            throw e;
        } finally {
            if (book != null) {
                book = null;
            }
        }
        return result;
    }

    @Override
    public List<Object> readByColumnCof(ExcelAppBean bean) throws Exception {
        List<Object> result = new ArrayList<Object>();
        HSSFWorkbook book = null;
        try {
            // 获得实体对象类模板
            Class<?> c = Class.forName(bean.getClassName());
            List<String> titles = bean.getTitles();
            // 获得实体对象类模板
            book = getBook(bean);
            // 判断是否需要事先通过宏来验证Excel
            if (ValidateUtil.isNotEmpty(bean.getCofSheetName())) {
                cofSheetName = bean.getCofSheetName();
            }
            validation(book.getSheet(cofSheetName), bean);
            HSSFSheet sheet = getSheet(book, bean);
            // 读取第一行
            HSSFRow row0 = sheet.getRow(0);
            Object obj = null;
            HSSFRow row = null;
            HSSFCell cell = null;
            String key = null;
            int col0;
            int startColNum = getColumnNumber(row0, bean.getColStartFlag());
            // 从指定的开始行号读起，实际情况： 有可能有多行表头
            for (int i = bean.getStartRow() - 1; i <= sheet.getLastRowNum(); i++) {
                boolean readOk = true;
                // 每一行都从起始列开始
                col0 = startColNum;
                obj = c.newInstance();
                row = sheet.getRow(i);
                if (row == null) {
                    return result;
                }
                // 读取指定的列，第一列为开始标示，读到最后一个配置列为止
                for (int j = 0; j < titles.size(); j++) {
                    key = titles.get(j);
                    cell = row.getCell(col0);
                    // 若单元为空，不再继续读取
                    if (cell == null) {
                        return result;
                    }
                    // 如果第一个要读取的单元格为空，则读取下一行
                    if (j == 0 && cell.getCellType() == CellType.BLANK) {
                        readOk = false;
                        break;
                    }
                    // 读取单元格值，并映射到实体对象
                    readCell(c, obj, cell, key);
                    // 向后移动单元格
                    col0++;
                }
                if (readOk) {
                    result.add(obj);
                }
            }
        } catch (ReflectiveOperationException e) {
            throw e;
        } finally {
            if (book != null) {
                book = null;
            }
        }
        return result;
    }

    @Override
    public void write(Object obj, ExcelAppBean bean) throws Exception {
        OutputStream out = null;
        HSSFWorkbook book = null;
        try {
            // 获取实体类的模板
            Class<?> c = Class.forName(bean.getClassName());
            book = getBook(bean);
            HSSFSheet sheet = getSheet(book, bean);
            LinkedHashMap<String, String> map = bean.getFields();

            // 列名，行号，列号
            List<String> titles = new ArrayList<String>();
            List<Integer> rowNums = new ArrayList<Integer>();
            List<Integer> colNums = new ArrayList<Integer>();
            // 通过配置信息，获取列名，行号，列号信息
            ExcelAppUtil.getCofInfo(map, titles, rowNums, colNums);

            HSSFRow row = null;
            HSSFCell cell = null;
            for (int j = 0; j < titles.size(); j++) {
                row = sheet.getRow(rowNums.get(j) - 1);
                cell = row.getCell(colNums.get(j) - 1);
                if (cell == null) {
                    continue;
                }
                // 将实体对象的值映射到单元格
                writeCell(c, obj, cell, titles.get(j));
            }
            out = new FileOutputStream(bean.getTargetFile());
            // 如果需要更改目标文件的sheet页面名
            String targetSheetName = bean.getTargetSheetName();
            if (targetSheetName != null && targetSheetName.length() > 0) {
                book.setSheetName(book.getSheetIndex(sheet.getSheetName()), targetSheetName);
            }
            book.write(out);

        } catch (IOException e) {
            throw e;
        } catch (ReflectiveOperationException e) {
            throw e;
        } finally {
            try {
                if (out != null) {
                    out.close();
                }
                if (book != null) {
                    book = null;
                }
            } catch (Exception e) {
                throw e;
            }
        }
    }

    @Override
    public ByteArrayOutputStream download(Object obj, ExcelAppBean bean) throws Exception {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        HSSFWorkbook book = null;
        try {
            // 获取实体类的模板
            Class<?> c = Class.forName(bean.getClassName());
            book = getBook(bean);
            HSSFSheet sheet = getSheet(book, bean);
            LinkedHashMap<String, String> map = bean.getFields();

            // 列名，行号，列号
            List<String> titles = new ArrayList<String>();
            List<Integer> rowNums = new ArrayList<Integer>();
            List<Integer> colNums = new ArrayList<Integer>();
            // 通过配置信息，获取列名，行号，列号信息
            ExcelAppUtil.getCofInfo(map, titles, rowNums, colNums);

            HSSFRow row = null;
            HSSFCell cell = null;
            for (int j = 0; j < titles.size(); j++) {
                row = sheet.getRow(rowNums.get(j) - 1);
                cell = row.getCell(colNums.get(j) - 1);
                if (cell == null) {
                    continue;
                }
                // 将实体对象的值映射到单元格
                writeCell(c, obj, cell, titles.get(j));
            }

            // 如果需要更改目标文件的sheet页面名
            String targetSheetName = bean.getTargetSheetName();
            if (targetSheetName != null && targetSheetName.length() > 0) {
                book.setSheetName(book.getSheetIndex(sheet.getSheetName()), targetSheetName);
            }
            book.write(out);

        } catch (IOException e) {
            throw e;
        } catch (ReflectiveOperationException e) {
            throw e;
        } finally {
            try {
                if (out != null) {
                    out.flush();
                    out.close();
                }
                if (book != null) {
                    book = null;
                }
            } catch (Exception e) {
                throw e;
            }
        }
        return out;
    }

    /**
     * @param bean bean
     * @return HSSFWorkbook
     * @throws Exception e
     * @Title: getBook
     * @Description: 获取Excel实例
     */
    private HSSFWorkbook getBook(ExcelAppBean bean) throws Exception {
        InputStream is = null;
        HSSFWorkbook book = null;
        try {
            String fileName = bean.getSourceFile();
            is = new BufferedInputStream(new FileInputStream(fileName), 1024);
            book = new HSSFWorkbook(is);
            this.evaluator = book.getCreationHelper().createFormulaEvaluator();
            return book;
        } catch (FileNotFoundException e) {
            throw e;
        } catch (IOException e) {
            throw e;
        } finally {
            try {
                if (is != null) {
                    is.close();
                    is = null;
                }
            } catch (Exception e) {
                throw e;
            }
        }
    }

    /**
     * @param book book
     * @param bean bean
     * @return HSSFSheet
     * @throws Exception e
     * @Title: getSheet
     * @Description: 获取Sheet, 如果没给定sheetName， 则返回第一个Sheet
     */
    private HSSFSheet getSheet(HSSFWorkbook book, ExcelAppBean bean) throws Exception {
        String sheetName = bean.getSourceSheetName();
        // 获取要读取的Excel的Sheet页
        HSSFSheet sheet = null;
        if (sheetName != null && sheetName.length() > 0) {
            sheet = book.getSheet(sheetName);
            if (sheet == null) {
                for (int i = 0; i < book.getNumberOfSheets(); i++) {
                    if (ExcelAppUtil.isMatchStr(book.getSheetName(i), sheetName)) {
                        sheet = book.getSheetAt(i);
                        break;
                    }
                }
            }
        } else {
            sheet = book.getSheetAt(0);
        }
        if (sheet == null) {
            throw new Exception("sheet is null   XLSApp 425");
        }
        return sheet;
    }

    /**
     * @param cell 单元格对象
     * @return String
     * @Title: getValue
     * @Description: 根据单元格类型获取值
     */
    private Object getCellValue(HSSFCell cell) {
        Object value = null;
        try {
            CellValue cellValue = this.evaluator.evaluate(cell);
            switch (cellValue.getCellType()) {
                case BOOLEAN:
                    value = cellValue.getBooleanValue();
                    break;
                case NUMERIC:
                    value = cellValue.getNumberValue();
                    break;
                case STRING:
                    value = cellValue.getStringValue();
                    break;
                case BLANK:
                    break;
                case ERROR:
                    break;
                case FORMULA:
                    break;
                default:
                    value = cellValue.getStringValue();
            }
        } catch (Exception e) {
            value = null;
        }
        return value;
    }

    /**
     * @param cell     单元格对象
     * @param objValue 要写入的值
     * @Title: getValue
     * @Description: 填写单元格
     */
    private void setCellValue(HSSFCell cell, Object objValue) {
        Object orgValue = this.getCellValue(cell);
        if (objValue instanceof Boolean) {
            cell.setCellValue((Boolean) objValue);
        } else if (objValue instanceof Date) {
            cell.setCellValue(ExcelAppUtil.getDateStr((Date) objValue));
        } else if (objValue instanceof Double) {
            cell.setCellValue((Double) objValue);
        } else {
            if (orgValue == null || "".equals(orgValue.toString().trim())) {
                cell.setCellValue(objValue.toString());
            } else {
                if (orgValue.toString().toUpperCase().endsWith("Input:".toUpperCase())) {
                    cell.setCellValue(orgValue.toString() + objValue.toString());
                } else if (orgValue.toString().toUpperCase().startsWith("Equipment:".toUpperCase())) {
                    String str = orgValue.toString().substring(0, orgValue.toString().indexOf("preheating") - 1);
                    String str2 = orgValue.toString().substring(orgValue.toString().indexOf("preheating") - 1, orgValue.toString().length());
                    cell.setCellValue(str + objValue.toString() + str2);
                }
            }
        }
    }

    /**
     * @param row        第一行
     * @param columnName 列名
     * @return Map<String, Integer>
     * @throws Exception e
     * @Title: getColumnNumber
     * @Description: 根据列名获取列索引号
     */
    private Integer getColumnNumber(HSSFRow row, String columnName) throws Exception {
        Integer colNum = null;
        HSSFCell cell = null;
        String cellValue = null;
        if (ValidateUtil.isInteger(columnName)) {
            colNum = Integer.valueOf(columnName);
        } else if (row != null) {
            for (int i = 0; i < row.getLastCellNum(); i++) {
                cell = row.getCell(i);
                cellValue = cell.getStringCellValue().trim();
                if (cellValue.replaceAll("\n", "").replaceAll(" ", "").trim().equalsIgnoreCase(columnName.trim())) {
                    colNum = cell.getColumnIndex();
                    break;
                }
            }
        }
        if (colNum == null) {
            throw new Exception("[XLSXAPP-getColumnNumber] The colNum invalid.");
        }
        return colNum;
    }

    /**
     * @param c    类模板
     * @param obj  实体对象
     * @param cell 单元格
     * @param key  字段
     * @throws Exception e
     * @Title: setValue
     * @Description: 调用实体对象的get方法获取值，并为单元给设置值
     */
    private void writeCell(Class<?> c, Object obj, HSSFCell cell, String key) throws Exception {
        try {
            Method method = null;
            try {
                method = c.getDeclaredMethod(ExcelAppUtil.becomeGetMethod(key));
                Object objValue = method.invoke(obj);
                if (objValue != null) {
                    setCellValue(cell, objValue);
                }
            } catch (NoSuchMethodException e) {
                c = c.getSuperclass();
                if (c != null) {
                    writeCell(c, obj, cell, key);
                } else {
                    throw e;
                }
            }
        } catch (ReflectiveOperationException e) {
            throw e;
        }
    }

    /**
     * @param c    类模板
     * @param obj  实体对象
     * @param cell 单元格
     * @param key  void
     * @throws Exception e
     * @Title: getValue
     * @Description: 获取单元格值，并调用实体对象的Set方法
     */
    private void readCell(Class<?> c, Object obj, HSSFCell cell, String key) throws Exception {
        try {
            Field field = null;
            try {
                field = c.getDeclaredField(key);
                Method method = c.getDeclaredMethod(ExcelAppUtil.becomeSetMethod(key), field.getType());

                String fieldType = field.getType().getName();
                Object valueObj = getCellValue(cell);
                Object value = null;
                // 根据实际要导入数据的类型做数据类型转换
                if (fieldType.equals(Double.class.getName())) {
                    value = valueObj;
                } else if (fieldType.equals(Integer.class.getName())) {
                    value = (int) Math.floor(((Double) valueObj).doubleValue());
                } else {
                    value = StringUtil.parseString(getCellValue(cell)).replaceAll("\n", "");
                }
                if (value != null) {
                    method.invoke(obj, value);
                }
            } catch (NoSuchFieldException e) {
                c = c.getSuperclass();
                if (c != null) {
                    readCell(c, obj, cell, key);
                } else {
                    throw e;
                }
            }
        } catch (ReflectiveOperationException e) {
            throw e;
        }
    }

    /**
     * @param c     类模板
     * @param obj   实体对象
     * @param cell  单元格
     * @param key   实体对象方法
     * @param sheet sheet
     * @param row   void
     * @Title: readMergeCell
     * @Description: 获取单元格值，并调用实体对象的Set方法
     */
    private void readMergeCell(Class<?> c, Object obj, HSSFCell cell, String key, HSSFSheet sheet, HSSFRow row) throws Exception {
        try {
            Field field = null;
            try {
                field = c.getDeclaredField(key);
                Method method = c.getDeclaredMethod(ExcelAppUtil.becomeSetMethod(key), field.getType());
                String value = getMergedRegionValue(sheet, row.getRowNum(), cell.getColumnIndex());

                if (value != null) {
                    method.invoke(obj, value);
                }
            } catch (NoSuchFieldException e) {
                c = c.getSuperclass();
                if (c != null) {
                    readMergeCell(c, obj, cell, key, sheet, row);
                } else {
                    throw e;
                }
            }
        } catch (ReflectiveOperationException e) {
            throw new Exception();
        }
    }

    /**
     * 获取合并单元格的值
     *
     * @param sheet  数据sheet
     * @param row    数据当前行
     * @param column 当前列
     * @return String
     */
    public String getMergedRegionValue(HSSFSheet sheet, int row, int column) {
        int sheetMergeCount = sheet.getNumMergedRegions();

        HSSFRow fRow = null;
        HSSFCell fCell = null;
        CellRangeAddress ca = null;
        for (int i = 0; i < sheetMergeCount; i++) {
            ca = sheet.getMergedRegion(i);
            int firstColumn = ca.getFirstColumn();
            int lastColumn = ca.getLastColumn();
            int firstRow = ca.getFirstRow();
            int lastRow = ca.getLastRow();

            if (row >= firstRow && row <= lastRow) {
                if (column >= firstColumn && column <= lastColumn) {
                    fRow = sheet.getRow(firstRow);
                    fCell = fRow.getCell(firstColumn);
                    return String.valueOf(getCellValue(fCell));
                }
            }
        }

        return null;
    }

    /**
     * @param sheet sheet
     * @param bean  bean
     * @throws Exception e
     * @Title: validation
     * @Description: 判断是否执行过宏
     */
    private void validation(HSSFSheet sheet, ExcelAppBean bean) throws Exception {
        boolean vaFlag = bean.isValidateFlag();
        if (!vaFlag) {
            return;
        }

        if (sheet == null) {
            throw new Exception();
        }
        HSSFRow row = sheet.getRow(0);
        if (row == null) {
            throw new Exception();
        }
        HSSFCell cell = row.getCell(0);
        if (cell == null) {
            throw new Exception();
        }
        String cofValue = cell.getStringCellValue();
        if (ValidateUtil.isEmpty(cofValue) || cofValue.equalsIgnoreCase(VALIDATE_NO)) {
            throw new Exception();
        }
    }
}
