package com.nationrel.modules.weibull.common.excel.app.impl;

import com.nationrel.modules.weibull.common.excel.app.ExcelApp;
import com.nationrel.modules.weibull.common.excel.bean.ExcelAppBean;
import com.nationrel.modules.weibull.common.excel.util.ExcelAppUtil;
import com.nationrel.modules.weibull.util.StringUtil;
import com.nationrel.modules.weibull.util.ValidateUtil;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.CellValue;
import org.apache.poi.ss.usermodel.FormulaEvaluator;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.*;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.*;

/**
 * @ClassName XLSXApp
 * @Description 读写2007以后版本的Excel
 * @Author sam
 * @CreateDate 2015-1-8
 * @version V1.0
 */
public class XLSXApp implements ExcelApp {
    /** 公式求值器 */
    private FormulaEvaluator evaluator;
    private String cofSheetName = "flagSheet";

    @Override
    public List<Object> readAllRow (ExcelAppBean bean) throws Exception {
        List<Object> result = new ArrayList<Object>();
        XSSFWorkbook book = null;
        try {
            // 获取实体对象类模板
            Class<?> c = Class.forName(bean.getClassName());
            // 获取Excel对象
            book = getBook(bean);
            // 判断是否需要事先通过宏来验证Excel
            if (ValidateUtil.isNotEmpty(bean.getCofSheetName())) {
                cofSheetName = bean.getCofSheetName();
            }
            validation(book.getSheet(cofSheetName), bean);
            XSSFSheet sheet = getSheet(book, bean);

            // 获取Excel映射信息
            LinkedHashMap<String, String> map = bean.getFields();
            // 列名，列号
            List<String> titles = new ArrayList<String>();
            List<Integer> colNums = new ArrayList<Integer>();
            // 通过配置信息，获取列名，列号信息
            ExcelAppUtil.getCofInfo(map, titles, null, colNums);

            // 读取行, 实际情况：有可能有多行表头，因此开始行号根据配置所得
            Object obj = null;
            XSSFRow row = null;
            XSSFCell cell = null;
            int tailRow = bean.getTailRow();
            int colNum;

            for (int rowNum = bean.getStartRow() - 1; rowNum < sheet.getLastRowNum()  - tailRow + 1; rowNum++) {
                obj =  c.newInstance();
                row = sheet.getRow(rowNum);
                if (row == null) {
                    return result;
                }
                // 读取列
                for (int j = 0; j < titles.size(); j++) {
                    colNum = colNums.get(j) - 1;
                    // 获取指定列
                    cell = row.getCell(colNum);
                    // 如果第一个要读取的单元格为空，则读取下一行
                    if (cell == null) {
                        continue;
                    }
                    // 读取单元格值并映射到实体对象
                    readCell(c, obj, cell, titles.get(j));
                }
                // 当前行成功读取
                result.add(obj);
            }
        } catch (ReflectiveOperationException e) {
            throw e;
        } finally {
            if (book != null) {
                book = null;
            }
        }
        return result;
    }
    @Override
    public List<Object> readByRowNum(ExcelAppBean bean) throws Exception {
        List<Object> result = new ArrayList<Object>();
        XSSFWorkbook book = null;
        try {
            // 获得实体对象类模板
            Class<?> c = Class.forName(bean.getClassName());
            // 获取Excel对象
            book = getBook(bean);
            // 判断是否需要事先通过宏来验证Excel
            if (ValidateUtil.isNotEmpty(bean.getCofSheetName())) {
                cofSheetName = bean.getCofSheetName();
            }
            validation(book.getSheet(cofSheetName), bean);
            // 获取要读取的Excel的Sheet页
            XSSFSheet sheet = getSheet(book, bean);
            // 获取Excel映射信息
            LinkedHashMap<String, String> map = bean.getFields();

            // 列名，行号，列号
            List<String> titles = new ArrayList<String>();
            List<Integer> rowNums = new ArrayList<Integer>();
            List<Integer> colNums = new ArrayList<Integer>();
            // 通过配置信息，获取列名，行号，列号信息
            ExcelAppUtil.getCofInfo(map, titles, rowNums, colNums);
            LinkedHashMap<Integer, List<Integer>> rowCols = ExcelAppUtil.getRowAndCol(rowNums, colNums);

            // 读取指定的行和列
            Set<Integer> keySet = rowCols.keySet();
            Iterator<Integer> it = keySet.iterator();
            Integer rowNum = null;
            List<Integer> cols = null;
            Object obj = null;
            XSSFRow row = null;
            XSSFCell cell = null;

            while (it.hasNext()) {
                obj =  c.newInstance();
                rowNum = it.next();
                cols = rowCols.get(rowNum);
                row = sheet.getRow(rowNum - 1);
                if (row == null) {
                    return result;
                }
                // 迭代每行对应的列号
                for (int i = 0; i < cols.size(); i++) {
                    int idx = cols.get(i);
                    cell = row.getCell(colNums.get(idx) - 1);
                    if (cell == null || cell.getCellType() == CellType.BLANK) {
                        continue;
                    }
                    readCell(c, obj, cell, titles.get(idx));
                }
                result.add(obj);
            }
        } catch (ReflectiveOperationException e) {
            throw e;
        } finally {
            if (book != null) {
                book = null;
            }
        }
        return result;
    }
    @Override
    public List<Object> readByColumnCof(ExcelAppBean bean) throws Exception {
        List<Object> result = new ArrayList<Object>();
        XSSFWorkbook book = null;
        try {
            // 获得实体对象类模板
            Class<?> c = Class.forName(bean.getClassName());
            List<String> titles = bean.getTitles();
            // 获得实体对象类模板
            book = getBook(bean);
            // 判断是否需要事先通过宏来验证Excel
            if (ValidateUtil.isNotEmpty(bean.getCofSheetName())) {
                cofSheetName = bean.getCofSheetName();
            }
            validation(book.getSheet(cofSheetName), bean);
            XSSFSheet sheet = getSheet(book, bean);
            // 读取第一行
            XSSFRow row0 = sheet.getRow(0);
            int startColNum = getColumnNumber(row0, bean.getColStartFlag());

            Object obj = null;
            XSSFRow row = null;
            XSSFCell cell = null;
            String key = null;
            int col0;
            boolean readOk = false;

            // 从指定的开始行号读起，实际情况： 有可能有多行表头
            for (int i = bean.getStartRow() - 1; i <= sheet.getLastRowNum(); i++) {
                readOk = true;
                // 每一行都从起始列开始
                col0 = startColNum;
                obj =  c.newInstance();
                row = sheet.getRow(i);
                if (row == null) {
                    return result;
                }
                // 读取指定的列，第一列为开始标示，读到最后一个配置列为止
                for (int j = 0; j < titles.size(); j++) {
                    key = titles.get(j);
                    cell = row.getCell(col0);
                    // 若单元为空，不再继续读取
                    /*if (cell == null) {
                        return result;
                    }*/
                    // 如果第一个要读取的单元格为空，则读取下一行
                    if (j == 0 && cell.getCellType() == CellType.BLANK) {
                        readOk = false;
                        break;
                    }
                    // 读取单元格值，并映射到实体对象
                    readCell(c, obj, cell, key);
                    // 向后移动单元格
                    col0++;
                }
                if (readOk) {
                    result.add(obj);
                }
            }
        } catch (ReflectiveOperationException e) {
            throw e;
        } finally {
            if (book != null) {
                try {
                    book = null;
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
        return result;
    }
    @Override
    public void write(Object obj, ExcelAppBean bean) throws Exception {
        OutputStream out = null;
        XSSFWorkbook book = null;
        try {
            // 获取实体类的模板
            Class<?> c = Class.forName(bean.getClassName());
            book = getBook(bean);
            XSSFSheet sheet = getSheet(book, bean);
            LinkedHashMap<String, String> map = bean.getFields();

            // 列名，行号，列号
            List<String> titles = new ArrayList<String>();
            List<Integer> rowNums = new ArrayList<Integer>();
            List<Integer> colNums = new ArrayList<Integer>();
            // 通过配置信息，获取列名，行号，列号信息
            ExcelAppUtil.getCofInfo(map, titles, rowNums, colNums);

            XSSFRow row = null;
            XSSFCell cell = null;
            for (int j = 0; j < titles.size(); j++) {
                row = sheet.getRow(rowNums.get(j) - 1);
                cell = row.getCell(colNums.get(j) - 1);
                if (cell == null) {
                    continue;
                }
                // 将实体对象的值映射到单元格
                writeCell(c, obj, cell, titles.get(j));
            }
            out = new FileOutputStream(bean.getTargetFile());
            // 如果需要更改目标文件的sheet页面名
            String targetSheetName = bean.getTargetSheetName();
            if (targetSheetName != null && targetSheetName.length() > 0) {
                book.setSheetName(book.getSheetIndex(sheet.getSheetName()), targetSheetName);
            }
            book.write(out);

        } catch (IOException e) {
            throw e;
        } catch (ReflectiveOperationException e) {
            throw e;
        } finally {
            try {
                if (out != null) {
                    out.close();
                }
                if (book != null) {
                    book = null;
                }
            } catch (Exception e) {
                throw e;
            }
        }
    }
    @Override
    public ByteArrayOutputStream download(Object obj, ExcelAppBean bean) throws Exception {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        XSSFWorkbook book = null;
        try {
            // 获取实体类的模板
            Class<?> c = Class.forName(bean.getClassName());
            book = getBook(bean);
            XSSFSheet sheet = getSheet(book, bean);
            LinkedHashMap<String, String> map = bean.getFields();

            // 列名，行号，列号
            List<String> titles = new ArrayList<String>();
            List<Integer> rowNums = new ArrayList<Integer>();
            List<Integer> colNums = new ArrayList<Integer>();
            // 通过配置信息，获取列名，行号，列号信息
            ExcelAppUtil.getCofInfo(map, titles, rowNums, colNums);

            XSSFRow row = null;
            XSSFCell cell = null;
            for (int j = 0; j < titles.size(); j++) {
                row = sheet.getRow(rowNums.get(j) - 1);
                cell = row.getCell(colNums.get(j) - 1);
                if (cell == null) {
                    continue;
                }
                // 将实体对象的值映射到单元格
                writeCell(c, obj, cell, titles.get(j));
            }

            // 如果需要更改目标文件的sheet页面名
            String targetSheetName = bean.getTargetSheetName();
            if (targetSheetName != null && targetSheetName.length() > 0) {
                book.setSheetName(book.getSheetIndex(sheet.getSheetName()), targetSheetName);
            }
            book.write(out);

        } catch (IOException e) {
            throw e;
        } catch (ReflectiveOperationException e) {
            throw e;
        } finally {
            try {
                if (out != null) {
                    out.flush();
                    out.close();
                }
                if (book != null) {
                    book = null;
                }
            } catch (Exception e) {
                throw e;
            }
        }
        return out;
    }
    /**
     * @Title: getBook
     * @Description: 获取Excel实例
     * @param bean bean
     * @return XSSFWorkbook
     * @exception Exception e
     */
    private XSSFWorkbook getBook (ExcelAppBean bean) throws Exception{
        InputStream is = null;
        XSSFWorkbook book = null;
        try {
            String fileName = bean.getSourceFile();
            //is = new BufferedInputStream(new FileInputStream(fileName), 1024);
            is = new BufferedInputStream(new FileInputStream(fileName), 8192);
            book = new XSSFWorkbook(is);
            this.evaluator = book.getCreationHelper().createFormulaEvaluator();
            return book;
        } catch (FileNotFoundException e) {
            throw e;
        } catch (IOException e) {
            throw e;
        } finally {
            try {
                if (is != null) {
                    is.close();
                    is = null;
                }
            } catch (Exception e) {
                throw e;
            }
        }
    }
    /**
     * @Title: getSheet
     * @Description: 获取Sheet, 如果没给定sheetName， 则返回第一个Sheet
     * @param book book
     * @param bean bean
     * @return XSSFSheet
     * @exception Exception e
     */
    private XSSFSheet getSheet(XSSFWorkbook book, ExcelAppBean bean) throws Exception {
        String sheetName = bean.getSourceSheetName();
        // 获取要读取的Excel的Sheet页
        XSSFSheet sheet = null;
        if (sheetName != null && sheetName.length() > 0) {
            sheet = book.getSheet(sheetName);
            if (sheet == null) {
                for (int i = 0; i < book.getNumberOfSheets(); i++) {
                    if (ExcelAppUtil.isMatchStr(book.getSheetName(i), sheetName)) {
                        sheet = book.getSheetAt(i);
                        break;
                    }
                }
            }
        } else {
            sheet = book.getSheetAt(0);
        }
        if (sheet == null) {
            throw new Exception("[XLSXApp-getSheet] The sheetName is invalid.");
        }
        return sheet;
    }
    /**
     * @Title: getValue
     * @Description: 根据单元格类型获取值
     * @param cell 单元格对象
     * @return String
     */
    private Object getCellValue(XSSFCell cell) {
        Object value = null;
        try {
            CellValue cellValue = this.evaluator.evaluate(cell);
            switch (cellValue.getCellType()) {
                case BOOLEAN:
                    value = cellValue.getBooleanValue();
                    break;
                case NUMERIC:
                    value = cellValue.getNumberValue();
                    break;
                case STRING:
                    value = cellValue.getStringValue();
                    break;
                case BLANK:
                    break;
                case ERROR:
                    break;
                case FORMULA:
                    break;
                default:
                    value = cellValue.getStringValue();
            }
        } catch (Exception e) {
            value = null;
        }
        return value;
    }
    /**
     * @Title: getValue
     * @Description: 填写单元格
     * @param cell 单元格对象
     * @param objValue 要写入的值
     */
    public void setCellValue(XSSFCell cell, Object objValue) {
        Object orgValue = this.getCellValue(cell);
        if (objValue instanceof Boolean) {
            cell.setCellValue((Boolean) objValue);
        } else if (objValue instanceof Date) {
            cell.setCellValue(ExcelAppUtil.getDateStr((Date) objValue));
        } else if (objValue instanceof Double) {
            cell.setCellValue((Double) objValue);
        } else {
            if (orgValue == null || "".equals(orgValue.toString().trim())) {
                cell.setCellValue(objValue.toString());
            } else {
                if (orgValue.toString().toUpperCase().endsWith("Input:".toUpperCase())) {
                    cell.setCellValue(orgValue.toString() + objValue.toString());
                } else if (orgValue.toString().toUpperCase().startsWith("Equipment:".toUpperCase())) {
                    String str = orgValue.toString().substring(0, orgValue.toString().indexOf("preheating") - 1);
                    String str2 = orgValue.toString().substring(orgValue.toString().indexOf("preheating") - 1, orgValue.toString().length());
                    cell.setCellValue(str + objValue.toString() + str2);
                }
            }
        }
    }

    /**
     * @Title: getColumnNumber
     * @Description: 根据列名获取列索引号
     * @param row 第一行
     * @param columnName 列名
     * @return Map<String,Integer>
     * @exception Exception e
     */
    private Integer getColumnNumber(XSSFRow row, String columnName) throws Exception {
        Integer colNum = null;
        XSSFCell cell = null;
        String cellValue = null;
        if (ValidateUtil.isInteger(columnName)) {
            colNum = Integer.valueOf(columnName);
        } else if (row != null) {
            for (int i = 0; i < row.getLastCellNum(); i++) {
                cell = row.getCell(i);
                cellValue = cell.getStringCellValue().trim();
                if (cellValue.replaceAll("\n", "").replaceAll(" ", "").trim().equalsIgnoreCase(columnName.trim())) {
                    colNum = cell.getColumnIndex();
                    break;
                }
            }
        }
        if (colNum == null) {
            throw new Exception("[XLSXAPP-getColumnNumber] The colNum invalid.");
        }
        return colNum;
    }
    /**
     * @Title: setValue
     * @Description: 调用实体对象的get方法获取值，并为单元给设置值
     * @param c 类模板
     * @param obj 实体对象
     * @param cell 单元格
     * @param key 字段
     * @exception Exception e
     */
    private void writeCell(Class<?> c, Object obj, XSSFCell cell, String key) throws Exception {
        try {
            Method method = null;
            try {
                method = c.getDeclaredMethod(ExcelAppUtil.becomeGetMethod(key));
                Object objValue = method.invoke(obj);
                if (objValue != null) {
                    setCellValue(cell, objValue);
                }
            } catch (NoSuchMethodException e) {
                c = c.getSuperclass();
                if (c != null) {
                    writeCell(c, obj, cell, key);
                } else {
                    throw e;
                }
            }
        } catch (ReflectiveOperationException e) {
            throw new Exception("[XLSXAPP-getColumnNumber] The colNum invalid.");
        }
    }
    /**
     * @Title: getValue
     * @Description: 获取单元格值，并调用实体对象的Set方法
     * @param c 类模板
     * @param obj 实体对象
     * @param cell 单元格
     * @param key void
     * @exception Exception e
     */
    private void readCell(Class<?> c, Object obj, XSSFCell cell, String key) throws Exception {
        try {
            Field field = null;
            try {
                field = c.getDeclaredField(key);
                Method method = c.getDeclaredMethod(ExcelAppUtil.becomeSetMethod(key), field.getType());
                String fieldType = field.getType().getName();
                Object valueObj = getCellValue(cell);
                Object value = null;
                // 根据实际要导入数据的类型做数据类型转换
                if (fieldType.equals(Double.class.getName())) {
                    value = Double.valueOf(valueObj.toString());
                } else if (fieldType.equals(Integer.class.getName())) {
                    value = (int) Math.floor(((Double) valueObj).doubleValue());
                } else {
                    value = StringUtil.parseString(getCellValue(cell)).replaceAll("\n", "");
                }
                if (value != null) {
                    method.invoke(obj, value);
                }
            } catch (NoSuchFieldException e) {
                c = c.getSuperclass();
                if (c != null) {
                    readCell(c, obj, cell, key);
                } else {
                    throw e;
                }
            }
        } catch (ReflectiveOperationException e) {
            throw new Exception("[XLSXAPP-getColumnNumber] The colNum invalid.");
        }
    }
    /**
     * @Title: validation
     * @Description: 判断是否执行过宏
     * @param sheet sheet
     * @param bean bean
     * @exception Exception e
     */
    private void validation(XSSFSheet sheet, ExcelAppBean bean) throws Exception {
        boolean vaFlag = bean.isValidateFlag();
        if (!vaFlag) {
            return;
        }

        if (sheet == null) {
            throw new Exception("MessageId.E009928");
        }
        XSSFRow row = sheet.getRow(0);
        if (row == null) {
            throw new Exception("MessageId.E009928");
        }
        XSSFCell cell = row.getCell(0);
        if (cell == null) {
            throw new Exception("MessageId.E009928");
        }
        String cofValue = cell.getStringCellValue();
        if (ValidateUtil.isEmpty(cofValue) || cofValue.equalsIgnoreCase(VALIDATE_NO)) {
            throw new Exception("MessageId.E009928");
        }
    }
}
