package com.nationrel.modules.weibull.common.excel.util;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellReference;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.*;
import java.util.*;

/**
 * @ClassName ExcelAppUtil
 * @Description Excel读写工具类
 * @version V1.0
 */
public class ExcelAppUtil {
    public static final int SXSSF_WINDOW_SIZE = 2000;

    /**
     * 返回Excel WorkBook 类似于DbUtil
     * @param fileTemplate
     * @return
     * @throws Exception
     */
    public static Workbook getSxssfWorkbook(String fileTemplate) throws Exception {
        InputStream is = null;
        try {
            is = new FileInputStream(new File(fileTemplate));
            XSSFWorkbook xssfwb = new XSSFWorkbook(is);
            Workbook wb = new SXSSFWorkbook(xssfwb, SXSSF_WINDOW_SIZE);
            return wb;
        } catch (Exception e) {
            throw e;
        } finally {
            try {
                if (is != null) {
                    is.close();
                    is = null;
                }
            } catch (Exception e) {
                throw e;
            }
        }
    }

    /**
     * 返回Excel Sheet 类似于DbUtil
     * @param book
     * @param sheetPos
     * @return
     * @throws Exception
     */
    public static Sheet getWorkSheet(Workbook book, int sheetPos) throws Exception {
        return book.getSheetAt(sheetPos);
    }

    /**
     * 设置是否有网格线
     * @param sheet
     * @param isLine
     */
    public static void setWorkSeetDisplayGridlines(Sheet sheet, boolean isLine) {
        sheet.setDisplayGridlines(isLine);
    }

//    /**
//     * 为Workbook缓存单元格样式
//     * @param wb
//     * @return
//     */
//    public static Map<Integer, CellStyle> cacheCellStyle(Workbook wb) {
//        Map<Integer, CellStyle> mapCellStyle = new HashMap<Integer, CellStyle>();
//        // CELL_TYPE_LEFT_NO_RIGHT_BOTTOM
//        CellStyle csDefault = wb.createCellStyle();
//        csDefault.setAlignment(CellStyle.ALIGN_LEFT);
//        csDefault.setVerticalAlignment(CellStyle.VERTICAL_TOP);
//        csDefault.setWrapText(true);
//        mapCellStyle.put(CellConstants.CELL_TYPE_DEFAULT, csDefault);
//
//        // CELL_TYPE_LEFT_NO_RIGHT_BOTTOM
//        CellStyle csLeftRightBottom = wb.createCellStyle();
//        csLeftRightBottom.setBorderLeft((short) 1);
//        csLeftRightBottom.setBorderRight((short) 1);
//        csLeftRightBottom.setBorderBottom((short) 1);
//        csLeftRightBottom.setAlignment(CellStyle.ALIGN_LEFT);
//        csLeftRightBottom.setVerticalAlignment(CellStyle.VERTICAL_TOP);
//        csLeftRightBottom.setWrapText(true);
//        mapCellStyle.put(CellConstants.CELL_TYPE_LEFT_NO_RIGHT_BOTTOM, csLeftRightBottom);
//
//        // CELL_TYPE_LEFT_NO_RIGHT_NO
//        CellStyle csLeftRight = wb.createCellStyle();
//        csLeftRight.setBorderLeft((short) 1);
//        csLeftRight.setBorderRight((short) 1);
//        csLeftRight.setAlignment(CellStyle.ALIGN_LEFT);
//        csLeftRight.setVerticalAlignment(CellStyle.VERTICAL_TOP);
//        csLeftRight.setWrapText(true);
//        mapCellStyle.put(CellConstants.CELL_TYPE_LEFT_NO_RIGHT_NO, csLeftRight);
//
//        // CELL_TYPE_NO_NO_RIGHT_BOTTOM
//        CellStyle csRightBottom = wb.createCellStyle();
//        csRightBottom.setBorderRight((short) 1);
//        csRightBottom.setBorderBottom((short) 1);
//        csRightBottom.setAlignment(CellStyle.ALIGN_LEFT);
//        csRightBottom.setVerticalAlignment(CellStyle.VERTICAL_TOP);
//        csRightBottom.setWrapText(true);
//        mapCellStyle.put(CellConstants.CELL_TYPE_NO_NO_RIGHT_BOTTOM, csRightBottom);
//
//        // CELL_TYPE_NO_NO_RIGHT_NO
//        CellStyle csRight = wb.createCellStyle();
//        csRight.setBorderRight((short) 1);
//        csRight.setAlignment(CellStyle.ALIGN_LEFT);
//        csRight.setVerticalAlignment(CellStyle.VERTICAL_TOP);
//        csRight.setWrapText(true);
//        mapCellStyle.put(CellConstants.CELL_TYPE_NO_NO_RIGHT_NO, csRight);
//
//        // CELL_TYPE_LEFT_NO_NO_BOTTOM
//        CellStyle csLeftBottom = wb.createCellStyle();
//        csLeftBottom.setBorderLeft((short) 1);
//        csLeftBottom.setBorderBottom((short) 1);
//        csLeftBottom.setAlignment(CellStyle.ALIGN_LEFT);
//        csLeftBottom.setVerticalAlignment(CellStyle.VERTICAL_TOP);
//        csLeftBottom.setWrapText(true);
//        mapCellStyle.put(CellConstants.CELL_TYPE_LEFT_NO_NO_BOTTOM, csLeftBottom);
//
//        // CELL_TYPE_NO_NO_NO_BOTTOM
//        CellStyle csBottom = wb.createCellStyle();
//        csBottom.setBorderBottom((short) 1);
//        csBottom.setAlignment(CellStyle.ALIGN_LEFT);
//        csBottom.setVerticalAlignment(CellStyle.VERTICAL_TOP);
//        csBottom.setWrapText(true);
//        mapCellStyle.put(CellConstants.CELL_TYPE_NO_NO_NO_BOTTOM, csBottom);
//
//        // CELL_TYPE_LEFT_TOP_RIGHT_BOTTOM
//        CellStyle csLeftTopRightBottom = wb.createCellStyle();
//        csLeftTopRightBottom.setBorderLeft((short) 1);
//        csLeftTopRightBottom.setBorderTop((short) 1);
//        csLeftTopRightBottom.setBorderRight((short) 1);
//        csLeftTopRightBottom.setBorderBottom((short) 1);
//        csLeftTopRightBottom.setAlignment(CellStyle.ALIGN_LEFT);
//        csLeftTopRightBottom.setVerticalAlignment(CellStyle.VERTICAL_TOP);
//        csLeftTopRightBottom.setWrapText(true);
//        mapCellStyle.put(CellConstants.CELL_TYPE_LEFT_TOP_RIGHT_BOTTOM, csLeftTopRightBottom);
//
//        return mapCellStyle;
//    }

    /**
     * 将Workbook写入到输出流
     * @param wb
     * @return
     * @throws Exception
     */
    public static OutputStream writeWbToStream(Workbook wb) throws Exception {
        OutputStream out = null;
        try {
            out = new ByteArrayOutputStream();
            wb.write(out);
            out.flush();
            out.close();
            return out;
        } catch (IOException e) {
            throw e;
        } finally {
            try {
                if (out != null) {
                    out.close();
                    out = null;
                }
            } catch (Exception e) {
                throw e;
            }
        }
    }

    /**
     * 写入单元格，值 和 样式
     * @param row
     * @param i
     * @param value
     * @param cellStyle
     */
    public static void writeCell(Row row, int i, String value, CellStyle cellStyle) {
        Cell cell = row.createCell(i);
        cell.setCellValue(value);
        if (cellStyle != null) {
            cell.setCellStyle(cellStyle);
        }
    }
    public static void writeCell(Sheet sh, CellReference cellReference,
                                 String value) {
        Row row = sh.getRow(cellReference.getRow());
        Cell cell = row.getCell(cellReference.getCol());
        if (StringUtils.isNotEmpty(value)) {
            cell.setCellValue(value);
        }
    }
    public static void writeCell(Sheet sh, CellReference cellReference,
                                 Date value) {
        Row row = sh.getRow(cellReference.getRow());
        Cell cell = row.getCell(cellReference.getCol());
        if (value != null) {
            cell.setCellValue(value);
        }
    }
    /**
     * @Title: getBook (.xlsx)
     * @Description: 获取Excel实例
     * @param fileName fileName
     * @return XSSFWorkbook
     * @exception Exception e
     */
    public static XSSFWorkbook getBook (String fileName) throws Exception {
        InputStream is = null;
        XSSFWorkbook book = null;
        try {
            is = new BufferedInputStream(new FileInputStream(fileName), 1024);
            book = new XSSFWorkbook(is);
            return book;
        } catch (FileNotFoundException e) {
            throw e;
        } catch (IOException e) {
            throw e;
        } finally {
            try {
                if (is != null) {
                    is.close();
                    is = null;
                }
            } catch (Exception e) {
                throw e;
            }
        }
    }

    /**
     * @Title: getBook (.xls)
     * @Description: 获取Excel实例
     * @return HSSFWorkbook
     * @exception Exception e
     */
    public static HSSFWorkbook getHssfWorkbook(String fileName) throws Exception{
        InputStream is = null;
        HSSFWorkbook book = null;
        try {
            is = new BufferedInputStream(new FileInputStream(fileName), 1024);
            book = new HSSFWorkbook(is);
            return book;
        } catch (FileNotFoundException e) {
            throw e;
        } catch (IOException e) {
            throw e;
        } finally {
            try {
                if (is != null) {
                    is.close();
                    is = null;
                }
            } catch (Exception e) {
                throw e;
            }
        }
    }

    /**
     * @Title: getCofInfo
     * @Description: 将配置信息转换为字段，行，列信息
     * @param map 字段和行号，列号的配置信息
     * @param titles 存放字段名
     * @param rowNums 存放要获取的行号
     * @param colNums 存放要获取的列号
     */
    public static void getCofInfo(LinkedHashMap<String, String> map, List<String> titles, List<Integer> rowNums, List<Integer> colNums) {
        Map.Entry<String, String> entry = null;
        Iterator<Map.Entry<String, String>> it = map.entrySet().iterator();
        String[] cols = new String[map.size()];
        int i = 0;
        String key = null;
        // 获取要读取的行，列信息
        while (it.hasNext()) {
            entry = it.next();
            key = entry.getKey();
            cols[i] = entry.getValue();
            if (rowNums != null && cols[i] != null && cols[i].contains(",")) {
                String[] c = cols[i].split(",");
                // 如果是一个区间: 2-3, 5
                if (c[0].contains("-")) {
                    String[] area = c[0].split("-");
                    int start = Integer.valueOf(area[0]).intValue();
                    int end = Integer.valueOf(area[1]).intValue();
                    for (int j = start; j <= end; j++) {
                        titles.add(key);
                        rowNums.add(j);
                        colNums.add(Integer.valueOf(c[1]));
                    }
                    // 不是区间: 2,3
                } else {
                    titles.add(key);
                    rowNums.add(Integer.valueOf(c[0]));
                    colNums.add(Integer.valueOf(c[1]));
                }
                // 只有列: 3
            } else {
                titles.add(key);
                colNums.add(Integer.valueOf(cols[i]));
            }
            i++;
        }
    }
    /**
     * @Title: getRowAndCol
     * @Description: 将行列拼装为类似一行多列的结构： 1:1,2,3; 2:4,5, key为行号，value为数组的索引号(非数组的值)
     * @param rowNums 行
     * @param colNums 列
     * @return Map<String,String>
     */
    public static LinkedHashMap<Integer, List<Integer>> getRowAndCol(List<Integer> rowNums, List<Integer> colNums) {
        LinkedHashMap<Integer, List<Integer>> result = new LinkedHashMap<Integer, List<Integer>>();
        Integer rowNum = null;
        List<Integer> colList = null;
        for (int i = 0; i < colNums.size(); i++) {
            rowNum = rowNums.get(i);
            if (result.get(rowNum) != null) {
                colList = result.get(rowNum);
            } else {
                colList = new ArrayList<Integer>();
                result.put(rowNum, colList);
            }
            // 记录列出现在数组里的索引位置
            colList.add(i);
        }
        return result;
    }
    /**
     * @Title: getDateStr
     * @Description: 格式化日期格式 : xxxx年xx月xx日
     * @param date date
     * @return String
     */
    public static String getDateStr (Date date) {
        return String.format("%1$tY/%1$tm/%1$td", date);
    }
    /**
     * @Title: isMatchStr
     * @Description: 检查reg是否可以匹配， 类似： reg=500, str=500h, 这种是匹配的
     * @param str 要检查的字符串
     * @param reg 规则
     * @return boolean
     */
    public static boolean isMatchStr (String str, String reg) {
        String[] strs = str.split(reg);
        boolean flag = false;
        if ((strs.length == 0 || "".equals(strs[0])) && strs[1] != null && !Character.isDigit(strs[1].charAt(0))) {
            flag = true;
        }
        return flag;
    }
    /**
     * @Title: becomeSetMethod
     * @Description: 为字段生成set方法字符串
     * @param field field
     * @return String
     */
    public static String becomeSetMethod (String field) {
        return "set" + field.replaceFirst(field.substring(0, 1), field.substring(0, 1).toUpperCase());
    }
    /**
     * @Title: becomeGetMethod
     * @Description: 为指定字段生成get方法字符串
     * @param field field
     * @return String
     */
    public static String becomeGetMethod (String field) {
        return "get" + field.replaceFirst(field.substring(0, 1), field.substring(0, 1).toUpperCase());
    }
}
