package com.nationrel.modules.weibull.dto;

import java.io.Serializable;


/**
 * @ClassName Pagination
 * @Description 分页接口
 * @author 范钦涛
 * @date 2013-6-3
 * @version V1.0
 */
public class Pagination implements Serializable {

    /** @Fields serialVersionUID : 序列ID */
    private static final long serialVersionUID = 1L;

    /** 数据行数 */
    private int rowTotal = 1;
    /** 当前页 */
    private int pageCurrent = 1;
    /** 每页行数 */
    private int pageSize = 10;

    /**
     * @return 取得数据总数
     */
    public int getRowTotal() {
        return rowTotal;
    }

    /**
     * @param 设定数据总数
     */
    public void setRowTotal(int rowTotal) {
        this.rowTotal = rowTotal;
        // 数据发生变化时回到第一页
        if (pageCurrent > this.getPageTotal() && this.getPageTotal() != 0) {
            pageCurrent = this.getPageTotal();
        }
    }

    /**
     * @return 取得开始行
     */
    public int getRowStart() {
        return (this.getPageCurrent() - 1) * this.getPageSize() + 1;
    }

    /**
     * @return 取得Mysql开始行
     */
    public int getMysqlRowStart() {
        return (this.getPageCurrent() - 1) * this.getPageSize();
    }

    /**
     * @return 取得结束行
     */
    public int getRowEnd() {
        return getRowStart() + this.getPageSize() - 1;
    }

    /**
     * @return 取得当前页
     */
    public int getPageCurrent() {
        return pageCurrent;
    }

    /**
     * @param 设定当前页
     */
    public void setPageCurrent(int pageCurrent) {
        this.pageCurrent = pageCurrent;
    }

    /**
     * @return 取得每页行数
     */
    public int getPageSize() {
        return pageSize;
    }

    /**
     * @param 设定每页行数
     */
    public void setPageSize(int pageSize) {
        this.pageSize = pageSize;
    }

    /**
     * @return 取得页数
     */
    public int getPageTotal() {
        int pageTotal = rowTotal / pageSize;
        if (rowTotal % pageSize > 0) {
            pageTotal++;
        }
        return pageTotal;
    }
}
