package com.nationrel.modules.weibull.function;

import org.apache.commons.math3.linear.AbstractRealMatrix;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.stat.StatUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aimee on 2020/9/22.
 */
public class AnalysisMatrix {
    private AbstractRealMatrix matrix = null;

    public AnalysisMatrix(double[][] dd) {
        this.matrix = new Array2DRowRealMatrix(dd);
    }

    public AnalysisMatrix(double x1, double x2, double x3, double x4) {
        double[][] matrixD = new double[2][2];
        matrixD[0][0] = x1;
        matrixD[0][1] = x3;
        matrixD[1][0] = x4;
        matrixD[1][1] = x2;
        this.matrix = new Array2DRowRealMatrix(matrixD);
    }

    public RealMatrix solve() {
        double[][] v = this.matrix.getData();
        this.solve(v, 2);
        return new Array2DRowRealMatrix(v);
    }

    public void solve(double[][] a, int n) {
        int[] p = new int[n];
        double[][] b = new double[n][n];

        int i;
        for(i = 0; i < n; ++i) {
            p[i] = i;
            b[i][i] = 1.0D;
        }

        int j;
        for(int k = 0; k < n; ++k) {
            double max = 0.0D;
            int col = i;
            int row = i;

            double temp;
            for(i = k; i < n; ++i) {
                for(j = k; j < n; ++j) {
                    temp = Math.abs(b[i][j]);
                    if (max < temp) {
                        max = temp;
                        row = i;
                        col = j;
                    }
                }
            }

            if (row != k) {
                for(j = 0; j < n; ++j) {
                    temp = a[row][j];
                    a[row][j] = a[k][j];
                    a[k][j] = temp;
                    temp = b[row][j];
                    b[row][j] = b[k][j];
                    b[k][j] = temp;
                }

                i = p[row];
                p[row] = p[k];
                p[k] = i;
            }

            if (col != k) {
                for(i = 0; i < n; ++i) {
                    temp = a[i][col];
                    a[i][col] = a[i][k];
                    a[i][k] = temp;
                }
            }

            for(j = k + 1; j < n; ++j) {
                a[k][j] /= a[k][k];
            }

            for(j = 0; j < n; ++j) {
                b[k][j] /= a[k][k];
            }

            a[k][k] = 1.0D;

            for(j = k + 1; j < n; ++j) {
                for(i = 0; i < k; ++i) {
                    a[i][j] -= a[i][k] * a[k][j];
                }

                for(i = k + 1; i < n; ++i) {
                    a[i][j] -= a[i][k] * a[k][j];
                }
            }

            for(j = 0; j < n; ++j) {
                for(i = 0; i < k; ++i) {
                    b[i][j] -= a[i][k] * b[k][j];
                }

                for(i = k + 1; i < n; ++i) {
                    b[i][j] -= a[i][k] * b[k][j];
                }
            }

            for(i = 0; i < k; ++i) {
                a[i][k] = 0.0D;
            }

            a[k][k] = 1.0D;
        }

        for(j = 0; j < n; ++j) {
            for(i = 0; i < n; ++i) {
                a[p[i]][j] = b[i][j];
            }
        }

    }

    public double getData(int row, int col) {
        return this.matrix.getData()[row][col];
    }

    public void put(int r, int c, double value) {
        if (this.matrix != null) {
            this.matrix.setEntry(r, c, value);
        }
    }

    public String toString() {
        return this.matrix.toString();
    }

    public double min() {
        int row = this.matrix.getRowDimension();
        int col = this.matrix.getColumnDimension();
        List<Double> temp = new ArrayList();

        int j;
        for(int i = 0; i < row; ++i) {
            for(j = 0; j < col; ++j) {
                double t = this.matrix.getEntry(i, j);
                if (t != 0.0D) {
                    temp.add(t);
                }
            }
        }

        double[] tempDouble = new double[temp.size()];

        for(j = 0; j < temp.size(); ++j) {
            tempDouble[j] = (Double)temp.get(j);
        }

        return StatUtils.min(tempDouble);
    }

    public double max() {
        int row = this.matrix.getRowDimension();
        int col = this.matrix.getColumnDimension();
        List<Double> temp = new ArrayList();

        int j;
        for(int i = 0; i < row; ++i) {
            for(j = 0; j < col; ++j) {
                double t = this.matrix.getEntry(i, j);
                if (t != 0.0D) {
                    temp.add(t);
                }
            }
        }

        double[] tempDouble = new double[temp.size()];

        for(j = 0; j < temp.size(); ++j) {
            tempDouble[j] = (Double)temp.get(j);
        }

        return StatUtils.max(tempDouble);
    }

    public void addCol(int colNum, double[] col) {
        this.matrix.setColumn(colNum, col);
    }

    public void addRow(int rowNum, double[] row) {
        this.matrix.setRow(rowNum, row);
    }

    public double[][] getData() {
        return this.matrix.getData();
    }

    public double[] getRow(int row) {
        return this.matrix.getRow(row);
    }

    public double[] getCol(int col) {
        return this.matrix.getColumn(col);
    }

    public int getColNum() {
        return this.matrix.getColumnDimension();
    }

    public int getRowNum() {
        return this.matrix.getRowDimension();
    }

    public AbstractRealMatrix getMatrix() {
        return this.matrix;
    }

    public void setMatrix(AbstractRealMatrix matrix) {
        this.matrix = matrix;
    }
}
