package com.nationrel.modules.weibull.util;

import java.util.Enumeration;
import java.util.Vector;

/**
 * @ClassName CSVLine
 * @Description CSV行类
 * @version V1.0
 */
public class CSVLine {
    private Vector<Element> items;

    /**
     * <p>Title: CSVLine</p>
     * <p>Description: CSV行类构造</p>
     */
    public CSVLine() {
        items = new Vector<Element>();
    }

    /**
     * @Title: addItem
     * @Description: 加节点
     * @param item void
     */
    public void addItem(String item) {
        addItem(item, false);
    }

    /**
     * @Title: addItem
     * @Description: 加节点
     * @param item
     * @param enquote void
     */
    public void addItem(String item, boolean enquote) {
        items.addElement(new Element(item, enquote));
    }

    /**
     * @Title: getLine
     * @Description: CSV行
     * @return String
     */
    public String getLine() {
        StringBuffer list = new StringBuffer();
        for (int n = 0; n < items.size(); n ++) {
            Element element = items.elementAt(n);
            String item = element.getItem();
            list.append(item);
            if (items.size() - 1 != n) {
                list.append(',');
            }
        }
        return new String(list);
    }

    /**
     * @Title: size
     * @Description: 节点个数
     * @return int
     */
    public int size() {
        return items.size();
    }

    /**
     * @Title: getItem
     * @Description: 获取节点
     * @param n
     * @return String
     */
    public String getItem(int n) {
        Element element = items.elementAt(n);
        return element.getRawItem();
    }

    /**
     * @Title: removeItem
     * @Description: 删除节点
     * @param n void
     */
    public void removeItem(int n) {
        items.removeElementAt(n);
    }

    /**
     * @Title: elements
     * @Description: CSV节点迭代
     * @return Enumeration<String>
     */
    public Enumeration<String> elements() {
        return new CSVLineEnumerator(items);
    }

    /**
     * @ClassName Element
     * @Description CSV节点
     * @Author vic.fan
     * @CreateDate 2013-10-9
     * @version V1.0
     */
    class Element {
        private String item;
        private boolean enquote;

        Element(String item) {
            this(item, false);
        }

        Element(String item, boolean enquote) {
            this.item = item;
            this.enquote = enquote;
        }

        public String getItem() {
            return enquote(item, enquote);
        }

        public String getRawItem() {
            return item;
        }
    }

    /**
     * @ClassName CSVLineEnumerator
     * @Description CSV行迭代
     * @Author vic.fan
     * @CreateDate 2013-10-9
     * @version V1.0
     */
    class CSVLineEnumerator implements Enumeration<String> {
        private Vector<Element> items;
        private int n;

        CSVLineEnumerator(Vector<Element> items) {
            this.items = items;
            n = 0;
        }

        @Override
        public String nextElement() {
            n ++;
            Element element = items.elementAt(n - 1);
            return element.getRawItem();
        }

        @Override
        public boolean hasMoreElements() {
            return n < items.size();
        }
    }

    /**
     * @Title: enquote
     * @Description: 转换CSV格式
     * @param item
     * @return String
     */
    public static String enquote(String item) {
        return enquote(item, false);
    }

    /**
     * @Title: enquote
     * @Description: 转换CSV格式
     * @param item
     * @param enquote
     * @return String
     */
    public static String enquote(String item, boolean enquote) {
        if (item.length() == 0) {
            return item;
        }
        if (item.indexOf('"') < 0 && item.indexOf(',') < 0 && !enquote) {
            return item;
        }

        StringBuffer sb = new StringBuffer(item.length() * 2 + 2);
        sb.append('"');
        for (int ind = 0; ind < item.length(); ind ++) {
            char ch = item.charAt(ind);
            if ('"' == ch) {
                sb.append("\"\"");
            } else {
                sb.append(ch);
            }
        }
        sb.append('"');

        return new String(sb);
    }
}
