package com.nationrel.modules.weibull.util;

import com.nationrel.modules.weibull.general.common.ResponseStatus;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.InputStream;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * 导入Excel工具类
 */
public class ImportExcelUtil {

    /**
     * 获取IO流中的数据，组装成List<List<Object>>对象
     * @param inStr
     * @param fileName
     * @return
     * @throws Exception
     */
    public static List<List<String>> getListByExcel(InputStream inStr, String fileName, Integer languageKey) throws Exception {
        List<List<String>> list = null;

        //创建Excel工作薄
        XSSFWorkbook wb = getWorkbook(inStr, fileName, languageKey);
        XSSFSheet sheet = null;
        XSSFRow row = null;
        XSSFCell cell = null;

        list = new ArrayList<>();
        //遍历Excel中所有的sheet
        for (int i = 0; i < wb.getNumberOfSheets(); i++) {
            sheet = wb.getSheetAt(i);
            if (sheet == null) {
                continue;
            }
            //遍历当前sheet中的所有行
            for (int j = sheet.getFirstRowNum(); j < sheet.getLastRowNum() + 1; j++) {
                row = sheet.getRow(j);
                if (row == null) {
                    continue;
                }
                //遍历所有的列
                List<String> li = new ArrayList<>();
                for (int y = 0; y < 9; y++) {
                    cell = row.getCell(y);
                    li.add(cell != null ? getCellValue(cell) : StringUtils.EMPTY);
                }
                list.add(li);
            }
        }
        return list;
    }

    /**
     * 根据文件后缀，自适应上传文件的版本
     * @param inStr
     * @param fileName
     * @return
     * @throws Exception
     */
    public static XSSFWorkbook getWorkbook(InputStream inStr, String fileName, Integer languageKey) throws Exception {
        XSSFWorkbook wb = null;
        if (fileName.toLowerCase().contains(EwbConstants.FILE_TYPE_XLSX)) {
            wb = new XSSFWorkbook(inStr);
        } else {
            throw new DescribeException(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ReliaqubeParamDataCheckEnum.请上传正确的文件格式xlsx, languageKey), ResponseStatus.WARN.getStatus());
        }
        return wb;
    }

    /**
     * 对表格中数值进行格式化
     * @param cell
     * @return
     */
    public static String getCellValue(XSSFCell cell) {
        String value = StringUtils.EMPTY;
        DecimalFormat df = new DecimalFormat("0");  //格式化number String字符
        SimpleDateFormat sdf = new SimpleDateFormat("yyy-MM-dd");  //日期格式化
        DecimalFormat df2 = new DecimalFormat("0.00");  //格式化数字

        switch (cell.getCellType()) {
            case STRING:
                value = cell.getRichStringCellValue().getString();
                break;
            case NUMERIC:
                if ("General".equals(cell.getCellStyle().getDataFormatString())) {
                    value = df.format((int) cell.getNumericCellValue());
                } else if ("m/d/yy".equals(cell.getCellStyle().getDataFormatString())) {
                    value = sdf.format(cell.getDateCellValue());
                } else {
                    value = df2.format((int) cell.getNumericCellValue());
                }
                break;
            case BOOLEAN:
                value = cell.getBooleanCellValue() + StringUtils.EMPTY;
                break;
//            case Cell.CELL_TYPE_BLANK:
//                value = "";
//                break;
//            default:
//                break;
        }
        return value;
    }

    /**
     * 集合转JSON字符串
     *
     * @param excelList
     * @return
     */
    public static List<String[]> excelToDatas(List<List<String>> excelList) {
        List<String[]> altaData = new ArrayList<>();
        for (int i = 1; i < excelList.size(); i++) {
            List<String> objects = excelList.get(i);
            altaData.add(new String[]{String.valueOf(i), getStr(objects, 0), getStr(objects, 1), getStr(objects, 2), getStr(objects, 3), getStr(objects, 4), getStr(objects, 5), getStr(objects, 6), getStr(objects, 7), getStr(objects, 8), getStr(objects, 9), getStr(objects, 10), getStr(objects, 11)});
        }
        return altaData;
    }

    /**
     * 取得：字符串
     *
     * @param objects 集合
     * @param index   索引
     * @return
     */
    private static String getStr(List<String> objects, int index) {
        if (index < objects.size()) {
            return objects.get(index);
        }
        return null;
    }
}
