package com.nationrel.modules.weibull.controller;

import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.bo.EwbAnalysisSettingBo;
import com.nationrel.modules.weibull.bo.EwbAnalysisSettingSaveBo;
import com.nationrel.modules.weibull.entity.EwbAnalysisDisWizard;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import com.nationrel.modules.weibull.vo.*;
import com.nationrel.modules.weibull.service.EwbAnalysisDisWizardService;
import com.nationrel.modules.weibull.service.EwbAnalysisService;
import com.nationrel.modules.weibull.service.EwbAnalysisSettingService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import jakarta.validation.Valid;
import jakarta.validation.constraints.NotBlank;
import java.util.List;

/**
 * @author Aimee on 2020/9/14.
 */
@Tag(name = "寿命数据分析-分析部分")
@Slf4j
@Validated
@RestController
@RequestMapping("/weibull/analysis")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisController {

    //    private final RedissonClient redissonClient;
    private final EwbAnalysisService ewbAnalysisService;
    private final EwbAnalysisSettingService ewbAnalysisSettingService;
    private final EwbAnalysisDisWizardService ewbAnalysisDisWizardService;

    @Operation(summary = "分析列表")
    @GetMapping("/list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", required = true, in = ParameterIn.HEADER),
            @Parameter(name = "analysisName", description = "分析名称", in = ParameterIn.QUERY),
            @Parameter(name = "wbId", description = "项目ID", in = ParameterIn.QUERY)
    })
    public ServerResponse<List<EwbAnalysisListVo>> analysisList(@RequestParam(required = false) String analysisName,
                                                                @RequestParam @NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        List<EwbAnalysisListVo> result = ewbAnalysisService.listByPage(wbId, analysisName, userInfo);
        return ServerResponse.createBySuccess(result);
    }

    @Operation(summary = "获取timeForRank")
    @GetMapping("/timeForRank")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "analysisName", description = "分析名称", in = ParameterIn.QUERY),
            @Parameter(name = "wbId", description = "项目ID", in = ParameterIn.QUERY)
    })
    public ServerResponse<String> timeForRank(@RequestParam@NotBlank(message = "DataIdNotEmpty") String dataId) {
        return ewbAnalysisService.timeForRank(dataId);
    }

    @Operation(summary = "分析创建")
    @PostMapping("/create")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse analysisCreate( @RequestBody @Valid EwbAnlaysisCreateVo param) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        String analysisId = UUIDUtil.getUuid();
        EwbAnalysisReqVo analysis = ewbAnalysisService.createAnalysis(userInfo, param.getWbId(), analysisId, param.getAnalysisName(), param.getObjectCode(), param.getObjectName(), param.getDescription());
        return ServerResponse.createBySuccess(analysis);
    }

    @Operation(summary = "分析编辑保存")
    @PostMapping("/save")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse analysisSave( @RequestBody @Valid EwbAnalysisSaveVo param) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return ewbAnalysisService.editProject(userInfo, param);
    }

    @Operation(summary = "无数据时直接画图")
    @PostMapping("/calcNoData")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse calcNoData( @RequestBody @Valid EwbCalcResultVo param) {
        return ewbAnalysisService.calcNoData(param);
    }

    @Operation(summary = "分析删除")
    @GetMapping("/del")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "analysisId", description = "分析ID", in = ParameterIn.QUERY)
    })
    public ServerResponse analysisCreate(@RequestParam @NotBlank(message = "AnalysisIdNotEmpty") String analysisId) {
        return ewbAnalysisService.delAnalysis(analysisId);
    }

    @Operation(summary = "获取参数估计")
    @PostMapping("/getAnalysis-setting")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse getAnalysisSetting(@RequestBody EwbAnalysisSettingBo param) {
        EwbAnalysisSettingVo setting = ewbAnalysisSettingService.getSetting(param);
        return ServerResponse.createBySuccess(setting);
    }

    @Operation(summary = "保存分析设置")
    @PostMapping("/analysis-setting-save")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse saveAnalysisSetting( @RequestBody EwbAnalysisSettingSaveBo param) {
        HeaderUserInfo user = GlobalExceptionHandler.parseUserInfo();
        ewbAnalysisSettingService.saveSetting(user, param);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "分析复制")
    @GetMapping("/copy")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "analysisId", description = "分析ID", in = ParameterIn.QUERY),
            @Parameter(name = "dataId", description = "数据ID", in = ParameterIn.QUERY),
            @Parameter(name = "settingId", description = "设置ID", in = ParameterIn.QUERY)
    })
    public ServerResponse analysisCopy(@NotBlank(message = "AnalysisIdNotEmpty") String analysisId,
                                       @NotBlank(message = "DataIdNotEmpty") String dataId,
                                       @NotBlank(message = "SetIdNotEmpty") String settingId) {
        ewbAnalysisService.analysisCopy(analysisId, dataId, settingId);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "获取分布向导列表")
    @GetMapping("/distribution-wizard")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "analysisId", description = "分析ID", in = ParameterIn.QUERY),
            @Parameter(name = "dataId", description = "数据ID", in = ParameterIn.QUERY)
    })
    public ServerResponse distributionWizard(@RequestParam(required = false) String dataId) {
        List<EwbAnalysisDisWizard> result = ewbAnalysisDisWizardService.getDistributionWizard(dataId);
        return ServerResponse.createBySuccess(result);
    }

    @Operation(summary = "计算分布向导")
    @PostMapping("/distribution-wizard-calc")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse distributionWizardCalc( @RequestBody EwbDistributionVo param) {
        HeaderUserInfo user = GlobalExceptionHandler.parseUserInfo();
        ewbAnalysisDisWizardService.getDistributionWizardCalc(param, user);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "计算数据")
    @PostMapping("/calcResult")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse calcResult( @RequestBody EwbCalcResultVo param) throws Exception {
        return ewbAnalysisDisWizardService.calcResult(param, GlobalExceptionHandler.parseUserInfo());
    }

    @Operation(summary = "选择分布向导")
    @PostMapping("/distribution-wizard-select")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse distributionWizardSelect( @RequestBody EwbDistributionSelectVo param) {
        HeaderUserInfo user = GlobalExceptionHandler.parseUserInfo();
        ewbAnalysisDisWizardService.distributionWizardSelect(param, user);
        return ServerResponse.createBySuccess();
    }

//    @Operation(summary = "分析编辑保存")
//    @PostMapping("/save")
//    @Parameters(value = {
//            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
//    })
//    public ServerResponse analysisSave( @RequestBody @Valid EwbAnalysisSaveVo param) {
//        HeaderUserInfo userInfo = GlobalExceptionHandler.parseHeader(request);
//        RLock rLock = null;
//        boolean isLock = false;
//        try {
//            rLock = redissonClient.getLock(param.getAnalysisId());
//            isLock = rLock.tryLock(redissonWaitTime, redissonLastTime, TimeUnit.MILLISECONDS);
//            if (isLock) {
//                return ewbAnalysisService.editProject(userInfo, param);
//            } else {
//                //其他人正在操作，请稍后处理!
//                return ServerResponse.createByErrorMessage(EwbParamterCheckEnum.getLanguageMessageByKey(EwbParamterCheckEnum.其他人正在操作数据, GlobalExceptionHandler.parseLK(request));
//            }
//        } catch (Exception e) {
//            log.error("节点删除 异常：%s", e);
//            return ServerResponse.createByErrorMessage(EwbParamterCheckEnum.getLanguageMessageByKey(EwbParamterCheckEnum.其他人正在操作数据, GlobalExceptionHandler.parseLK(request));
//        } finally {
//            if (isLock) {
//                rLock.unlock();//释放Redisson锁
//            }
//        }
//    }
}
