package com.nationrel.modules.weibull.controller;

import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.entity.EwbAnalysisSolveDataCreateVo;
import com.nationrel.modules.weibull.entity.EwbAnalysisSolveDataUpdateVo;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import com.nationrel.modules.weibull.service.EwbAnalysisSolveDataService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@Tag(name = "可靠性测试设计 历史数据")
@RestController
@RequestMapping("/weibull/solveData")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisSolveDataController {

    private final EwbAnalysisSolveDataService solveDataService;

    @Operation(summary = "列表")
    @GetMapping
    @Parameters(value = {
            @Parameter(name = "solveName", description = "计算名称"),
            @Parameter(name = "pageIndex", description = "当前页码"),
            @Parameter(name = "pageSize", description = "每页条数")
    })
    public ServerResponse list(
            @RequestParam(required = false) String solveName,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return solveDataService.listByPage(userInfo, solveName, pageIndex, pageSize);
    }

    @Operation(summary = "根据id查询")
    @GetMapping("/{id}")
    @Parameters(value = {
            @Parameter(name = "solveId", description = "计算ID")
    })
    public ServerResponse selectOne(@PathVariable(value = "id") String solveId) {
        return solveDataService.selectOneBySolveId(solveId);
    }

    @Operation(summary = "创建单个")
    @PostMapping
    public ServerResponse create(@RequestBody @Valid EwbAnalysisSolveDataCreateVo createVo) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return solveDataService.create(userInfo, createVo);
    }

    @Operation(summary = "根据id更新")
    @PutMapping
    public ServerResponse update(@RequestBody @Valid EwbAnalysisSolveDataUpdateVo updateVo) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return solveDataService.update(userInfo, updateVo);
    }

    @Operation(summary = "根据id删除")
    @DeleteMapping("/{id}")
    @Parameters(value = {
            @Parameter(name = "solveId", description = "计算ID")
    })
    public ServerResponse delete(@PathVariable(value = "id") String solveId) {
        return solveDataService.deleteBySolveId(solveId);
    }
}
