package com.nationrel.modules.weibull.controller;

import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.entity.EwbInspectInterval;
import com.nationrel.modules.weibull.entity.EwbReplaceInterval;
import com.nationrel.modules.weibull.vo.EwbCostOptimizeCreateVo;
import com.nationrel.modules.weibull.service.EwbCostOptimizeService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import jakarta.validation.constraints.NotBlank;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;

@Tag(name = "成本优化分析")
@Validated
@RestController
@RequestMapping("/weibull/cost")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbCostOptimizeController {

    private final EwbCostOptimizeService optimizeService;

    @Operation(summary = "更换成本创建")
    @PostMapping("/replace")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse createReplace(@RequestBody @Valid EwbCostOptimizeCreateVo param) {
        return ServerResponse.createBySuccess(optimizeService.createReplace(param, GlobalExceptionHandler.parseUserInfo()));
    }

    @Operation(summary = "更换成本创建")
    @GetMapping("/replace")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse replaceList(@NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return optimizeService.selectReplace(wbId);
    }

    @Operation(summary = "获取单个更换成本")
    @GetMapping("/replace/{replaceIntervalId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse getReplace(@PathVariable @NotBlank(message = "AnalysisIdNotEmpty") String replaceIntervalId) {
        return optimizeService.getReplace(replaceIntervalId);
    }

    @Operation(summary = "删除单个更换成本")
    @DeleteMapping("/replace/{replaceIntervalId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse deleteReplace(@PathVariable @NotBlank(message = "AnalysisIdNotEmpty") String replaceIntervalId) {
        return optimizeService.deleteReplace(replaceIntervalId);
    }

    @Operation(summary = "更换成本名称修改")
    @PutMapping("/replace")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse deleteReplace(@RequestBody @Valid EwbReplaceInterval replaceInterval) {
        return optimizeService.editReplace(replaceInterval);
    }

    @Operation(summary = "检测成本创建")
    @PostMapping("/inspect")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse createInspect(@RequestBody @Valid EwbCostOptimizeCreateVo param) {
        return ServerResponse.createBySuccess(optimizeService.createInspect(param, GlobalExceptionHandler.parseUserInfo()));
    }

    @Operation(summary = "检测成本列表")
    @GetMapping("/inspect")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse inspectList(@NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return optimizeService.selectInspect(wbId);
    }

    @Operation(summary = "获取单个更换成本")
    @GetMapping("/inspect/{inspectIntervalId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse getInspect(@PathVariable @NotBlank(message = "AnalysisIdNotEmpty") String inspectIntervalId) {
        return optimizeService.getInspect(inspectIntervalId);
    }

    @Operation(summary = "删除单个更换成本")
    @DeleteMapping("/inspect/{inspectIntervalId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse deleteInspect(@PathVariable @NotBlank(message = "AnalysisIdNotEmpty") String inspectIntervalId) {
        return optimizeService.deleteInspect(inspectIntervalId);
    }

    @Operation(summary = "更换成本名称修改")
    @PutMapping("/inspect")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse editInspect(@RequestBody @Valid EwbInspectInterval inspectInterval) {
        return optimizeService.editInspect(inspectInterval);
    }

}
