package com.nationrel.modules.weibull.controller;

import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.vo.*;
import com.nationrel.modules.weibull.service.*;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import jakarta.validation.constraints.NotBlank;
import java.io.IOException;

/**
 * 退化分析
 */
@Tag(name = "退化分析")
@Validated
@RestController
@RequestMapping("/weibull/degradation")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbDegradationController {

    private final EwbDaInfoService ewbDaInfoService;
    private final EwbDaDataDetailService ewbDaDataDetailService;
    private final EwbDaSettingService ewbDaSettingService;
    private final EwbAnalysisService ewbAnalysisService;
    private final EwbAnalysisSettingService ewbAnalysisSettingService;
    private final EwbAnalysisDataDetailService ewbAnalysisDataDetailService;

    @Operation(summary = "退化创建")
    @PostMapping("/create")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse create(@RequestBody @Valid EwbDegradationCreateVo param) {
        String daId = ewbDaInfoService.create(param.getName(), param.getWbId(), GlobalExceptionHandler.parseUserInfo());
        return ServerResponse.createBySuccess(daId);
    }

    @Operation(summary = "退化列表")
    @GetMapping("/list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "wbId", description = "项目ID", in = ParameterIn.QUERY)
    })
    public ServerResponse list(@NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return ServerResponse.createBySuccess(ewbDaInfoService.list(GlobalExceptionHandler.parseUserInfo(), wbId));
    }

    @Operation(summary = "退化名称修改-删除(修改名称传退化名称，删除时del_flag=1)")
    @PostMapping("/edit")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse edit(@RequestBody @Valid EwbDaInfoSaveVo param) {
        ewbDaInfoService.edit(GlobalExceptionHandler.parseUserInfo(), param);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "数据列表")
    @GetMapping("/data-list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
    })
    public ServerResponse dataList(
            @RequestParam String daId,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "100000000") int pageSize) {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.dataList(daId, pageIndex, pageSize));
    }

    @Operation(summary = "获取分析结果列表")
    @GetMapping("/getResultList")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse getResultList(@NotBlank(message = "") String daId) {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.getResultList(daId));
    }

    @Operation(summary = "数据保存")
    @PostMapping("/data-save")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
    })
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse dataList(@RequestBody @Valid EwbDaDataDetailSaveVo param) {
        ewbDaDataDetailService.dataSave(GlobalExceptionHandler.parseUserInfo(), param);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "获取设置")
    @GetMapping("/get-setting")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
    })
    public ServerResponse getSetting(@NotBlank(message = "DataIdNotEmpty") String daId) {
        return ServerResponse.createBySuccess(ewbDaSettingService.getSetting(daId));
    }

    @Operation(summary = "保存设置")
    @PostMapping("/save-setting")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
    })
    public ServerResponse saveSetting(@RequestBody EwbDaSettingVo param) {
        ewbDaSettingService.update(GlobalExceptionHandler.parseUserInfo().getUserId(), param);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "分析")
    @GetMapping("/analysis")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
    })
    public ServerResponse analysis(@NotBlank(message = "DataIdNotEmpty") String daId, Double xmin, Double xmax, Double ymin, Double ymax) throws Exception {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.analysis(GlobalExceptionHandler.parseUserInfo().getUserId(), daId, xmin, xmax, ymin, ymax, GlobalExceptionHandler.parseUserInfo().getLanguageKey()));
    }

    @Operation(summary = "导入")
    @PostMapping("/importData")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse importData(@RequestParam("file") MultipartFile file, @RequestParam("daId") String daId) throws Exception {
        ewbDaDataDetailService.importData(GlobalExceptionHandler.parseUserInfo(), file, daId);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "导入模板下载")
    @GetMapping("/import-template")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public void importTemplate(HttpServletResponse response) throws IOException {
        ewbDaDataDetailService.importTemplate(response);
    }

    //    @Operation(summary = "调用画图API")
//    @GetMapping("/getResultPlot")
//    @Parameters(value = {
//            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
//    })
    public ServerResponse getResultPlot(@RequestParam("daId") String daId) throws Exception {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.getResultPlot(daId));
    }

    @Operation(summary = "获取退化数据（单元数量、数据量、测量值）")
    @GetMapping("/getDataInfo")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse getDataInfo(@NotBlank(message = "DataIdNotEmpty") String daId) {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.getDataInfo(daId));
    }

    @Operation(summary = "计算模型向导")
    @GetMapping("/calc")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse calc(@NotBlank(message = "DataIdNotEmpty") String daId) throws Exception {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.calc(daId));
    }

    @Operation(summary = "选择模型向导")
    @GetMapping("/selectModel")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse selectModel(@NotBlank(message = "DataIdNotEmpty") String daId, @RequestParam("model") String model) {
        ewbDaSettingService.selectModel(daId, model);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "创建寿命数据分析")
    @PostMapping("/createWeibull")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse createWeibull(@RequestBody EwbDegradationToWeibullVo param) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        String analysisId = UUIDUtil.getUuid();
        EwbAnalysisReqVo analysis = ewbAnalysisService.createAnalysis(userInfo, param.getWbId(), analysisId, param.getAnalysisName(), null, null, null);
        // 更新设置
        param.getSetting().setAnalysisId(analysisId);
        param.getSetting().setDataId(analysis.getDataId());
        param.getSetting().setSettingId(analysis.getSettingId());
        ewbAnalysisSettingService.saveSetting(userInfo, param.getSetting());
        // 更新数据
        param.getDataList().setAnalysisId(analysisId);
        param.getDataList().setDataId(analysis.getDataId());
        if (param.getDataList().getHotDataChange() == null) {
            param.getDataList().setHotDataChange(2);
        }
        if (param.getDataList().getHotDataRankChange() == null) {
            param.getDataList().setHotDataRankChange(0);
        }
        if (StringUtils.isBlank(param.getDataList().getTimeForRank())) {
            param.getDataList().setTimeForRank(EwbConstants.FLAG_NO_CHECKED);
        }
        ewbAnalysisDataDetailService.saveData(userInfo, param.getDataList(), false);
        return ServerResponse.createBySuccess(analysis);
    }

    @Operation(summary = "复制")
    @GetMapping("/copy")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse copy(@NotBlank(message = "DataIdNotEmpty") String daId) {
        return ServerResponse.createBySuccess(ewbDaInfoService.copy(daId));
    }

    @Operation(summary = "导出")
    @GetMapping("/export")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public void export(HttpServletResponse response, @NotBlank(message = "DataIdNotEmpty") String daId) {
        ewbDaDataDetailService.export(response, daId, GlobalExceptionHandler.parseLk());
    }
}
