package com.nationrel.modules.weibull.controller;

import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.bo.EwbGroupListSaveBo;
import com.nationrel.modules.weibull.service.EwbGroupService;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import jakarta.validation.constraints.NotBlank;

/**
 * @author Aimee on 2020/11/23.
 */
@Tag(name = "寿命数据分组")
@RestController
@RequestMapping("/weibull/group")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbGroupController {
    private final EwbGroupService ewbGroupService;

    @Operation(summary = "分组列表")
    @GetMapping("/list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse getGroupList(@NotBlank(message = "DataIdNotEmpty") String dataId) {
        return ServerResponse.createBySuccess(ewbGroupService.getGroupList(dataId));
    }

    @Operation(summary = "分组列表保存")
    @PostMapping("/save")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse saveGroupList(@RequestBody @Valid EwbGroupListSaveBo param) {
        ewbGroupService.saveGroupList(GlobalExceptionHandler.parseUserInfo(), param);
        return ServerResponse.createBySuccess();
    }

}
