package com.nationrel.modules.weibull.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.entity.EwbAnalysisInfo;
import com.nationrel.modules.weibull.vo.EwbAnalysisInfoCreateVo;
import com.nationrel.modules.weibull.vo.EwbAnalysisInfoSaveVo;
import com.nationrel.modules.weibull.service.EwbAnalysisInfoService;
import com.nationrel.modules.weibull.service.EwbAnalysisService;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import org.apache.ibatis.annotations.Param;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author Aimee on 2020/10/19.
 */
@Tag(name = "寿命数据分析-项目部分")
@Validated
@RestController
@RequestMapping("/weibull/project")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbProjectController {
    private final EwbAnalysisService analysisService;
    private final EwbAnalysisInfoService ewbAnalysisInfoService;

    @Operation(summary = "项目创建")
    @PostMapping("/create")
    @RequiresPermissions("weibull:ewb_analysis_info:add")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public Result projectCreate(@RequestBody @Valid EwbAnalysisInfoCreateVo param) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return ewbAnalysisInfoService.createAnalysisInfo(userInfo, param.getWbName(), param.getObjectCode(), param.getObjectName(), param.getDescription(), param.getAnalysisType().replaceAll("/lifeData/", ""));
    }

    @Operation(summary = "项目删除")
    @DeleteMapping("/delete")
    @RequiresPermissions("weibull:ewb_analysis_info:delete")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "wbId", description = "项目编号", in = ParameterIn.QUERY)
    })
    public Result delete(@RequestParam(name = "wbId", required = true) String wbId) {
        ewbAnalysisInfoService.delete(wbId);
        return Result.ok();
    }

    @Operation(summary = "项目列表")
    @GetMapping("/list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "analysisName", description = "分析名称", in = ParameterIn.QUERY),
            @Parameter(name = "analysisType", description = "分析类型(life:综合分析;weibull:weibull分析;degradation:退化分析;costOpti:成本优化分析;nevadaNRI:内华达分析NRI;nevadaRI:内华达分析RI;)", in = ParameterIn.QUERY),
            @Parameter(name = "pageIndex", description = "当前页码", in = ParameterIn.QUERY),
            @Parameter(name = "pageSize", description = "每页条数", in = ParameterIn.QUERY)
    })
    public Result<IPage<EwbAnalysisInfo>> projectList(HttpServletRequest request,
                                                                                              @RequestParam(required = false) String analysisName,
                                                                                              @Param("analysisType") String analysisType,
                                                                                              @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
                                                                                              @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        analysisType = analysisType.replace("/lifeData/", "");
        return Result.OK(ewbAnalysisInfoService.listByPage(analysisName, userInfo, analysisType, pageIndex, pageSize));
    }

    @Operation(summary = "项目列表")
    @GetMapping("/statistic")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse statistic() {
        return analysisService.statistic(GlobalExceptionHandler.parseUserInfo());
    }

    @Operation(summary = "项目编辑保存")
    @PostMapping("/save")
    @RequiresPermissions("weibull:ewb_analysis_info:edit")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public Result projectSave(@RequestBody @Valid EwbAnalysisInfoSaveVo param) {
        return ewbAnalysisInfoService.editProject(GlobalExceptionHandler.parseUserInfo(), param.getWbId(), param.getWbName(), param.getObjectCode(), param.getObjectName(), param.getDescription());
    }

}
