package com.nationrel.modules.weibull.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.*;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.general.util.common.BasicDataUtil;
import com.nationrel.modules.weibull.general.util.common.NumberUtil;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import com.nationrel.modules.weibull.mapper.EwbAnalysisCalcHistoryMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisSolveDataDetailMapper;
import com.nationrel.modules.weibull.service.*;
import com.nationrel.modules.weibull.util.StringUtil;
import com.nationrel.modules.weibull.util.ValidateUtil;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.vo.*;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import org.apache.http.client.utils.URIBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 可靠性测试设计
 */
@Tag(name = "可靠性测试设计")
@RestController
@RequestMapping("/weibull/reliatest")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbReliaTestController {

    private final RgaApiService rgaApiService;
    private final EwbValidatorService validatorService;
    private final EwbAnalysisCalcHistoryMapper calcHistoryMapper;
    private final EwbAnalysisSolveDataService solveDataService;
    private final EwbAnalysisSolveDataDetailMapper solveDataDetailMapper;
    private final EwbAnalysisSolveDataDetailService solveDataDetailService;
    private final AllVariableConfig allVariableConfig;


    @Operation(summary = "计算结果列表")
    @GetMapping("/list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "calcType", description = "计算类型", in = ParameterIn.QUERY)
    })
    public ServerResponse<List<EwbAnalysisCalcHistory>> projectList(HttpServletRequest request, @RequestParam int calcType) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return ServerResponse.createBySuccess(calcHistoryMapper.selectList(new QueryWrapper<EwbAnalysisCalcHistory>().lambda().eq(EwbAnalysisCalcHistory::getUserName, userInfo.getUserName()).eq(EwbAnalysisCalcHistory::getCalcType, calcType).orderByDesc(EwbAnalysisCalcHistory::getInsertTime).last("limit 10")));
    }

    @Operation(summary = "参数二项式-可靠性")
    @PostMapping("/solveTimeRel")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse solveTimeRel(HttpServletRequest request, @RequestBody SolveTimeRelVo param) throws Exception {
        String reliaQubeUrl = allVariableConfig.getReliaQubeUrl();
        String serverSystem = allVariableConfig.getServerSystem();
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = userInfo.getLanguageKey();
        StringBuilder parameter = new StringBuilder(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.可靠度, languageKey) + "  ");
        parameter.append(EwbDistributingCodeEnum.getByLaguageKey(param.getDistributingCode(), languageKey)).append("  ");
        if (ValidateUtil.isEmpty(param.getCalcCode()) || (!"1".equals(param.getCalcCode()) && !"2".equals(param.getCalcCode()))) {
            throw new DescribeException(EwbCustomPromptEnum.请选择正确的计算方式, languageKey);
        }
        validateRange(parameter, param.getConfidenceLevel(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信水平, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        validateRange(parameter, param.getReliability(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.可靠度, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        validateRange(parameter, param.getTimeRdemo(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.要求时间, languageKey), EwbConstants.TAB_FAILURE_END_TIME_MIN, EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);
        validateRange(parameter, param.getAllowFailures(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.失效, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);
        validateRange(parameter, param.getSampleSize(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.测试单元, languageKey), new BigDecimal(param.getAllowFailures()), EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);
        if (!param.getDistributingCode().equals(EwbDistributingCodeEnum.Exponential.getIndex())) {
            String checkName = "";
            if (param.getDistributingCode().equals(EwbDistributingCodeEnum.Normal.getIndex())) {
                checkName = "Std";
            } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode())) {
                checkName = "LogStd";
            } else if (param.getDistributingCode().equals(EwbDistributingCodeEnum.Weibull.getIndex())) {
                checkName = "β";
            }
            validateRange(parameter, param.getShape(), checkName, EwbConstants.CALC_COMMON_MIN, EwbConstants.CALC_COMMON_MAX, languageKey);
        }
        EwbAnalysisSolveData solveData = JSONObject.parseObject(JSON.toJSONString(param), EwbAnalysisSolveData.class);
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setUpdateTime(new Date());
        solveData.setVerficationCode(EwbVerficationTypeEnum.可靠度.getIndex());
        solveData.setSolveCode(EwbSolveTypeEnum.参数二项式.getIndex());

        param.setDistributingCode(EwbDistributingCodeEnum.getName(param.getDistributingCode()));
        URIBuilder uri = new URIBuilder();
        if ("1".equals(param.getCalcCode())) {
            uri = new URIBuilder(reliaQubeUrl + EwbTestDesignEnum.getPathByIndex(EwbTestDesignEnum.SOLVE_TIME_REL.getIndex(), serverSystem));
            uri.setParameter("SampleSize", param.getSampleSize());
        } else if ("2".equals(param.getCalcCode())) {
            uri = new URIBuilder(reliaQubeUrl + EwbTestDesignEnum.getPathByIndex(EwbTestDesignEnum.SOLVE_TIME_REL_SIZE.getIndex(), serverSystem));
            uri.setParameter("TimeTest", param.getSampleSize());
        }
        uri.setParameter("distribution", param.getDistributingCode());
        uri.setParameter("Rdemo", StringUtil.parseString(WeibullNumberUtil.parseDouble(param.getReliability()) / 100));
        uri.setParameter("ConfLevel", StringUtil.parseString(WeibullNumberUtil.parseDouble(param.getConfidenceLevel()) / 100));
        uri.setParameter("TimeRdemo", param.getTimeRdemo());
        uri.setParameter("AllowFailures", param.getAllowFailures());
        uri.setParameter("Shape", ValidateUtil.isEmpty(param.getShape()) ? "1" : param.getShape());
        String resultsCallApi = rgaApiService.getResultsCallApi("", uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        if (resultsCallApi.indexOf("error") > 0 || resultsCallApi.indexOf("Exception") > 0) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.无法计算, userInfo.getLanguageKey()));
        }
        calcHistoryMapper.insert(EwbAnalysisCalcHistory.builder().userName(userInfo.getUserName()).calcType(1).parameter(parameter.toString()).result(resultsCallApi).insertTime(new Date()).build());
        HashMap<String, Object> result = createCalcResult(resultsCallApi, userInfo.getUserName(), 1);

        solveDataService.solveSave(result, solveData, param.getSolveId(), userInfo);
        return ServerResponse.createBySuccess(result);
    }

    @Operation(summary = "参数二项式-MTTF")
    @PostMapping("/solveTimeMTTF")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse solveTimeMttf(HttpServletRequest request, @RequestBody SolveTimeMTTFVo param) throws Exception {
        String reliaQubeUrl = allVariableConfig.getReliaQubeUrl();
        String serverSystem = allVariableConfig.getServerSystem();
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = userInfo.getLanguageKey();
        StringBuilder parameter = new StringBuilder("MTTF  ");
        if (ValidateUtil.isEmpty(param.getCalcCode()) || (!"1".equals(param.getCalcCode()) && !"2".equals(param.getCalcCode()))) {
            throw new DescribeException(EwbCustomPromptEnum.请选择正确的计算方式, languageKey);
        }
        validateRange(parameter, param.getMTTF(), "MTTF", EwbConstants.TAB_FAILURE_END_TIME_MIN, EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);
        validateRange(parameter, param.getConfidenceLevel(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信水平, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        validateRange(parameter, param.getAllowFailures(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.失效, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);
        validateRange(parameter, param.getSampleSize(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.测试单元, languageKey), new BigDecimal(param.getAllowFailures()), EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);
        if (!param.getDistributingCode().equals(EwbDistributingCodeEnum.Exponential.getIndex())) {
            String checkName = "";
            if (param.getDistributingCode().equals(EwbDistributingCodeEnum.Normal.getIndex())) {
                checkName = "Std";
            } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode())) {
                checkName = "LogStd";
            } else if (param.getDistributingCode().equals(EwbDistributingCodeEnum.Weibull.getIndex())) {
                checkName = "β";
            }
            validateRange(parameter, param.getShape(), checkName, EwbConstants.CALC_COMMON_MIN, EwbConstants.CALC_COMMON_MAX, languageKey);
        }

        EwbAnalysisSolveData solveData = JSONObject.parseObject(JSON.toJSONString(param), EwbAnalysisSolveData.class);
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setUpdateTime(new Date());
        solveData.setVerficationCode(EwbVerficationTypeEnum.MTTF.getIndex());
        solveData.setTimeRdemo((double) 0);
        solveData.setSolveCode(EwbSolveTypeEnum.参数二项式.getIndex());

        param.setDistributingCode(EwbDistributingCodeEnum.getName(param.getDistributingCode()));
        URIBuilder uri = new URIBuilder();
        if ("1".equals(param.getCalcCode())) {
            uri = new URIBuilder(reliaQubeUrl + EwbTestDesignEnum.getPathByIndex(EwbTestDesignEnum.SOLVE_TIME_MTTF.getIndex(), serverSystem));
            uri.setParameter("SampleSize", param.getSampleSize());
        } else if ("2".equals(param.getCalcCode())) {
            uri = new URIBuilder(reliaQubeUrl + EwbTestDesignEnum.getPathByIndex(EwbTestDesignEnum.SOLVE_TIME_MTTF_SIZE.getIndex(), serverSystem));
            uri.setParameter("TimeTest", param.getSampleSize());
        }
        uri.setParameter("distribution", param.getDistributingCode());
        uri.setParameter("MTTF", param.getMTTF());
        uri.setParameter("ConfLevel", StringUtil.parseString(WeibullNumberUtil.parseDouble(param.getConfidenceLevel()) / 100));
        uri.setParameter("AllowFailures", param.getAllowFailures());
        uri.setParameter("Shape", ValidateUtil.isEmpty(param.getShape()) ? "1" : param.getShape());
        String resultsCallApi = rgaApiService.getResultsCallApi("", uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        if (resultsCallApi.indexOf("error") > 0 || resultsCallApi.indexOf("Exception") > 0) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.无法计算, userInfo.getLanguageKey()));
        }
        calcHistoryMapper.insert(EwbAnalysisCalcHistory.builder().userName(userInfo.getUserName()).calcType(1).parameter(parameter.toString()).result(resultsCallApi).insertTime(new Date()).build());
        HashMap<String, Object> result = createCalcResult(resultsCallApi, userInfo.getUserName(), 1);

        solveDataService.solveSave(result, solveData, param.getSolveId(), userInfo);
        return ServerResponse.createBySuccess(result);
    }

    @Operation(summary = "非参数二项式-可靠性")
    @PostMapping("/solveReliability")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse solveReliability(HttpServletRequest request, @RequestBody SolveReliabilityVo param) throws Exception {
        String serverSystem = allVariableConfig.getServerSystem();
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = userInfo.getLanguageKey();
        StringBuilder parameter = new StringBuilder(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.可靠度, languageKey) + "  ");
        validateRange(parameter, param.getConfidenceLevel(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信水平, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        validateRange(parameter, param.getSampleSize(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.测试单元, languageKey), new BigDecimal(param.getAllowFailures()), EwbConstants.CALC_SAMPLE_SIZE_MAX, languageKey);
        validateRange(parameter, param.getAllowFailures(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.失效数, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);

        EwbAnalysisSolveData solveData = JSONObject.parseObject(JSON.toJSONString(param), EwbAnalysisSolveData.class);
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setUpdateTime(new Date());
        solveData.setVerficationCode(EwbVerficationTypeEnum.可靠度.getIndex());
        solveData.setSolveCode(EwbSolveTypeEnum.非参数二项式.getIndex());

        URIBuilder uri = new URIBuilder(allVariableConfig.getReliaQubeUrl() + EwbTestDesignEnum.getPathByIndex(EwbTestDesignEnum.SOLVE_RELIABILITY.getIndex(), serverSystem));
        uri.setParameter("ConfLevel", StringUtil.parseString(WeibullNumberUtil.parseDouble(param.getConfidenceLevel()) / 100));
        uri.setParameter("SampleSize", param.getSampleSize());
        uri.setParameter("AllowFailures", param.getAllowFailures());
        String resultsCallApi = rgaApiService.getResultsCallApi("", uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        if (resultsCallApi.indexOf("error") > 0 || resultsCallApi.indexOf("Exception") > 0) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.无法计算, userInfo.getLanguageKey()));
        }
        calcHistoryMapper.insert(EwbAnalysisCalcHistory.builder().userName(userInfo.getUserName()).calcType(2).parameter(parameter.toString()).result(resultsCallApi).insertTime(new Date()).build());
        HashMap<String, Object> result = createCalcResult(StringUtil.parseString(WeibullNumberUtil.parseDouble(resultsCallApi) * 100), userInfo.getUserName(), 2);

        solveDataService.solveSave(result, solveData, param.getSolveId(), userInfo);
        return ServerResponse.createBySuccess(result);
    }

    @Operation(summary = "非参数二项式-置信水平")
    @PostMapping("/solveConfidence")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse solveConfidence(HttpServletRequest request, @RequestBody SolveConfidenceVo param) throws Exception {
        String serverSystem = allVariableConfig.getServerSystem();
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = userInfo.getLanguageKey();
        StringBuilder parameter = new StringBuilder(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信水平, languageKey) + "  ");
        validateRange(parameter, param.getReliability(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.可靠性, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        validateRange(parameter, param.getSampleSize(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.测试单元, languageKey), new BigDecimal(param.getAllowFailures()), EwbConstants.CALC_SAMPLE_SIZE_MAX, languageKey);
        validateRange(parameter, param.getAllowFailures(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.失效数, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);

        EwbAnalysisSolveData solveData = JSONObject.parseObject(JSON.toJSONString(param), EwbAnalysisSolveData.class);
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setUpdateTime(new Date());
        solveData.setVerficationCode(EwbVerficationTypeEnum.置信水平.getIndex());
        solveData.setSolveCode(EwbSolveTypeEnum.非参数二项式.getIndex());

        URIBuilder uri = new URIBuilder(allVariableConfig.getReliaQubeUrl() + EwbTestDesignEnum.getPathByIndex(EwbTestDesignEnum.SOLVE_CONFIDENCE.getIndex(), serverSystem));
        uri.setParameter("Reliability", StringUtil.parseString(WeibullNumberUtil.parseDouble(param.getReliability()) / 100));
        uri.setParameter("SampleSize", param.getSampleSize());
        uri.setParameter("AllowFailures", param.getAllowFailures());
        String resultsCallApi = rgaApiService.getResultsCallApi("", uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        if (resultsCallApi.indexOf("error") > 0 || resultsCallApi.indexOf("Exception") > 0) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.无法计算, userInfo.getLanguageKey()));
        }
        calcHistoryMapper.insert(EwbAnalysisCalcHistory.builder().userName(userInfo.getUserName()).calcType(2).parameter(parameter.toString()).result(resultsCallApi).insertTime(new Date()).build());
        HashMap<String, Object> result = createCalcResult(resultsCallApi, userInfo.getUserName(), 2);

        solveDataService.solveSave(result, solveData, param.getSolveId(), userInfo);
        return ServerResponse.createBySuccess(result);
    }

    @Operation(summary = "非参数二项式-单元数量")
    @PostMapping("/solveSampleSize")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse solveSampleSize(HttpServletRequest request, @RequestBody SolveSampleSizeVo param) throws Exception {
        String serverSystem = allVariableConfig.getServerSystem();
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = userInfo.getLanguageKey();
        StringBuilder parameter = new StringBuilder(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.单元数量, languageKey) + "  ");
        validateRange(parameter, param.getReliability(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.可靠性, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);
        validateRange(parameter, param.getConfidenceLevel(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信水平, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        validateRange(parameter, param.getAllowFailures(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.失效数, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, new BigDecimal("10000"), languageKey);

        EwbAnalysisSolveData solveData = JSONObject.parseObject(JSON.toJSONString(param), EwbAnalysisSolveData.class);
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setUpdateTime(new Date());
        solveData.setVerficationCode(EwbVerficationTypeEnum.单元数量.getIndex());
        solveData.setSolveCode(EwbSolveTypeEnum.非参数二项式.getIndex());

        URIBuilder uri = new URIBuilder(allVariableConfig.getReliaQubeUrl() + EwbTestDesignEnum.getPathByIndex(EwbTestDesignEnum.SOLVE_SAMPLE_SIZE.getIndex(), serverSystem));
        uri.setParameter("Reliability", StringUtil.parseString(WeibullNumberUtil.parseDouble(param.getReliability()) / 100));
        uri.setParameter("ConfLevel", StringUtil.parseString(WeibullNumberUtil.parseDouble(param.getConfidenceLevel()) / 100));
        uri.setParameter("AllowFailures", param.getAllowFailures());
        String resultsCallApi = rgaApiService.getResultsCallApi("", uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        if (resultsCallApi.indexOf("error") > 0 || resultsCallApi.indexOf("Exception") > 0) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.无法计算, userInfo.getLanguageKey()));
        }
        calcHistoryMapper.insert(EwbAnalysisCalcHistory.builder().userName(userInfo.getUserName()).calcType(2).parameter(parameter.toString()).result(resultsCallApi).insertTime(new Date()).build());
        HashMap<String, Object> result = createCalcResult(resultsCallApi, userInfo.getUserName(), 2);

        solveDataService.solveSave(result, solveData, param.getSolveId(), userInfo);
        return ServerResponse.createBySuccess(result);
    }


    @Operation(summary = "指数卡方-可靠性")
    @PostMapping("/solveTimeChiSquareRel")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse solveTimeChiSquareRel(HttpServletRequest request, @RequestBody SolveTimeChiSquareRelVo param) throws Exception {
        String serverSystem = allVariableConfig.getServerSystem();
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = userInfo.getLanguageKey();
        StringBuilder parameter = new StringBuilder(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.可靠度, languageKey) + "  ");
        validateRange(parameter, param.getReliability(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.可靠性, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        validateRange(parameter, param.getConfidenceLevel(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信水平, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        validateRange(parameter, param.getAllowFailures(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.允许的失效数, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, new BigDecimal("100000"), languageKey);
        validateRange(parameter, param.getTime(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.时间, languageKey), EwbConstants.TAB_FAILURE_END_TIME_MIN, EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);

        EwbAnalysisSolveData solveData = JSONObject.parseObject(JSON.toJSONString(param), EwbAnalysisSolveData.class);
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setUpdateTime(new Date());
        solveData.setVerficationCode(EwbVerficationTypeEnum.可靠度.getIndex());
        solveData.setSolveCode(EwbSolveTypeEnum.指数卡方.getIndex());

        URIBuilder uri = new URIBuilder(allVariableConfig.getReliaQubeUrl() + EwbTestDesignEnum.getPathByIndex(EwbTestDesignEnum.SOLVE_TIME_CHI_SQUARE_REL.getIndex(), serverSystem));
        uri.setParameter("rdemo", StringUtil.parseString(WeibullNumberUtil.parseDouble(param.getReliability()) / 100));
        uri.setParameter("confidenceLevel", StringUtil.parseString(WeibullNumberUtil.parseDouble(param.getConfidenceLevel()) / 100));
        uri.setParameter("time", param.getTime());
        uri.setParameter("allowedFailures", param.getAllowFailures());
        String resultsCallApi = rgaApiService.getResultsCallApi("", uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        if (resultsCallApi.indexOf("error") > 0 || resultsCallApi.indexOf("Exception") > 0) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.无法计算, userInfo.getLanguageKey()));
        }
        calcHistoryMapper.insert(EwbAnalysisCalcHistory.builder().userName(userInfo.getUserName()).calcType(3).parameter(parameter.toString()).result(resultsCallApi).insertTime(new Date()).build());
        HashMap<String, Object> result = createCalcResult(resultsCallApi, userInfo.getUserName(), 3);

        solveDataService.solveSave(result, solveData, param.getSolveId(), userInfo);
        return ServerResponse.createBySuccess(result);
    }


    @Operation(summary = "指数卡方-MTTF")
    @PostMapping("/solveTimeChiSquareMTTF")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse solveTimeChiSquareMttf(HttpServletRequest request, @RequestBody SolveTimeChiSquareMTTFVo param) throws Exception {
        String serverSystem = allVariableConfig.getServerSystem();
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = userInfo.getLanguageKey();
        StringBuilder parameter = new StringBuilder("MTTF  ");
        validateRange(parameter, param.getMttf(), "MTTF", EwbConstants.TAB_FAILURE_END_TIME_MIN, EwbConstants.TAB_FAILURE_END_TIME_MAX, languageKey);
        validateRange(parameter, param.getConfidenceLevel(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信水平, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        validateRange(parameter, param.getAllowFailures(), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.允许的失效数, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, new BigDecimal("100000"), languageKey);

        EwbAnalysisSolveData solveData = JSONObject.parseObject(JSON.toJSONString(param), EwbAnalysisSolveData.class);
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setUpdateTime(new Date());
        solveData.setVerficationCode(EwbVerficationTypeEnum.MTTF.getIndex());
        solveData.setSolveCode(EwbSolveTypeEnum.指数卡方.getIndex());

        URIBuilder uri = new URIBuilder(allVariableConfig.getReliaQubeUrl() + EwbTestDesignEnum.getPathByIndex(EwbTestDesignEnum.SOLVE_TIME_CHI_SQUARE_MTTF.getIndex(), serverSystem));
        uri.setParameter("MTTF", param.getMttf());
        uri.setParameter("ConfLevel", StringUtil.parseString(WeibullNumberUtil.parseDouble(param.getConfidenceLevel()) / 100));
        uri.setParameter("AllowFailures", param.getAllowFailures());
        String resultsCallApi = rgaApiService.getResultsCallApi("", uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        if (resultsCallApi.indexOf("error") > 0 || resultsCallApi.indexOf("Exception") > 0) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.无法计算, userInfo.getLanguageKey()));
        }
        calcHistoryMapper.insert(EwbAnalysisCalcHistory.builder().userName(userInfo.getUserName()).calcType(3).parameter(parameter.toString()).result(resultsCallApi).insertTime(new Date()).build());
        HashMap<String, Object> result = createCalcResult(resultsCallApi, userInfo.getUserName(), 3);

        solveDataService.solveSave(result, solveData, param.getSolveId(), userInfo);
        return ServerResponse.createBySuccess(result);
    }


    @Operation(summary = "指数卡方-Weibull")
    @PostMapping("/solveWeibullChiSquareRel")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse solveWeibullChiSquareRel(HttpServletRequest request, @RequestBody SolveDemoRelWeibullChiSquareRelVo param) throws Exception {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = userInfo.getLanguageKey();
        StringBuilder parameter = new StringBuilder(EwbWeibullChiSquaredEnum.getLanguageMessageByKey(EwbWeibullChiSquaredEnum.getWeibullChiSquaredEnum(param.getVerficationCode()), languageKey) + "  ");
        validateRange(parameter, param.getBeta(), "β", EwbConstants.CALC_COMMON_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);

        //如果未修改表格数据，取库中数据计算
        isUpdateTableData(param);

        URIBuilder uri = new URIBuilder(allVariableConfig.getReliaQubeUrl() + EwbWeibullChiSquaredEnum.getPathByIndex(param.getVerficationCode(), allVariableConfig.getServerSystem()));
        uri.setParameter("beta", param.getBeta());
        if (EwbWeibullChiSquaredEnum.SOLVEDEMOREL_WEIBULLCHISQUARED.getIndex().equals(param.getVerficationCode())) {
            validateRange(parameter, param.getConfidenceLevel(), EwbWeibullChiSquaredParamEnum.getLanguageMessageByKey(EwbWeibullChiSquaredParamEnum.ConfidenceLevel, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            validateRange(parameter, param.getConfidenceLevel(), EwbWeibullChiSquaredParamEnum.getLanguageMessageByKey(EwbWeibullChiSquaredParamEnum.TestTimeperSys, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
            uri.setParameter("ConfidenceLevel", Double.parseDouble(param.getConfidenceLevel()) / 100 + "");
            uri.setParameter("TimeforR", param.getTime());
        } else if (EwbWeibullChiSquaredEnum.SOLVECONFIDENCE_WEIBULLCHISQUARED.getIndex().equals(param.getVerficationCode())) {
            validateRange(parameter, param.getReliability(), EwbWeibullChiSquaredParamEnum.getLanguageMessageByKey(EwbWeibullChiSquaredParamEnum.ConfidenceLevel, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            validateRange(parameter, param.getConfidenceLevel(), EwbWeibullChiSquaredParamEnum.getLanguageMessageByKey(EwbWeibullChiSquaredParamEnum.TestTimeperSys, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
            uri.setParameter("Reliability", Double.parseDouble(param.getConfidenceLevel()) / 100 + "");
            uri.setParameter("TimeforR", param.getTime());
        } else if (EwbWeibullChiSquaredEnum.SOLVEMTTF_WEIBULLCHISQUARED.getIndex().equals(param.getVerficationCode())) {
            validateRange(parameter, param.getConfidenceLevel(), EwbWeibullChiSquaredParamEnum.getLanguageMessageByKey(EwbWeibullChiSquaredParamEnum.ConfidenceLevel, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            uri.setParameter("ConfidenceLevel", Double.parseDouble(param.getConfidenceLevel()) / 100 + "");
        } else if (EwbWeibullChiSquaredEnum.SOLVECONFIDENCE_WEIBULLCHISQUAREDMTTF.getIndex().equals(param.getVerficationCode())) {
            validateRange(parameter, param.getMttf(), "MTTF", EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
            uri.setParameter("MTTF", param.getMttf());
        }

        // 验证数据的合法性
        validatorService.validatorAnalysisDetailData(param.getDataDetails(), EwbConstants.FLAG_CHECKED, param.getStartDataNo(), EwbConstants.FLAG_CHECKED, param.getEndDataNo(), BasicDataUtil.EMPTY_STR, languageKey);
        // 获取分析明细数据
        List<EwbAnalysisDataDetail> collect = param.getDataDetails().stream().map(item -> {
            EwbAnalysisDataDetail e = new EwbAnalysisDataDetail();
            e.setQuantity(WeibullNumberUtil.parseInteger(item[0]));
            e.setFailureEndTime(WeibullNumberUtil.parseDouble(item[1]));
            e.setFailureSuspension(item[2]);
            e.setComment1(item[3]);
            return e;
        }).collect(Collectors.toList());
        List<EwbAnalysisDataDetail> listDataDetails = CollectionUtils.isEmpty(param.getAnalysisDataDetails()) ? collect : param.getAnalysisDataDetails();
        int totalNumFailureAndSuspensions = 0;
        int totalNumSuspensions = 0;
        int totalNumFailures = 0;
        int totalNumofRows = 0;
        int totalFailureRows = 0;
        int totalSuspensionRows = 0;
        if (listDataDetails != null && listDataDetails.size() > 0) {
            Iterator<EwbAnalysisDataDetail> sListIterator = listDataDetails.iterator();
            while (sListIterator.hasNext()) {
                EwbAnalysisDataDetail weiMaterial = sListIterator.next();
                // 删除集合中WeiMaterial对象属性fratios为空的对象s
                if (weiMaterial.getFailureEndTime() == null) {
                    sListIterator.remove();
                } else {
                    totalNumFailureAndSuspensions += weiMaterial.getQuantity();
                    totalNumofRows += 1;
                    if (EwbConstants.STATE_SUSPENSION.equalsIgnoreCase(weiMaterial.getFailureSuspension())) {
                        totalSuspensionRows += 1;
                        totalNumSuspensions += weiMaterial.getQuantity();
                        weiMaterial.setFailureSuspension(EwbConstants.STATE_S);
                    } else if (EwbConstants.STATE_FAILURE.equalsIgnoreCase(weiMaterial.getFailureSuspension())) {
                        totalFailureRows += 1;
                        totalNumFailures += weiMaterial.getQuantity();
                        weiMaterial.setFailureSuspension(EwbConstants.STATE_F);
                    }
                }
            }
        }
        param.setTotalNumFailureAndSuspensions(totalNumFailureAndSuspensions);
        param.setTotalNumSuspensions(totalNumSuspensions);
        param.setTotalNumFailures(totalNumFailures);
        param.setTotalNumofRows(totalNumofRows);
        param.setTotalFailureRows(totalFailureRows);
        param.setTotalSuspensionRows(totalSuspensionRows);
        param.setAnalysisDataDetails(listDataDetails);

        //表格详细数据
        String dataId = solveDataDetailService.saveWeibullDatas(param.getDataId(), listDataDetails);

        EwbAnalysisSolveData solveData = JSONObject.parseObject(JSON.toJSONString(param), EwbAnalysisSolveData.class);
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setUpdateTime(new Date());
        solveData.setDataId(dataId);
        solveData.setSolveCode(EwbSolveTypeEnum.指数卡方Weibull.getIndex());

        JSONObject requestLifeData = rgaApiService.getRequestLifeDataJson(BasicDataUtil.EMPTY_STR,
                rgaApiService.getDataType(EwbConstants.FLAG_NO_CHECKED, EwbConstants.FLAG_CHECKED), param, false);
        if (requestLifeData == null) {
            return ServerResponse.createBySuccess();
        }
        JSONObject requsetJson = new JSONObject();
        requsetJson.put("Data", requestLifeData);
        String resultsCallApi = rgaApiService.getResultsCallApi(requsetJson.toString(), uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        if (resultsCallApi.indexOf("error") > 0 || resultsCallApi.indexOf("Exception") > 0) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.无法计算, userInfo.getLanguageKey()));
        }
        calcHistoryMapper.insert(EwbAnalysisCalcHistory.builder().userName(userInfo.getUserName()).calcType(5).parameter(parameter.toString()).result(resultsCallApi).insertTime(new Date()).build());
        HashMap<String, Object> result = createCalcResult(resultsCallApi, userInfo.getUserName(), 5);

        solveDataService.solveSave(result, solveData, param.getSolveId(), userInfo);
        result.put("dataId", dataId);
        return ServerResponse.createBySuccess(result);
    }

    @Operation(summary = "修正有效性验证测试")
    @PostMapping("/solveFixEffectiviness")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse solveFixEffectiviness(HttpServletRequest request, @RequestBody SolveFixEffectivinessVo param) throws Exception {
        String serverSystem = allVariableConfig.getServerSystem();
        // ===================================================
        // 此计算共需要计算3步
        // 1、计算FailureRank
        // (1)、处理计算FailureRank需要的数据
        // (2)、拼接调用计算接口的url
        // (3)、计算FailureRank
        // ===================================================
        // 2、计算各类型的计算结果
        // (1)、处理计算各类型需要的数据
        // (2)、拼接调用计算接口的url
        // (3)、计算各类型计算结果
        // ===================================================
        // 3、计算修正后的数据的计算结果
        // (1)、处理计算修正后数据需要的数据
        // (2)、拼接调用计算接口的url
        // (3)、计算结果
        // ===================================================
        // 4、处理计算结果数据
        // (1)、处理第二步计算结果的数据，根据不同的设置返回计算结果
        // (2)、处理第三步计算结果的数据
        // ===================================================
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = userInfo.getLanguageKey();
        StringBuilder parameter = new StringBuilder(EwbFixEffectivinessEnum.getLanguageMessageByKey(EwbFixEffectivinessEnum.SOLVE_FIXEFFECTIVINESS, languageKey) + "  ");
        //如果未修改表格数据，取库中数据计算
        isUpdateTableData(param);
        // 1、计算FailureRank
        // (1)、处理计算FailureRank需要的数据
        SolveFixEffectivinessVo effectivinessVo = new SolveFixEffectivinessVo(param.getDefectStartDataNo(), param.getDefectEndDataNo(), param.getFixStartDataNo(), param.getFixEndDataNo(), param.getDataDetails(), param.getDefectDatas(), param.getFixDatas(), param.getDefectEndDataNo(), 0, 0, param.getDefectEndDataNo(), 0, 0);
        List<EwbAnalysisDataDetail> listDefectDetails = SolveFixEffectivinessVo.parseDefectDatas(effectivinessVo, userInfo.getLanguageKey());
        SolveDemoRelWeibullChiSquareRelVo chiSquareRelVo = new SolveDemoRelWeibullChiSquareRelVo(1, effectivinessVo.getDefectEndDataNo(), listDefectDetails, effectivinessVo.getDefectEndDataNo(), effectivinessVo.getTotalNumSuspensions(), effectivinessVo.getTotalNumFailures(), effectivinessVo.getTotalNumOfRows(), effectivinessVo.getTotalNumFailures(), effectivinessVo.getTotalSuspensionRows());
        // (2)、拼接调用计算接口的url
        String uriFailureRank = allVariableConfig.getReliaQubeUrl() + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_FAILURERANK);
        // (3)、计算FailureRank
        JSONObject requestLifeData = rgaApiService.getRequestLifeDataJson(BasicDataUtil.EMPTY_STR, rgaApiService.getDataType(EwbConstants.FLAG_CHECKED, EwbConstants.FLAG_NO_CHECKED), chiSquareRelVo, false);
        if (requestLifeData == null) {
            return ServerResponse.createBySuccess();
        }
        requestLifeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.endTime), 0);
        String responseFailureRank = rgaApiService.getResultsCallApi(requestLifeData.toString(), uriFailureRank, EwbConstants.RGA_API_CONTENTTYPE_1, null);

        // 2、计算各类型的计算结果
        // (1)、拼接调用计算接口的url
        StringBuilder uriDefect = new StringBuilder(allVariableConfig.getReliaQubeUrl());
        uriDefect.append(EwbFixEffectivinessParamEnum.getPathByIndex(param.getVerficationCode(), serverSystem));
        parameter.append(EwbFixEffectivinessParamEnum.getLanguageMessageByKey(EwbFixEffectivinessParamEnum.getFixEffectivinessParamEnum(param.getVerficationCode()), languageKey)).append(" ");
        if (EwbConstants.SERVER_SYSTEM_LINUX.equals(serverSystem)){
            uriDefect.append("01".equals(param.getEstimationMethodCode()) ? "?analysisType=RRX" : "?analysisType=MLE");
        } else {
            uriDefect.append("01".equals(param.getEstimationMethodCode()) ? "/RRX" : "/MLE");
        }
        parameter.append("01".equals(param.getEstimationMethodCode()) ? "RRX" : "MLE" + " ");
        // (2)、处理计算各类型需要的数据
        requestLifeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), JSONObject.parse(responseFailureRank));
        // (3)、计算各类型计算结果
        String responseDefectResult = rgaApiService.getResultsCallApi(requestLifeData.toString(), new URIBuilder(uriDefect.toString()).build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);

        if (responseDefectResult.indexOf("error") > 0) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.无法使用所选分布分析数据, languageKey);
        }

        // 3、计算修正后的数据的计算结果
        // (1)、处理计算修正后数据需要的数据
        List<EwbAnalysisDataDetail> listFixDetails = SolveFixEffectivinessVo.parseFixDatas(effectivinessVo, userInfo.getLanguageKey());
        chiSquareRelVo = new SolveDemoRelWeibullChiSquareRelVo(1, effectivinessVo.getFixEndDataNo(), listFixDetails, 0, 0, 0, 0, 0, 0);
        requestLifeData = rgaApiService.getRequestLifeDataJson(BasicDataUtil.EMPTY_STR, rgaApiService.getDataType(EwbConstants.FLAG_NO_CHECKED, EwbConstants.FLAG_NO_CHECKED), chiSquareRelVo, false);
        if (requestLifeData == null) {
            return ServerResponse.createBySuccess();
        }
        requestLifeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), JSONObject.parse(responseFailureRank));
        JSONObject requestFixData = new JSONObject();
        requestFixData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data), requestLifeData);
        requestFixData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results), JSONObject.parse(responseDefectResult));
        // (2)、拼接调用计算接口的url
        URIBuilder uriFix = new URIBuilder(allVariableConfig.getReliaQubeUrl() + EwbTestDesignEnum.getPathByIndex(EwbTestDesignEnum.SOLVEMTBF_NHPPCHISQUARED.getIndex(), serverSystem));
        // (3)、计算结果
        String responseFixResult = rgaApiService.getResultsCallApi(requestFixData.toString(), new URIBuilder(uriFix.toString()).build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);

        // 4、处理计算结果数据
        // (1)、处理第二步计算结果的数据，根据不同的设置返回计算结果
        List<EwbCalcResultKeyValueVo> resultVos = EwbAnalysisDisWizardService.handleCalcResult(serverSystem, JSONObject.parseObject(responseDefectResult), new EwbAnalysisResult(), rgaApiService, EwbCalcResultVo.builder().distributingCode(param.getVerficationCode().equals(1) || param.getVerficationCode().equals(2) ? EwbDistributingCodeEnum.Weibull.getIndex() : param.getVerficationCode().equals(3) ? EwbDistributingCodeEnum.Exponential.getIndex() : param.getVerficationCode().equals(4) ? EwbDistributingCodeEnum.Normal.getIndex() : EwbDistributingCodeEnum.Lognormal.getIndex()).parameterCode(param.getVerficationCode().equals(2) ? EwbParameterCodeEnum.三参数.getIndex() : EwbParameterCodeEnum.二参数.getIndex()).build(), false);
        resultVos.forEach(resultVo -> parameter.append(resultVo.getKey()).append(":").append(resultVo.getValue()).append(" "));
        String calcResult = NumberUtil.formatDouble(Double.parseDouble(responseFixResult) * 100, 2) + "%";
        // (2)、处理第三步计算结果的数据
        calcHistoryMapper.insert(EwbAnalysisCalcHistory.builder().userName(userInfo.getUserName()).calcType(EwbDRTCalcTypeEnum.修正有效性试验.getIndex()).parameter(parameter.toString()).result(calcResult).insertTime(new Date()).build());

        //保存两个表格数据
        HashMap<String, String> map = solveDataDetailService.saveFixEffectivinessDatas(param.getDefectdataId(), param.getFixDataId(), param.getDefectDatas(), param.getFixDatas());

        EwbAnalysisSolveData solveData = JSONObject.parseObject(JSON.toJSONString(param), EwbAnalysisSolveData.class);
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setUpdateTime(new Date());
        solveData.setDefectdataId(map.get("defuuid"));
        solveData.setFixDataId(map.get("fixuuid"));
        solveData.setSolveCode(EwbSolveTypeEnum.修正有效性验证测试.getIndex());

        solveData.setCalcResult(calcResult);
        solveData.setCalcResults(JSONObject.toJSONString(resultVos));

        String solveId = solveDataService.solveSave(null, solveData, param.getSolveId(), userInfo);
        return ServerResponse.createBySuccess(SolveFixEffectivinessCalcResultVo.builder().calucResult(calcResult).calucResults(resultVos).calucHistory(calcHistoryMapper.selectList(new QueryWrapper<EwbAnalysisCalcHistory>().lambda().eq(EwbAnalysisCalcHistory::getUserName, userInfo.getUserName()).eq(EwbAnalysisCalcHistory::getCalcType, EwbDRTCalcTypeEnum.修正有效性试验.getIndex()).orderByDesc(EwbAnalysisCalcHistory::getInsertTime).last("limit 10"))).defectdataId(map.get("defuuid")).fixDataId(map.get("fixuuid")).solveId(solveId).build());
    }

    @Operation(summary = "可修系统分布")
    @PostMapping("/solveRepairePoisson")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse solveRepairePoisson(HttpServletRequest request, @RequestBody SolveRepairePoissonVo param) throws Exception {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = userInfo.getLanguageKey();
        StringBuilder parameter = new StringBuilder(EwbRepairePoissionEnum.getLanguageMessageByKey(EwbRepairePoissionEnum.getRepairePoissionEnum(param.getVerficationCode()), languageKey) + "  ");
        validateRange(parameter, param.getTimeForMtbf(), EwbRepairePoissionParamEnum.getLanguageMessageByKey(EwbRepairePoissionParamEnum.timeforMTBF, languageKey), EwbConstants.TAB_FAILURE_END_TIME_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
        validateRange(parameter, param.getBeta(), "β", EwbConstants.CALC_COMMON_MIN, EwbConstants.CALC_BETA_MAX, languageKey);
        validateRange(parameter, param.getNumofF(), EwbRepairePoissionParamEnum.getLanguageMessageByKey(EwbRepairePoissionParamEnum.timeforMTBF, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);

        URIBuilder uri = new URIBuilder(allVariableConfig.getReliaQubeUrl() + EwbRepairePoissionEnum.getPathByIndex(param.getVerficationCode(), allVariableConfig.getServerSystem()));
        uri.setParameter("beta", param.getBeta());
        uri.setParameter("NumofF", param.getNumofF());
        uri.setParameter("timeforMTBF", param.getTimeForMtbf());
        if (EwbRepairePoissionEnum.SOLVETIME_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode()) || EwbRepairePoissionEnum.SOLVENUMOFSYST_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode()) || EwbRepairePoissionEnum.SOLVEMTBF_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode())) {
            validateRange(parameter, param.getVerficationCode(), EwbRepairePoissionParamEnum.getLanguageMessageByKey(EwbRepairePoissionParamEnum.ConfidenceLevel, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            uri.setParameter("ConfidenceLevel", Double.parseDouble(param.getVerficationCode()) / 100 + "");
        }
        if (EwbRepairePoissionEnum.SOLVENUMOFSYST_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode()) || EwbRepairePoissionEnum.SOLVEMTBF_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode()) || EwbRepairePoissionEnum.SOLVECONFIDENCE_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode())) {
            validateRange(parameter, param.getTestTimeperSys(), EwbRepairePoissionParamEnum.getLanguageMessageByKey(EwbRepairePoissionParamEnum.TestTimeperSys, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
            uri.setParameter("TestTimeperSys", param.getTestTimeperSys());
        }
        if (EwbRepairePoissionEnum.SOLVETIME_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode()) || EwbRepairePoissionEnum.SOLVENUMOFSYST_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode()) || EwbRepairePoissionEnum.SOLVECONFIDENCE_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode())) {
            validateRange(parameter, param.getMtbf(), "MTBF", EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
            uri.setParameter("MTBF", param.getMtbf());
        }
        if (EwbRepairePoissionEnum.SOLVETIME_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode()) || EwbRepairePoissionEnum.SOLVEMTBF_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode()) || EwbRepairePoissionEnum.SOLVECONFIDENCE_NHPPCHISQUARED.getIndex().equals(param.getVerficationCode())) {
            validateRange(parameter, param.getMtbf(), EwbRepairePoissionParamEnum.getLanguageMessageByKey(EwbRepairePoissionParamEnum.numofSyst, languageKey), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
            uri.setParameter("numofSyst", param.getNumofSyst());
        }

        EwbAnalysisSolveData solveData = JSONObject.parseObject(JSON.toJSONString(param), EwbAnalysisSolveData.class);
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setUpdateTime(new Date());
        solveData.setSolveCode(EwbSolveTypeEnum.可修系统分布.getIndex());

        String resultsCallApi = rgaApiService.getResultsCallApi("", uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        if (resultsCallApi.indexOf("error") > 0 || resultsCallApi.indexOf("Exception") > 0) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.无法计算, userInfo.getLanguageKey()));
        }
        calcHistoryMapper.insert(EwbAnalysisCalcHistory.builder().userName(userInfo.getUserName()).calcType(4).parameter(parameter.toString()).result(resultsCallApi).insertTime(new Date()).build());
        HashMap<String, Object> result = createCalcResult(resultsCallApi, userInfo.getUserName(), 4);

        solveData.setCalcResult((result.get("result").toString()));

        solveDataService.solveSave(result, solveData, param.getSolveId(), userInfo);
        return ServerResponse.createBySuccess(result);
    }

    public HashMap<String, Object> createCalcResult(String resultsCallApi, String userName, int calacType) {
        HashMap<String, Object> result = new HashMap<>();
        result.put("result", resultsCallApi);
        result.put("calcHistory", calcHistoryMapper.selectList(new QueryWrapper<EwbAnalysisCalcHistory>().lambda().eq(EwbAnalysisCalcHistory::getUserName, userName).eq(EwbAnalysisCalcHistory::getCalcType, calacType).orderByDesc(EwbAnalysisCalcHistory::getInsertTime).last("limit 10")));
        return result;
    }

    private void validateRange(StringBuilder parameter, String val, String name, BigDecimal min, BigDecimal max, Integer languageKey) {
        parameter.append(name).append(":").append(val).append("  ");
        // 范围验证
        if (ValidateUtil.isEmpty(val)) {
            throw new DescribeException(name, EwbCustomPromptEnum.不可为空, languageKey);
        }
        if (!ValidateUtil.isNumber(val)) {
            throw new DescribeException(name, EwbCustomPromptEnum.必须为数字, languageKey);
        }
        if (name.equals(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.失效, languageKey))) {
            if (WeibullNumberUtil.parseBigDecimal(val).compareTo(min) < 0 || WeibullNumberUtil.parseBigDecimal(val).compareTo(max) > 0) {
                throw new DescribeException(name, min, max, EwbCustomPromptEnum.范围必须是, -1);
            }
        } else {
            if (WeibullNumberUtil.parseBigDecimal(val).compareTo(min) < 0 || WeibullNumberUtil.parseBigDecimal(val).compareTo(max) > 0) {
                throw new DescribeException(name, min, max, EwbCustomPromptEnum.范围必须是, -1);
            }
        }
    }

    /**
     * 根据表格数据是否修改过标识，取数据库中数据
     * @param param
     */
    private void isUpdateTableData(SolveFixEffectivinessVo param) {
        // 1:修改过，不查询
        if (!"1".equals(param.getIsUpdateDefectTableData())) {
            List<String[]> defectDatasStringList = EwbAnalysisSolveDataDetail.getDefectDatasStringList(solveDataDetailMapper.selectList(new QueryWrapper<EwbAnalysisSolveDataDetail>().lambda().eq(EwbAnalysisSolveDataDetail::getDataId, param.getDefectdataId())));
            param.setDefectDatas(defectDatasStringList);
            param.setDefectEndDataNo(defectDatasStringList.size());
        }
        if (!"1".equals(param.getIsUpdateFixTableData())) {
            List<String[]> fixDatasStringList = EwbAnalysisSolveDataDetail.getFixDatasStringList(solveDataDetailMapper.selectList(new QueryWrapper<EwbAnalysisSolveDataDetail>().lambda().eq(EwbAnalysisSolveDataDetail::getDataId, param.getFixDataId())));
            param.setFixDatas(fixDatasStringList);
            param.setFixEndDataNo(fixDatasStringList.size());
        }
    }

    /**
     * 根据表格数据是否修改过标识，取数据库中数据
     * @param param
     */
    private void isUpdateTableData(SolveDemoRelWeibullChiSquareRelVo param) {
        // 1:修改过，不查询
        if (!"1".equals(param.getIsUpdateHotData())) {
            List<String[]> hotDatasStringList = EwbAnalysisSolveDataDetail.getDatasStringList(solveDataDetailMapper.selectList(new QueryWrapper<EwbAnalysisSolveDataDetail>().lambda().eq(EwbAnalysisSolveDataDetail::getDataId, param.getDataId())));
            param.setDataDetails(hotDatasStringList);
            param.setEndDataNo(hotDatasStringList.size());
        }
    }
}
