package com.nationrel.modules.weibull.controller;

import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.vo.EwbStressStrengthCreateVo;
import com.nationrel.modules.weibull.vo.EwbStressStrengthEditNameVo;
import com.nationrel.modules.weibull.vo.EwbStressStrengthEditVo;
import com.nationrel.modules.weibull.vo.EwbStressStrengthVo;
import com.nationrel.modules.weibull.service.EwbStressStrengthService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;

import jakarta.validation.constraints.NotBlank;
import java.util.List;

/**
 * ss曲线分析
 */
@Tag(name = "ss曲线分析")
@Validated
@RestController
@RequestMapping("/weibull/stress")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbStressStrengthController {

    private final EwbStressStrengthService stressStrengthService;

    @Operation(summary = "ss曲线分析列表")
    @GetMapping("/list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "wbId", description = "项目ID", in = ParameterIn.QUERY)
    })
    public ServerResponse<List<EwbStressStrengthVo>> analysisList(@RequestParam @NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return stressStrengthService.listByPage(wbId);
    }

    @Operation(summary = "分析列表")
    @GetMapping("/analysisList")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "wbId", description = "项目ID", in = ParameterIn.QUERY),
            @Parameter(name = "analysisId", description = "分析ID", in = ParameterIn.QUERY),
            @Parameter(name = "distributingCode", description = "分布类型", in = ParameterIn.QUERY),
            @Parameter(name = "type", description = "应力强度类型", in = ParameterIn.QUERY)
    })
    public ServerResponse analysisList(@RequestParam @NotBlank(message = "AnalysisIdNotEmpty") String wbId, @RequestParam(required = false) String analysisId, @RequestParam @NotBlank(message = "DistributingCodeNotEmpty") String distributingCode, @RequestParam @NotBlank(message = "StressStrengthNotEmpty") String type) {
        return stressStrengthService.selectAnalysisList(wbId, analysisId, distributingCode, type);
    }

    @Operation(summary = "ss曲线分析创建")
    @PostMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse create(@RequestBody @Valid EwbStressStrengthCreateVo param) {
        return ServerResponse.createBySuccess(stressStrengthService.create(param.getName(), param.getWbId(), GlobalExceptionHandler.parseUserInfo()));
    }

    @Operation(summary = "ss曲线分析更新")
    @PutMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse edit(@RequestBody @Valid EwbStressStrengthEditVo entity) {
        return stressStrengthService.edit(entity);
    }

    @Operation(summary = "ss曲线分析更新名称")
    @PutMapping("/name")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse editName(@RequestBody @Valid EwbStressStrengthEditNameVo entity) {
        return stressStrengthService.editName(entity);
    }

    @Operation(summary = "ss曲线单条信息获取")
    @GetMapping("/{id}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "id", description = "分析ID", in = ParameterIn.QUERY)
    })
    public ServerResponse create(@PathVariable(value = "id") String id) {
        return stressStrengthService.getById(id);
    }

    @Operation(summary = "ss曲线单条信息删除")
    @DeleteMapping("/{id}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "id", description = "分析ID", in = ParameterIn.QUERY)
    })
    public ServerResponse delete(@PathVariable(value = "id") String id) {
        return stressStrengthService.deleteById(id);
    }
}
