package com.nationrel.modules.weibull.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.dto.Pagination;
import com.nationrel.modules.weibull.entity.EwbAnalysisDataDetail;
import com.nationrel.modules.weibull.entity.EwbAnalysisRank;
import com.nationrel.modules.weibull.vo.EwbAnalysisDataDetailVo;
import com.nationrel.modules.weibull.vo.EwbAnalysisDataMaxMinSVo;
import com.nationrel.modules.weibull.vo.EwbGroupListVo;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.util.List;

public interface EwbAnalysisDataDetailMapper extends BaseMapper<EwbAnalysisDataDetail> {

    List<EwbAnalysisDataDetailVo> searchEwbDetailData(@Param("dataId") String dataId);

    /**
     * @Title: deleteByDataId
     * @Description: 根据dataId删除随机数
     * @param dataId void
     */
    int deleteByDataId(@Param("dataId") String dataId);

    /**
     * @Title: failureTimeByComplete
     * @Description: 获取结束时间都相等的完整数据类型的结束时间
     * @param dataId 数据ID
     * @return double failureTime
     */
    double failureTimeByComplete(@Param("dataId") String dataId);

    /**
     * @Title: failureTimeByRight
     * @Description: 获取结束时间都相等的右删失类型的结束时间
     * @param dataId 数据ID
     * @return double 数据量
     */
    double failureTimeByRight(@Param("dataId") String dataId);

    /**
     * @Title: countLeftByFtEqu
     * @Description: 获取结束时间都相等的左删失数据类型的数据量
     * @param dataId 数据ID
     * @return int 数据量
     */
    int countLeftByFtEqu(@Param("dataId") String dataId);

    /**
     * @Title: countIntervalByFtEqu
     * @Description: 获取结束时间都相等的区间数据类型的数据量
     * @param dataId 数据ID
     * @return int 数据量
     */
    int countIntervalByFtEqu(@Param("dataId") String dataId);

    /**
     * @Title: countRightByFtEqu
     * @Description: 获取结束时间都相等的右删失类型的数据量
     * @param dataId 数据ID
     * @return int 数据量
     */
    int countRightByFtEqu(@Param("dataId") String dataId);

    /**
     * @Title: countRdataType
     * @Description: 获取右删失类型的数据量
     * @param dataId 数据ID
     * @return int 数据量
     */
    int countRdataType(@Param("dataId") String dataId);

    /**
     * @Title: countCompleteByFtEqu
     * @Description: 获取结束时间都相等的完整数据类型的数据量
     * @param dataId 数据ID
     * @return int 数据量
     */
    int countCompleteByFtEqu(@Param("dataId") String dataId);
    @Select("select count(distinct failure_end_time) from ewb_ANALYSIS_DATA_DETAIL where  DATA_ID = #{dataId} and F_S = '1'")
    int countfCompleteByFtEqu(@Param("dataId") String dataId);
    @Select("select count(1) from ewb_ANALYSIS_DATA_DETAIL where  DATA_ID = #{dataId} and F_S = '0'")
    int countsCompleteByFtEqu(@Param("dataId") String dataId);

    /**
     * @Title: sumCompleteQuantity
     * @Description: 统计完整数据类型的数据量总和
     * @param dataId 数据ID
     * @return int 数据量
     */
    int sumCompleteQuantity(@Param("dataId") String dataId);


    /**
     * @Title: sumLeftQuantity
     * @Description: 统计左删失数据类型的数据量总和
     * @param dataId 数据ID
     * @return int 数据量
     */
    int sumLeftQuantity(@Param("dataId") String dataId);

    /**
     * @Title: sumIntervalQuantity
     * @Description: 统计区间数据类型的数据量总和
     * @param dataId 数据ID
     * @return int 数据量
     */
    int sumIntervalQuantity(@Param("dataId") String dataId);

    /**
     * @Title: countCquantity
     * @Description: 获取完整数据类型的数据量
     * @param dataId 数据ID
     * @return int 数据量
     */
    int countCquantity(@Param("dataId") String dataId);

    /**
     * @Title: countRquantity
     * @Description: 获取右删失数据类型的数据量
     * @param dataId 数据ID
     * @return int 数据量
     */
    int countRquantity(@Param("dataId") String dataId);

    /**
     * @Title: countLquantity
     * @Description: 获取左删失数据类型的数据量
     * @param dataId 数据ID
     * @return int 数据量
     */
    int countLquantity(@Param("dataId") String dataId);

    /**
     * @Title: countIquantity
     * @Description: 获取区间数据类型的数据量
     * @param dataId 数据ID
     * @return int 数据量
     */
    int countIquantity(@Param("dataId") String dataId);

    /**
     * @Title: updateDataTypeEqC
     * @Description: 更新符合完整数据的数据类型为完整数据
     * @param dataId 数据ID
     */
    void updateDataTypeEqC(@Param("dataId") String dataId);

    /**
     * @Title: updateDataTypeEqR
     * @Description: 更新符合右删失数据的数据类型为右删失数据
     * @param dataId 数据ID
     */
    void updateDataTypeEqR(@Param("dataId") String dataId);

    /**
     * @Title: updateDataTypeEqL
     * @Description: 更新符合左删失数据的数据类型为左删失数据
     * @param dataId 数据ID
     */
    void updateDataTypeEqL(@Param("dataId") String dataId);

    /**
     * @Title: updateDataTypeEqI
     * @Description: 更新符合区间数据的数据类型为区间数据
     * @param dataId 数据ID
     */
    void updateDataTypeEqI(@Param("dataId") String dataId);

    /**
     * @Title: updateLastInspectTime
     * @Description: 将明细数据的最近检查时间更新为"最近检查时间=结束时间"数据
     * @param dataId 数据ID
     */
    int updateLastInspectTime(@Param("dataId") String dataId);

    /**
     * @Title: updateQuantityEqDefault
     * @Description: 将明细数据的数量更新为默认值等于1的
     * @param dataId 数据ID
     */
    int updateQuantityEqDefault(@Param("dataId") String dataId);

    /**
     * @Title: batchInsert
     * @Description: 批量插入分析明细数据
     * @param detailData 明细数据
     * @param userInfo 用户信息
     */
    void batchInsert(
            @Param("userInfo") HeaderUserInfo userInfo,
            @Param("detailData")List<EwbAnalysisDataDetail> detailData);

    /**
     * @Title: updateDetailDataNum
     * @Description: 修改明细数据的数据编号
     * @param changeCount 新增或者删除的数据条数
     * @param dataId 数据ID
     * @param endDataNum 结束数据编号
     */
    void updateDetailDataNum(@Param("changeCount") Integer changeCount, @Param("dataId") String dataId, @Param("endDataNum") Integer endDataNum);

    /**
     * @Title: delDetailDataByDataId
     * @Description: 根据数据ID--删除分析明细数据
     * @param startDataNum 删除开始数据编号
     * @param endDataNum 删除结束数据编号
     * @param dataId 数据ID
     */
    void delDetailDataByDataId(@Param("dataId") String dataId , @Param("startDataNum") Integer startDataNum , @Param("endDataNum") Integer endDataNum);

    /**
     * @Title: searchSampleByDataIdForFailure
     * @Description: 根据数据ID、最大数据量、X最小值、X范围对分析秩抽样 仅针对0失效
     * @param minX X最小值
     * @param rangeX X范围
     * @param rankMaximum 最大数据量
     * @param dataId 数据ID
     * @return List<RldAnalysisRank>
     */
    List<EwbAnalysisRank> searchSampleByDataIdForFailure(
            @Param("minX") double minX,
            @Param("rangeX") double rangeX,
            @Param("rankMaximum") int rankMaximum,
            @Param("dataId") String dataId);

    /**
     * @Title: searchOneByDataIdForFailure
     * @Description: 根据数据ID搜索一条明细数据(仅针对0失效)
     * @param dataId 数据ID
     * @return RldAnalysisRank
     */
    EwbAnalysisRank searchOneByDataIdForFailure(@Param("dataId") String dataId);

    /**
     * @Title: searchByDataIdForFailure
     * @Description: 根据数据ID搜索分析数据(仅针对0失效)
     * @param dataId 数据ID
     * @return List<RldAnalysisRank>
     */
    List<EwbAnalysisRank> searchByDataIdForFailure(@Param("dataId") String dataId);
    /**
     * @Title: searchSampleByDataIdForFailureExp
     * @Description: 根据数据ID、最大数据量、X最小值、X范围对分析秩抽样 (仅针对0失效指数1参数)
     * @param minX X最小值
     * @param rangeX X范围
     * @param rankMaximum 最大数据量
     * @param dataId 数据ID
     * @return List<RldAnalysisRank>
     */
    List<EwbAnalysisRank> searchSampleByDataIdForFailureExp(
            @Param("minX") double minX,
            @Param("rangeX") double rangeX,
            @Param("rankMaximum") int rankMaximum,
            @Param("dataId") String dataId);

    /**
     * @Title: searchMaxAndMinFailureTime
     * @Description: 根据数据ID搜索分析明细数据最大失效时间的对数、最小失效时间的对数
     * @param dataId 数据ID
     * @return RldAnalysisDataDetailModel
     */
    EwbAnalysisDataDetailVo searchMaxAndMinFailureTime(@Param("dataId") String dataId);

    /**
     * 查找最大f值和最小s值
     * @param dataId
     * @return
     */
    @Select("select max(failure_end_time) from ewb_ANALYSIS_DATA_DETAIL where DATA_ID = #{dataId} and F_S = '1'")
    double searchMaxFtime(@Param("dataId") String dataId);

    @Select("select max(failure_end_time) as maxSuspensionTime,min(failure_end_time) as minSuspensionTime from ewb_ANALYSIS_DATA_DETAIL where DATA_ID = #{dataId} and F_S = '0'")
    EwbAnalysisDataMaxMinSVo searchMaxAndMinStime(@Param("dataId") String dataId);
    /**
     * @Title: countLdaDatas
     * @Description: 统计寿命数据量
     * @param dataId 数据Id
     * @return int
     */
    int countEwbDatas(@Param("dataId") String dataId);
    /**
     * @Title: selectDatasByDataId
     * @Description: 获取寿命数据
     * @param pagination 分页
     * @param dataId 数据Id
     * @return List<RldAnalysisDataDetail>
     */
    List<EwbAnalysisDataDetail> selectDatasByDataId(
            @Param("dataId") String dataId,
            @Param("pagination") Pagination pagination);

    /**
     * @Title: selectByDataIdOrderByTime
     * @Description: 根据主键获取排序后数据集合
     * @param dataId 数据ID
     * @return List<RldAnalysisDataDetail>
     */
    List<EwbAnalysisDataDetail> selectByDataIdOrderByTime(@Param("dataId") String dataId);

    /**
     * @Title: sumQuantity
     * @Description: 产品数量和
     * @param dataId 数据ID
     * @return int
     */
    Integer sumQuantity(@Param("dataId") String dataId);

    /**
     * @Title: sumRightQuantity
     * @Description: 统计右删失数据类型的数据量总和
     * @param dataId 数据ID
     * @return int 数据量
     */
    Integer sumRightQuantity(@Param("dataId") String dataId);

    /**
     * @Title: sumRightQuantity
     * @Description: 统计S数据类型的数据量总和
     * @param dataId 数据ID
     * @return int 数据量
     */
    @Select("select if(sum(QUANTITY) is null,'0', sum(QUANTITY)) from ewb_ANALYSIS_DATA_DETAIL where DATA_ID = #{dataId,jdbcType=CHAR} and F_S = 0")
    int sumSuspensionQuantity(@Param("dataId") String dataId);

    /**
     * @Title: searchOneByDataIdForFailureExp
     * @Description: 根据数据ID搜索一条明细数据(仅针对0失效指数1参数)
     * @param dataId 数据ID
     * @return RldAnalysisRank
     */
    EwbAnalysisRank searchOneByDataIdForFailureExp(@Param("dataId") String dataId);
    /**
     * @Title: countFdata
     * @Description: 获取失效数据
     * @param dataId 数据ID
     * @return int 数据量
     */
    int countFdata(@Param("dataId") String dataId);

    /**
     * @Title: searchByDataIdForFailureExp
     * @Description: 根据数据ID搜索分析数据(仅针对0失效指数1参数)
     * @param dataId 数据ID
     * @return List<RldAnalysisRank>
     */
    List<EwbAnalysisRank> searchByDataIdForFailureExp(@Param("dataId") String dataId);

    /**
     * @Title: updateF
     * @Description: 将明细数据"F或者S状态"修改为F
     * @param dataId 数据ID
     */
    int updateF(@Param("dataId") String dataId);

    @Insert("INSERT into ewb_analysis_data_detail(DATA_ID, DATA_NO, QUANTITY, LAST_INSPECT_TIME, F_S, FAILURE_END_TIME, FAILURE_TIME, DATA_TYPE, COMMENT1, COMMENT2, COMMENT3, COMMENT4, COMMENT5) " +
            "select #{targetDataId}, DATA_NO, QUANTITY, LAST_INSPECT_TIME, F_S, FAILURE_END_TIME, FAILURE_TIME, DATA_TYPE, COMMENT1, COMMENT2, COMMENT3, COMMENT4, COMMENT5 from ewb_analysis_data_detail where DATA_ID = #{dataId}")
    void copyData(@Param("dataId") String dataId, @Param("targetDataId") String targetDataId);

    List<EwbAnalysisDataDetailVo> searchExportDetailData(@Param("dataId") String dataId);

    int statSumQuantity(@Param("dataId") String dataId, @Param("startDataNo") Integer startDataNo, @Param("endDataNo") Integer endDataNo);

    @Select("select COMMENT1 as name, count(DATA_ID) as quantity from ewb_analysis_data_detail where DATA_ID = #{dataId} GROUP BY COMMENT1")
    List<EwbGroupListVo> getGroupList(@Param("dataId") String dataId);

    void copyGddToAdd(
            @Param("analysisDataId") String analysisDataId,
            @Param("groupingDataId") String groupingDataId,
            @Param("comment") String comment,
            @Param("calcOperatorType") String calcOperatorType);

    /**
     * @Title: copyGddToAddS
     * @Description: 复制分组明细表数据到分析明细数据表(copy为S的数据)
     * @param analysisDataId 分析数据ID
     * @param groupingDataId 分组数据ID
     * @param comment 分组值
     * @param calcOperatorType 运算符号
     */
    void copyGddToAddS(
            @Param("analysisDataId") String analysisDataId,
            @Param("groupingDataId") String groupingDataId,
            @Param("comment") String comment,
            @Param("calcOperatorType") String calcOperatorType);

    /**
     * @Title: copyGddToAddF
     * @Description: 复制分组明细表数据到分析明细数据表(copy为F的数据)
     * @param analysisDataId 分析数据ID
     * @param groupingDataId 分组数据ID
     * @param comment 分组值
     * @param calcOperatorType 运算符号
     */
    void copyGddToAddF(
            @Param("analysisDataId") String analysisDataId,
            @Param("groupingDataId") String groupingDataId,
            @Param("comment") String comment,
            @Param("calcOperatorType") String calcOperatorType);

    /**
     * 将 FAILURE_END_TIME>time的数据置为s
     * @param dataId
     * @param time
     */
    @Update("update ewb_analysis_data_detail set F_S = '0' where DATA_ID = #{dataId} and FAILURE_END_TIME >= #{time}")
    int updateSusByTime(@Param("dataId") String dataId, @Param("time") double time);

    @Select("SELECT count( 1 )  FROM ewb_analysis_data_detail WHERE DATA_ID = #{dataId} AND F_S = '1' GROUP BY FAILURE_END_TIME")
    List<Integer> selectDiffPointCount(@Param("dataId") String dataId);

    @Update("update ewb_analysis_data_detail set F_S = 1 where DATA_ID = #{dataId} and F_S = '0'")
    void updatefByAnalysisId(String dataId);

    @Update("update ewb_analysis_data_detail set last_inspect_time = failure_end_time where DATA_ID = #{dataId}")
    void updateLitByAnalysisId(String dataId);

    @Update("update ewb_analysis_data_detail set quantity = 1 where DATA_ID = #{dataId}")
    void updateQuantityByAnalysisId(String dataId);
}