package com.nationrel.modules.weibull.mapper;


import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.nationrel.modules.weibull.entity.EwbAnalysis;
import com.nationrel.modules.weibull.vo.EwbAnalysisListVo;
import com.nationrel.modules.weibull.vo.EwbAnalysisStartVo;
import com.nationrel.modules.weibull.vo.EwbChartDataModel;
import com.nationrel.modules.weibull.vo.EwbMultiChartAnalysisListVo;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.util.List;

public interface EwbAnalysisMapper extends BaseMapper<EwbAnalysis> {

    List<EwbAnalysisListVo> selectSearch(@Param("wbId") String wbId, @Param("userId") String userId, @Param("analysisName") String analysisName);


    @Update("update ewb_analysis set end_time_name = #{endTimeName} where analysis_id = #{analysisId}")
    void updateEndTimeName(String analysisId, String endTimeName);
    /**
     * @Title: searchChartDataByAnalysisId
     * @Description: 根据分析ID搜索画图数据
     * @param analysisId 分析ID
     * @return List<LdChartDataModel>
     */
    List<EwbChartDataModel> searchChartDataByAnalysisId(@Param("analysisId") String analysisId, @Param("plotType") String plotType);

    void updateByPrimaryKey(EwbAnalysis ewbAnalysis);

    @Insert("INSERT into ewb_analysis(ANALYSIS_ID, ANALYSIS_NAME, WB_ID, PARENT_ID, DEL_FLAG, INSERT_TIME, INSERT_USER_ID, UPDATE_TIME, UPDATE_USER_ID, VERSION_NO)" +
            "select #{targetAnalysisId}, CONCAT(ANALYSIS_NAME,'_copy'), WB_ID, PARENT_ID, DEL_FLAG, now(), INSERT_USER_ID, UPDATE_TIME, UPDATE_USER_ID, VERSION_NO from ewb_analysis where ANALYSIS_ID = #{sourceId}")
    void copyData(@Param("sourceId") String sourceId, @Param("targetAnalysisId") String targetAnalysisId);

    @Select("select " +
            "analysis.WB_ID as wbId, " +
            "analysis.ANALYSIS_ID as analysisId, " +
            "analysis.ANALYSIS_NAME as analysisName, " +
            "analysis_setting.SETTING_ID as settingId, " +
            "analysis_setting.DISTRIBUTING_CODE as distributingCode, " +
            "IF(analysis_setting.xaxis_start IS NULL, '0', '1' ) plotCount, " +
            "'0' as checkFlag " +
            "from ewb_analysis analysis " +
            "left join ewb_analysis_data analysis_data on analysis_data.ANALYSIS_ID = analysis.ANALYSIS_ID " +
            "left join ewb_analysis_setting analysis_setting on analysis_setting.DATA_ID = analysis_data.DATA_ID " +
            "where analysis.insert_user_id = #{userId} and analysis.del_flag = '0' and analysis.ANALYSIS_NAME != '' and analysis.ANALYSIS_NAME IS NOT null and analysis_setting.calc_result IS NOT null and analysis_setting.calc_result != '' "  +
            "group by analysis.WB_ID, analysis.ANALYSIS_ID,analysis.ANALYSIS_NAME,analysis_setting.SETTING_ID,analysis_setting.DISTRIBUTING_CODE")
    List<EwbMultiChartAnalysisListVo> selectAllAnalysisList(@Param("userId") String userId);

    @Select("select " +
            "analysis.WB_ID as wbId, " +
            "analysis.ANALYSIS_ID as analysisId, " +
            "analysis.ANALYSIS_NAME as analysisName, " +
            "analysis_setting.SETTING_ID as settingId, " +
            "analysis_setting.DISTRIBUTING_CODE as distributingCode, " +
            "IF( multi_chart_detail.ANALYSIS_ID IS NULL, '0', '1' ) AS checkFlag, " +
            "IF(analysis_setting.xaxis_start IS NULL, '0', '1' ) plotCount  " +
            "from ewb_analysis analysis " +
            "left join ewb_analysis_data analysis_data on analysis_data.ANALYSIS_ID = analysis.ANALYSIS_ID " +
            "left join ewb_analysis_setting analysis_setting on analysis_setting.DATA_ID = analysis_data.DATA_ID " +
            "LEFT JOIN ewb_multi_chart_detail multi_chart_detail on multi_chart_detail.ANALYSIS_ID = analysis.ANALYSIS_ID and multi_chart_detail.MULTI_CHART_ID = #{multiChartId} " +
            "where analysis.WB_ID = #{wbId} and analysis.del_flag = '0' and analysis.ANALYSIS_NAME != '' and analysis.ANALYSIS_NAME IS NOT null "  +
            "group by analysis.WB_ID, analysis.ANALYSIS_ID,analysis.ANALYSIS_NAME,analysis_setting.SETTING_ID,analysis_setting.DISTRIBUTING_CODE, multi_chart_detail.ANALYSIS_ID")
    List<EwbMultiChartAnalysisListVo> selectAnalysisList(@Param("wbId") String wbId, @Param("multiChartId") String multiChartId);

    @Select("select " +
            "analysis.ANALYSIS_NAME as analysisName, " +
            "analysis.ANALYSIS_ID as analysisId, " +
            "analysis_data.DATA_ID as dataId, " +
            "analysis_data.IS_RANK_OR_NODATA as isRankOrNodata, " +
            "analysis_setting.SETTING_ID as settingId, " +
            "analysis_setting.BELIEVE_CODE as boundSided, " +
            "analysis_setting.BELIEVE_PERCENT as confidenceLevel, " +
            "analysis_setting.xaxis_start as xaxisStart, " +
            "analysis_setting.xaxis_end as xaxisEnd, " +
            "analysis_setting.yaxis_start as yaxisStart, " +
            "analysis_setting.yaxis_end as yaxisEnd, " +
            "analysis_setting.BELIEVE_METHOD_CODE as method " +
            "from ewb_analysis analysis " +
            "left join ewb_analysis_data analysis_data on analysis.ANALYSIS_ID = analysis_data.ANALYSIS_ID " +
            "left join ewb_analysis_setting analysis_setting on analysis_setting.DATA_ID = analysis_data.DATA_ID " +
            "where analysis.ANALYSIS_ID = #{analysisId}")
    EwbAnalysisStartVo selectParamId(@Param("analysisId") String analysisId);

    @Select("select wb_id from ewb_analysis where analysis_id = #{analysisId}")
    String selectWbIdByAnalysisId(String analysisId);

    @Select("select analysis_id from ewb_analysis where parent_id = #{analysisId}")
    List<String> selectMultiAnalysisIds(String analysisId);
}
