package com.nationrel.modules.weibull.mapper;


import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.nationrel.modules.weibull.entity.EwbAnalysisRank;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

public interface EwbAnalysisRankMapper extends BaseMapper<EwbAnalysisRank> {

    /**
     * @Title: deleteByDataId
     * @Description: 根据数据Id删除分析秩表数据
     * @param dataId 数据Id
     */
    void deleteByDataId(@Param("dataId") String dataId);

    /**
     * @Title: searchRankBySettingId
     * @Description: 根据分析设定ID搜索分析秩
     * @param settingId 分析设定ID
     * @return List<RldAnalysisRank>
     */
    List<EwbAnalysisRank> searchRankBySettingId(@Param("settingId") String settingId);

    /**
     * @Title: searchRankSample
     * @Description: 根据分析设定ID、最大数据量、X最小值、X范围对分析秩抽样
     * @param minX X最小值
     * @param rangeX X范围
     * @param rankMaximum 最大数据量
     * @param settingId 分析设定ID
     * @return List<RldAnalysisRank>
     */
    List<EwbAnalysisRank> searchRankSample(
            @Param("minX") double minX,
            @Param("rangeX") double rangeX,
            @Param("rankMaximum") int rankMaximum,
            @Param("settingId") String settingId);

    /**
     * @Title: searchRankSampleByY
     * @Description: 根据分析设定ID、最大数据量、Y最小值、Y范围对分析秩抽样(当X值全部相等时)
     * @param minY Y最小值
     * @param rangeY Y范围
     * @param rankMaximum 最大数据量
     * @param settingId 分析设定ID
     * @return List<RldAnalysisRank>
     */
    List<EwbAnalysisRank> searchRankSampleByY(
            @Param("minY") double minY,
            @Param("rangeY") double rangeY,
            @Param("rankMaximum") int rankMaximum,
            @Param("settingId") String settingId);

    /**
     * @Title: searchBySettingId
     * @Description: 根据分析ID搜索一条分析秩(分析秩抽样时，分析秩X值相等、Y值相等)
     * @param settingId 分析ID
     * @return RldAnalysisRank
     */
    EwbAnalysisRank searchOneBySettingId(@Param("settingId") String settingId);

    /**
     * @Title: countRankBySettingId
     * @Description: 根据分析设定ID统计rank数
     * @param settingId 分析设定ID
     * @return int
     */
    int countRankBySettingId(@Param("settingId") String settingId);

    /**
     * @Title: searchMinXAndY
     * @Description: 根据分析设定ID搜索分析秩XY值的最小值和最大值
     * @param settingId 分析设定ID
     * @return Map<String, Object>
     */
    Map<String, Object> searchExtremum(@Param("settingId") String settingId);

    @Insert("INSERT into ewb_analysis_rank(DATA_ID, ADRI, RANK, DATA_NO, FAILURE_TIME, X, Y) " +
            "select #{targetDataId}, ADRI, RANK, DATA_NO, FAILURE_TIME, X, Y from ewb_analysis_rank where DATA_ID = #{dataId}")
    void copyData(@Param("dataId") String dataId, @Param("targetDataId") String targetDataId);
}