package com.nationrel.modules.weibull.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.nationrel.modules.weibull.entity.EwbAnalysisSetting;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Update;

public interface EwbAnalysisSettingMapper extends BaseMapper<EwbAnalysisSetting> {

    /**
     * @Title: selectByPageInput
     * @Description: 根据页面输入值获取分析设定对象
     * @param settingId 设定ID
     * @param distributingCode 分布类型
     * @param parameterCode 参数类型
     * @param estimationMethodCode 参数估计方法
     * @param believeCode 置信类型
     * @param believeMethodCode 置信法
     * @param believePercent 置信度
     * @return RldAnalysisSetting 设定对象
     */
    EwbAnalysisSetting selectByPageInput(@Param("settingId") String settingId,
                                         @Param("distributingCode") String distributingCode,
                                         @Param("parameterCode") String parameterCode,
                                         @Param("estimationMethodCode") String estimationMethodCode,
                                         @Param("believeCode") String believeCode,
                                         @Param("believeMethodCode") String believeMethodCode,
                                         @Param("believePercent") Double believePercent);

    @Insert("INSERT into ewb_ANALYSIS_SETTING(SETTING_ID, DATA_ID, DISTRIBUTING_CODE, PARAMETER_CODE, " +
            "ESTIMATION_METHOD_CODE, RANK_METHOD_CODE, BELIEVE_CODE, BELIEVE_METHOD_CODE, BELIEVE_PERCENT, " +
            "INSERT_TIME, INSERT_USER_ID, UPDATE_TIME, UPDATE_USER_ID, VERSION_NO, BETA, SIGMA, LOGSIGMA) " +
            "select #{targetSettingId}, #{targetDataId}, DISTRIBUTING_CODE, PARAMETER_CODE, ESTIMATION_METHOD_CODE, RANK_METHOD_CODE, BELIEVE_CODE, BELIEVE_METHOD_CODE, BELIEVE_PERCENT, now(), INSERT_USER_ID, UPDATE_TIME, UPDATE_USER_ID, VERSION_NO, BETA, SIGMA, LOGSIGMA from ewb_ANALYSIS_SETTING where SETTING_ID = #{settingId}")
    void copyData(@Param("settingId") String settingId, @Param("targetSettingId") String targetSettingId, @Param("targetDataId") String targetDataId);

    void deleteByPrimaryKey(@Param("settingId") String settingId);

    void updateByPrimaryKey(EwbAnalysisSetting analysisSetting);

    @Update("update ewb_ANALYSIS_SETTING set paramter = #{paramter} where setting_id = #{settingId}")
    void updateParamter(String settingId, String paramter);

    @Update("update ewb_ANALYSIS_SETTING set CALC_RESULT_ALL = #{calcResultAll} where setting_id = #{settingId}")
    void updateCalcResultAll(String settingId, String calcResultAll);

    @Update("update ewb_ANALYSIS_SETTING set FAILURE_RANK = #{failureRank} where setting_id = " +
            "#{settingId}")
    void updateFailureRankAndRow(String settingId, String failureRank);

    @Update("update ewb_ANALYSIS_SETTING set believe_percent = #{believePercent} where setting_id = #{settingId}")
    void updateBelievePercent(String settingId, String believePercent);

    @Update("update ewb_ANALYSIS_SETTING set CALC_RESULT_ALL = '',CALC_RESULT = '' where setting_id = #{settingId}")
    void deleteAllCalcResult(String settingId);

    @Update("update ewb_ANALYSIS_SETTING set xaxis_end = null,xaxis_start = null," +
            "yaxis_end = null,yaxis_start = null,time = null,un_reliability = null,beta_mark = null,eta_mark = null,mu_mark = null,sigma_mark = null,lambda_mark = null,target_time = null,target_reliability = null where setting_id = #{settingId}")
    void deleteMarkSetting(String settingId);
}