package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.EwbParameterCodeEnum;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.util.ExportExcelUtil;
import com.nationrel.modules.weibull.util.ImportExcelUtil;
import com.nationrel.modules.weibull.util.NumberParseUtil;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.vo.EwbAnalysisDataDetailVo;
import com.nationrel.modules.weibull.vo.EwbAnalysisDataRankVo;
import com.nationrel.modules.weibull.vo.EwbAnalysisDataSaveVo;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * @author Aimee on 2020/9/15.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisDataDetailService {

    private final EwbAnalysisDataDetailMapper analysisDataDetailMapper;
    private final EwbAnalysisDataRankMapper analysisDataRankMapper;
    private final SaveDataDetailService saveService;
    private final EwbAnalysisSettingMapper analysisSettingMapper;
    private final EwbAnalysisDataMapper analysisDataMapper;
    private final EwbAnalysisInfoMapper analysisInfoMapper;
    private final EwbAnalysisMapper analysisMapper;
    private final EwbAnalysisService analysisService;
    private final EwbValidatorService validatorService;
    private final AllVariableConfig allVariableConfig;

    public List<EwbAnalysisDataDetail> listDataDetail(String dataId) {
        return analysisDataDetailMapper.selectList(new LambdaQueryWrapper<EwbAnalysisDataDetail>().eq(EwbAnalysisDataDetail::getDataId, dataId).eq(EwbAnalysisDataDetail::getFailureSuspension, EwbConstants.STATE_S).orderByDesc(EwbAnalysisDataDetail::getDataNo));
    }

    public ServerResponse listByPage(HeaderUserInfo userInfo, String dataId, int pageIndex, int pageSize) {
//        PageHelper.startPage(pageIndex, pageSize);
        List<EwbAnalysisDataDetailVo> list = analysisDataDetailMapper.searchEwbDetailData(dataId);
        List<EwbAnalysisDataRankVo> listRank = analysisDataRankMapper.selectByDataId(dataId);
        // 简化数据以减少网络数据传输
        List<String[]> datas = new ArrayList<>(list.size());
        List<String[]> datasRank = new ArrayList<>(listRank.size());
        list.forEach(analysisDataDetailVo -> datas.add(EwbAnalysisDataDetailVo.toStringStr(analysisDataDetailVo)));
        listRank.forEach(analysisDataRankVo -> datasRank.add(EwbAnalysisDataRankVo.toStringStr(analysisDataRankVo)));
        EwbAnalysisData analysisData = analysisDataMapper.selectOne(new QueryWrapper<EwbAnalysisData>().lambda().eq(EwbAnalysisData::getDataId, dataId));
        Map<String, Object> result = new HashMap<>();
        result.put("setting", analysisData);
//        boolean isAdmin = isAdmin(userInfo);
        boolean isAdmin = true;
        result.put("dataNumber", isAdmin ? 20000 : allVariableConfig.getWeibullDataNumber());
        result.put("list", datas);
        result.put("listRank", datasRank);
//        result.put("list", list.stream().map(ewbAnalysisDataDetailVo -> new String[]{ewbAnalysisDataDetailVo.getDataId(), ewbAnalysisDataDetailVo.getDataNo(), ewbAnalysisDataDetailVo.getQuantity(), ewbAnalysisDataDetailVo.getFailureEndTime(), ewbAnalysisDataDetailVo.getFailureSuspension(), ewbAnalysisDataDetailVo.getLastInspectTime()}).collect(Collectors.toList()));
        if (analysisData != null) {
            result.put("endTimeName", analysisService.getAnalysis(analysisData.getAnalysisId()).getEndTimeName());
        }
        return ServerResponse.createBySuccess(result);
    }

    private boolean isAdmin(HeaderUserInfo userInfo) {
        boolean isAdmin = false;
        for (int i = 0; i < userInfo.getRoleInfos().size(); i++) {
            if ("admin".equals(userInfo.getRoleInfos().get(i).getRoleCode())) {
                isAdmin = true;
                break;
            }
        }
        return isAdmin;
    }

    /**
     * 保存weibull数据
     * 添加isValidData参数，判断是否是已经解析完成的数据
     *
     * @param userInfo
     * @param param
     * @param isValidData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse saveData(HeaderUserInfo userInfo, EwbAnalysisDataSaveVo param, boolean isValidData) {
        // 数据ID
        String dataId = param.getDataId();
        // 分析Id
        String analysisId = param.getAnalysisId();
        EwbAnalysisSetting analysisSetting = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().lambda().select(EwbAnalysisSetting::getSettingId).eq(EwbAnalysisSetting::getDataId, dataId));
        EwbAnalysisData analysisData = analysisDataMapper.selectById(dataId);
        String settingId = analysisSetting.getSettingId();
        if (EwbConstants.FLAG_NO_CHECKED.equals(param.getTimeForRank())) {
            // 非概率VS时间数据
            boolean changeFlag;
            // 页面复选框"包含区间删失数据或左删失数据"选中标记
            String leftFlag = param.getLeftFlag();
            // 页面复选框"包含右删失数据"选中标记
            String rightFlag = param.getRightFlag();
            // 页面复选框"分组数据"选中标记
            String groupingFlag = param.getGroupingFlag();
            // ==================
            // 1.修改寿命数据分析数据表信息
            // ==================
            if (param.getHotDataChange() > 1) {
                // 解析新的数组结构的数据，将其解析为对象
                List<EwbAnalysisDataDetail> analysisDataDetails = new ArrayList<>();
                if (isValidData) {
                    analysisDataDetails = param.getAnalysisDataDetails();
                } else {
                    EwbAnalysis analysis = analysisService.getAnalysis(analysisId);
                    String endTimeName = StringUtils.isBlank(analysis.getEndTimeName()) ? EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, userInfo.getLanguageKey()) : analysis.getEndTimeName();
                    for (int i = 0; i < param.getAnalysisDatas().size(); i++) {
                        String[] data = param.getAnalysisDatas().get(i);
                        if (data[0] != null && data[1] != null) {
                            // 判断填入数值是否为大于0的数字
                            if (!WeibullNumberUtil.isNumeric(data[4]) || WeibullNumberUtil.parseDouble(data[4]) <= 0) {
                                throw new DescribeException((i + 1), endTimeName, EwbCustomPromptEnum.必须为大于零数字, userInfo.getLanguageKey());
                            }
                            // 判断最近检查时间填入数值是否为大于0的数字
                            if (leftFlag.equals(EwbConstants.FLAG_CHECKED) && (!WeibullNumberUtil.isNumeric(data[2]) || WeibullNumberUtil.parseDouble(data[2]) < 0)) {
                                throw new DescribeException((i + 1), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.最近检查时间, userInfo.getLanguageKey()), EwbCustomPromptEnum.必须为大于零数字, userInfo.getLanguageKey());
                            }
                            // 判断最近检查时间是否合法
                            if (leftFlag.equals(EwbConstants.FLAG_CHECKED) && WeibullNumberUtil.parseDouble(data[2]) > WeibullNumberUtil.parseDouble(data[4])) {
                                throw new DescribeException((i + 1), endTimeName, EwbCustomPromptEnum.必须大于等于最近检查时间, userInfo.getLanguageKey());
                            }
                            analysisDataDetails.add(EwbAnalysisDataDetail.builder().dataId(dataId).dataNo(WeibullNumberUtil.parseInteger(data[0])).quantity(WeibullNumberUtil.parseInteger(data[1])).lastInspectTime(WeibullNumberUtil.parseDouble(data[2])).failureSuspension(data[3]).failureEndTime(WeibullNumberUtil.parseDouble(data[4])).failureTime(WeibullNumberUtil.parseDouble(data[5])).comment1(data[6]).comment2(data[7]).comment3(data[8]).comment4(data[9]).comment5(data[10]).build());
                        }
                    }
                }
                param.setAnalysisDataDetails(analysisDataDetails);
                param.setDataDetails(StringUtils.isNotBlank(param.getDataDetails()) ? param.getDataDetails() : JSON.toJSONString(analysisDataDetails));
                validatorService.validatorAnalysisDetailData(param.getDataDetails(), param.getGroupingFlag(), param.getStartDataNo(), param.getLeftFlag(), param.getRightFlag(), param.getEndDataNo(), param.getDataId(), userInfo.getLanguageKey());
                // 获取分析明细数据
                List<EwbAnalysisDataDetail> listDataDetails = CollectionUtils.isEmpty(param.getAnalysisDataDetails()) ? JSONObject.parseArray(param.getDataDetails(), EwbAnalysisDataDetail.class) : param.getAnalysisDataDetails();
                if (listDataDetails != null && listDataDetails.size() > 0) {
                    // 删除集合中WeiMaterial对象属性fratios为空的对象s
                    listDataDetails.removeIf(weiMaterial -> weiMaterial.getFailureEndTime() == null);
                }
//                boolean isAdmin = isAdmin(userInfo);
                boolean isAdmin = true;
                if (Objects.requireNonNull(listDataDetails).size() > (isAdmin ? 20000 : allVariableConfig.getWeibullDataNumber())) {
                    return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.非vip用户仅支持少量数据计算, userInfo.getLanguageKey()) + allVariableConfig.getWeibullDataNumber() + "!");
                }
                // 起始数据编号
                int startDataNo = param.getStartDataNo();
                // ==================
                // 2.保存分析明细数据
                // ==================
                saveService.saveAnalysisDataDetail(dataId, listDataDetails, startDataNo, leftFlag);
                analysisService.deleteAnalysisResultData(dataId, settingId, analysisId);
            }
            if (param.getHotDataChange() > 1 || !param.getTimeForRank().equals(analysisData.getTimeForRank())) {
                // 删除各种中间数据
                analysisService.deleteAnalysisResultData(dataId, settingId, analysisId);
                analysisData.setTimeForRank(param.getTimeForRank());
                // 寿命数据分析数据表修改
                analysisDataMapper.updateByPrimaryKey(analysisData);
            }
            // ==================
            // 3.根据页面复选框的选中情况更新明细数据
            // ==================
            changeFlag = saveService.updateAnalysisDataDetail(dataId, leftFlag, rightFlag, groupingFlag);

            // ==================
            // 4.更新寿命数据分析数据表的各种类型的数据量
            // ==================
            saveService.updateAnalysisData(userInfo, dataId);
            // 设定ID
            if (changeFlag) {
                analysisService.deleteAnalysisResultData(dataId, settingId, analysisId);
            }

            // 包含删失数据(右删失数据)
            analysisData.setRFlag(rightFlag);
            // 包含删失数据(右删失数据)
            analysisData.setLFlag(leftFlag);
            // 分组数据
            analysisData.setGroupingFlag(groupingFlag);
            // 更新者
            analysisData.setUpdateUserId(userInfo.getUserId());
            // 寿命数据分析数据表修改
            analysisDataMapper.updateByPrimaryKey(analysisData);
            // ==================
            // 5.更新寿命数据分析主表信息
            // ==================
            EwbAnalysisInfoService.updateUpdateInfo(userInfo.getUserName(), param.getAnalysisId(), analysisInfoMapper, analysisMapper);
            // 返回结果
            if (changeFlag) {
//                PageHelper.startPage(1, 1000000);
                return ServerResponse.createBySuccess();
            }
        } else {
            // 概率VS时间数据保存
            if (param.getHotDataRankChange() > 1) {
                List<String> differentTime = new ArrayList<>();
                List<String> differentRank = new ArrayList<>();
                // 解析新的数组结构的数据，将其解析为对象
                List<EwbAnalysisDataRank> analysisDataRanks = new ArrayList<>();
                for (int i = 0; i < param.getAnalysisDatasRank().size(); i++) {
                    String[] data = param.getAnalysisDatasRank().get(i);
                    if (data[0] != null && data[1] != null) {
                        analysisDataRanks.add(EwbAnalysisDataRank.builder().dataId(dataId).dataNo(i).failureTime(WeibullNumberUtil.parseDouble(data[1])).failureRank(WeibullNumberUtil.parseDouble(data[2])).build());
                        if (!differentTime.contains(data[1])) {
                            differentTime.add(data[1]);
                        }
                        if (!differentRank.contains(data[2])) {
                            differentRank.add(data[2]);
                        }
                    }
                }
                // 依据当前选择的计算模型判断当前传入的数据是否合法
                if (EwbParameterCodeEnum.一参数.getIndex().equals(analysisSetting.getParameterCode())) {
                    if (differentTime.size() == 0) {
                        return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.Rank一参数需要至少一个数据, userInfo.getLanguageKey());
                    }
                } else if (differentTime.size() < 2 || differentRank.size() < 2) {
                    return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.Rank二参数三参数需要至少一个数据, userInfo.getLanguageKey());
                }
                // ==================
                // 2.保存分析明细数据
                // ==================
                saveService.saveAnalysisDataRank(dataId, analysisDataRanks, 1);
            }
            if (param.getHotDataRankChange() > 1 || !param.getTimeForRank().equals(analysisData.getTimeForRank())) {
                // 删除各种中间数据
                analysisService.deleteAnalysisResultData(dataId, settingId, analysisId);
                analysisData.setTimeForRank(param.getTimeForRank());
                // 寿命数据分析数据表修改
                analysisDataMapper.updateByPrimaryKey(analysisData);
            }
//            PageHelper.startPage(1, 1000000);
            return ServerResponse.createBySuccess();
        }
        return ServerResponse.createBySuccess();
    }

    /**
     * 导入数据
     * 2021-12-20 添加关闭流操作
     *
     * @param userInfo
     * @param file
     * @param dataId
     * @param analysisId
     * @throws IOException
     */
    @Transactional(rollbackFor = Exception.class)
    public void importData(HeaderUserInfo userInfo, MultipartFile file, String dataId, String analysisId) throws Exception {

        // 取得：Excel数据。
        List<List<String>> excelList = ImportExcelUtil.getListByExcel(file.getInputStream(), file.getOriginalFilename(), userInfo.getLanguageKey());
        if (CollectionUtils.isEmpty(excelList)) {
            return;
        }

        List<String[]> datas = ImportExcelUtil.excelToDatas(excelList);

        List<EwbAnalysisDataDetail> analysisDataDetails = new ArrayList<>();

        // 解析新的数组结构的数据，将其解析为对象
        for (int rowNum = 0; rowNum < datas.size(); rowNum++) {
            String[] data = datas.get(rowNum);
            // 判断填入数值是否为大于0的数字
            if (StringUtils.isNotBlank(data[1]) || StringUtils.isNotBlank(data[2]) || StringUtils.isNotBlank(data[3]) || StringUtils.isNotBlank(data[4])) {
                if (StringUtils.isBlank(data[4]) || !NumberParseUtil.isNumeric(data[4]) || NumberParseUtil.parseDouble(data[4]) <= 0) {
                    throw new DescribeException((rowNum + 2), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, userInfo.getLanguageKey()), EwbCustomPromptEnum.必须为大于零数字, userInfo.getLanguageKey());
                }
                // 判断最近检查时间填入数值是否为大于0的数字
                if (StringUtils.isNotBlank(data[2]) && (!NumberParseUtil.isNumeric(data[2]) || NumberParseUtil.parseDouble(data[2]) <= 0)) {
                    throw new DescribeException((rowNum + 2), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.最近检查时间, userInfo.getLanguageKey()), EwbCustomPromptEnum.必须为大于零数字, userInfo.getLanguageKey());
                } else if (StringUtils.isNotBlank(data[2])) {
                    // 判断最近检查时间是否合法
                    if (NumberParseUtil.parseDouble(data[2]) > NumberParseUtil.parseDouble(data[4])) {
                        throw new DescribeException((rowNum + 2), ReliaqubeParamDataCheckEnum.结束时间必须大于等于最近检查时间, userInfo.getLanguageKey());
                    }
                } else {
                    // 最近检查时间的默认值为结束时间
                    data[2] = data[4];
                }
                analysisDataDetails.add(EwbAnalysisDataDetail.builder()
                        .dataId(dataId)
                        .dataNo(rowNum + 1)
                        .quantity(StringUtils.isNotBlank(data[1]) && NumberParseUtil.parseInteger(data[1]) > 1 ? NumberParseUtil.parseInteger(data[1]) : 1)
                        .lastInspectTime(NumberParseUtil.parseDouble(data[2]))
                        .failureSuspension(EwbConstants.STATE_SUSPENSION.equalsIgnoreCase(data[3]) ? EwbConstants.STATE_SUSPENSION : EwbConstants.STATE_FAILURE)
                        .failureEndTime(NumberParseUtil.parseDouble(data[4]))
                        .failureTime(EwbConstants.STATE_SUSPENSION.equalsIgnoreCase(data[3]) ? NumberParseUtil.parseDouble(data[4]) : (NumberParseUtil.parseDouble(data[2]) + NumberParseUtil.parseDouble(data[4])) / 2)
                        .dataType(EwbConstants.COMPLETE_DATA_TYPE)
                        .comment1(data[5]).build());
            }
        }

        // 更新原有数据
        EwbAnalysisData analysisData = analysisDataMapper.selectById(dataId);
        analysisData.setLFlag(EwbConstants.FLAG_CHECKED);
        analysisData.setRFlag(EwbConstants.FLAG_CHECKED);
        analysisData.setGroupingFlag(EwbConstants.FLAG_CHECKED);
        analysisDataMapper.updateById(analysisData);
        saveData(userInfo, EwbAnalysisDataSaveVo.builder().dataDetails(JSONObject.toJSONString(analysisDataDetails)).rightFlag(EwbConstants.FLAG_CHECKED).leftFlag(EwbConstants.FLAG_CHECKED).groupingFlag(EwbConstants.FLAG_CHECKED).timeForRank(EwbConstants.FLAG_NO_CHECKED).startDataNo(1).endDataNo(analysisDataDetails.size()).analysisId(analysisId).dataId(dataId).hotDataChange(2).analysisDataDetails(analysisDataDetails).build(), true);
        EwbAnalysisInfoService.updateUpdateInfo(userInfo.getUserName(), analysisId, analysisInfoMapper, analysisMapper);
    }

    /**
     * 将weibull数据导出为excel数据
     * 2021-12-20 修改导出的内容，保证导出的数据可以直接导入系统
     *
     * @param response
     * @param dataId
     * @throws IOException
     */
    public void exportData(HttpServletResponse response, String dataId) throws IOException {
        // 获取导出寿命数据
        List<EwbAnalysisDataDetailVo> expList = analysisDataDetailMapper.searchExportDetailData(dataId);
        // 定义：sheet名称
        String sheetName = "weibull-data";
        // 取得：Excel标题Map
        LinkedHashMap<String, String> fieldMap = this.buildExcelTitleMap();
        // 导出：Excel
        ExportExcelUtil.export(sheetName, expList, fieldMap, response);
    }

    /**
     * 导入模板下载
     *
     * @param response
     * @throws IOException
     */
    public void importTemplate(HttpServletResponse response) {
        // 取得：Excel标题Map
        LinkedHashMap<String, String> excelTitleMap = this.buildExcelTitleMap();
        // 导出：Excel
        ExportExcelUtil.export("weibull-data", new ArrayList<>(100), excelTitleMap, response);
    }

    /**
     * 取得：Excel标题Map
     *
     * @return：Map
     */
    private LinkedHashMap<String, String> buildExcelTitleMap() {
        LinkedHashMap<String, String> excelTitleMap = new LinkedHashMap<>();
        excelTitleMap.put("quantity", "数量");
        excelTitleMap.put("lastInspectTime", "最近检查时间");
        excelTitleMap.put("failureSuspension", "F或S状态");
        excelTitleMap.put("failureEndTime", "结束时间");
        excelTitleMap.put("comment1", "分组编号");
        return excelTitleMap;
    }

    /**
     * 获取分析设置
     *
     * @param settingId
     * @return
     */
    public EwbAnalysisSetting getAnalysisSetting(String settingId) {
        return analysisSettingMapper.selectById(settingId);
    }

}
