package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.EwbCalcParametersEnum;
import com.nationrel.modules.weibull.enums.EwbDistributingCodeEnum;
import com.nationrel.modules.weibull.enums.EwbParameterCodeEnum;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.general.domain.enums.common.WhetherFlagEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.util.StringUtil;
import com.nationrel.modules.weibull.util.ValidateUtil;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.vo.*;
import lombok.RequiredArgsConstructor;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aimee on 2020/10/20.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisDisWizardService {

    private final EwbAnalysisDisWizardMapper analysisDisWizardMapper;
    private final EwbAnalysisDataMapper analysisDataMapper;
    private final RgaApiService rgaApiService;
    private final EwbAnalysisSettingMapper analysisSettingMapper;
    private final EwbAnalysisDataDetailMapper analysisDataDetailMapper;
    private final EwbAnalysisFrBelieveMapper analysisFrBelieveMapper;
    private final EwbAnalysisPrBelieveMapper analysisPrBelieveMapper;
    private final EwbAnalysisRankMapper analysisRankMapper;
    private final EwbAnalysisResultMapper analysisResultMapper;
    private final EwbAnalysisResultDetailMapper analysisResultDetailMapper;
    private final EwbNonNevadaSettingMapper nonNevadaSettingMapper;
    private final StringRedisTemplate redisTemplate;
    private final EwbValidatorService validatorService;

    private final EwbAnalysisService analysisService;
    private final AllVariableConfig allVariableConfig;

    public List<EwbAnalysisDisWizard> getDistributionWizard(String dataId) {

        // 点击计算验证失败后,页面选中的复选框的值
//        List<EwbAnalysisDisWizard> checkedAnalysisDisWizards = new ArrayList<EwbAnalysisDisWizard>();
//        if (ValidateUtils.isNotEmpty(input.getCheckeStr())) {
//            // 将Json字符窜转换为Json数组
//            JSONArray jsonArray = JSONArray.parse(input.getCheckeStr());
//            // 将Json数组转为泛型List<RldAnalysisDisWizard>
//            checkedAnalysisDisWizards = JSONArray.toList(jsonArray, new RldAnalysisDisWizard(), new JsonConfig());
//        }

        // 已计算过的分布向导
        List<EwbAnalysisDisWizard> wizardCalculated = analysisDisWizardMapper.selectByDataId(dataId);

        // 所有分布向导
        List<EwbAnalysisDisWizard> wizards = getWizardTemplate(dataId);

        for (int i = 0; i < wizards.size(); i++) {
            String distributing = wizards.get(i).getDistributingCode();
            String parameter = wizards.get(i).getParameterCode();
            for (EwbAnalysisDisWizard ewbAnalysisDisWizard : wizardCalculated) {
                if (ewbAnalysisDisWizard.getDistributingCode().equals(distributing) && ewbAnalysisDisWizard.getParameterCode().equals(parameter)) {
                    wizards.set(i, ewbAnalysisDisWizard);
                    break;
                }
            }
        }

        // 保存分析设置
//        if (input.getDistributingCode() != null) {
//            RldAnalysisSetting analysisSetting = new RldAnalysisSetting();
//            analysisSetting.setDistributingCode(input.getDistributingCode());
//            analysisSetting.setParameterCode(input.getParameterCode());
//            analysisSetting.setEstimationMethodCode(input.getEstimationMethodCode());
//            analysisSetting.setRankMethodCode(input.getRankMethodCode());
//            analysisSetting.setBelieveCode(input.getBelieveCode());
//            analysisSetting.setBelieveMethodCode(input.getBelieveMethodCode());
//            analysisSetting.setBelievePercent(NumberUtils.parseDouble(input.getBelievePercent()));
//            output.setAnalysisSetting(analysisSetting);
//        }

//        String dataFlag = input.getDataFlag();
//        if (RldConstants.DATA_FLAG_CREATE_SUCCESS.equals(dataFlag)) {
//            output.setPage(RldConstants.PATH_WIZARD_CREATE);
//            dataFlag = RldConstants.DATA_FLAG_EDIT;
//        } else {
//            output.setPage(RldConstants.PATH_WIZARD_EDIT);
//        }

        // 点击计算验证失败后，加入已经选中复选框的值
//        if (checkedAnalysisDisWizards.size() > 0) {
//            for (RldAnalysisDisWizard checkedData : checkedAnalysisDisWizards) {
//                for (RldAnalysisDisWizard analysisDiz : wizards) {
//                    if (analysisDiz.getDistributingCode().equals(checkedData.getDistributingCode()) && analysisDiz.getParameterCode().equals(checkedData.getParameterCode())) {
//                        analysisDiz.setCheckeFlag(checkedData.getCheckeFlag());
//                    }
//                }
//            }
//        }

        // 输出所有分布向导
        return wizards;
    }


    /**
     * @param dataId 数据ID
     * @return List<RldAnalysisDisWizard>
     * @Title: getWizardTemplate
     * @Description: 生成分布向导模板
     */
    private List<EwbAnalysisDisWizard> getWizardTemplate(String dataId) {

        List<EwbAnalysisDisWizard> wizards = new ArrayList<>();

        // Weibull 1p
//        EwbAnalysisDisWizard disWizardWeibull1p = new EwbAnalysisDisWizard();
//        disWizardWeibull1p.setDataId(dataId);
//        disWizardWeibull1p.setDistributingCode(EwbConstants.V_LD_DISTRIBUTING_WEIBULL);
//        disWizardWeibull1p.setParameterCode(EwbConstants.V_LD_PARAMETER_1);
//        disWizardWeibull1p.setCheckeFlag(EwbConstants.FLAG_CHECKED);
//        wizards.add(disWizardWeibull1p);

        // Weibull 2p
        EwbAnalysisDisWizard disWizardWeibull2p = new EwbAnalysisDisWizard();
        disWizardWeibull2p.setDataId(dataId);
        disWizardWeibull2p.setDistributingCode(EwbDistributingCodeEnum.Weibull.getIndex());
        disWizardWeibull2p.setParameterCode(EwbParameterCodeEnum.二参数.getIndex());
        disWizardWeibull2p.setCheckeFlag(EwbConstants.FLAG_CHECKED);
        wizards.add(disWizardWeibull2p);

        // Weibull 3p
        EwbAnalysisDisWizard disWizardWeibull3p = new EwbAnalysisDisWizard();
        disWizardWeibull3p.setDataId(dataId);
        disWizardWeibull3p.setDistributingCode(EwbDistributingCodeEnum.Weibull.getIndex());
        disWizardWeibull3p.setParameterCode(EwbParameterCodeEnum.三参数.getIndex());
        disWizardWeibull3p.setCheckeFlag(EwbConstants.FLAG_CHECKED);
        wizards.add(disWizardWeibull3p);

        // Lognormal 2p
        EwbAnalysisDisWizard disWizardLognormal2p = new EwbAnalysisDisWizard();
        disWizardLognormal2p.setDataId(dataId);
        disWizardLognormal2p.setDistributingCode(EwbDistributingCodeEnum.Lognormal.getIndex());
        disWizardLognormal2p.setParameterCode(EwbParameterCodeEnum.二参数.getIndex());
        disWizardLognormal2p.setCheckeFlag(EwbConstants.FLAG_CHECKED);
        wizards.add(disWizardLognormal2p);

        // Lognormal 1p
//        EwbAnalysisDisWizard disWizardLognormal1p = new EwbAnalysisDisWizard();
//        disWizardLognormal1p.setDataId(dataId);
//        disWizardLognormal1p.setDistributingCode(EwbConstants.V_LD_DISTRIBUTING_LOGNORMAL);
//        disWizardLognormal1p.setParameterCode(EwbConstants.V_LD_PARAMETER_1);
//        disWizardLognormal1p.setCheckeFlag(EwbConstants.FLAG_CHECKED);
//        wizards.add(disWizardLognormal1p);

        // Normal 2p
        EwbAnalysisDisWizard disWizardNormal2p = new EwbAnalysisDisWizard();
        disWizardNormal2p.setDataId(dataId);
        disWizardNormal2p.setDistributingCode(EwbDistributingCodeEnum.Normal.getIndex());
        disWizardNormal2p.setParameterCode(EwbParameterCodeEnum.二参数.getIndex());
        disWizardNormal2p.setCheckeFlag(EwbConstants.FLAG_CHECKED);
        wizards.add(disWizardNormal2p);

        // Normal 1p
//        EwbAnalysisDisWizard disWizardNormal1p = new EwbAnalysisDisWizard();
//        disWizardNormal1p.setDataId(dataId);
//        disWizardNormal1p.setDistributingCode(EwbConstants.V_LD_DISTRIBUTING_NORMAL);
//        disWizardNormal1p.setParameterCode(EwbConstants.V_LD_PARAMETER_1);
//        disWizardNormal1p.setCheckeFlag(EwbConstants.FLAG_CHECKED);
//        wizards.add(disWizardNormal1p);

        // Exponential 1p
        EwbAnalysisDisWizard disWizardExponential1p = new EwbAnalysisDisWizard();
        disWizardExponential1p.setDataId(dataId);
        disWizardExponential1p.setDistributingCode(EwbDistributingCodeEnum.Exponential.getIndex());
        disWizardExponential1p.setParameterCode(EwbParameterCodeEnum.一参数.getIndex());
        disWizardExponential1p.setCheckeFlag(EwbConstants.FLAG_CHECKED);
        wizards.add(disWizardExponential1p);

        // Exponential 2p
//        EwbAnalysisDisWizard disWizardExponential2p = new EwbAnalysisDisWizard();
//        disWizardExponential2p.setDataId(dataId);
//        disWizardExponential2p.setDistributingCode(EwbConstants.V_LD_DISTRIBUTING_EXPONENTIAL);
//        disWizardExponential2p.setParameterCode(EwbParameterCodeEnum.twoParameter.getIndex());
//        disWizardExponential2p.setCheckeFlag(EwbConstants.FLAG_CHECKED);
//        wizards.add(disWizardExponential2p);

        return wizards;
    }

    public void getDistributionWizardCalc(EwbDistributionVo param, HeaderUserInfo userInfo) {
        //===========
        // 2.计算并保存分布向导
        //===========
        // 数据ID
        List<EwbAnalysisDisWizard> ldAnalysisDisWizards = param.getEwbAnalysisDisWizards();
        if (ldAnalysisDisWizards == null || ldAnalysisDisWizards.size() <= 0) {
            throw new DescribeException(EwbCustomPromptEnum.请选择分布, userInfo.getLanguageKey());
        }
        Long dataCount = analysisDataDetailMapper.selectCount(new QueryWrapper<EwbAnalysisDataDetail>().lambda().eq(EwbAnalysisDataDetail::getDataId, param.getDataId()));
        if (dataCount == null || dataCount <= 0) {
            throw new DescribeException(EwbCustomPromptEnum.请先输入数据, userInfo.getLanguageKey());
        }
        try {
            EwbAnalysisData analysisData = analysisDataMapper.selectOne(new QueryWrapper<EwbAnalysisData>().lambda().eq(EwbAnalysisData::getDataId, param.getDataId()));
            if (!CollectionUtils.isEmpty(ldAnalysisDisWizards)) {
                analysisDisWizardMapper.delete(new QueryWrapper<EwbAnalysisDisWizard>().lambda().eq(EwbAnalysisDisWizard::getDataId, param.getDataId()));
                for (EwbAnalysisDisWizard ldAnalysisDisWizard : ldAnalysisDisWizards) {
                    // 分布类型
                    String distributingCode = ldAnalysisDisWizard.getDistributingCode();
                    // 参数类型
                    String parameterCode = ldAnalysisDisWizard.getParameterCode();
                    String p1 = ldAnalysisDisWizard.getP1() == null ? "" : String.valueOf(ldAnalysisDisWizard.getP1());

                    // ==================
                    // 调用外部接口获取相关参数并保存分布向导结果
                    // ==================
                    // 接口调用路径
                    String apiLifeDataUrl = rgaApiService.getApiLifeDataUrl(distributingCode, param.getEstimationMethodCode(), parameterCode, p1, false, null);
//                    String apiLifeDataUrl = rgaApiService.getApiLifeDataUrl(distributingCode, EwbEvaluateCodeEnum.rrx.getIndex(), parameterCode, p1, false, null);
                    // 数据类型
                    String apiDataType = rgaApiService.getDataType(analysisData.getRFlag(), analysisData.getLFlag());
                    // 准备RGA计算函数数据
                    JSONObject requestFunctionData = new JSONObject();
                    // 调用api的输入参数
                    JSONObject requestLifeData = rgaApiService.getRequestLifeDataJson(param.getDataId(), apiDataType, new SolveDemoRelWeibullChiSquareRelVo(), false);
                    requestFunctionData.put("data", requestLifeData.toString());
                    // 调用外部API并保存分布向导计算结果
                    rgaApiService.excuteApiSaveAnalysisDisWizard(requestLifeData, apiLifeDataUrl, distributingCode, param.getDataId(), parameterCode, userInfo);

                }
            }
        } catch (Exception e) {
            throw new DescribeException(EwbCustomPromptEnum.计算失败请检查数据, userInfo.getLanguageKey());
        }

    }

    @Transactional(rollbackFor = Exception.class)
    public void distributionWizardSelect(EwbDistributionSelectVo param, HeaderUserInfo userInfo) {
        // 分析设定ID
        String settingId = param.getSettingId();
        // 数据ID
        String dataId = param.getDataId();
        // 分析设定
        EwbAnalysisSetting analysisSetting = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().lambda().eq(EwbAnalysisSetting::getSettingId, param.getSettingId()));
        // 分布
        String distributingCode = param.getDistributingCode();
        // 参数
        String parameterCode = param.getParameterCode();
        // Normal和LogNormal时默认2参数
        if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode) || EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode)) {
            parameterCode = EwbParameterCodeEnum.二参数.getIndex();
        }
        // 置信类型
        //String believeCode = analysisSetting.getBelieveCode();
//        // 置信度默认0.9
//        Double believePercent = EwbConstants.BELIEVEPERCENT_DEFAULT * Math.pow(EwbConstants.HUNDRED, -1);
//        // 置信法
//        String believeMethodCode = analysisSetting.getBelieveMethodCode();
//        if (believeCode == null || EwbConstants.CONFIDENCE_TYPE_NOTHING.equals(believeCode)) {
//            believeMethodCode = null;
//            believePercent = null;
//        }

        // ==================
        // 1.更新寿命数据分析设定表
        // ==================
//        EwbAnalysisSetting setting = ldAnalysisSettingMapper.selectByPrimaryKey(settingId);
        // 原分布和参数
//        String oldDis = analysisSetting.getDistributingCode();
//        String oldPar = analysisSetting.getParameterCode();
        analysisSetting.setSettingId(settingId);
        analysisSetting.setDataId(dataId);
        analysisSetting.setDistributingCode(distributingCode);
        analysisSetting.setParameterCode(parameterCode);
        analysisSetting.setCalcResult("");
        analysisSetting.setCalcResultAll("");
//        analysisSetting.setUpdatePgmId(userInfo.getPgmId());
        analysisSetting.setUpdateUserId(userInfo.getUserId());
        analysisSettingMapper.updateById(analysisSetting);

//        // ==================
//        // 2.更新寿命数据分析主表信息
//        // ==================
//        // 获取寿命数据分析主表信息
//        String analysisId = param.getAnalysisId();
//        RldAnalysis ldAnalysis = analysisMapper.selectByPrimaryKey(analysisId);
//        ldAnalysis.setUpdateUserId(userInfo.getUserId());
//        ldAnalysis.setUpdatePgmId(userInfo.getPgmId());
//        // 修改寿命数据分析主表信息
//        analysisMapper.updateByPrimaryKey(ldAnalysis);

        // 重置分析数据
        analysisRankMapper.deleteByDataId(dataId);
        analysisResultMapper.deleteByPrimaryKey(settingId);
        analysisResultDetailMapper.deleteAnalysisResultDetail(settingId);
        analysisPrBelieveMapper.deleteBySettingId(settingId);
        analysisFrBelieveMapper.deleteBySettingId(settingId);
//        ewbAnalysisSettingMapper.deleteByPrimaryKey(settingId);
//        ewbAnalysisDisWizardMapper.deleteByDataId(dataId);
//        apiAnalysisResultMapper.delete(new QueryWrapper<EwbApiAnalysisResult>().lambda().eq(EwbApiAnalysisResult::getDataId, dataId));

        redisTemplate.delete(param.getAnalysisId());

    }

    /**
     * 计算单个模型结果
     *
     * @param param
     */
    @SuppressWarnings("DuplicatedCode")
    @Synchronized
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<List<EwbCalcResultKeyValueVo>> calcResult(EwbCalcResultVo param, HeaderUserInfo userInfo) throws Exception {
        EwbAnalysisData analysisData = analysisDataMapper.selectById(param.getDataId());
        if (!param.getTimeForRank().equals(analysisData.getTimeForRank())) {
            // 当切换数据模型后进行计算，则清除原模型的中间数据
            analysisData.setTimeForRank(param.getTimeForRank());
            analysisDataMapper.update(null, new UpdateWrapper<EwbAnalysisData>().lambda().eq(EwbAnalysisData::getDataId, analysisData.getDataId()).set(EwbAnalysisData::getTimeForRank, param.getTimeForRank()));
            analysisService.deleteAnalysisResultData(analysisData.getDataId(), param.getSettingId(), analysisData.getAnalysisId());
        }
        param.setRightFlag(analysisData.getRFlag());
        // 接口调用路径
        //根据distributingCode获取参数值
        String paramter = EwbDistributingCodeEnum.Weibull.getIndex().equals(param.getDistributingCode()) ? param.getBeta() : (EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getDistributingCode()) ? param.getLogSigma() : param.getSigma());
        String apiLifeDataUrl;
        // 正常数据与概率VS时间数据的不同验证方式
        if (analysisData.getTimeForRank().equals(EwbConstants.FLAG_NO_CHECKED)) {
            // 正常数据验证方式
            System.err.println("--1--：" + System.currentTimeMillis());
            validatorService.validatorDataDetail(param.getDataId(), param.getSettingId(), param.getParameterCode(), param.getDistributingCode(), param.getEstimationMethodCode(), analysisData.getLFlag(), userInfo);
            System.err.println("--2--：" + System.currentTimeMillis());
            Long failureCount = analysisDataDetailMapper.selectCount(new QueryWrapper<EwbAnalysisDataDetail>().lambda().eq(EwbAnalysisDataDetail::getDataId, param.getDataId()).eq(EwbAnalysisDataDetail::getFailureSuspension, EwbConstants.STATE_F));
            // ==================
            // 调用外部接口获取相关参数并保存分布向导结果
            // ==================
            if (failureCount == 0) {
                if (StringUtils.isBlank(param.getBelievePercent()) || !ValidateUtil.isNumber(param.getBelievePercent()) || Double.parseDouble(param.getBelievePercent()) < 0 || Double.parseDouble(param.getBelievePercent()) > 100) {
                    throw new DescribeException(EwbCustomPromptEnum.请输入正确的置信下限, userInfo.getLanguageKey());
                } else {
                    analysisSettingMapper.updateBelievePercent(param.getSettingId(), param.getBelievePercent());
                }
            }
            apiLifeDataUrl = rgaApiService.getApiLifeDataUrl(param.getDistributingCode(), param.getEstimationMethodCode(), param.getParameterCode(), paramter, failureCount == 0, StringUtils.isNotBlank(param.getBelievePercent()) ? Double.parseDouble(param.getBelievePercent()) / 100.0 + "" : null);
        } else {
            //概率VS时间数据的验证方式
            apiLifeDataUrl = rgaApiService.getApiLifeDataUrl(param.getDistributingCode(), param.getEstimationMethodCode(), param.getParameterCode(), paramter, false, StringUtils.isNotBlank(param.getBelievePercent()) ? Double.parseDouble(param.getBelievePercent()) / 100.0 + "" : null);
        }
        if (StringUtils.isNotBlank(paramter)) {
            analysisSettingMapper.updateParamter(param.getSettingId(), paramter);
        }
        EwbAnalysisSetting analysisSetting = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select("setting_id", "calc_result", "beta", "sigma", "logsigma", "time", "un_reliability", "beta_mark", "eta_mark", "mu_mark", "sigma_mark", "lambda_mark", "target_time", "target_reliability").lambda().eq(EwbAnalysisSetting::getSettingId, param.getSettingId()).eq(EwbAnalysisSetting::getDistributingCode, param.getDistributingCode()).eq(EwbAnalysisSetting::getEstimationMethodCode, param.getEstimationMethodCode()).eq(EwbAnalysisSetting::getParameterCode, param.getParameterCode()).eq(EwbAnalysisSetting::getBelievePercent, Double.parseDouble(param.getBelievePercent())));
        String results = "";
        if (analysisSetting != null && EwbParameterCodeEnum.一参数.getIndex().equals(param.getParameterCode())) {
            if ((EwbDistributingCodeEnum.Weibull.getIndex().equals(param.getDistributingCode()) && Double.parseDouble(param.getBeta()) == analysisSetting.getBeta()) || (EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getDistributingCode()) && Double.parseDouble(param.getLogSigma()) == analysisSetting.getLogsigma()) || (EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode()) && Double.parseDouble(param.getSigma()) == analysisSetting.getSigma())) {
                Long failureCount = analysisDataDetailMapper.selectCount(new QueryWrapper<EwbAnalysisDataDetail>().lambda().eq(EwbAnalysisDataDetail::getDataId, param.getDataId()).eq(EwbAnalysisDataDetail::getFailureSuspension, EwbConstants.STATE_F));
                if (failureCount != 0) {
                    results = StringUtils.isBlank(analysisSetting.getCalcResult()) ? "" : analysisSetting.getCalcResult();
                }
            }
        } else if (analysisSetting != null && StringUtils.isNotBlank(analysisSetting.getCalcResult())) {
            results = analysisSetting.getCalcResult();
        }
        if (StringUtils.isBlank(results)) {
            // 数据类型
            String apiDataType = rgaApiService.getDataType(analysisData.getRFlag(), analysisData.getLFlag());
            // 准备RGA计算函数数据
            JSONObject requestFunctionData = new JSONObject();
            // 调用api的输入参数
            JSONObject requestLifeData = rgaApiService.getRequestLifeDataJson(param.getDataId(), apiDataType, new SolveDemoRelWeibullChiSquareRelVo(), !analysisData.getTimeForRank().equals(EwbConstants.FLAG_NO_CHECKED));
            if (requestLifeData == null) {
                return ServerResponse.createBySuccess();
            }
            requestFunctionData.put("data", requestLifeData.toString());
            results = rgaApiService.getResultsCallApi(requestLifeData.toString(), apiLifeDataUrl, EwbConstants.RGA_API_CONTENTTYPE_1, null);
//            apiAnalysisResultMapper.delete(new QueryWrapper<EwbApiAnalysisResult>().lambda().eq(EwbApiAnalysisResult::getDataId, param.getDataId()));
            redisTemplate.delete(analysisData.getAnalysisId());
        }
        analysisSetting = analysisSetting == null ? new EwbAnalysisSetting() : analysisSetting;
        if (StringUtils.isBlank(analysisSetting.getSettingId())) {
            // 清除原来的标记信息
            analysisSettingMapper.deleteMarkSetting(param.getSettingId());
        }
        analysisSetting.setCalcResult(results);
        analysisSetting.setParameterCode(param.getParameterCode());
        analysisSetting.setDistributingCode(param.getDistributingCode());
        analysisSetting.setBelievePercent(param.getBelievePercent() == null ? null : Double.parseDouble(param.getBelievePercent()));
        analysisSetting.setEstimationMethodCode(param.getEstimationMethodCode());
        analysisSetting.setBeta(param.getBeta() == null ? null : Double.parseDouble(param.getBeta()));
        analysisSetting.setSigma(param.getSigma() == null ? null : Double.parseDouble(param.getSigma()));
        analysisSetting.setLogsigma(param.getLogSigma() == null ? null : Double.parseDouble(param.getLogSigma()));
        analysisSetting.setSettingId(param.getSettingId());
        // 更新计算结果部分的数据，不删除点数据，降低后续使用的接口调用数量
        JSONObject calcResultAll = JSON.parseObject(analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select("setting_Id", "calc_result_all").lambda().eq(EwbAnalysisSetting::getSettingId, param.getSettingId())).getCalcResultAll());
        if (null != calcResultAll) {
            calcResultAll.put(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.results), JSONObject.parse(results));
            analysisSetting.setCalcResultAll(calcResultAll.toJSONString());
        } else {
            calcResultAll = new JSONObject();
            calcResultAll.put(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.results), JSONObject.parse(results));
        }
        System.err.println("--3--：" + System.currentTimeMillis());
        analysisSettingMapper.updateById(analysisSetting);
        System.err.println("--4--：" + System.currentTimeMillis());
        // 此处判断是否存在方差为负数的数据，若存在方差为负数的数据，则需要将isRankOrNodata置成1，并将前端计算器tab页面的状态修改为readOnly
        analysisDataMapper.update(null, new UpdateWrapper<EwbAnalysisData>().lambda().eq(EwbAnalysisData::getDataId, analysisData.getDataId()).set(EwbAnalysisData::getIsRankOrNodata, EwbPlotService.judgeUclLclInterval(allVariableConfig.getServerSystem(), calcResultAll, param.getDistributingCode()) ? EwbConstants.FLAG_CHECKED : EwbConstants.FLAG_NO_CHECKED));
        EwbAnalysisResult analysisResult = new EwbAnalysisResult();
        System.err.println("--5--：" + System.currentTimeMillis());
        if (results.indexOf("error") >= 1 || results.indexOf("Exception") >= 1) {
            throw new DescribeException(EwbCustomPromptEnum.无法计算, userInfo.getLanguageKey());
        }
        List<EwbCalcResultKeyValueVo> result = handleCalcResult(allVariableConfig.getServerSystem(), JSONObject.parseObject(results), analysisResult, rgaApiService, param, analysisData.getTimeForRank().equals(EwbConstants.FLAG_CHECKED));
        // 添加了控制前端显示置信水平的字段
        result.add(new EwbCalcResultKeyValueVo("failureCount", apiLifeDataUrl.contains(param.getBeta() + "&CL=" + (Double.parseDouble(param.getBelievePercent()) / 100.0 + "")) ? param.getBelievePercent() : 0));
        // 将最新的状态返回给前端，确保计算器的置信区间的readOnly状态正确
        result.add(new EwbCalcResultKeyValueVo("isRankOrNodata", analysisData.getIsRankOrNodata()));
        EwbAnalysisService.updateAnalysisResult(param, analysisResult, analysisResultMapper);
        System.err.println("--6--：" + System.currentTimeMillis());
        // 处理内华达的计算状态
        EwbNonNevadaSetting nonNevadaSetting = nonNevadaSettingMapper.selectOne(new QueryWrapper<EwbNonNevadaSetting>().lambda().eq(EwbNonNevadaSetting::getNevadaId,analysisData.getAnalysisId()));
        if (nonNevadaSetting != null) {
            nonNevadaSetting.setIsComplete(WhetherFlagEnum.YES.getIndex());
            nonNevadaSettingMapper.updateById(nonNevadaSetting);
        }
        return ServerResponse.createBySuccess(result);
    }

    /**
     * 处理计算结果数据
     *
     * @param jsonObj
     * @param analysisResult
     * @param rgaApiService
     * @param param
     * @return
     */
    public static List<EwbCalcResultKeyValueVo> handleCalcResult(String serverSystem, JSONObject jsonObj, EwbAnalysisResult analysisResult, RgaApiService rgaApiService, EwbCalcResultVo param, boolean isRankOrNodata) {
        double beta = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.beta));
        double eta = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.eta));
        double gamma = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.gamma));
        double varBeta = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varBeta));
        double varEta = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varEta));
        double covBetaEta = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.covBetaEta));
        double varMu = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varMu));
        double varSigma = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varSigma));
        double covMuSigma = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.covMuSigma));
        double varLogMu = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varLogMu));
        double varLogSigma = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varLogSigma));
        double covLogMuSigma = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.covLogMuSigma));
        double varLambda = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varLambda));
        double mu = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mu));
        double sigma = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.sigma));
        double logMu = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logMu));
        double logSigma = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logSigma));
        double lambda = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lambda));
        double lkv = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lkv));
        double pho = StringUtil.parseJsonDouble(jsonObj, EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.pho));

        JSONArray mixBeta = jsonObj.getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixBeta)) == null ? new JSONArray() : jsonObj.getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixBeta));
        JSONArray mixEta = jsonObj.getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixEta)) == null ? new JSONArray() : jsonObj.getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixEta));
        JSONArray mixP = jsonObj.getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixP)) == null ? new JSONArray() : jsonObj.getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixP));

        double p1, p2, p3 = 0, rao, lk, db = 0, de = 0, dbe = 0, b10, mttf, mixBeta1, mixBeta2, mixBeta3, mixEta1, mixEta2, mixEta3, mixP1 = 0, mixP2 = 0, mixP3 = 0;
        rao = Double.isInfinite(pho) ? -99999999
                : WeibullNumberUtil.round(pho, EwbConstants.NUM_DIGIT_6);
        lk = WeibullNumberUtil.round(lkv, EwbConstants.NUM_DIGIT_6);

        List<EwbCalcResultKeyValueVo> result = new ArrayList<>();
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(param.getDistributingCode())) {
            if (!EwbParameterCodeEnum.混合2参数.getIndex().equals(param.getParameterCode())
                    && !EwbParameterCodeEnum.混合3参数.getIndex().equals(param.getParameterCode())) {
                p1 = WeibullNumberUtil.round(beta, EwbConstants.NUM_DIGIT_6);
                p2 = WeibullNumberUtil.round(eta, EwbConstants.NUM_DIGIT_6);
                p3 = WeibullNumberUtil.round(gamma, EwbConstants.NUM_DIGIT_6);
                mttf = rgaApiService.calcMttf(param.getDistributingCode(), p1, p2);
                b10 = rgaApiService.calcB10(p1, p2);
                db = WeibullNumberUtil.round(varBeta, EwbConstants.NUM_DIGIT_9);
                de = WeibullNumberUtil.round(varEta, EwbConstants.NUM_DIGIT_9);
                dbe = WeibullNumberUtil.round(covBetaEta, EwbConstants.NUM_DIGIT_9);
                result.add(new EwbCalcResultKeyValueVo("β", p1));
                result.add(new EwbCalcResultKeyValueVo("η", p2));
                if (EwbParameterCodeEnum.三参数.getIndex().equals(param.getParameterCode())) {
                    result.add(new EwbCalcResultKeyValueVo("γ", p3));
                }
                result.add(new EwbCalcResultKeyValueVo("LK", isRankOrNodata ? "N/A" : lk));
                result.add(new EwbCalcResultKeyValueVo("ρ", rao == -99999999 ? "-" : rao));
            } else {
                mixBeta1 = mixBeta.getDouble(0);
                mixBeta2 = mixBeta.getDouble(1);
                mixBeta3 = mixBeta.getDouble(2);
                mixEta1 = mixEta.getDouble(0);
                mixEta2 = mixEta.getDouble(1);
                mixEta3 = mixEta.getDouble(2);
                mixP1 = mixP.getDouble(0);
                mixP2 = 1 - mixP1;

                p1 = WeibullNumberUtil.round(mixBeta1, EwbConstants.NUM_DIGIT_6);
                p2 = WeibullNumberUtil.round(mixEta1, EwbConstants.NUM_DIGIT_6);
                p3 = WeibullNumberUtil.round(mixBeta2, EwbConstants.NUM_DIGIT_6);
                //db = NumberUtils.round(mixEta2, EwbConstants.NUM_DIGIT_6); // Used this variable or
                mttf = rgaApiService.calcMttf(param.getDistributingCode(), p1, p2);
                b10 = rgaApiService.calcB10(p1, p2);
                db = WeibullNumberUtil.round(mixEta2, EwbConstants.NUM_DIGIT_6); // Used this variable or
                // column for Mixed Eta 2
                // value
                de = WeibullNumberUtil.round(mixBeta3, EwbConstants.NUM_DIGIT_6); // Used this variable or
                // column for Mixed Beta 3
                // value
                dbe = WeibullNumberUtil.round(mixEta3, EwbConstants.NUM_DIGIT_6); // Used this variable or
                // column for Mixed Eta 3
                // value
                mixP1 = WeibullNumberUtil.round(mixP1, EwbConstants.NUM_DIGIT_6);
                result.add(new EwbCalcResultKeyValueVo("β1", p1));
                result.add(new EwbCalcResultKeyValueVo("η1", p2));
                result.add(new EwbCalcResultKeyValueVo("P1", mixP1));
                result.add(new EwbCalcResultKeyValueVo("β2", p3));
                result.add(new EwbCalcResultKeyValueVo("η2", db));
                if (EwbParameterCodeEnum.混合3参数.getIndex().equals(param.getParameterCode())) {
                    mixP2 = mixP.getDouble(1);
                    mixP3 = 1 - mixP1 - mixP2;
                    result.add(new EwbCalcResultKeyValueVo("P2", WeibullNumberUtil.round(mixP2, EwbConstants.NUM_DIGIT_6)));
                    result.add(new EwbCalcResultKeyValueVo("P3", WeibullNumberUtil.round(mixP3, EwbConstants.NUM_DIGIT_6)));
                    result.add(new EwbCalcResultKeyValueVo("β3", de));
                    result.add(new EwbCalcResultKeyValueVo("η3", dbe));
                } else {
                    result.add(new EwbCalcResultKeyValueVo("P2", WeibullNumberUtil.round(mixP2, EwbConstants.NUM_DIGIT_6)));
                }
                result.add(new EwbCalcResultKeyValueVo("LK", isRankOrNodata ? "N/A" : lk));
                result.add(new EwbCalcResultKeyValueVo("ρ", rao == -99999999 ? "-" : rao));
            }
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getDistributingCode())) {
            p1 = WeibullNumberUtil.round(logMu, EwbConstants.NUM_DIGIT_6);
            p2 = WeibullNumberUtil.round(logSigma, EwbConstants.NUM_DIGIT_6);
            lk = WeibullNumberUtil.round(lkv, EwbConstants.NUM_DIGIT_6);
            mttf = rgaApiService.calcMttf(param.getDistributingCode(), p1, p2);
            b10 = rgaApiService.calcB10(p1, p2);
            db = WeibullNumberUtil.round(varLogMu, EwbConstants.NUM_DIGIT_9);
            de = WeibullNumberUtil.round(varLogSigma, EwbConstants.NUM_DIGIT_9);
            dbe = WeibullNumberUtil.round(covLogMuSigma, EwbConstants.NUM_DIGIT_9);
            result.add(new EwbCalcResultKeyValueVo("u'", p1));
            result.add(new EwbCalcResultKeyValueVo("σ'", p2));
            result.add(new EwbCalcResultKeyValueVo("LK", isRankOrNodata ? "N/A" : lk));
            result.add(new EwbCalcResultKeyValueVo("ρ", rao == -99999999 ? "-" : rao));
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode())) {
            p1 = WeibullNumberUtil.round(mu, EwbConstants.NUM_DIGIT_6);
            p2 = WeibullNumberUtil.round(sigma, EwbConstants.NUM_DIGIT_6);
            lk = WeibullNumberUtil.round(lkv, EwbConstants.NUM_DIGIT_6);
            mttf = rgaApiService.calcMttf(param.getDistributingCode(), p1, p2);
            b10 = rgaApiService.calcB10(p1, p2);
            if (!"NaN".equals(String.valueOf(varMu))) {
                db = WeibullNumberUtil.round(varMu, EwbConstants.NUM_DIGIT_9);
            }
            if (!"NaN".equals(String.valueOf(varSigma))) {
                de = WeibullNumberUtil.round(varSigma, EwbConstants.NUM_DIGIT_9);
            }
            if (!"NaN".equals(String.valueOf(covMuSigma))) {
                dbe = WeibullNumberUtil.round(covMuSigma, EwbConstants.NUM_DIGIT_9);
            }
            result.add(new EwbCalcResultKeyValueVo("u", p1));
            result.add(new EwbCalcResultKeyValueVo("σ", p2));
            result.add(new EwbCalcResultKeyValueVo("LK", isRankOrNodata ? "N/A" : lk));
            result.add(new EwbCalcResultKeyValueVo("ρ", rao == -99999999 ? "-" : rao));
        } else {
            lambda = eta > 0.0 ? (1 / eta) : lambda;
            p1 = WeibullNumberUtil.round(lambda, EwbConstants.NUM_DIGIT_6);
            p2 = WeibullNumberUtil.round(lambda, EwbConstants.NUM_DIGIT_6);
            lk = WeibullNumberUtil.round(lkv, EwbConstants.NUM_DIGIT_6);
            result.add(new EwbCalcResultKeyValueVo("λ", p1));
            if (EwbParameterCodeEnum.二参数.getIndex().equals(param.getParameterCode())) {
                p3 = WeibullNumberUtil.round(gamma, EwbConstants.NUM_DIGIT_6);
                result.add(new EwbCalcResultKeyValueVo("γ", WeibullNumberUtil.round(gamma, EwbConstants.NUM_DIGIT_6)));
            }
            mttf = rgaApiService.calcMttf(param.getDistributingCode(), lambda, lambda);
            b10 = rgaApiService.calcB102(p1, jsonObj.getString("lambda"));
            db = WeibullNumberUtil.round(varEta, EwbConstants.NUM_DIGIT_9);
            de = WeibullNumberUtil.round(varLambda, EwbConstants.NUM_DIGIT_9);
            dbe = WeibullNumberUtil.round(covMuSigma, EwbConstants.NUM_DIGIT_9);
            result.add(new EwbCalcResultKeyValueVo("LK", isRankOrNodata ? "N/A" : lk));
            result.add(new EwbCalcResultKeyValueVo("ρ", rao == -99999999 ? "-" : rao));
        }
        analysisResult.setP1(p1);
        analysisResult.setP2(p2);
        analysisResult.setP3(p3);
        analysisResult.setRao(rao);
        // -10000.0设定成固定N/A
        analysisResult.setLk(isRankOrNodata ? 0.0 : lk);
        analysisResult.setMttf(mttf);
        analysisResult.setB10(b10);
        analysisResult.setDb(db);
        analysisResult.setDe(de);
        analysisResult.setDbe(dbe);
        analysisResult.setInterval1(mixP1);
        analysisResult.setInterval2(mixP2);
        analysisResult.setInterval3(mixP3);
        return result;
    }
}
