package com.nationrel.modules.weibull.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.common.WhetherFlagEnum;
import com.nationrel.modules.weibull.general.util.common.BasicDataUtil;
import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.dto.EwbNevadaCreateDto;
import com.nationrel.modules.weibull.entity.EwbAnalysisInfo;
import com.nationrel.modules.weibull.entity.EwbNonNevadaSetting;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.enums.EwbConfidenceBoundEnum;
import com.nationrel.modules.weibull.enums.EwbConfidenceTypeCodeEnum;
import com.nationrel.modules.weibull.enums.EwbNevadaForecastTypeEnum;
import com.nationrel.modules.weibull.vo.EwbAnalysisReqVo;
import com.nationrel.modules.weibull.vo.NevadaCreateVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * @author Aimee on 2020/9/15.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisInfoService {

    private final EwbAnalysisInfoMapper analysisInfoMapper;
    private final EwbNonNevadaDataMapper nonNevadaDataMapper;
    private final EwbNonNevadaSettingMapper nonNevadaSettingMapper;
    private final EwbAnalysisService analysisService;
    private final EwbDaInfoService daInfoService;

    @Transactional(rollbackFor = Exception.class)
    public Result createAnalysisInfo(HeaderUserInfo userInfo, String wbName, String objectCode, String objectName, String description, String analysisType) {

        // 创建项目信息
        String wbId = UUIDUtil.getUuid();
        EwbAnalysisInfo analysisInfo = EwbAnalysisInfo.builder().wbId(wbId)
                .wbName(wbName)
                .objectCode(objectCode)
                .analysisType(analysisType)
                .objectName(objectName)
                .description(description)
                .createTime(new Date())
                .createBy(userInfo.getUserName())
                .updateTime(new Date())
                .versionNo(0)
                .delFlag("0").build();
        analysisInfoMapper.insert(analysisInfo);

        // 创建分析信息
        String analysisId = UUIDUtil.getUuid();
        switch (analysisType) {
            case "life":
            case "weibull":
                analysisService.createAnalysis(userInfo, wbId, analysisId, "分析1", null, null, null);
                break;
            case "degradation":
                daInfoService.create(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.分析, userInfo.getLanguageKey()) + "1", wbId, userInfo);
                break;
            case "costOpti":
                //daInfoService.create("分析1", wbId, userInfo);
                break;
            case "nevadaNRI":
                EwbNevadaCreateDto createDto = EwbNonNevadaService.createDto(NevadaCreateVo.builder().wbId(wbId).analysisName(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.分析, userInfo.getLanguageKey()) + "1").build(), userInfo);
                EwbNonNevadaSetting nevadaSetting = createDto.getNonNevadaSetting();
                EwbAnalysisReqVo analysis = analysisService.createAnalysis(userInfo, wbId, nevadaSetting.getNevadaId(), "", null, null, null);
                nevadaSetting.setDataId(analysis.getDataId());
                nevadaSetting.setSettingId(analysis.getSettingId());
                nevadaSetting.setForecastType(EwbNevadaForecastTypeEnum.未设置.getIndex());
                nevadaSetting.setConfidenceBound(EwbConfidenceBoundEnum.双侧.getIndex());
                nevadaSetting.setConfidenceLevel(EwbConfidenceBoundEnum.双侧.getConfidenceLevel());
                nevadaSetting.setBoundSided(EwbConfidenceTypeCodeEnum.无.getIndex());
                nevadaSetting.setIsComplete(WhetherFlagEnum.NO.getIndex());
                nonNevadaSettingMapper.insert(nevadaSetting);
                nonNevadaDataMapper.insertBatch(createDto.getNonNevadaData());
                break;
            case "nevadaRI":
                EwbNevadaService.createDto(NevadaCreateVo.builder().wbId(wbId).analysisName(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.分析, userInfo.getLanguageKey()) + "1").nevadaType("nevadaRI").build(), userInfo);
                break;
            default:
                break;
        }

        return Result.ok(analysisInfo);
    }

    public IPage<EwbAnalysisInfo> listByPage(String analysisName, HeaderUserInfo userInfo, String analysisType, int pageIndex, int pageSize) {
        analysisName = BasicDataUtil.parseEmptyStr(analysisName);
        QueryWrapper<EwbAnalysisInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(EwbAnalysisInfo::getDelFlag, "0");
        queryWrapper.lambda().eq(EwbAnalysisInfo::getCreateBy, userInfo.getUserName());
        queryWrapper.lambda().eq(EwbAnalysisInfo::getAnalysisType, analysisType);
        if (StringUtils.isNotBlank(analysisName)) {
            queryWrapper.lambda().like(EwbAnalysisInfo::getWbName, analysisName);
        }
        Page<EwbAnalysisInfo> page = new Page<>(pageIndex, pageSize);
        return analysisInfoMapper.selectPage(page, queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    public Result editProject(HeaderUserInfo userInfo, String wbId, String wbName, String objectCode, String objectName, String description) {
        EwbAnalysisInfo analysisInfo = analysisInfoMapper.selectById(wbId);
        if (analysisInfo == null) {
            return Result.error(EwbCustomPromptEnum.getLanguageMessageByKey(EwbCustomPromptEnum.项目信息不存在, userInfo.getLanguageKey()));
        }
        EwbAnalysisInfo newEwbAnalysis = EwbAnalysisInfo.builder()
                .wbId(wbId)
                .wbName(wbName)
                .objectCode(objectCode)
                .objectName(objectName)
                .description(description)
                .updateTime(new Date())
                .updateBy(userInfo.getUserName())
                .updateTime(new Date())
                .delFlag("0").build();
        return Result.ok(analysisInfoMapper.updateById(newEwbAnalysis));
    }

    @Transactional(rollbackFor = Exception.class)
    public void delete(String wbId) {
        analysisInfoMapper.deleteWb(wbId);
    }

    /**
     * 更新修改者信息
     *
     * @param userName
     * @param analysisId
     */
    public static void updateUpdateInfo(String userName, String analysisId, EwbAnalysisInfoMapper analysisInfoMapper, EwbAnalysisMapper analysisMapper) {
        String wbId = analysisMapper.selectWbIdByAnalysisId(analysisId);
        analysisInfoMapper.updateUpdateInfo(new Date(), userName, wbId);
    }
}
