package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.bo.EwbAnalysisSettingSaveBo;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.*;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.util.common.BasicDataUtil;
import com.nationrel.modules.weibull.general.util.common.DateTimeUtil;
import com.nationrel.modules.weibull.general.util.common.ObjectActionUtil;
import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.util.StringUtil;
import com.nationrel.modules.weibull.util.ValidateUtil;
import com.nationrel.modules.weibull.vo.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author Aimee on 2020/9/15.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisService {

    private final RgaApiService rgaApiService;
    private final EwbAnalysisMapper analysisMapper;
    private final StringRedisTemplate redisTemplate;
    private final EwbDaInfoMapper daInfoMapper;
    private final EwbNevadaSettingMapper nevadaSettingMapper;
    private final EwbNonNevadaSettingMapper nonNevadaSettingMapper;
    private final EwbAnalysisInfoMapper analysisInfoMapper;
    private final EwbAnalysisDataMapper analysisDataMapper;
    private final EwbAnalysisRankMapper analysisRankMapper;
    private final EwbAnalysisDataService analysisDataService;
    private final EwbAnalysisResultMapper analysisResultMapper;
    private final EwbReplaceIntervalMapper replaceIntervalMapper;
    private final EwbInspectIntervalMapper inspectIntervalMapper;
    private final EwbAnalysisSettingMapper analysisSettingMapper;
    private final EwbAnalysisSettingService analysisSettingService;
    private final EwbMultiChartDetailMapper multiChartDetailMapper;
    private final EwbAnalysisDisWizardMapper analysisDisWizardMapper;
    private final EwbAnalysisFrBelieveMapper analysisFrBelieveMapper;
    private final EwbAnalysisPrBelieveMapper analysisPrBelieveMapper;
    //    private final EwbApiAnalysisResultMapper apiAnalysisResultMapper;
    private final EwbAnalysisDataRankMapper analysisDataRankMapper;
    private final EwbAnalysisDataDetailMapper analysisDataDetailMapper;
    private final EwbAnalysisResultDetailMapper analysisResultDetailMapper;
    private final EwbAnalysisNodataSettingMapper analysisNodataSettingMapper;

    private final AllVariableConfig allVariableConfig;

    @Transactional(rollbackFor = Exception.class)
    public void analysisCopy(String analysisId, String dataId, String settingId) {
        String targetAnalysisId = UUIDUtil.getUuid();
        String targetDataId = UUIDUtil.getUuid();
        String targetSettingId = UUIDUtil.getUuid();
        analysisMapper.copyData(analysisId, targetAnalysisId);
        analysisDataMapper.copyData(analysisId, targetAnalysisId, targetDataId);
        analysisDataDetailMapper.copyData(dataId, targetDataId);
        analysisDisWizardMapper.copyData(dataId, targetDataId);
        analysisFrBelieveMapper.copyData(settingId, targetSettingId);
        analysisPrBelieveMapper.copyData(settingId, targetSettingId);
//        analysisRankMapper.copyData(dataId, targetDataId);
        analysisResultMapper.copyData(settingId, targetSettingId);
        analysisResultDetailMapper.copyData(settingId, targetSettingId);
        EwbAnalysisSetting setting = analysisSettingMapper.selectById(settingId);
        setting.setSettingId(targetSettingId);
        setting.setDataId(targetDataId);
        analysisSettingMapper.insert(setting);
//        apiAnalysisResultMapper.copyData(dataId, targetAnalysisId, targetDataId);
    }

    @Transactional(rollbackFor = Exception.class)
    public EwbAnalysisReqVo createAnalysis(HeaderUserInfo userInfo, String wbId, String analysisId, String analysisName,
                                           String objectCode, String objectName, String description) {

        EwbAnalysisReqVo analysisReqVo = new EwbAnalysisReqVo();
        EwbAnalysis analysis = EwbAnalysis.builder().analysisId(analysisId).analysisName(analysisName).wbId(wbId)
                .objectCode(objectCode).objectName(objectName).description(description).versionNo(0)
                .insertTime(new Date()).updateTime(new Date()).insertUserId(userInfo.getUserId()).delFlag("0").build();
        analysisMapper.insert(analysis);
        // 创建设置信息
        EwbAnalysisSettingSaveBo analysisSettingSaveBo = new EwbAnalysisSettingSaveBo();
        String settingId = UUIDUtil.getUuid();
        String dataId = UUIDUtil.getUuid();
        analysisSettingSaveBo.setSettingId(settingId);
        analysisSettingSaveBo.setDistributingCode(EwbDistributingCodeEnum.Weibull.getIndex());
        analysisSettingSaveBo.setParameterCode(EwbParameterCodeEnum.二参数.getIndex());
        analysisSettingSaveBo.setEstimationMethodCode(EwbEvaluateCodeEnum.mle.getIndex());
        analysisSettingSaveBo.setRankMethodCode("01");
        analysisSettingSaveBo.setBelieveCode(EwbConfidenceTypeCodeEnum.双侧置信限.getIndex());
        analysisSettingSaveBo.setBelieveMethodCode(EwbConfidenceMethodCodeEnum.fishermatrix.getIndex());
        analysisSettingSaveBo.setBelievePercent("63.2");
        analysisSettingService.createAnalysisSetting(userInfo, analysisSettingSaveBo, dataId);
        // 创建数据信息
        analysisDataService.createAnalysisData(userInfo, dataId, analysisId);
        EwbAnalysisInfoService.updateUpdateInfo(userInfo.getUserName(), analysisId, analysisInfoMapper, analysisMapper);

        BeanUtils.copyProperties(analysis, analysisReqVo);
        analysisReqVo.setDataId(dataId);
        analysisReqVo.setSettingId(settingId);
        return analysisReqVo;
    }

    public JSONObject getApiResult(EwbAnalysisStartVo param) throws Exception {
        // redis存在则直接获取
        String resultAll = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select(EwbConstants.SETTING_ID, "calc_result_all").lambda().eq(EwbAnalysisSetting::getSettingId, param.getSettingId())).getCalcResultAll();
        // 不同的系统环境数据判断方式不一样
        if (ValidateUtil.isNotEmpty(resultAll)) {
            if (EwbConstants.SERVER_SYSTEM_WINDOWS.equals(allVariableConfig.getServerSystem())) {
                return JSONObject.parseObject(resultAll);
            } else {
                JSONObject requestPlotData = JSONObject.parseObject(resultAll);
                if (requestPlotData.getJSONObject("data") != null) {
                    if (requestPlotData.getJSONObject("data").get("failureRank") == null
                            || requestPlotData.getJSONObject("data").get("failureRank").equals("[]")) {
                        requestPlotData.getJSONObject("data").put("failureRank", new double[0]);
                    }
                    return requestPlotData;
                }
            }
        }
        EwbAnalysisSetting analysisSetting = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select(EwbConstants.SETTING_ID, "data_id", "distributing_code", "parameter_code", "estimation_method_code", "rank_method_code", "believe_code", "believe_method_code", "paramter", "believe_percent", "beta", "sigma", "logsigma").lambda().eq(EwbAnalysisSetting::getSettingId, param.getSettingId()));

        // ==================
        // 调用外部接口获取画图参数值
        // ==================
        EwbAnalysisData analysisData = analysisDataMapper.selectById(param.getDataId());
        boolean isRank = analysisData.getTimeForRank().equals(EwbConstants.FLAG_CHECKED) && analysisDataRankMapper.selectCount(new QueryWrapper<EwbAnalysisDataRank>().lambda().eq(EwbAnalysisDataRank::getDataId, param.getDataId())) > 0;
        String distributingCode = analysisSetting.getDistributingCode();
        String parameter = rgaApiService.getParameter(distributingCode, StringUtil.parseString(param.getBeta()), StringUtil.parseString(param.getSigma()), StringUtil.parseString(param.getLogsigma()), analysisSetting);
        if (StringUtils.isNotBlank(parameter)) {
            analysisSettingMapper.updateParamter(param.getSettingId(), parameter);
        }
        Integer rightQuantity = analysisDataDetailMapper.sumRightQuantity(param.getDataId());
        Integer sumQuantity = analysisDataDetailMapper.sumQuantity(param.getDataId());
        if (sumQuantity == null || rightQuantity == null || rightQuantity.equals(sumQuantity)) {
            analysisSetting.setEstimationMethodCode("00");
        }
        // 数据类型
        String apiDataType = rgaApiService.getDataType(analysisData.getRFlag(), analysisData.getLFlag());
        // 准备RGA计算函数数据
        JSONObject requestFunctionData = new JSONObject();
        // 调用api的输入参数
        JSONObject requestLifeData = rgaApiService.getRequestLifeDataJson(param.getDataId(), apiDataType, new SolveDemoRelWeibullChiSquareRelVo(), isRank);
        requestFunctionData.put("data", requestLifeData.toString());
        // 第一步：执行调用接口并获取调用API返回结果
        String responseResultsForLifeData = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select(EwbConstants.SETTING_ID, "calc_result").lambda().eq(EwbAnalysisSetting::getSettingId, param.getSettingId())).getCalcResult();
        // 第二步：调用接口获取失效率
        String apiFailureRankUrl = allVariableConfig.getReliaQubeUrl() + EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.API_LIFEDATA_FAILURERANK);
        String responseResultsForFailureRank = rgaApiService.getResultsCallApi(requestLifeData.toString(), apiFailureRankUrl, EwbConstants.RGA_API_CONTENTTYPE_1, null);
        JSONObject finalResultsJsonForFailureRank = JSONObject.parseObject(requestFunctionData.get("data").toString());
        finalResultsJsonForFailureRank.remove(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.failureRank));
        finalResultsJsonForFailureRank.put(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.failureRank), isRank ? requestLifeData.getJSONArray(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.failureRank)).toJSONString() : JSONArray.parseArray(responseResultsForFailureRank));
        // 第三步：准备画图参数
        String believeCode = analysisSetting.getBelieveCode();
        String believeMethodCode = analysisSetting.getBelieveMethodCode();
        String believePercent = StringUtil.parseString(analysisSetting.getBelievePercent());
        JSONObject requestPlotData = rgaApiService.getApiPlotArgs(believeCode, believeMethodCode, believePercent, responseResultsForLifeData, finalResultsJsonForFailureRank);
        requestLifeData.put(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.failureRank), isRank ? requestLifeData.getJSONArray(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.failureRank)).toJSONString() : JSONArray.parseArray(responseResultsForFailureRank));
        // 更新FailureRank以及DataRow信息
        analysisSettingMapper.updateFailureRankAndRow(param.getSettingId(), !isRank ? responseResultsForFailureRank : requestLifeData.getJSONArray(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.failureRank)).toJSONString());
        requestPlotData.put("maxTime", requestLifeData.get(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.endTime)));
        if (EwbConstants.SERVER_SYSTEM_LINUX.equals(allVariableConfig.getServerSystem()) && requestPlotData.getJSONObject("data") != null && requestPlotData.getJSONObject("data").get("failureRank") == null) {
            requestPlotData.getJSONObject("data").put("failureRank", new double[0]);
        }
        return requestPlotData;
    }

    public List<EwbAnalysisListVo> listByPage(String wbId, String analysisName, HeaderUserInfo userInfo) {
        analysisName = BasicDataUtil.parseEmptyStr(analysisName);
        List<EwbAnalysisListVo> list = analysisMapper.selectSearch(wbId, userInfo.getUserId(), analysisName);
        List<EwbAnalysisListVo> result = new ArrayList<>();
        for (EwbAnalysisListVo analysisListVo : list) {
            if (ValidateUtil.isEmpty(analysisListVo.getParentId())) {
                result.add(analysisListVo);
                analysisListVo.setChildren(treeMenuList(list, analysisListVo.getAnalysisId()));
            }
        }
        return result;
    }

    /**
     * 获取某个父节点下面的所有子节点
     *
     * @param menuList
     * @param pid
     * @return
     */
    private List<EwbAnalysisListVo> treeMenuList(List<EwbAnalysisListVo> menuList, String pid) {
        // 子节点
        List<EwbAnalysisListVo> childMenu = new ArrayList<>();
        for (EwbAnalysisListVo mu : menuList) {
            // 遍历出父id等于参数的id，add进子节点集合
            if (mu.getParentId() != null && mu.getParentId().equals(pid)) {
                // 递归遍历下一级
                // treeMenuList(menuList, mu.getParentId());
                childMenu.add(mu);
            }
        }
        return childMenu;
    }

    @Transactional(rollbackFor = Exception.class)
    public ServerResponse editProject(HeaderUserInfo userInfo, EwbAnalysisSaveVo param) {
        EwbAnalysis analysis = analysisMapper.selectById(param.getAnalysisId());
        if (analysis == null) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.项目信息不存在, userInfo.getLanguageKey());
        }
        if (StringUtils.isNotBlank(param.getEndTimeName())) {
            analysisMapper.updateEndTimeName(param.getAnalysisId(), param.getEndTimeName());
        } else {
            EwbAnalysis newEwbAnalysis = EwbAnalysis.builder().analysisId(param.getAnalysisId())
                    .analysisName(param.getAnalysisName()).objectCode(param.getObjectCode())
                    .objectName(param.getObjectName()).description(param.getDescription()).updateTime(new Date())
                    .updateUserId(userInfo.getUserId()).delFlag("0").build();
            analysisMapper.updateById(newEwbAnalysis);
        }
//        Thread.sleep(100000);
        EwbAnalysisInfoService.updateUpdateInfo(userInfo.getUserName(), param.getAnalysisId(), analysisInfoMapper, analysisMapper);

        return ServerResponse.createBySuccess();
    }

    /**
     * 删除分析数据
     *
     * @param analysisid
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse delAnalysis(String analysisid) {
        EwbAnalysis analysis = analysisMapper.selectById(analysisid);
        // 所有分组分析的id
        List<String> analysisIds = analysisMapper.selectMultiAnalysisIds(analysisid);
        analysisIds.add(analysisid);
        // 存放所有分组分析的dataId
        List<String> dataIds = new ArrayList<>(analysisIds.size());
        List<String> settingIds = new ArrayList<>(analysisIds.size());
        analysisIds.forEach(analysisId -> {
            EwbAnalysisData analysisData = analysisDataMapper.selectOne(new QueryWrapper<EwbAnalysisData>().lambda().eq(EwbAnalysisData::getAnalysisId, analysisId));
            EwbAnalysisSetting analysisSetting = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().lambda().eq(EwbAnalysisSetting::getDataId, analysisData.getDataId()));
            deleteAnalysisResultData(analysisData.getDataId(), analysisSetting.getSettingId(), analysisId);
            dataIds.add(analysisData.getDataId());
            settingIds.add(analysisSetting.getSettingId());
        });
        // 删除列表详细数据
        analysisDataRankMapper.delete(new QueryWrapper<EwbAnalysisDataRank>().lambda().in(EwbAnalysisDataRank::getDataId, dataIds));
        analysisDataDetailMapper.delete(new QueryWrapper<EwbAnalysisDataDetail>().lambda().eq(EwbAnalysisDataDetail::getDataId, dataIds));
        analysisNodataSettingMapper.delete(new QueryWrapper<EwbAnalysisNodataSetting>().lambda().eq(EwbAnalysisNodataSetting::getSettingId, settingIds));
        analysisMapper.deleteById(analysis.getAnalysisId());

        // 删除多图中关联的分析
        multiChartDetailMapper.delete(new QueryWrapper<EwbMultiChartDetail>().lambda().eq(EwbMultiChartDetail::getAnalysisId, analysisIds));
        // 删除成本分析中关联的分析
        inspectIntervalMapper.delete(new QueryWrapper<EwbInspectInterval>().lambda().eq(EwbInspectInterval::getAnalysisId, analysisIds));
        replaceIntervalMapper.delete(new QueryWrapper<EwbReplaceInterval>().lambda().eq(EwbReplaceInterval::getAnalysisId, analysisIds));

        return ServerResponse.createBySuccess();
    }

    /**
     * 删除中间数据
     *
     * @param dataId
     * @param settingId
     * @param analysisId
     */
    public void deleteAnalysisResultData(String dataId, String settingId, String analysisId) {
        // 重置分析数据
        analysisRankMapper.deleteByDataId(dataId);
        analysisResultMapper.deleteByPrimaryKey(settingId);
        analysisResultDetailMapper.deleteAnalysisResultDetail(settingId);
        analysisPrBelieveMapper.deleteBySettingId(settingId);
        analysisFrBelieveMapper.deleteBySettingId(settingId);
        // analysisSettingMapper.deleteByPrimaryKey(settingId);
        analysisDisWizardMapper.deleteByDataId(dataId);
//        apiAnalysisResultMapper.delete(new QueryWrapper<EwbApiAnalysisResult>().lambda().eq(EwbApiAnalysisResult::getDataId, dataId));
        analysisSettingMapper.deleteAllCalcResult(settingId);
        analysisSettingMapper.deleteMarkSetting(settingId);

        redisTemplate.delete(analysisId);
    }

    public EwbAnalysis getAnalysis(String analysisId) {
        EwbAnalysis analysis = analysisMapper.selectById(analysisId);
        if (ObjectActionUtil.isEmpty(analysis)) {
            return new EwbAnalysis();
        }
        return analysis;
    }

    /**
     * 获取所有分析的数据统计集合
     *
     * @param userInfo
     * @return
     */
    public ServerResponse<EwbAnalysisStatisticVo> statistic(HeaderUserInfo userInfo) {
        // 查询出所有基于weibull的分析数据
        List<EwbAnalysisInfo> analysisInfos = analysisInfoMapper.selectList(new QueryWrapper<EwbAnalysisInfo>().lambda().eq(EwbAnalysisInfo::getCreateBy, userInfo.getUserName()));
        // 查询出所有的weibull分析
        List<EwbAnalysis> analysises = analysisMapper.selectList(new QueryWrapper<EwbAnalysis>().lambda().eq(EwbAnalysis::getInsertUserId, userInfo.getUserId()));
        // 查询出所有的退化分析
        List<EwbDaInfo> daInfos = daInfoMapper.selectList(new QueryWrapper<EwbDaInfo>().lambda().eq(EwbDaInfo::getInsertUserId, userInfo.getUserId()));
        // 查询出所有的成本优化分析
        List<EwbInspectInterval> inspectIntervals = inspectIntervalMapper.selectList(new QueryWrapper<EwbInspectInterval>().lambda().eq(EwbInspectInterval::getInsertUserId, userInfo.getUserId()));
        List<EwbReplaceInterval> replaceIntervals = replaceIntervalMapper.selectList(new QueryWrapper<EwbReplaceInterval>().lambda().eq(EwbReplaceInterval::getInsertUserId, userInfo.getUserId()));
        // 查询出所有的nevadaNRI分析
        List<EwbNonNevadaSetting> nonNevadaSettings = nonNevadaSettingMapper.selectList(new QueryWrapper<EwbNonNevadaSetting>().lambda().eq(EwbNonNevadaSetting::getInsertUserId, userInfo.getUserId()));
        // 查询出所有的nevadaRI分析
        List<EwbNevadaSetting> nevadaSettings = nevadaSettingMapper.selectList(new QueryWrapper<EwbNevadaSetting>().lambda().eq(EwbNevadaSetting::getInsertUserId, userInfo.getUserId()));

        EwbAnalysisStatisticNumVo eWeibull = EwbAnalysisStatisticNumVo.builder().analysisNum(0).itemNum(0).lastDate(new Date(0)).lastDateStr("-").build();
        EwbAnalysisStatisticNumVo weibull = EwbAnalysisStatisticNumVo.builder().analysisNum(0).itemNum(0).lastDate(new Date(0)).lastDateStr("-").build();
        EwbAnalysisStatisticNumVo degradation = EwbAnalysisStatisticNumVo.builder().analysisNum(0).itemNum(0).lastDate(new Date(0)).lastDateStr("-").build();
        EwbAnalysisStatisticNumVo costOpti = EwbAnalysisStatisticNumVo.builder().analysisNum(0).itemNum(0).lastDate(new Date(0)).lastDateStr("-").build();
        EwbAnalysisStatisticNumVo nevadaNri = EwbAnalysisStatisticNumVo.builder().analysisNum(0).itemNum(0).lastDate(new Date(0)).lastDateStr("-").build();
        EwbAnalysisStatisticNumVo nevadaRi = EwbAnalysisStatisticNumVo.builder().analysisNum(0).itemNum(0).lastDate(new Date(0)).lastDateStr("-").build();
        // 循环上述查出的泪容进行分类计数处理，生成最终需要获得的数据总数
        if (!CollectionUtils.isEmpty(analysisInfos)) {
            StringBuilder eWeibullStr = new StringBuilder();
            StringBuilder weibullStr = new StringBuilder();
            StringBuilder degradationStr = new StringBuilder();
            StringBuilder costOptiStr = new StringBuilder();
            StringBuilder nevadaNriStr = new StringBuilder();
            StringBuilder nevadaRiStr = new StringBuilder();

            for (EwbAnalysisInfo analysisInfo : analysisInfos) {
                switch (analysisInfo.getAnalysisType()) {
                    case "life":
                        eWeibullStr.append(analysisInfo.getWbId());
                        eWeibull.setAnalysisNum(eWeibull.getAnalysisNum() + 1);
                        if (analysisInfo.getUpdateTime() != null && analysisInfo.getUpdateTime().getTime() > eWeibull.getLastDate().getTime()) {
                            eWeibull.setLastDate(analysisInfo.getUpdateTime());
                            eWeibull.setLastDateStr(DateTimeUtil.dateToStr(analysisInfo.getUpdateTime(), DateTimeUtil.STANDARD_FORMAT_NOS));
                        }
                        break;
                    case "weibull":
                        weibullStr.append(analysisInfo.getWbId());
                        weibull.setAnalysisNum(weibull.getAnalysisNum() + 1);
                        if (analysisInfo.getUpdateTime() != null && analysisInfo.getUpdateTime().getTime() > weibull.getLastDate().getTime()) {
                            weibull.setLastDate(analysisInfo.getUpdateTime());
                            weibull.setLastDateStr(DateTimeUtil.dateToStr(analysisInfo.getUpdateTime(), DateTimeUtil.STANDARD_FORMAT_NOS));
                        }
                        break;
                    case "degradation":
                        degradationStr.append(analysisInfo.getWbId());
                        degradation.setAnalysisNum(degradation.getAnalysisNum() + 1);
                        if (analysisInfo.getUpdateTime() != null && analysisInfo.getUpdateTime().getTime() > degradation.getLastDate().getTime()) {
                            degradation.setLastDate(analysisInfo.getUpdateTime());
                            degradation.setLastDateStr(DateTimeUtil.dateToStr(analysisInfo.getUpdateTime(), DateTimeUtil.STANDARD_FORMAT_NOS));
                        }
                        break;
                    case "costOpti":
                        costOptiStr.append(analysisInfo.getWbId());
                        costOpti.setAnalysisNum(costOpti.getAnalysisNum() + 1);
                        if (analysisInfo.getUpdateTime() != null && analysisInfo.getUpdateTime().getTime() > costOpti.getLastDate().getTime()) {
                            costOpti.setLastDate(analysisInfo.getUpdateTime());
                            costOpti.setLastDateStr(DateTimeUtil.dateToStr(analysisInfo.getUpdateTime(), DateTimeUtil.STANDARD_FORMAT_NOS));
                        }
                        break;
                    case "nevadaNRI":
                        nevadaNriStr.append(analysisInfo.getWbId());
                        nevadaNri.setAnalysisNum(nevadaNri.getAnalysisNum() + 1);
                        if (analysisInfo.getUpdateTime() != null && analysisInfo.getUpdateTime().getTime() > nevadaNri.getLastDate().getTime()) {
                            nevadaNri.setLastDate(analysisInfo.getUpdateTime());
                            nevadaNri.setLastDateStr(DateTimeUtil.dateToStr(analysisInfo.getUpdateTime(), DateTimeUtil.STANDARD_FORMAT_NOS));
                        }
                        break;
                    case "nevadaRI":
                        nevadaRiStr.append(analysisInfo.getWbId());
                        nevadaRi.setAnalysisNum(nevadaRi.getAnalysisNum() + 1);
                        if (analysisInfo.getUpdateTime() != null && analysisInfo.getUpdateTime().getTime() > nevadaRi.getLastDate().getTime()) {
                            nevadaRi.setLastDate(analysisInfo.getUpdateTime());
                            nevadaRi.setLastDateStr(DateTimeUtil.dateToStr(analysisInfo.getUpdateTime(), DateTimeUtil.STANDARD_FORMAT_NOS));
                        }
                        break;
                    default:
                        break;
                }
            }

            if (!CollectionUtils.isEmpty(analysises)) {
                for (EwbAnalysis analysis : analysises) {
                    if (StringUtils.isNotBlank(analysis.getWbId()) && eWeibullStr.toString().contains(analysis.getWbId())) {
                        eWeibull.setItemNum(eWeibull.getItemNum() + 1);
                    } else if (StringUtils.isNotBlank(analysis.getWbId()) && weibullStr.toString().contains(analysis.getWbId())) {
                        weibull.setItemNum(eWeibull.getItemNum() + 1);
                    }
                }
            }

            if (!CollectionUtils.isEmpty(daInfos)) {
                for (EwbDaInfo daInfo : daInfos) {
                    if (degradationStr.toString().contains(daInfo.getWbId())) {
                        degradation.setItemNum(degradation.getItemNum() + 1);
                    }
                }
            }

            if (!CollectionUtils.isEmpty(inspectIntervals)) {
                for (EwbInspectInterval inspectInterval : inspectIntervals) {
                    if (costOptiStr.toString().contains(inspectInterval.getWbId())) {
                        costOpti.setItemNum(costOpti.getItemNum() + 1);
                    }
                }
            }

            if (!CollectionUtils.isEmpty(replaceIntervals)) {
                for (EwbReplaceInterval replaceInterval : replaceIntervals) {
                    if (costOptiStr.toString().contains(replaceInterval.getWbId())) {
                        costOpti.setItemNum(costOpti.getItemNum() + 1);
                    }
                }
            }

            if (!CollectionUtils.isEmpty(nonNevadaSettings)) {
                for (EwbNonNevadaSetting nonNevadaSetting : nonNevadaSettings) {
                    if (nevadaNriStr.toString().contains(nonNevadaSetting.getWbId())) {
                        nevadaNri.setItemNum(nevadaNri.getItemNum() + 1);
                    }
                }
            }

            if (!CollectionUtils.isEmpty(nevadaSettings)) {
                for (EwbNevadaSetting nevadaSetting : nevadaSettings) {
                    if (nevadaRiStr.toString().contains(nevadaSetting.getWbId())) {
                        nevadaRi.setItemNum(nevadaRi.getItemNum() + 1);
                    }
                }
            }
        }
        return ServerResponse.createBySuccess(EwbAnalysisStatisticVo.builder().eWeibull(eWeibull).weibull(weibull).degradation(degradation).costOpti(costOpti).nevadaNRI(nevadaNri).nevadaRI(nevadaRi).build());
    }

    /**
     * 输入数据进行计算时更新设置信息，拼接对应的计算数据
     *
     * @param param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse calcNoData(EwbCalcResultVo param) {
        // 更新概率VS时间的开关
        EwbAnalysisData analysisData = analysisDataMapper.selectById(param.getDataId());
        analysisData.setTimeForRank(param.getTimeForRank());
        analysisDataMapper.updateById(analysisData);
        EwbAnalysisSetting setting = analysisSettingMapper.selectById(param.getSettingId());
        // 默认删除以前全部配置以及数据
        deleteAnalysisResultData(setting.getDataId(), setting.getSettingId(), "analysisId");
        int deleteNum;
        if (param.getTimeForRank().equals(EwbConstants.FLAG_NO_CHECKED)) {
            deleteNum = analysisDataDetailMapper.deleteByDataId(setting.getDataId());
        } else {
            deleteNum = analysisDataRankMapper.deleteByDataId(setting.getDataId());
        }
        if (deleteNum > 0) {
            // 若删除了data数据，则肯定清除setting信息
            analysisSettingMapper.deleteMarkSetting(param.getSettingId());
            setting = analysisSettingMapper.selectById(param.getSettingId());
        }

        JSONObject directCalcData = EwbConstants.DIRECT_CALC_DATA;
        String serverSystem = allVariableConfig.getServerSystem();
        // 生成通用的计算需要的参数数据
        directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.beta), StringUtils.isBlank(param.getBeta()) ? 0 : Double.parseDouble(param.getBeta()));
        directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.eta), StringUtils.isBlank(param.getEta()) ? 0 : Double.parseDouble(param.getEta()));
        directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lambda), StringUtils.isBlank(param.getLambda()) ? 1 : Double.parseDouble(param.getLambda()));
        if (EwbDistributingCodeEnum.Exponential.getIndex().equals(param.getDistributingCode())) {
            directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.eta), 1 / Double.parseDouble(directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).get(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lambda)).toString()));
        }
        directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mu), StringUtils.isBlank(param.getMu()) ? 0 : Double.parseDouble(param.getMu()));
        directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.sigma), StringUtils.isBlank(param.getSigma()) ? 0 : Double.parseDouble(param.getSigma()));
        directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logMu), StringUtils.isBlank(param.getLogMu()) ? 0 : Double.parseDouble(param.getLogMu()));
        directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logSigma), StringUtils.isBlank(param.getLogSigma()) ? 0 : Double.parseDouble(param.getLogSigma()));
        directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.gamma), StringUtils.isBlank(param.getGamma()) ? 0 : Double.parseDouble(param.getGamma()));
        EwbAnalysisNodataSetting nodataSetting = analysisNodataSettingMapper.selectById(param.getSettingId());
        nodataSetting = nodataSetting == null ? new EwbAnalysisNodataSetting() : nodataSetting;
        // --------- 特殊参数Start ---------
        // 混合两参数三参数需要输入MixP
        if (param.getDistributingCode().equals(EwbDistributingCodeEnum.Weibull.getIndex())) {
            if (param.getParameterCode().equals(EwbParameterCodeEnum.混合2参数.getIndex())) {
                directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixP), new double[]{StringUtils.isBlank(param.getProportion()) ? 0 : Double.parseDouble(param.getProportion()), 0, 0});
                directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixBeta), new double[]{StringUtils.isBlank(param.getBeta()) ? 0 : Double.parseDouble(param.getBeta()), StringUtils.isBlank(param.getBetaTwo()) ? 0 : Double.parseDouble(param.getBetaTwo()), 0, 0});
                directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixEta), new double[]{StringUtils.isBlank(param.getBeta()) ? 0 : Double.parseDouble(param.getEta()), StringUtils.isBlank(param.getBetaTwo()) ? 0 : Double.parseDouble(param.getEtaTwo()), 0, 0});
            }
            if (param.getParameterCode().equals(EwbParameterCodeEnum.混合3参数.getIndex())) {
                directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixP), new double[]{StringUtils.isBlank(param.getProportion()) ? 0 : Double.parseDouble(param.getProportion()), StringUtils.isBlank(param.getProportionTwo()) ? 0 : Double.parseDouble(param.getProportionTwo()), 0});
                directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixBeta), new double[]{StringUtils.isBlank(param.getBeta()) ? 0 : Double.parseDouble(param.getBeta()), StringUtils.isBlank(param.getBetaTwo()) ? 0 : Double.parseDouble(param.getBetaTwo()), StringUtils.isBlank(param.getBetaThree()) ? 0 : Double.parseDouble(param.getBetaThree()), 0});
                directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mixEta), new double[]{StringUtils.isBlank(param.getBeta()) ? 0 : Double.parseDouble(param.getEta()), StringUtils.isBlank(param.getBetaTwo()) ? 0 : Double.parseDouble(param.getEtaTwo()), StringUtils.isBlank(param.getBetaThree()) ? 0 : Double.parseDouble(param.getBetaThree()), 0});
            }
        }
        // ---------- 特殊参数End ----------
        setting.setCalcResult(directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).toString());
        setting.setCalcResultAll(directCalcData.toString());
        // 如果信息发生变化，则清除原有的设置
        if (!setting.getParameterCode().equals(param.getParameterCode()) || !setting.getDistributingCode().equals(param.getDistributingCode()) || !setting.getEstimationMethodCode().equals(param.getEstimationMethodCode()) || !isChangedNumber(param, nodataSetting)) {
            // 更新最新的配置信息
            setting.setParameterCode(param.getParameterCode());
            setting.setDistributingCode(param.getDistributingCode());
            setting.setEstimationMethodCode(param.getEstimationMethodCode());
            // 更新原有设置以及此画图模式下的特殊参数
            analysisSettingMapper.updateById(setting);
            analysisSettingMapper.deleteMarkSetting(param.getSettingId());
        } else {
            // 更新原有设置以及此画图模式下的特殊参数
            analysisSettingMapper.updateById(setting);
        }
        analysisNodataSettingMapper.deleteById(setting.getSettingId());
        EwbAnalysisNodataSetting.copyProperties(param, nodataSetting);
        analysisNodataSettingMapper.insert(nodataSetting);
        // 插入计算结果数据
        EwbAnalysisResult analysisResult = new EwbAnalysisResult();
        EwbAnalysisDisWizardService.handleCalcResult(allVariableConfig.getServerSystem(), directCalcData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)), analysisResult, rgaApiService, param, true);
        updateAnalysisResult(param, analysisResult, analysisResultMapper);
        return ServerResponse.createBySuccess();
    }

    /**
     * 画图前判断设置是否已发生修改
     *
     * @param param
     * @param nodataSetting
     * @return
     */
    public boolean isChangedNumber(EwbCalcResultVo param, EwbAnalysisNodataSetting nodataSetting) {
        // 判断weibull各参数配置下的参数是否修改
        if (param.getDistributingCode().equals(EwbDistributingCodeEnum.Weibull.getIndex())) {
            if (param.getParameterCode().equals(EwbParameterCodeEnum.二参数.getIndex())) {
                return isSameWeibullTwo(param, nodataSetting);
            } else if (param.getParameterCode().equals(EwbParameterCodeEnum.三参数.getIndex())) {
                return isSameWeibullMixTwo(param, nodataSetting) && isSameStringDouble(param.getGamma(), nodataSetting.getGamma());
            } else if (param.getParameterCode().equals(EwbParameterCodeEnum.混合2参数.getIndex())) {
                return isSameWeibullMixTwo(param, nodataSetting);
            } else if (param.getParameterCode().equals(EwbParameterCodeEnum.混合3参数.getIndex())) {
                return isSameWeibullMixTwo(param, nodataSetting) && isSameStringDouble(param.getBetaThree(), nodataSetting.getBetaThree()) && isSameStringDouble(param.getEtaThree(), nodataSetting.getEtaThree()) && isSameStringDouble(param.getProportionTwo(), nodataSetting.getProportionTwo());
            }
            // 判断Normal各参数配置下的参数是否修改
        } else if (param.getDistributingCode().equals(EwbDistributingCodeEnum.Normal.getIndex())) {
            return isSameStringDouble(param.getMu(), nodataSetting.getMu()) && isSameStringDouble(param.getSigma(), nodataSetting.getSigma());
            // 判断Lognormal各参数配置下的参数是否修改
        } else if (param.getDistributingCode().equals(EwbDistributingCodeEnum.Lognormal.getIndex())) {
            return isSameStringDouble(param.getLogMu(), nodataSetting.getLogMu()) && isSameStringDouble(param.getLogSigma(), nodataSetting.getLogSigma());
            // 判断Exponential各参数配置下的参数是否修改
        } else if (param.getDistributingCode().equals(EwbDistributingCodeEnum.Exponential.getIndex())) {
            return isSameStringDouble(param.getLambda(), nodataSetting.getLambda());
        }
        return false;
    }

    public boolean isSameWeibullTwo(EwbCalcResultVo param, EwbAnalysisNodataSetting nodataSetting) {
        return isSameStringDouble(param.getBeta(), nodataSetting.getBeta()) && isSameStringDouble(param.getEta(), nodataSetting.getEta());
    }

    public boolean isSameWeibullMixTwo(EwbCalcResultVo param, EwbAnalysisNodataSetting nodataSetting) {
        return isSameWeibullTwo(param, nodataSetting) && isSameStringDouble(param.getBetaTwo(), nodataSetting.getBetaTwo()) && isSameStringDouble(param.getEtaTwo(), nodataSetting.getEtaTwo()) && isSameStringDouble(param.getProportion(), nodataSetting.getEtaTwo());
    }

    public boolean isSameStringDouble(String value1, Double value2) {
        return value2 != null && value2.equals(Double.parseDouble(value1));
    }

    /**
     * 更新计算结果数据
     *
     * @param param
     * @param analysisResult
     * @param analysisResultMapper
     */
    static void updateAnalysisResult(EwbCalcResultVo param, EwbAnalysisResult analysisResult, EwbAnalysisResultMapper analysisResultMapper) {
        analysisResult.setSettingId(param.getSettingId());
        analysisResult.setStepFlag(EwbConstants.STEP_2);
        EwbAnalysisResult ewbAnalysisResult = analysisResultMapper.selectById(param.getSettingId());
        if (ewbAnalysisResult == null || StringUtils.isBlank(ewbAnalysisResult.getSettingId())) {
            analysisResultMapper.insert(analysisResult);
        } else {
            analysisResultMapper.updateById(analysisResult);
        }
    }

    /**
     * 获取概率VS时间
     *
     * @param dataId
     * @return
     */
    public ServerResponse<String> timeForRank(String dataId) {
        return ServerResponse.createBySuccess(analysisDataMapper.selectById(dataId).getTimeForRank());
    }
}
