package com.nationrel.modules.weibull.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.bo.EwbAnalysisSettingBo;
import com.nationrel.modules.weibull.bo.EwbAnalysisSettingSaveBo;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.EwbConfidenceTypeCodeEnum;
import com.nationrel.modules.weibull.enums.EwbDistributingCodeEnum;
import com.nationrel.modules.weibull.enums.EwbParameterCodeEnum;
import com.nationrel.modules.weibull.vo.EwbAnalysisModel;
import com.nationrel.modules.weibull.vo.EwbAnalysisSettingVo;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.util.ValidateUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;

/**
 * @author Aimee on 2020/9/22.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisSettingService {

    private final EwbAnalysisMapper analysisMapper;
    private final StringRedisTemplate redisTemplate;
    private final EwbAnalysisDataMapper analysisDataMapper;
    private final EwbAnalysisInfoMapper analysisInfoMapper;
    private final EwbAnalysisRankMapper analysisRankMapper;
    private final EwbAnalysisResultMapper analysisResultMapper;
    private final EwbAnalysisSettingMapper analysisSettingMapper;
    private final EwbAnalysisDisWizardMapper analysisDisWizardMapper;
    private final EwbAnalysisFrBelieveMapper analysisFrBelieveMapper;
    private final EwbAnalysisPrBelieveMapper analysisPrBelieveMapper;
//    private final EwbApiAnalysisResultMapper apiAnalysisResultMapper;
    private final EwbAnalysisDataRankMapper analysisDataRankMapper;
    private final EwbAnalysisDataDetailMapper analysisDataDetailMapper;
    private final EwbAnalysisResultDetailMapper analysisResultDetailMapper;
    private final EwbAnalysisNodataSettingMapper analysisNodataSettingMapper;


    public EwbAnalysisSettingVo getSetting(EwbAnalysisSettingBo param) {
        EwbAnalysisSettingVo output = new EwbAnalysisSettingVo();
        // 设定ID
        String settingId = param.getSettingId();
        // 分析ID
        String analysisId = param.getAnalysisId();
        // 获取寿命数据-数据量信息
        EwbAnalysisModel analysisModel = analysisDataMapper.searchAanlysisDataByAnalysisId(analysisId);
        int failureQuantity = analysisModel.getFailureQuantity();
        output.setAnalysisId(analysisId);
        if (ValidateUtil.isNotEmpty(settingId)) {
            // 根据数据ID获取分析设置信息
            EwbAnalysisSetting analysisSetting = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select("setting_id", "data_id", "distributing_code", "parameter_code", "estimation_method_code", "rank_method_code", "believe_code", "believe_method_code", "paramter", "believe_percent", "beta", "sigma", "logsigma").lambda().eq(EwbAnalysisSetting::getSettingId, settingId));
            EwbAnalysisResult analysisResult = analysisResultMapper.selectById(settingId);
            String distributing = analysisSetting.getDistributingCode();
            String parameter = analysisSetting.getParameterCode();
            if (analysisResult != null) {
                if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributing) && EwbParameterCodeEnum.一参数.getIndex().equals(parameter)) {
                    output.setBeta(analysisResult.getP1());
                } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributing) && EwbParameterCodeEnum.一参数.getIndex().equals(parameter)) {
                    output.setSigma(analysisResult.getP2());
                } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributing) && EwbParameterCodeEnum.一参数.getIndex().equals(parameter)) {
                    output.setLogsigma(analysisResult.getP2());
                }
            } else {
                Double paramNum = StringUtils.isNotBlank(analysisSetting.getParamter()) ? Double.parseDouble(analysisSetting.getParamter()) : null;
                if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributing) && EwbParameterCodeEnum.一参数.getIndex().equals(parameter)) {
                    output.setBeta(paramNum);
                } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributing) && EwbParameterCodeEnum.一参数.getIndex().equals(parameter)) {
                    output.setSigma(paramNum);
                } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributing) && EwbParameterCodeEnum.一参数.getIndex().equals(parameter)) {
                    output.setLogsigma(paramNum);
                }
            }
            // 设置输出值
            setOutputValue(output, analysisSetting, failureQuantity);
        } else {
            // 如果没有失效时，置信百分比默认值为63
            if (EwbConstants.N_ZERO == failureQuantity) {
                output.setBelievePercent(EwbConstants.NO_F_BELIEVEPERCENT_DEFAULT);
            } else {
                // 默认置信百分比
                output.setBelievePercent(EwbConstants.BELIEVEPERCENT_DEFAULT);
            }
        }

        output.setDistributingCode(ValidateUtil.isEmpty(output.getDistributingCode()) ? EwbDistributingCodeEnum.Weibull.getIndex() : output.getDistributingCode());
        output.setParameterCode(ValidateUtil.isEmpty(output.getParameterCode()) ? EwbParameterCodeEnum.二参数.getIndex() : output.getParameterCode());
        EwbAnalysisData analysisData = analysisDataMapper.selectById(analysisModel.getDataId());
        output.setTimeForRank(analysisData.getTimeForRank());
        output.setDataQuantity(StringUtils.isBlank(analysisData.getTimeForRank()) || analysisData.getTimeForRank().equals(EwbConstants.FLAG_NO_CHECKED) ? analysisDataDetailMapper.selectCount(new QueryWrapper<EwbAnalysisDataDetail>().lambda().eq(EwbAnalysisDataDetail::getDataId, analysisModel.getDataId())) : analysisDataRankMapper.selectCount(new QueryWrapper<EwbAnalysisDataRank>().lambda().eq(EwbAnalysisDataRank::getDataId, analysisModel.getDataId())));
        output.setFailureQuantity(failureQuantity);

        // 添加无数据的参数
        EwbAnalysisNodataSetting nodataSetting = analysisNodataSettingMapper.selectById(param.getSettingId());
        output.setNodataSetting(nodataSetting == null ? new EwbAnalysisNodataSetting() : nodataSetting);
        return output;
    }

    /**
     * @param output          输出
     * @param analysisSetting 分析设定表
     * @param failureQuantity 失效数据量
     * @Title: setOutputValue
     * @Description: 设置输出值
     */
    public void setOutputValue(EwbAnalysisSettingVo output, EwbAnalysisSetting analysisSetting, Integer failureQuantity) {
        // 分布类型
        output.setDistributingCode(analysisSetting.getDistributingCode());
        // 参数类型
        output.setParameterCode(analysisSetting.getParameterCode());
        // 参数估计方法
        output.setEstimationMethodCode(analysisSetting.getEstimationMethodCode());
        // 求秩方法
        output.setRankMethodCode(analysisSetting.getRankMethodCode());
        // 置信类型
        output.setBelieveCode(analysisSetting.getBelieveCode());
        // 置信法
        output.setBelieveMethodCode(analysisSetting.getBelieveMethodCode());
        if (analysisSetting.getBelievePercent() == null) {
            // 如果没有失效时，置信百分比默认值为63
            if (EwbConstants.N_ZERO.equals(failureQuantity)) {
                output.setBelievePercent(EwbConstants.NO_F_BELIEVEPERCENT_DEFAULT);
            } else {
                // 默认置信百分比
                output.setBelievePercent(EwbConstants.BELIEVEPERCENT_DEFAULT);
            }
        } else {
            // 置信度
            output.setBelievePercent(analysisSetting.getBelievePercent());
        }
    }

    public void saveSetting(HeaderUserInfo userInfo, EwbAnalysisSettingSaveBo param) {
        if ("null".equals(param.getEstimationMethodCode())) {
            param.setEstimationMethodCode("00");
        }

        // 获取分析Id
        String analysisId = param.getAnalysisId();
        // 获取设定Id
        String settingId = param.getSettingId();
        // 获取分析设定对象
//        EwbAnalysisSetting analysisSetting = analysisSettingMapper.selectById(settingId);
        // 如果analysisSetting为空表示明细数据发生变化，设定表数据已经被删除，则需要重新创建设定表数据
        // 编辑分析设定表数据
        // 置信度
        Double believePercent = null;
        // 置信法
        String believeMethodCode = null;
        // 参数类型
        String parameterCode = param.getParameterCode();
        // 如果置信类型选择非"无"时,则获取置信度
        if (!EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getBelieveCode())) {
            believePercent = WeibullNumberUtil.round(Double.valueOf(param.getBelievePercent()), EwbConstants.NUM_DIGIT_2);
            believeMethodCode = param.getBelieveMethodCode();
        }

        // 根据页面输入值获取分析设定对象
        EwbAnalysisSetting changeSetting = analysisSettingMapper.selectByPageInput(settingId, param.getDistributingCode(), parameterCode, param.getEstimationMethodCode(), param.getBelieveCode(), believeMethodCode, believePercent);
        // 分布类型
        String distributingCode = param.getDistributingCode();
        // 当选择weibull1参数时，还需要检查β是否发生变化
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode) && EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
            if (ValidateUtil.isEmpty(param.getBeta())) {
                throw new DescribeException("β", EwbCustomPromptEnum.不可为空, userInfo.getLanguageKey());
            } else if (!ValidateUtil.isNumber(param.getBeta())) {
                throw new DescribeException("β", EwbCustomPromptEnum.必须为数字, userInfo.getLanguageKey());
            } else if (Double.parseDouble(param.getBeta()) < 0.0001 || Double.parseDouble(param.getBeta()) > 50) {
                throw new DescribeException("β", EwbConstants.TAB_FAILURE_END_TIME_MIN, new BigDecimal("50"), EwbCustomPromptEnum.范围请输入, userInfo.getLanguageKey());
            }
            // 根据页面输入值获取分析结果对象
            EwbAnalysisResult ldAnalysisResult = analysisResultMapper.selectBySettingIdPageInput(settingId, Double.valueOf(param.getBeta()));
            // 判断页面数据是否发生变化,如果发生变化则修改设定表数据并且删除相关的分析结果数据
            if (changeSetting == null || ldAnalysisResult == null) {
                // 修改分析相关数据
                modifyChangeData(userInfo, param, analysisId, settingId, parameterCode);
            }
        } else {
            if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode) && EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                if (ValidateUtil.isEmpty(param.getLogsigma())) {
                    throw new DescribeException(EwbConstants.LOG_SIGMA, EwbCustomPromptEnum.不可为空, userInfo.getLanguageKey());
                } else if (!ValidateUtil.isNumber(param.getLogsigma())) {
                    throw new DescribeException(EwbConstants.LOG_SIGMA, EwbCustomPromptEnum.必须为数字, userInfo.getLanguageKey());
                } else if (Double.parseDouble(param.getLogsigma()) < 0.0001 || Double.parseDouble(param.getLogsigma()) > 5) {
                    throw new DescribeException(EwbConstants.LOG_SIGMA, EwbConstants.TAB_FAILURE_END_TIME_MIN, new BigDecimal("5"), EwbCustomPromptEnum.范围请输入, userInfo.getLanguageKey());
                }
            } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode) && EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                if (ValidateUtil.isEmpty(param.getSigma())) {
                    throw new DescribeException("σ", EwbCustomPromptEnum.不可为空, userInfo.getLanguageKey());
                } else if (!ValidateUtil.isNumber(param.getSigma())) {
                    throw new DescribeException("σ", EwbCustomPromptEnum.必须为数字, userInfo.getLanguageKey());
                } else if (Double.parseDouble(param.getSigma()) < 0.0001 || Double.parseDouble(param.getSigma()) > 1000000000) {
                    throw new DescribeException("σ", EwbConstants.TAB_FAILURE_END_TIME_MIN, new BigDecimal("1000000000"), EwbCustomPromptEnum.范围请输入, userInfo.getLanguageKey());
                }
            }
            // 判断页面数据是否发生变化,如果发生变化则修改设定表数据并且删除相关的分析结果数据
            if (changeSetting == null) {
                // 修改分析相关数据
                modifyChangeData(userInfo, param, analysisId, settingId, parameterCode);
            }
        }
        // 重置分析数据
        if (changeSetting == null) {
            analysisRankMapper.deleteByDataId(param.getDataId());
            analysisResultMapper.deleteByPrimaryKey(settingId);
            analysisResultDetailMapper.deleteAnalysisResultDetail(settingId);
            analysisPrBelieveMapper.deleteBySettingId(settingId);
            analysisFrBelieveMapper.deleteBySettingId(settingId);
//        analysisSettingMapper.deleteByPrimaryKey(settingId);
            analysisDisWizardMapper.deleteByDataId(param.getDataId());
//            apiAnalysisResultMapper.delete(new QueryWrapper<EwbApiAnalysisResult>().lambda().eq(EwbApiAnalysisResult::getDataId, param.getDataId()));
            redisTemplate.delete(analysisId);
//            changeSetting = EwbAnalysisSetting.builder().settingId(settingId).parameterCode(parameterCode).distributingCode(param.getDistributingCode()).estimationMethodCode(param.getEstimationMethodCode()).believeCode(param.getBelieveCode()).believeMethodCode(believeMethodCode).believePercent(believePercent).build();
//            analysisSettingMapper.insert(changeSetting);
        }
        EwbAnalysisInfoService.updateUpdateInfo(userInfo.getUserName(), analysisId, analysisInfoMapper, analysisMapper);
    }

    /**
     * @param userInfo          当前登录的用户
     * @param param             输入
     * @param analysisId        分析Id
     * @param settingId         设定Id
     * @param parameterCode     参数个数
     * @Title: modifyChangeData
     * @Description:修改分析相关数据
     */
    public void modifyChangeData(HeaderUserInfo userInfo, EwbAnalysisSettingSaveBo param, String analysisId, String settingId, String parameterCode) {
        // ==================
        // 1.修改分析设定数据
        // ==================
        EwbAnalysisSetting analysisSetting = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select("setting_id", "data_id", "distributing_code", "parameter_code", "estimation_method_code", "rank_method_code", "believe_code", "believe_method_code", "paramter", "believe_percent", "beta", "sigma", "logsigma").lambda().eq(EwbAnalysisSetting::getSettingId, settingId));

        // 分布类型
        analysisSetting.setDistributingCode(param.getDistributingCode());
        // 参数类型
        analysisSetting.setParameterCode(parameterCode);
        // 参数估计方法
        analysisSetting.setEstimationMethodCode(param.getEstimationMethodCode());
        analysisSetting.setUpdateUserId(userInfo.getUserId());
//        analysisSetting.setUpdatePgmId(userInfo.getPgmId());
        // 分析设定表修改
        analysisSettingMapper.update(analysisSetting, new QueryWrapper<EwbAnalysisSetting>().lambda().eq(EwbAnalysisSetting::getSettingId, settingId));

        // ==================
        // 2.更新寿命数据分析主表信息
        // ==================
        // 获取寿命数据分析主表信息
        EwbAnalysis ldAnalysis = analysisMapper.selectById(analysisId);
        ldAnalysis.setUpdateUserId(userInfo.getUserId());
        ldAnalysis.setUpdateTime(new Date());
//        ldAnalysis.setUpdatePgmId(userInfo.getPgmId());
        // 修改寿命数据分析主表信息
        analysisMapper.update(ldAnalysis, new QueryWrapper<EwbAnalysis>().lambda().eq(EwbAnalysis::getAnalysisId, analysisId));

        //==================
        //3.页面分析设定数据发生变化时删除分析结果相关联表的数据
        //==================
//        // 数据ID
//        String dataId = param.getDataId();
//        analysisRankMapper.deleteByDataId(dataId);
//        analysisResultMapper.deleteById(settingId);
//        analysisResultDetailMapper.deleteAnalysisResultDetail(settingId);
//        analysisPrBelieveMapper.deleteBySettingId(settingId);
//        analysisFrBelieveMapper.deleteBySettingId(settingId);

    }

    /**
     * @param userInfo 当前登录的用户
     * @param param    输入
     * @param dataId   数据Id
     * @Title: createAnalysisSetting
     * @Description:创建分析设定表数据
     */
    public void createAnalysisSetting(HeaderUserInfo userInfo, EwbAnalysisSettingSaveBo param, String dataId) {
        // ==================
        // 1.创建分析设定表数据
        // ==================
        EwbAnalysisSetting analysisSetting = new EwbAnalysisSetting();
        // 分析设定ID
        String settingId = param.getSettingId();
        analysisSetting.setSettingId(settingId);
        param.setSettingId(settingId);
        BeanUtils.copyProperties(param, analysisSetting);
        // 数据ID
        analysisSetting.setDataId(dataId);
        // 分布类型
        analysisSetting.setDistributingCode(param.getDistributingCode());
        // 参数类型
        analysisSetting.setParameterCode(param.getParameterCode());
        // 参数估计方法
        analysisSetting.setEstimationMethodCode(param.getEstimationMethodCode());
        // 求秩方法
        analysisSetting.setRankMethodCode(param.getRankMethodCode());
        // 置信类型
        analysisSetting.setBelieveCode(param.getBelieveCode());
        // 置信法
        analysisSetting.setBelieveMethodCode(param.getBelieveMethodCode());
        // beta
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(param.getDistributingCode()) && EwbParameterCodeEnum.一参数.getIndex().equals(param.getParameterCode())) {
            analysisSetting.setBeta(StringUtils.isBlank(param.getBeta()) ? null : WeibullNumberUtil.round(Double.parseDouble(param.getBeta()), EwbConstants.NUM_DIGIT_6));
            analysisSetting.setParamter(analysisSetting.getBeta().toString());
        }else if (EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode()) && EwbParameterCodeEnum.一参数.getIndex().equals(param.getParameterCode())) {
            // logsigma
            analysisSetting.setSigma(StringUtils.isBlank(param.getSigma()) ? null : WeibullNumberUtil.round(Double.parseDouble(param.getSigma()), EwbConstants.NUM_DIGIT_6));
            analysisSetting.setParamter(analysisSetting.getSigma().toString());
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getDistributingCode()) && EwbParameterCodeEnum.一参数.getIndex().equals(param.getParameterCode())) {
            // sigma
            analysisSetting.setLogsigma(StringUtils.isBlank(param.getLogsigma()) ? null : WeibullNumberUtil.round(Double.parseDouble(param.getLogsigma()), EwbConstants.NUM_DIGIT_6));
            analysisSetting.setParamter(analysisSetting.getLogsigma().toString());
        }
        // 置信度
        if (!EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getBelieveCode())) {
            analysisSetting.setBelievePercent(WeibullNumberUtil.round(Double.valueOf(param.getBelievePercent()), EwbConstants.NUM_DIGIT_2));
        } else {
            analysisSetting.setBelievePercent(null);
        }
        // 创建者
        analysisSetting.setInsertUserId(userInfo.getUserId());
        analysisSetting.setInsertTime(new Date());
        analysisSetting.setVersionNo(0);
        // 创建程序ID
//        analysisSetting.setInsertPgmId(userInfo.getPgmId());
        // 分析设定表插入
        analysisSettingMapper.insert(analysisSetting);
    }
}
