package com.nationrel.modules.weibull.service;

import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.mapper.EwbAnalysisSolveDataDetailMapper;
import com.nationrel.modules.weibull.entity.EwbAnalysisDataDetail;
import com.nationrel.modules.weibull.entity.EwbAnalysisSolveDataDetail;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisSolveDataDetailService {

    private final EwbAnalysisSolveDataDetailMapper solveDataDetailMapper;

    /**
     * 存储指数卡方Weibull表格详细数据
     *
     * @param dataId
     * @param listDataDetails
     * @return
     */
    public String saveWeibullDatas(String dataId, List<EwbAnalysisDataDetail> listDataDetails) {
        ArrayList<EwbAnalysisSolveDataDetail> solveDataDetails = new ArrayList<>();
        if (StringUtils.isEmpty(dataId)) {
            dataId = UUIDUtil.getUuid();
        } else {
            solveDataDetailMapper.deleteById(dataId);
        }
        if (ObjectUtils.isEmpty(listDataDetails)) {
            return dataId;
        }
        int i = 1;
        for (EwbAnalysisDataDetail dataDetail : listDataDetails) {
            EwbAnalysisSolveDataDetail solveDataDetail = new EwbAnalysisSolveDataDetail();
            solveDataDetail.setDataId(dataId);
            solveDataDetail.setDataNo(i++);
            solveDataDetail.setQuantity(dataDetail.getQuantity());
            solveDataDetail.setTime(dataDetail.getFailureEndTime());
            if (EwbConstants.STATE_S.equals(dataDetail.getFailureSuspension())) {
                solveDataDetail.setFailureSuspension(EwbConstants.STATE_SUSPENSION);
            } else if (EwbConstants.STATE_F.equals(dataDetail.getFailureSuspension())) {
                solveDataDetail.setFailureSuspension(EwbConstants.STATE_FAILURE);
            }
            solveDataDetail.setGroupNo(dataDetail.getComment1());
            solveDataDetails.add(solveDataDetail);
        }
        solveDataDetailMapper.batchInsert(solveDataDetails);
        return dataId;
    }

    /**
     * 存储修正有效性验证测试两张表格的数据
     * @param defuuid 有缺陷的数据表Id
     * @param fixuuid 修正后的数据表Id
     * @param defectDatasStringArrayList 有缺陷的数据表
     * @param fixDatasStringArrayList 修正后的数据表
     * @return
     */
    public HashMap<String, String> saveFixEffectivinessDatas(String defuuid, String fixuuid, List<String[]> defectDatasStringArrayList, List<String[]> fixDatasStringArrayList) {
        //处理表格详细数据并保存start
        //处理有缺陷的数据表数据并保存
        if (StringUtils.isEmpty(defuuid)) {
            defuuid = UUIDUtil.getUuid();
        } else {
            solveDataDetailMapper.deleteById(defuuid);
        }
        ArrayList<EwbAnalysisSolveDataDetail> solveDataDetails = new ArrayList<>();
        for (int i = 0; i < defectDatasStringArrayList.size(); i++) {
            String[] data = defectDatasStringArrayList.get(i);
            if (WeibullNumberUtil.isNull(WeibullNumberUtil.parseDouble(data[0])) || (WeibullNumberUtil.isNull(WeibullNumberUtil.parseDouble(data[2])))) {
                continue;
            }
            EwbAnalysisSolveDataDetail solveDataDetail = new EwbAnalysisSolveDataDetail();
            solveDataDetail.setStartTime(WeibullNumberUtil.parseDouble(data[0]));
            solveDataDetail.setFailureSuspension(data[1]);
            solveDataDetail.setEndTime(WeibullNumberUtil.parseDouble(data[2]));
            solveDataDetail.setTime(WeibullNumberUtil.parseDouble(data[3]));
            solveDataDetail.setGroupNo(data[4]);
            solveDataDetail.setDataId(defuuid);
            solveDataDetail.setDataNo(i + 1);
            solveDataDetails.add(solveDataDetail);
        }
        solveDataDetailMapper.batchInsert(solveDataDetails);
        //处理修正后的数据表数据并保存
        if (StringUtils.isEmpty(fixuuid)) {
            fixuuid = UUIDUtil.getUuid();
        } else {
            solveDataDetailMapper.deleteById(fixuuid);
            fixuuid = fixuuid;
        }
        // 详细数据数组集合转对象集合
        solveDataDetails = new ArrayList<>();
        for (int i = 0; i < fixDatasStringArrayList.size(); i++) {
            String[] data = fixDatasStringArrayList.get(i);
            if (WeibullNumberUtil.isNull(WeibullNumberUtil.parseDouble(data[0])) || (WeibullNumberUtil.isNull(WeibullNumberUtil.parseDouble(data[1])))) {
                continue;
            }
            EwbAnalysisSolveDataDetail solveDataDetail = new EwbAnalysisSolveDataDetail();
            solveDataDetail.setStartTime(WeibullNumberUtil.parseDouble(data[0]));
            solveDataDetail.setEndTime(WeibullNumberUtil.parseDouble(data[1]));
            solveDataDetail.setTime(WeibullNumberUtil.parseDouble(data[2]));
            solveDataDetail.setGroupNo(data[3]);
            solveDataDetail.setDataId(fixuuid);
            solveDataDetail.setDataNo(i + 1);
            solveDataDetails.add(solveDataDetail);
        }
        solveDataDetailMapper.batchInsert(solveDataDetails);
        //处理表格详细数据并保存end
        HashMap<String, String> result = new HashMap<>();
        result.put("defuuid",defuuid);
        result.put("fixuuid",fixuuid);
        return result;
    }

}
