package com.nationrel.modules.weibull.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.mapper.EwbAnalysisSolveDataDetailMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisSolveDataMapper;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.*;
import com.nationrel.modules.weibull.util.StringUtil;
import lombok.RequiredArgsConstructor;
import org.jeecg.common.api.IPageExtField;
import org.jeecg.common.api.PageExtField;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisSolveDataService {

    private final EwbAnalysisSolveDataMapper solveDataMapper;
    private final EwbAnalysisSolveDataDetailMapper solveDataDetailMapper;

    /**
     * 分页查询
     *
     * @param userInfo
     * @param solveName
     * @param pageIndex
     * @param pageSize
     * @return
     */
    public ServerResponse listByPage(HeaderUserInfo userInfo, String solveName, int pageIndex, int pageSize) {
        Integer languageKey = userInfo.getLanguageKey();
        IPageExtField<EwbAnalysisSolveData> page = new PageExtField<>(EwbAnalysisSolveData.class, pageIndex, pageSize);
        QueryWrapper<EwbAnalysisSolveData> qw = new QueryWrapper<>();
        //筛选条件
        if (!StringUtils.isEmpty(solveName)) {
            qw.lambda().like(EwbAnalysisSolveData::getSolveName, solveName);
        }
        qw.lambda().eq(EwbAnalysisSolveData::getInsertUserId, userInfo.getUserId())
                .orderBy(true, false, EwbAnalysisSolveData::getUpdateTime);
        IPageExtField<EwbAnalysisSolveData> list = solveDataMapper.selectPage(page, qw);
        IPageExtField<EwbAnalysisSolveDataVo> result = null;

        BeanUtils.copyProperties(list, result);
        result.setRecords(packingSolveData(list.getRecords(), languageKey));
        return ServerResponse.createBySuccess(result);
    }

    /**
     * 根据languageKey将Code编码包装为Name
     *
     * @param list
     * @param languageKey
     * @return
     */
    public List<EwbAnalysisSolveDataVo> packingSolveData(List<EwbAnalysisSolveData> list, Integer languageKey) {
        return list.stream().map(item -> {
            EwbAnalysisSolveDataVo vo = new EwbAnalysisSolveDataVo();
            vo.setSolveCodeName(EwbSolveTypeEnum.getLanguageMessageByKey(item.getSolveCode(), languageKey));
            vo.setSolveRouterPath(EwbSolveTypeEnum.getEnglish(item.getSolveCode()));
            if (EwbSolveTypeEnum.修正有效性验证测试.getIndex().equals(item.getSolveCode())) {
                item.setVerficationCode(EwbFixEffectivinessParamEnum.getLanguageMessageByKey(Integer.valueOf(item.getVerficationCode()), languageKey));
            } else if (EwbSolveTypeEnum.可修系统分布.getIndex().equals(item.getSolveCode())) {
                item.setVerficationCode(EwbSolveVerficationTypeEnum.getLanguageMessageByKey(item.getVerficationCode(), languageKey));
            } else if (EwbSolveTypeEnum.指数卡方Weibull.getIndex().equals(item.getSolveCode()) || EwbSolveTypeEnum.指数卡方.getIndex().equals(item.getSolveCode())) {
                item.setVerficationCode(EwbWeibullChiSquaredEnum.getLanguageMessageByKey(item.getVerficationCode(), languageKey));
            } else if (EwbSolveTypeEnum.非参数二项式.getIndex().equals(item.getSolveCode()) || EwbSolveTypeEnum.参数二项式.getIndex().equals(item.getSolveCode())) {
                item.setVerficationCode(EwbVerficationTypeEnum.getLanguageMessageByKey(item.getVerficationCode(), languageKey));
            }
            BeanUtils.copyProperties(item, vo);
            return vo;
        }).collect(Collectors.toList());
    }

    /**
     * 根据id查询
     *
     * @param solveId
     * @return
     */
    public ServerResponse selectOneBySolveId(String solveId) {
        QueryWrapper<EwbAnalysisSolveData> qw = new QueryWrapper<>();
        qw.lambda().eq(EwbAnalysisSolveData::getSolveId, solveId);
        EwbAnalysisSolveData solveData = solveDataMapper.selectOne(qw);
        solveData.setInsertUserId(null);
        solveData.setUpdateUserId(null);
        solveData.setInsertTime(null);
        solveData.setVersionNo(null);
        EwbAnalysisSolveDataDetailVo solveDataVo = new EwbAnalysisSolveDataDetailVo();
        BeanUtils.copyProperties(solveData, solveDataVo);
        //查询对应表格详细数据 返回数组集合
        //指数卡方weibull表格数据
        if (!ObjectUtils.isEmpty(solveData) && !StringUtils.isEmpty(solveData.getDataId())) {
            List<EwbAnalysisSolveDataDetail> datas = solveDataDetailMapper.selectList(new QueryWrapper<EwbAnalysisSolveDataDetail>().lambda().eq(EwbAnalysisSolveDataDetail::getDataId, solveData.getDataId()));
            if (!ObjectUtils.isEmpty(datas)) {
                List<String[]> result = datas.stream().map(item -> {
                    return new String[]{StringUtil.parseString(item.getQuantity()), StringUtil.parseString(item.getTime()), item.getFailureSuspension(), item.getGroupNo()};
                }).collect(Collectors.toList());
                solveDataVo.setDataDetails(result);
            }
        }
        //修正有效性测试有缺陷表格数据
        if (!ObjectUtils.isEmpty(solveData) && !StringUtils.isEmpty(solveData.getDefectdataId())) {
            List<EwbAnalysisSolveDataDetail> defects = solveDataDetailMapper.selectList(new QueryWrapper<EwbAnalysisSolveDataDetail>().lambda().eq(EwbAnalysisSolveDataDetail::getDataId, solveData.getDefectdataId()));
            List<String[]> result = defects.stream().map(item -> {
                return new String[]{StringUtil.parseString(item.getStartTime()), item.getFailureSuspension(), StringUtil.parseString(item.getEndTime()), StringUtil.parseString(item.getTime()), item.getGroupNo()};
            }).collect(Collectors.toList());
            solveDataVo.setDefectDatas(result);
        }
        //修正有效性测试修正后表格数据
        if (!ObjectUtils.isEmpty(solveData) && !StringUtils.isEmpty(solveData.getFixDataId())) {
            List<EwbAnalysisSolveDataDetail> fixs = solveDataDetailMapper.selectList(new QueryWrapper<EwbAnalysisSolveDataDetail>().lambda().eq(EwbAnalysisSolveDataDetail::getDataId, solveData.getFixDataId()));
            List<String[]> result = fixs.stream().map(item -> {
                return new String[]{StringUtil.parseString(item.getStartTime()), StringUtil.parseString(item.getEndTime()), StringUtil.parseString(item.getTime()), item.getGroupNo()};
            }).collect(Collectors.toList());
            solveDataVo.setFixDatas(result);
        }
        return ServerResponse.createBySuccess(solveDataVo);
    }

    /**
     * 根据id删除
     *
     * @param solveId
     * @return
     */
    public ServerResponse deleteBySolveId(String solveId) {
        EwbAnalysisSolveData solveData = solveDataMapper.selectOne(new QueryWrapper<EwbAnalysisSolveData>().lambda().eq(EwbAnalysisSolveData::getSolveId, solveId).select(EwbAnalysisSolveData::getSolveId, EwbAnalysisSolveData::getDataId, EwbAnalysisSolveData::getFixDataId));
        if (ObjectUtils.isEmpty(solveData)) {
            return ServerResponse.createByError();
        }
        //删除对应表格详细数据
        if (!StringUtils.isEmpty(solveData.getDataId())) {
            solveDataDetailMapper.deleteById(solveData.getDataId());
        }
        if (!StringUtils.isEmpty(solveData.getFixDataId())) {
            solveDataDetailMapper.deleteById(solveData.getFixDataId());
        }
        solveDataMapper.deleteById(solveId);
        return ServerResponse.createBySuccess();
    }

    /**
     * 根据solveId是否存在 新增或更新
     *
     * @param result
     * @param solveData
     * @param solveId
     * @param userInfo
     * @return
     */
    public String solveSave(HashMap<String, Object> result, EwbAnalysisSolveData solveData, String solveId, HeaderUserInfo userInfo) {
        if (!ObjectUtils.isEmpty(result)) {
            solveData.setCalcResult(String.valueOf(result.get("result")));
        }
        if (StringUtils.isEmpty(solveId)) {
            // save
            solveId = UUIDUtil.getUuid();
            solveData.setSolveId(solveId);
            solveData.setInsertUserId(userInfo.getUserId());
            solveData.setInsertTime(new Date());
            solveDataMapper.insert(solveData);
        } else {
            // update
            solveData.setSolveId(solveId);
            solveDataMapper.updateById(solveData);
        }
        if (!ObjectUtils.isEmpty(result)) {
            result.put("solveId", solveId);
        }
        return solveId;
    }

    /**
     * 创建单个
     *
     * @param userInfo
     * @param createVo
     * @return
     */
    public ServerResponse create(HeaderUserInfo userInfo, EwbAnalysisSolveDataCreateVo createVo) {
        EwbAnalysisSolveData solveData = new EwbAnalysisSolveData();
        solveData.setSolveId(UUIDUtil.getUuid());
        solveData.setSolveName(createVo.getSolveName());
        solveData.setSolveCode(createVo.getSolveCode());
        solveData.setInsertUserId(userInfo.getUserId());
        solveData.setUpdateUserId(userInfo.getUserId());
        solveData.setInsertTime(new Date());
        solveData.setUpdateTime(new Date());
        solveDataMapper.insert(solveData);
        return ServerResponse.createBySuccess();
    }

    /**
     * 根据id更新
     *
     * @param userInfo
     * @param updateVo
     * @return
     */
    public ServerResponse update(HeaderUserInfo userInfo, EwbAnalysisSolveDataUpdateVo updateVo) {
        EwbAnalysisSolveData update = new EwbAnalysisSolveData();
        update.setSolveId(updateVo.getSolveId());
        update.setSolveName(updateVo.getSolveName());
        update.setUpdateUserId(userInfo.getUserId());
        update.setUpdateTime(new Date());
        solveDataMapper.updateById(update);
        return ServerResponse.createBySuccess();
    }
}
