package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.bo.EwbCalcParamBo;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.dto.EwbCalcResultDetailDto;
import com.nationrel.modules.weibull.dto.EwbCalcResultDto;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.*;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.general.util.common.DateTimeUtil;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.util.ValidateUtil;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.vo.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author Aimee on 2020/9/22.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbCalcService {

    private final RgaApiService rgaApiService;
    private final EwbAnalysisMapper analysisMapper;
    private final EwbValidatorService validatorService;
    private final EwbChartDataService chartDataService;
    private final EwbAnalysisDataMapper analysisDataMapper;
    private final EwbAnalysisInfoMapper analysisInfoMapper;
    private final EwbDataHandlerService dataHandlerService;
    private final EwbAnalysisResultMapper analysisResultMapper;
    private final EwbAnalysisSettingMapper analysisSettingMapper;
    private final EwbAnalysisCalcResultMapper analysisCalcResultMapper;
    private final EwbAnalysisDataDetailMapper analysisDataDetailMapper;

    private final AllVariableConfig allVariableConfig;

    public ServerResponse calc(String userId, EwbCalcParamBo param, HeaderUserInfo user, boolean isNevada) throws Exception {
        // 获取dataId
        String dataId = param.getDataId();

        // 获取settingId
        String settingId = param.getSettingId();

        EwbAnalysisData ldAnalysisData = analysisDataMapper.selectById(dataId);

        validatorService.validatorDataDetail(param.getDataId(), param.getSettingId(), null, null, null, "", user);

        // 获取寿命数据分析结果表
        EwbAnalysisResult ldAnalysisResult = analysisResultMapper.selectById(settingId);

        // 获取寿命数据分析设定表
        EwbAnalysisSetting ldAnalysisSetting = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select("setting_id", "data_id", "distributing_code", "parameter_code", "estimation_method_code", "rank_method_code", "believe_code", "believe_method_code", "paramter", "believe_percent", "beta", "sigma", "logsigma").lambda().eq(EwbAnalysisSetting::getSettingId, settingId));
        String distributingCode = ldAnalysisSetting.getDistributingCode();
        String parameterCode = ldAnalysisSetting.getParameterCode();

        String dataType;
        EwbAnalysisCalcResult analysisCalcResult = new EwbAnalysisCalcResult();
        BeanUtils.copyProperties(ldAnalysisSetting, analysisCalcResult);
        StringBuilder confidenceParam = new StringBuilder();
        StringBuilder calcMethodParam = new StringBuilder();

        // 判断数据类型
        if (Objects.equals(ldAnalysisData.getCQuantity(), ldAnalysisData.getQuantity())) {
            dataType = EwbConstants.COMPLETE_DATA_TYPE;
        } else if (ldAnalysisData.getCQuantity() + ldAnalysisData.getRQuantity() == ldAnalysisData.getQuantity()) {
            dataType = EwbConstants.RIGHT_DATA_TYPE;
        } else {
            dataType = EwbConstants.INTERVAL_DATA_TYPE;
        }

        EwbParameterVo calcParameterModel = new EwbParameterVo();
        // 条件可靠度 ——附加时间
        calcParameterModel.setAddT(WeibullNumberUtil.parseDouble(param.getAdditionalTime()));
        if (StringUtils.isNotBlank(param.getAdditionalTime())) {
            calcMethodParam.append("/").append(param.getAdditionalTime());
        }

        // 置信类型
        String confidenceType = param.getConfidenceType();
        // 置信度
        String confidencePercent = param.getConfidencePercent();
        // 置信法
        String confidenceMethod = param.getConfidenceMethod();
        if (EwbConfidenceTypeCodeEnum.无.getIndex().equals(confidenceType)) {
            confidencePercent = null;
            confidenceMethod = null;
        }
        // 置信法
        calcParameterModel.setBelieveMethod(confidenceMethod);
        // 参数
        calcParameterModel.setParameterCode(parameterCode);
        // 置信百分比
        Double believePercent = WeibullNumberUtil.parseDouble(confidencePercent);
        if (believePercent != null) {
            believePercent = believePercent * Math.pow(EwbConstants.HUNDRED, -1);
            analysisCalcResult.setBelievePercent(believePercent);
        }

        calcParameterModel.setBelievePercent(believePercent);
        // 计算类型
        calcParameterModel.setCalcType(param.getCalcType());
        // 数据类型，区间 or 完整 or 右删失
        calcParameterModel.setDataType(dataType);
        // db T_LD_ANALYSIS_RESULT——方差DB
        calcParameterModel.setDb(ldAnalysisResult.getDb());
        // dbe T_LD_ANALYSIS_RESULT——方差DBE
        calcParameterModel.setDbe(ldAnalysisResult.getDbe());
        // de T_LD_ANALYSIS_RESULT——方差DE
        calcParameterModel.setDe(ldAnalysisResult.getDe());
        // 分布 weibull
        calcParameterModel.setDistributing(distributingCode);
        // 结束时间
        calcParameterModel.setEndT(WeibullNumberUtil.parseDouble(param.getEndTime()));
        if (StringUtils.isNotBlank(param.getEndTime())) {
            calcMethodParam.append(param.getEndTime());
        }
        // 失效百分比
        Double failPercent = WeibullNumberUtil.parseDouble(param.getFailurePercent());
        if (StringUtils.isNotBlank(param.getFailurePercent())) {
            calcMethodParam.append(param.getFailurePercent());
        }
        if (failPercent != null) {
            failPercent = failPercent * Math.pow(EwbConstants.HUNDRED, -1);
            calcMethodParam.append(failPercent);
        }
        calcParameterModel.setFailPercent(failPercent);
        // 似然比 T_LD_ANALYSIS_RESULT-LK
        calcParameterModel.setLk(ldAnalysisResult.getLk());
        // 参数1 T_LD_ANALYSIS_RESULT
        calcParameterModel.setP1(ldAnalysisResult.getP1());
        // 参数2 T_LD_ANALYSIS_RESULT
        calcParameterModel.setP2(ldAnalysisResult.getP2());
        // 参数3 T_LD_ANALYSIS_RESULT
        calcParameterModel.setP3(ldAnalysisResult.getP3());
        // side
        calcParameterModel.setSide(param.getConfidenceType());
        // 开始时间
        calcParameterModel.setStartT(WeibullNumberUtil.parseDouble(param.getStartTime()));
        if (StringUtils.isNotBlank(param.getStartTime())) {
            calcMethodParam.append(param.getStartTime());
        }

        // 获取寿命数据,做似然比运算
        if (param.getConfidenceMethod().equals(EwbConfidenceMethodCodeEnum.likelihoodradio.getIndex())) {
            EwbCalcResultDetailDto ldModel = dataHandlerService.handlerLd(ldAnalysisSetting);
            calcParameterModel.setT1(ldModel.getT1());
            calcParameterModel.setT2(ldModel.getT2());
            calcParameterModel.setTu(ldModel.getTu());
            calcParameterModel.setTl(ldModel.getTl());
            calcParameterModel.setN1(ldModel.getN1());
            calcParameterModel.setN2(ldModel.getN2());
            calcParameterModel.setNl(ldModel.getNl());
            calcParameterModel.setNu(ldModel.getNu());
        }

        if (EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
            if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode)) {
                param.setBeta(String.valueOf(ldAnalysisResult.getP1()));
            } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode)) {
                param.setLogsigma(String.valueOf(ldAnalysisResult.getP2()));
            } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode)) {
                param.setSigma(String.valueOf(ldAnalysisResult.getP2()));
            }
        }

        // ==================
        // 调用RGA——API,准备输入参数
        // ==================
        EwbCalcResultDto ewbCalcResultDto = new EwbCalcResultDto();
        EwbAnalysisData analysisData = analysisDataMapper.selectById(dataId);

        String parameter = rgaApiService.getParameter(distributingCode, param.getBeta(), param.getSigma(), param.getLogsigma(), ldAnalysisSetting);
        String apiLifeDataUrl = rgaApiService.getApiLifeDataUrl(distributingCode, ldAnalysisSetting.getEstimationMethodCode(), param.getParameterCode(), parameter, analysisDataDetailMapper.selectCount(new QueryWrapper<EwbAnalysisDataDetail>().lambda().eq(EwbAnalysisDataDetail::getFailureSuspension, EwbConstants.STATE_F).eq(EwbAnalysisDataDetail::getDataId, dataId)) == 0, param.getConfidencePercent());
        String apiDataType = rgaApiService.getDataType(analysisData.getRFlag(), analysisData.getLFlag());
        JSONObject requestFunctionData = new JSONObject();
        // 调用api的输入参数
        JSONObject requestLifeData = rgaApiService.getRequestLifeDataJson(dataId, apiDataType, new SolveDemoRelWeibullChiSquareRelVo(), false);
        if (requestLifeData == null) {
            return ServerResponse.createBySuccess();
        }
        requestFunctionData.put("data", requestLifeData.toString());
        // 第一步：执行调用接口并获取调用API返回结果
        String responseResults = rgaApiService.getResultsCallApi(requestLifeData.toString(), apiLifeDataUrl, EwbConstants.RGA_API_CONTENTTYPE_1, null);
        // 第二步：调用接口获取失效率
        String apiFailureRankUrl = allVariableConfig.getReliaQubeUrl() + EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.API_LIFEDATA_FAILURERANK);
        String resultsFailureRank = rgaApiService.getResultsCallApi(requestLifeData.toString(), apiFailureRankUrl, EwbConstants.RGA_API_CONTENTTYPE_1, null);
        JSONObject newDataJson = JSONObject.parseObject(requestFunctionData.get("data").toString());
        String serverSystem = allVariableConfig.getServerSystem();
        newDataJson.remove(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank));
        if (EwbConstants.SERVER_SYSTEM_WINDOWS.equals(serverSystem)) {
            newDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), "null".equals(resultsFailureRank) ? "[]" : resultsFailureRank);
        } else {
            if (StringUtils.isNotBlank(resultsFailureRank) && !"null".equals(resultsFailureRank) && !resultsFailureRank.contains("error")) {
                newDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), JSONObject.parseArray(resultsFailureRank));
            } else {
                newDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), new double[0]);
            }
        }
        // 第三步：准备计算器参数并调用计算器接口
        JSONObject requestCalcFunctionData = JSONObject.parseObject(responseResults);
        requestCalcFunctionData.remove(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.analysisType));
        requestCalcFunctionData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.analysisType), rgaApiService.getEstimationMethod(ldAnalysisSetting.getEstimationMethodCode()));

        JSONObject requestCalcBoundsData = new JSONObject();
        requestCalcBoundsData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results), requestCalcFunctionData);
        requestCalcBoundsData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data), newDataJson);

        // ==================
        // 调用API 获取各种计算结果(可靠度、不可靠度、平均失效时间、失效率 、参数、BX%寿命、条件可靠度)
        // ==================
        if (EwbConstants.CALC_TYPE_PARAMETER.equals(param.getCalcType())) {
            JSONObject jsonObj = JSONObject.parseObject(responseResults);
            setParameter(distributingCode, ewbCalcResultDto, jsonObj);
        } else {
            String apiCalcFunctionUrl = rgaApiService.getApiCalcFunctionUrl(param.getCalcType(), param.getEndTime(), param.getAdditionalTime(), distributingCode, param.getStartTime(), param.getFailurePercent(), parameterCode);
//            System.out.println(requestCalcFunctionData);
            if (EwbConstants.SERVER_SYSTEM_LINUX.equals(serverSystem)) {
                apiCalcFunctionUrl = apiCalcFunctionUrl.replace(EwbDistributingCodeEnum.Exponential.getName() + EwbDistributionTypeEnum.oneP.getName(), EwbDistributingCodeEnum.Exponential.getName());
            }
            String responseResultCalcFunction = rgaApiService.getResultsCallApi(requestCalcFunctionData.toString(), apiCalcFunctionUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
            ewbCalcResultDto.setMean(ValidateUtil.isNumber(responseResultCalcFunction) || ValidateUtil.isScientificNotation(responseResultCalcFunction) ? WeibullNumberUtil.round(Double.parseDouble(responseResultCalcFunction), EwbConstants.NUM_DIGIT_10) : 0.0);
        }

        // ==================
        // 调用API 获取计算器各种参数的置信区间
        // ==================
        if (!EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
            confidenceParam.append(EwbConfidenceMethodCodeEnum.getValue(param.getConfidenceMethod())).append(" / ").append(param.getConfidencePercent()).append("% / ").append(EwbConfidenceTypeCodeEnum.getValue(param.getConfidenceType()));
            if (EwbConstants.CALC_TYPE_PARAMETER.equals(param.getCalcType())) {
                if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode)) {
                    param.setParameterType(EwbConstants.RGA_API_PARAMETER_TYPE_BETA);
                } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode) || EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode)) {
                    param.setParameterType(EwbConstants.RGA_API_PARAMETER_TYPE_MU);
                } else if (EwbDistributingCodeEnum.Exponential.getIndex().equals(distributingCode)) {
                    param.setParameterType(EwbConstants.RGA_API_PARAMETER_TYPE_LAMBDA);
                }
                String apiCalcBoundsP1Url = rgaApiService.getApiBoundsUrl(param);
                String responseResultP1Bounds = rgaApiService.getResultsCallApi(requestCalcBoundsData.toString(), apiCalcBoundsP1Url, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
                String strP1Lb = responseResultP1Bounds.substring(1, responseResultP1Bounds.indexOf(","));
                ewbCalcResultDto.setP1L(ValidateUtil.isNumber(strP1Lb) ? WeibullNumberUtil.round(Double.parseDouble(strP1Lb), EwbConstants.NUM_DIGIT_6) : 0.0);
                String strP1Ub = responseResultP1Bounds.substring(responseResultP1Bounds.indexOf(",") + 1, responseResultP1Bounds.length() - 1);
                ewbCalcResultDto.setP1U(ValidateUtil.isNumber(strP1Ub) ? WeibullNumberUtil.round(Double.parseDouble(strP1Ub), EwbConstants.NUM_DIGIT_6) : 0.0);

                if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode)) {
                    param.setParameterType(EwbConstants.RGA_API_PARAMETER_TYPE_ETA);
                } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode) || EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode)) {
                    param.setParameterType(EwbConstants.RGA_API_PARAMETER_TYPE_SIGMA);
                } else if (EwbDistributingCodeEnum.Exponential.getIndex().equals(distributingCode)) {
                    param.setParameterType(EwbConstants.RGA_API_PARAMETER_TYPE_LAMBDA);
                }
                String apiCalcBoundsP2Url = rgaApiService.getApiBoundsUrl(param);
                String responseResultP2Bounds = rgaApiService.getResultsCallApi(requestCalcBoundsData.toString(), apiCalcBoundsP2Url, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
                String strP2Lb = responseResultP2Bounds.substring(1, responseResultP2Bounds.indexOf(","));
                ewbCalcResultDto.setP2L(ValidateUtil.isNumber(strP2Lb) ? WeibullNumberUtil.round(Double.parseDouble(strP2Lb), EwbConstants.NUM_DIGIT_6) : 0.0);
                String strP2Ub = responseResultP2Bounds.substring(responseResultP2Bounds.indexOf(",") + 1, responseResultP2Bounds.length() - 1);
                ewbCalcResultDto.setP2U(ValidateUtil.isNumber(strP2Ub) ? WeibullNumberUtil.round(Double.parseDouble(strP2Ub), EwbConstants.NUM_DIGIT_6) : 0.0);
            } else {
                String apiCalcBoundsUrl = rgaApiService.getApiBoundsUrl(param);
                String responseResultBounds = rgaApiService.getResultsCallApi(requestCalcBoundsData.toString(), apiCalcBoundsUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
                String strLb = responseResultBounds.substring(1, responseResultBounds.indexOf(","));
                ewbCalcResultDto.setLb(ValidateUtil.isNumeric1(strLb) ? WeibullNumberUtil.round(Double.parseDouble(strLb), isNevada ? EwbConstants.NUM_DIGIT_18 : EwbConstants.NUM_DIGIT_6) : 0.0);
                String strUb = responseResultBounds.substring(responseResultBounds.indexOf(",") + 1, responseResultBounds.length() - 1);
                ewbCalcResultDto.setUb(ValidateUtil.isNumeric1(strUb) ? WeibullNumberUtil.round(Double.parseDouble(strUb), isNevada ? EwbConstants.NUM_DIGIT_18 : EwbConstants.NUM_DIGIT_6) : 0.0);
            }
        }
        HashMap<String, Object> jsonResult = new HashMap<>();
        if (isNevada) {
            return ServerResponse.createBySuccess(ewbCalcResultDto.getMean() + "," + ewbCalcResultDto.getUb() + "," + ewbCalcResultDto.getLb());
        }
        jsonResult.put("resultUpperBound_text", ewbCalcResultDto.getUb());
        jsonResult.put("resultMean_text", ewbCalcResultDto.getMean() + "");
        jsonResult.put("resultLowerBound_text", ewbCalcResultDto.getLb());
        jsonResult.put("resultBetaUpperBound_text", ewbCalcResultDto.getP1U());
        jsonResult.put("resultBetaMean_text", ewbCalcResultDto.getP1());
        jsonResult.put("resultBetaLowerBound_text", ewbCalcResultDto.getP1L());
        jsonResult.put("resultEtaUpperBound_text", ewbCalcResultDto.getP2U());
        jsonResult.put("resultEtaMean_text", ewbCalcResultDto.getP2());
        jsonResult.put("resultEtaLowerBound_text", ewbCalcResultDto.getP2L());
        analysisCalcResult.setConfidenceParam(confidenceParam.toString());
        analysisCalcResult.setCalcMethodParam(calcMethodParam.toString());
        analysisCalcResult.setAnalysisId(ldAnalysisData.getAnalysisId());
        analysisCalcResult.setCalcResult(param.getCalcResult());
        analysisCalcResult.setInsertTime(new Date());
//        StringBuilder calcResultCa = new StringBuilder();
//        String parameterValue = "";
//        if (EwbCalcTypeEnum.可靠度.getIndex().equals(param.getCalcType()) || EwbCalcTypeEnum.不可靠度.getIndex().equals(param.getCalcType()) || EwbCalcTypeEnum.失效率.getIndex().equals(param.getCalcType())) {
//            parameterValue = "(" + param.getEndTime() + ")：";
//        } else if (EwbCalcTypeEnum.BX寿命.getIndex().equals(param.getCalcType()) || EwbCalcTypeEnum.可靠寿命.getIndex().equals(param.getCalcType())) {
//            parameterValue = "(" + param.getFailurePercent() + ")：";
//        } else if (EwbCalcTypeEnum.条件可靠度.getIndex().equals(param.getCalcType()) || EwbCalcTypeEnum.条件不可靠度.getIndex().equals(param.getCalcType())) {
//            parameterValue = "(" + param.getStartTime() + "/" + param.getAdditionalTime() + ")：";
//        } else if (EwbCalcTypeEnum.PDF.getIndex().equals(param.getCalcType())) {
//            parameterValue = "(" + param.getStartTime() + ")：";
//        } else if (EwbCalcTypeEnum.平均失效时间.getIndex().equals(param.getCalcType()) || EwbCalcTypeEnum.参数.getIndex().equals(param.getCalcType())) {
//            parameterValue = "：";
//        }
//        calcResultCa = handleConfidence(calcResultCa, user.getLanguageKey(), ewbCalcResultDto, parameterValue, param.getCalcType());
//        analysisCalcResult.setCalcResultCa(calcResultCa.toString());
//        analysisCalcResultMapper.insert(analysisCalcResult);

        EwbAnalysisInfoService.updateUpdateInfo(userId, ldAnalysisData.getAnalysisId(), analysisInfoMapper, analysisMapper);
        return ServerResponse.createBySuccess(EwbAnalysisCalcVo.builder().analysisCalcResults(calcResult(ldAnalysisData.getAnalysisId(), user.getLanguageKey())).calcResults(jsonResult).build());
    }

//    private StringBuilder handleConfidence(StringBuilder calcResultCa, Integer languageKey, EwbCalcResultDto ewbCalcResultDto, String parameterValue, String calcType) {
//        if (ewbCalcResultDto.getUb() != 0.0) {
//            calcResultCa.append(" ").append(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.上限, languageKey)).append("：").append(ewbCalcResultDto.getUb()).append("；");
//        }
//        if (ewbCalcResultDto.getMean() != 0.0) {
//            calcResultCa.append(" ").append(EwbCalcTypeEnum.getByLaguageKey(calcType, languageKey)).append(parameterValue).append(ewbCalcResultDto.getMean()).append("；");
//        }
//        if (ewbCalcResultDto.getLb() != 0.0) {
//            calcResultCa.append(" ").append(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.下限, languageKey)).append("：").append(ewbCalcResultDto.getLb()).append("；");
//        }
//        if (ewbCalcResultDto.getP1U() != 0.0) {
//            calcResultCa.append(" β").append(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.上限, languageKey)).append("：").append(ewbCalcResultDto.getUb()).append("；");
//        }
//        if (ewbCalcResultDto.getP1() != 0.0) {
//            calcResultCa.append(" β：").append(parameterValue).append(ewbCalcResultDto.getP1()).append("；");
//        }
//        if (ewbCalcResultDto.getP1L() != 0.0) {
//            calcResultCa.append(" β").append(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.下限, languageKey)).append("：").append(ewbCalcResultDto.getLb()).append("；");
//        }
//        if (ewbCalcResultDto.getP2U() != 0.0) {
//            calcResultCa.append(" η").append(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.上限, languageKey)).append("：").append(ewbCalcResultDto.getUb()).append("；");
//        }
//        if (ewbCalcResultDto.getP2() != 0.0) {
//            calcResultCa.append(" η：").append(parameterValue).append(ewbCalcResultDto.getP2()).append("；");
//        }
//        if (ewbCalcResultDto.getP2L() != 0.0) {
//            calcResultCa.append(" η").append(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.下限, languageKey)).append("：").append(ewbCalcResultDto.getLb()).append("；");
//        }
//        return calcResultCa;
//    }

    private void setParameter(String distributingCode, EwbCalcResultDto ewbCalcResultDto, JSONObject jsonObj) {
        String serverSystem = allVariableConfig.getServerSystem();
        double beta = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.beta));
        double eta = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.eta));
        double gamma = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.gamma));
        double mu = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mu));
        double sigma = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.sigma));
        double logMu = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logMu));
        double logSigma = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logSigma));
        double lambda = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lambda));
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode)) {
            ewbCalcResultDto.setP1(WeibullNumberUtil.round(beta, EwbConstants.NUM_DIGIT_6));
            ewbCalcResultDto.setP2(WeibullNumberUtil.round(eta, EwbConstants.NUM_DIGIT_6));
            /*calcResultModel.setP3(NumberUtils.round(gamma, EwbConstants.NUM_DIGIT_6));*/
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode)) {
            ewbCalcResultDto.setP1(WeibullNumberUtil.round(mu, EwbConstants.NUM_DIGIT_6));
            ewbCalcResultDto.setP2(WeibullNumberUtil.round(sigma, EwbConstants.NUM_DIGIT_6));
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode)) {
            ewbCalcResultDto.setP1(WeibullNumberUtil.round(logMu, EwbConstants.NUM_DIGIT_6));
            ewbCalcResultDto.setP2(WeibullNumberUtil.round(logSigma, EwbConstants.NUM_DIGIT_6));
        } else if (EwbDistributingCodeEnum.Exponential.getIndex().equals(distributingCode)) {
            ewbCalcResultDto.setP1(WeibullNumberUtil.round(lambda, EwbConstants.NUM_DIGIT_6));
            ewbCalcResultDto.setP2(WeibullNumberUtil.round(gamma, EwbConstants.NUM_DIGIT_6));
        }
    }

    public Map<String, Object> calcSetting(String userId, String settingId, String analysisId) {
        Map<String, Object> resultMap = new HashMap<>();
        // 分析设置
        EwbAnalysisSetting ldAnalysisSetting = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select("setting_id", "data_id", "distributing_code", "parameter_code", "estimation_method_code", "rank_method_code", "believe_code", "believe_method_code", "paramter", "believe_percent", "beta", "sigma", "logsigma").lambda().eq(EwbAnalysisSetting::getSettingId, settingId));
//        output.setLdAnalysisSetting(ldAnalysisSetting);
        resultMap.put("setting", ldAnalysisSetting);

        // 分析结果 p1 p2
        EwbAnalysisResult ldAnalysisResult = analysisResultMapper.selectOne(new QueryWrapper<EwbAnalysisResult>().lambda().eq(EwbAnalysisResult::getSettingId, settingId));
//        output.setLdAnalysisResult(ldAnalysisResult);
        resultMap.put("result", ldAnalysisResult);

        // 获取寿命数据-数据量信息
        EwbAnalysisModel analysisModel = analysisDataMapper.searchAanlysisDataByAnalysisId(analysisId);
        // 失效数据量
//        int failureQuantity = analysisModel.getFailureQuantity();
//        output.setFailureQuantity(failureQuantity);
        resultMap.put("failureQuantity", analysisModel.getFailureQuantity());


        EwbChartDataModel ldChartDataModel = chartDataService.getProbabilityChartData(analysisId, EwbConstants.PLOT_TYPE_PROBABLITY).get(0);
        // weibull2p
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
            resultMap.put("β", ldChartDataModel.getP1());
            resultMap.put("η", ldChartDataModel.getP2());
            // weibull3p
            if (EwbParameterCodeEnum.三参数.getIndex().equals(ldChartDataModel.getParamCode())) {
                resultMap.put("γ", ldChartDataModel.getP3());
            } else if (EwbParameterCodeEnum.混合2参数.getIndex().equals(ldChartDataModel.getParamCode())) {
                resultMap.remove("β");
                resultMap.remove("η");
                resultMap.put("β1", ldChartDataModel.getP1());
                resultMap.put("η1", ldChartDataModel.getP2());
                // weibull混合2p
                resultMap.put("β2", ldChartDataModel.getP3());
                resultMap.put("η2", ldChartDataModel.getP4());
            } else if (EwbParameterCodeEnum.混合3参数.getIndex().equals(ldChartDataModel.getParamCode())) {
                resultMap.remove("β");
                resultMap.remove("η");
                resultMap.put("β1", WeibullNumberUtil.round(ldChartDataModel.getP1(), EwbConstants.NUM_DIGIT_4));
                resultMap.put("η1", WeibullNumberUtil.round(ldChartDataModel.getP2(), EwbConstants.NUM_DIGIT_4));
                // weibull混合3p
                resultMap.put("β2", WeibullNumberUtil.round(ldChartDataModel.getP3(), EwbConstants.NUM_DIGIT_4));
                resultMap.put("η2", WeibullNumberUtil.round(ldChartDataModel.getP4(), EwbConstants.NUM_DIGIT_4));
                resultMap.put("β3", WeibullNumberUtil.round(ldChartDataModel.getP5(), EwbConstants.NUM_DIGIT_4));
                resultMap.put("η3", WeibullNumberUtil.round(ldChartDataModel.getP6(), EwbConstants.NUM_DIGIT_4));
            }
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
            resultMap.put("logMu", ldChartDataModel.getP1());
            resultMap.put("logSigma", ldChartDataModel.getP2());
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
            resultMap.put("u", ldChartDataModel.getP1());
            resultMap.put("σ", ldChartDataModel.getP2());
        } else if (EwbDistributingCodeEnum.Exponential.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
            resultMap.put("λ", ldChartDataModel.getP2());
            if (EwbParameterCodeEnum.二参数.getIndex().equals(ldChartDataModel.getParamCode())) {
                resultMap.put("γ", ldChartDataModel.getP2());
            }
        }
        resultMap.put("LK", ldAnalysisResult.getLk());
        if (EwbEvaluateCodeEnum.rrx.getIndex().equals(ldAnalysisSetting.getEstimationMethodCode()) || EwbEvaluateCodeEnum.rry.getIndex().equals(ldAnalysisSetting.getEstimationMethodCode())) {
            resultMap.put("ρ", ldAnalysisResult.getRao());
        }
        EwbAnalysisInfoService.updateUpdateInfo(userId, analysisId, analysisInfoMapper, analysisMapper);
        return resultMap;
    }

    public List<EwbAnalysisCalcResultVo> calcResult(String analysisId, Integer languageKey) {
        List<EwbAnalysisCalcResult> analysisCalcResults = analysisCalcResultMapper.selectList(new QueryWrapper<EwbAnalysisCalcResult>().lambda().eq(EwbAnalysisCalcResult::getAnalysisId, analysisId).orderByDesc(EwbAnalysisCalcResult::getInsertTime).last("limit 10"));
        List<EwbAnalysisCalcResultVo> analysisCalcResultVos = new ArrayList<>(analysisCalcResults.size());
        for (EwbAnalysisCalcResult analysisCalcResult : analysisCalcResults) {
            EwbAnalysisCalcResultVo analysisCalcResultVo = new EwbAnalysisCalcResultVo();
            analysisCalcResultVo.setCalcSetting(EwbDistributingCodeEnum.getName(analysisCalcResult.getDistributingCode()) + "," + EwbParameterCodeEnum.getByLaguageKey(analysisCalcResult.getParameterCode(), languageKey) + " " + EwbEvaluateCodeEnum.getByLaguageKey(analysisCalcResult.getEstimationMethodCode(), languageKey) + analysisCalcResult.getCalcResult());
            analysisCalcResultVo.setDateStr(DateTimeUtil.dateToStr(analysisCalcResult.getInsertTime()));
            analysisCalcResultVo.setCalcResult1(analysisCalcResult.getConfidenceParam());
            analysisCalcResultVo.setCalcResult2(analysisCalcResult.getCalcResultCa());
            analysisCalcResultVos.add(analysisCalcResultVo);
        }
        return analysisCalcResultVos;
    }
}
