package com.nationrel.modules.weibull.service;

import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.entity.EwbAnalysisRank;
import com.nationrel.modules.weibull.entity.EwbAnalysisResult;
import com.nationrel.modules.weibull.entity.EwbAnalysisSetting;
import com.nationrel.modules.weibull.enums.EwbDistributingCodeEnum;
import com.nationrel.modules.weibull.enums.EwbParameterCodeEnum;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.vo.EwbAnalysisDataDetailVo;
import com.nationrel.modules.weibull.vo.EwbAnalysisModel;
import com.nationrel.modules.weibull.vo.EwbChartDataModel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Aimee on 2020/9/22.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbChartDataService {

    private final EwbAnalysisMapper ldAnalysisMapper;
    private final EwbAnalysisPrBelieveMapper ldAnalysisPrBelieveMapper;
    private final EwbAnalysisRankMapper ldAnalysisRankMapper;
    private final EwbAnalysisDataMapper analysisDataMapper;
    private final EwbAnalysisDataDetailMapper ldAnalysisDataDetailMapper;
    private final EwbAnalysisResultMapper ldAnalysisResultMapper;
    private final EwbAnalysisSettingMapper ldAnalysisSettingMapper;

    private final AllVariableConfig allVariableConfig;

    /* (non-Javadoc)
     * <p>Title: getProbabilityChartData</p>
     * <p>Description: 概率图作图数据获取</p>
     * @param analysisId
     * @return
     * @throws Exception
     */
    public List<EwbChartDataModel> getProbabilityChartData(String analysisId, String plotType) {
        List<EwbChartDataModel> ldChartDataModels = ldAnalysisMapper.searchChartDataByAnalysisId(analysisId, plotType);
        for (EwbChartDataModel ldChartDataModel : ldChartDataModels) {
            // 分析秩
//            ldChartDataModel.setRank(getRankDataSample(ldChartDataModel.getSettingId(), ldChartDataModel.getFailureQuantity()==null?0:ldChartDataModel.getFailureQuantity(), ldChartDataModel.getAnalysisId()));
            // 概率图置信限
            ldChartDataModel.setPr(ldAnalysisPrBelieveMapper.selectPrBelieveBySettingId(ldChartDataModel.getSettingId()));
        }
        appendMoreParameters(ldChartDataModels);
        return ldChartDataModels;
    }

    /**
     * @Title: getRankDataSample
     * @Description: 取得分析秩数据抽样
     * @param settingId 分析设定ID
     * @param failureQuantity 失效数据量
     * @param analysisId 分析ID
     * @return List<LdAnalysisRank>
     */
    private List<EwbAnalysisRank> getRankDataSample(String settingId, int failureQuantity, String analysisId) {
        // 获取寿命数据-数据量信息
        EwbAnalysisModel analysisModel = analysisDataMapper.searchAanlysisDataByAnalysisId(analysisId);
        // 数据ID
        String dataId = analysisModel.getDataId();
        List<EwbAnalysisRank> rankSample = new ArrayList<>();
        // 分析明细数据为0失效，则进行特殊处理
        if (failureQuantity == 0) {
            // 根据数据ID搜索分析明细数据最大log失效时间、最小log失效时间
            EwbAnalysisDataDetailVo dataDetailModel = ldAnalysisDataDetailMapper.searchMaxAndMinFailureTime(dataId);
            EwbAnalysisSetting ldAnalysisSetting = ldAnalysisSettingMapper.selectById(settingId);

            if (dataDetailModel != null) {
                // 如果是0失效指数分布，则执行以下
                if (EwbDistributingCodeEnum.Exponential.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
                    // 获取评估结果值
                    EwbAnalysisResult ldAnalysisResult = ldAnalysisResultMapper.selectById(settingId);
                    double minX = 0.0;
                    double maxX = (double) Math.round(-(Math.log(0.5) / ldAnalysisResult.getP1()));
                    Double minY = 0.1;
                    Double maxY = 1.0;
                    // 值间距
                    double rangeX = maxX - minX;
                    double rangeY = maxY - minY;

                    // 作图时，分析秩抽样的最大数据量
                    int rankMaximum = Integer.parseInt(allVariableConfig.getRankMaximum());

                    // 分析秩总数
                    int rankNum = ldAnalysisDataDetailMapper.countEwbDatas(dataId);

                    // 分析秩数据量小于rankMaximum时取得全部数据
                    if (rankNum <= rankMaximum) {
                        rankSample = ldAnalysisDataDetailMapper.searchByDataIdForFailureExp(dataId);
                        // X值全部相等、Y值全部相等时，只取一条
                    } else if (rangeX == 0 && rangeY == 0) {
                        EwbAnalysisRank rank = ldAnalysisDataDetailMapper.searchOneByDataIdForFailureExp(dataId);
                        rankSample.add(rank);
                    } else {
                        rankSample = ldAnalysisDataDetailMapper.searchSampleByDataIdForFailureExp(minX, rangeX, rankMaximum, dataId);
                    }
                    rankSample.get(0).setX(maxX);
                } else {
                    Double minX = dataDetailModel.getMinFailureTime();
                    Double maxX = dataDetailModel.getMaxFailureTime();
                    Double minY = 0.0;
                    Double maxY = 0.0;

                    if (minX != null) {
                        // 值间距
                        double rangeX = maxX - minX;
                        double rangeY = maxY - minY;

                        // 作图时，分析秩抽样的最大数据量
                        int rankMaximum = Integer.parseInt(allVariableConfig.getRankMaximum());

                        // 分析秩总数
                        int rankNum = ldAnalysisDataDetailMapper.countEwbDatas(dataId);

                        // 分析秩数据量小于rankMaximum时取得全部数据
                        if (rankNum <= rankMaximum) {
                            rankSample = ldAnalysisDataDetailMapper.searchByDataIdForFailure(dataId);
                            // X值全部相等、Y值全部相等时，只取一条
                        } else if (rangeX == 0 && rangeY == 0) {
                            EwbAnalysisRank rank = ldAnalysisDataDetailMapper.searchOneByDataIdForFailure(dataId);
                            rankSample.add(rank);
                        } else {
                            rankSample = ldAnalysisDataDetailMapper.searchSampleByDataIdForFailure(minX, rangeX, rankMaximum, dataId);
                        }
                    }
                }
            }
            // 分析明细数据为非0失效，则进入正常处理
        } else {
            // 取得x轴变换坐标和y轴变换坐标的最大值和最小值
            Map<String, Object> extremum = ldAnalysisRankMapper.searchExtremum(settingId);
            if (extremum != null && !extremum.isEmpty()) {

                Double minX = WeibullNumberUtil.parseDouble(extremum.get(EwbConstants.KEY_RANK_MINX).toString());
                Double maxX = WeibullNumberUtil.parseDouble(extremum.get(EwbConstants.KEY_RANK_MAXX).toString());
                Double minY = WeibullNumberUtil.parseDouble(extremum.get(EwbConstants.KEY_RANK_MINY).toString());
                Double maxY = WeibullNumberUtil.parseDouble(extremum.get(EwbConstants.KEY_RANK_MAXY).toString());

                if (minX != null && minY != null) {

                    // 值间距
                    double rangeX = maxX - minX;
                    double rangeY = maxY - minY;

                    // 作图时，分析秩抽样的最大数据量
                    int rankMaximum = Integer.parseInt(allVariableConfig.getRankMaximum());

                    // 分析秩总数
                    int rankNum = ldAnalysisRankMapper.countRankBySettingId(settingId);

                    // 分析秩数据量小于rankMaximum时取得全部数据
                    if (rankNum <= rankMaximum) {
                        rankSample = ldAnalysisRankMapper.searchRankBySettingId(settingId);
                        // X值全部相等、Y值全部相等时，只取一条
                    } else if (rangeX == 0 && rangeY == 0) {
                        EwbAnalysisRank rank = ldAnalysisRankMapper.searchOneBySettingId(settingId);
                        rankSample.add(rank);
                        // X值全部相等，根据Y值抽样
                    } else if (rangeX == 0) {
                        rankSample = ldAnalysisRankMapper.searchRankSampleByY(minY, rangeY, rankMaximum, settingId);
                        // 根据X值抽样
                    } else {
                        rankSample = ldAnalysisRankMapper.searchRankSample(minX, rangeX, rankMaximum, settingId);
                    }
                }
            }
        }

        return rankSample;
    }


    /* (non-Javadoc)
     * <p>Title: getContourMapChartData</p>
     * <p>Description: 等高图作图数据获取</p>
     * @param analysisId
     * @return
     * @throws Exception
     */
    public List<EwbChartDataModel> getContourMapChartData(String analysisId, String plotType) {
        List<EwbChartDataModel> ldChartDataModels = ldAnalysisMapper.searchChartDataByAnalysisId(analysisId, plotType);
        appendMoreParameters(ldChartDataModels);
        return ldChartDataModels;
    }

    public void appendMoreParameters(List<EwbChartDataModel> chartData) {
        for (EwbChartDataModel cd : chartData) {
            if (EwbParameterCodeEnum.混合2参数.getIndex().equals(cd.getParamCode()) || EwbParameterCodeEnum.混合3参数.getIndex().equals(cd.getParamCode())) {
                EwbAnalysisResult ldAnalysisResult = ldAnalysisResultMapper.selectById(cd.getSettingId());
                cd.setP4(ldAnalysisResult.getDb()); // Eta 2
                cd.setP5(ldAnalysisResult.getDe()); // Beta 3
                cd.setP6(ldAnalysisResult.getDbe()); // Eta 3
                cd.setP7(ldAnalysisResult.getInterval1()); // P 1
                cd.setP8(ldAnalysisResult.getInterval2()); // P 1
                cd.setP9(ldAnalysisResult.getInterval3()); // P 3
            }
        }
    }
}
