package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.EwbCalcParametersEnum;
import com.nationrel.modules.weibull.enums.EwbDaModelCodeEnum;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.general.util.common.DateTimeUtil;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.service.damodel.DaFactory;
import com.nationrel.modules.weibull.service.damodel.DaModelI;
import com.nationrel.modules.weibull.util.*;
import com.nationrel.modules.weibull.vo.*;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.utils.URIBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Aimee on 2020/11/23.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbDaDataDetailService {

    private final EwbDaDataDetailMapper ewbDaDataDetailMapper;
    private final EwbDaUnitResultsMapper ewbDaUnitResultsMapper;
    private final EwbDaInfoMapper ewbDaInfoMapper;
    private final EwbAnalysisInfoMapper ewbAnalysisInfoMapper;
    private final EwbDaDataMapper ewbDaDataMapper;
    private final RgaApiService rgaApiService;
    private final EwbDaSettingMapper ewbDaSettingMapper;
    private final EwbDaPlotMapper ewbDaPlotMapper;
    private final EwbDaChartDataMapper ewbDaChartDataMapper;
    private final StringRedisTemplate redisTemplate;
    private final AllVariableConfig allVariableConfig;

    public Map<String, Object> dataList(String daId, int pageIndex, int pageSize) {
//        PageHelper.startPage(pageIndex, pageSize);
        Map<String, Object> result = new HashMap<>();
        List<EwbDaDataDetail> list = ewbDaDataDetailMapper.selectList(new QueryWrapper<EwbDaDataDetail>().lambda().eq(EwbDaDataDetail::getDaId, daId));
        result.put("list", list);
        result.put("dataNumber", allVariableConfig.getWeibullDataNumber());
        return result;
    }

    public void dataSave(HeaderUserInfo userInfo, EwbDaDataDetailSaveVo param) {
        // 获取分析明细数据
        List<EwbDaDataDetail> ewbDaDataDetails = JSONObject.parseArray(param.getDataDetails(), EwbDaDataDetail.class);
        validateData(ewbDaDataDetails, userInfo);
        if (ewbDaDataDetails.size() > 0) {
            // 删除集合中WeiMaterial对象属性fratios为空的对象s
            ewbDaDataDetails.removeIf(weiMaterial -> weiMaterial.getInspectTime() == null);
        }

        // 起始数据编号
        int startDataNo = param.getStartDataNo();
        // 结束数据编号
        int endDataNo = param.getEndDataNo();
        // ==================
        // 1.删除当前页面分析明细数据表数据
        // ==================
        ewbDaDataDetailMapper.delete(new QueryWrapper<EwbDaDataDetail>().lambda().eq(EwbDaDataDetail::getDaId, param.getDaId()));

        // ==================
        // 2.更新大于等于当前页起始数据编号的明细表数据
        // ==================
        // 计算删除或者新增数据的条数
        int changeCount = ewbDaDataDetails.size() - (endDataNo - startDataNo + 1);
        // 如果明细数据条数发生变化则 更新明细数据编号
        ewbDaDataDetailMapper.updateDetailDataNum(changeCount, param.getDaId(), endDataNo + 1);
        // 插入起始数据编号
        int newStartDataNo = startDataNo;
        if (startDataNo > 0) {
            newStartDataNo = startDataNo - 1;
        }
        Set set = new HashSet();
        if (ewbDaDataDetails.size() > 0) {
            // ==================
            // 3.保存退化分析明细数据
            // ==================
            // 分析明细数据处理

            List<EwbDaDataDetail> dataDetails = new ArrayList<>();
            double maxInspectTime = 0;
            double minInspectTime = 0;
            double maxInspectValue = 0;
            double minInspectValue = 0;
            List<Double> inspectTimes = new ArrayList<>();
            List<Double> inspectValues = new ArrayList<>();
            for (EwbDaDataDetail dataDetail : ewbDaDataDetails) {
                if (!inspectTimes.contains(dataDetail.getInspectTime())) {
                    inspectTimes.add(dataDetail.getInspectTime());
                }
                if (!inspectValues.contains(dataDetail.getInspectValue())) {
                    inspectValues.add(dataDetail.getInspectValue());
                }
                set.add(dataDetail.getUnitNo());
                // 获取最大最小值
                if (maxInspectTime == 0 || dataDetail.getInspectTime() > maxInspectTime) {
                    maxInspectTime = dataDetail.getInspectTime();
                }
                if (minInspectTime == 0 || dataDetail.getInspectTime() < minInspectTime) {
                    minInspectTime = dataDetail.getInspectTime();
                }
                if (maxInspectValue == 0 || dataDetail.getInspectValue() > maxInspectValue) {
                    maxInspectValue = dataDetail.getInspectValue();
                }
                if (minInspectValue == 0 || dataDetail.getInspectValue() < minInspectValue) {
                    minInspectValue = dataDetail.getInspectValue();
                }

                newStartDataNo++;
                // 数据编号
                dataDetail.setDataNo(newStartDataNo);
                // 数据ID
                dataDetail.setDaId(param.getDaId());
                dataDetails.add(dataDetail);
                if (dataDetails.size() > 100) {
                    // 批量插入分析明细数据
                    ewbDaDataDetailMapper.batchInsert(dataDetails);
                    dataDetails.clear();
                }
            }
//            if (inspectValues.size() < 3 || inspectTimes.size() < 3) {
//                throw new DescribeException("退化分析至少需要3条不同的检测时间和3条不同的测量值数据！", 1);
//            }
            if (dataDetails.size() > 0) {
                // 批量插入分析明细数据
                ewbDaDataDetailMapper.batchInsert(dataDetails);
                dataDetails.clear();
            }
            EwbDaData ewbDaData = ewbDaDataMapper.selectOne(new QueryWrapper<EwbDaData>().lambda().eq(EwbDaData::getDaId, param.getDaId()));
            ewbDaData.setQuantity(ewbDaDataDetails.size());
            ewbDaData.setUnitQuantity(set.size());
            ewbDaData.setMaxInspectTime(maxInspectTime);
            ewbDaData.setMinInspectTime(minInspectTime);
            ewbDaData.setMaxInspectValue(maxInspectValue);
            ewbDaData.setMinInspectValue(minInspectValue);
            ewbDaDataMapper.updateById(ewbDaData);

        }
        String wbId = ewbDaInfoMapper.selectWbIdByDaId(param.getDaId());
        ewbAnalysisInfoMapper.updateUpdateInfo(new Date(), userInfo.getUserId(), wbId);
        clearPlotDb(param.getDaId());
    }

    private void validateData(List<EwbDaDataDetail> ewbDaDataDetails, HeaderUserInfo userInfo) {
        ewbDaDataDetails.removeIf(ewbDaDataDetail -> ewbDaDataDetail.getInspectValue() == null && ewbDaDataDetail.getInspectTime() == null && ewbDaDataDetail.getUnitNo() == null);
        if (ewbDaDataDetails.size() > 100000) {
            throw new DescribeException(EwbCustomPromptEnum.导入数据不可超过100000, userInfo.getLanguageKey());
        }
        ewbDaDataDetails.stream().collect(Collectors.groupingBy(EwbDaDataDetail::getUnitNo)).forEach((unitNo, list) -> {
            Map<Double, List<EwbDaDataDetail>> map = list.stream().collect(Collectors.groupingBy(EwbDaDataDetail::getInspectTime));
            if (map.size() < 2) {
                throw new DescribeException(EwbCustomPromptEnum.每个单元编号至少有两条不相同的数据, userInfo.getLanguageKey());
            }
        });
        Set<String> set = new HashSet<>();
        for (EwbDaDataDetail ewbDaDataDetail : ewbDaDataDetails) {
            set.add(ewbDaDataDetail.getUnitNo());
            if (ValidateUtil.isEmpty(ewbDaDataDetail.getUnitNo())) {
                throw new DescribeException(EwbCommonUsedEnum.单元编号, userInfo.getLanguageKey(), EwbCustomPromptEnum.不能为空);
            }
            if (ewbDaDataDetail.getInspectTime() == null) {
                throw new DescribeException(EwbCommonUsedEnum.检测时间, userInfo.getLanguageKey(), EwbCustomPromptEnum.不能为空);
            } else if (!ValidateUtil.isNumber(ewbDaDataDetail.getInspectTime().toString())) {
                throw new DescribeException(EwbCommonUsedEnum.检测时间, userInfo.getLanguageKey(), EwbCustomPromptEnum.必须为数字);
            } else if (ewbDaDataDetail.getInspectTime() < 0 || ewbDaDataDetail.getInspectTime() > 999999999) {
                throw new DescribeException(EwbCommonUsedEnum.检测时间, new BigDecimal("0"), new BigDecimal("999999999"), EwbCustomPromptEnum.范围必须是, userInfo.getLanguageKey());
            }
            if (ewbDaDataDetail.getInspectValue() == null) {
                throw new DescribeException(EwbCommonUsedEnum.测量值, userInfo.getLanguageKey(), EwbCustomPromptEnum.不能为空);
            } else if (ewbDaDataDetail.getInspectValue() == 0) {
                throw new DescribeException(EwbCommonUsedEnum.测量值, userInfo.getLanguageKey(), EwbCustomPromptEnum.不能为零);
            } else if (ewbDaDataDetail.getInspectValue() < 0 || ewbDaDataDetail.getInspectValue() > 999999999) {
                throw new DescribeException(EwbCommonUsedEnum.测量值, EwbCustomPromptEnum.范围必须是, "0-999999999", userInfo.getLanguageKey());
            }
        }
//        if (set.size() < 2) {
//            throw new DescribeException("至少需要2组数据", -1);
//        }
    }

    /**
     * 清除画图数据
     *
     * @param daId
     */
    public void clearPlotDb(String daId) {
        ewbDaUnitResultsMapper.delete(new QueryWrapper<EwbDaUnitResults>().lambda().eq(EwbDaUnitResults::getDaId, daId));
        ewbDaChartDataMapper.delete(new QueryWrapper<EwbDaChartData>().lambda().eq(EwbDaChartData::getDaId, daId));
        Set<String> keys = redisTemplate.keys(daId + "*");
        redisTemplate.delete(Objects.requireNonNull(keys));
    }

    /**
     * 导入数据
     * 2021-12-20 添加流的关闭
     *
     * @param file
     * @param daId
     * @throws IOException
     */
    @Transactional(rollbackFor = Exception.class)
    public void importData(HeaderUserInfo userInfo, MultipartFile file, String daId) throws Exception {

        // 取得：Excel数据。
        List<List<String>> excelList = ImportExcelUtil.getListByExcel(file.getInputStream(), file.getOriginalFilename(), userInfo.getLanguageKey());
        if (CollectionUtils.isEmpty(excelList)) {
            return;
        }

        List<String[]> datas = ImportExcelUtil.excelToDatas(excelList);

        List<EwbDaDataDetail> ewbDaDataDetails = new ArrayList<>();
        // 解析新的数组结构的数据，将其解析为对象
        for (int rowNum = 0; rowNum < datas.size(); rowNum++) {
            String[] data = datas.get(rowNum);
            ewbDaDataDetails.add(EwbDaDataDetail.builder()
                    .daId(daId)
                    .dataNo(rowNum + 1)
                    .unitNo(data[1])
                    .inspectTime(NumberParseUtil.parseDouble(data[2]))
                    .inspectValue(NumberParseUtil.parseDouble(data[3]))
                    .build());
        }
        if (!CollectionUtils.isEmpty(ewbDaDataDetails)) {
            validateData(ewbDaDataDetails, userInfo);
            ewbDaDataDetailMapper.delete(new QueryWrapper<EwbDaDataDetail>().lambda().eq(EwbDaDataDetail::getDaId, daId));
            dataSave(userInfo, EwbDaDataDetailSaveVo.builder().dataDetails(JSONObject.toJSONString(ewbDaDataDetails)).startDataNo(0).endDataNo(ewbDaDataDetails.size()).daId(daId).build());
        }
        String wbId = ewbDaInfoMapper.selectWbIdByDaId(daId);
        ewbAnalysisInfoMapper.updateUpdateInfo(new Date(), userInfo.getUserId(), wbId);

    }

//    private List<List<EwbDaChartData>> dataGroup(List<EwbDaChartData> list) {
//        //创建一个map,来让list分组把一个List<User>分成多个List<User>
//        Map<String, List<EwbDaChartData>> map = new HashMap<>();
//        //遍历传进来的list
//        for (EwbDaChartData user : list) {
//            //如果map的某个key值已经存在同样的userId,则把这个user添加到这个key所在的List<User>
//            if (map.containsKey(user.getUnitNo())) {
//                map.get(user.getUnitNo()).add(user);
//            } else {
//                //如果没有相同的id,就新创建一个List
//                List<EwbDaChartData> newList = new ArrayList<>();
//                //添加数据
//                newList.add(user);
//                map.put(user.getUnitNo(), newList);
//            }
//        }
//        List<List<EwbDaChartData>> lines = new ArrayList<>();
//        Collection<List<EwbDaChartData>> values = map.values();
//        Iterator<List<EwbDaChartData>> iterator = values.iterator();
//        while (iterator.hasNext()) {
//            lines.add(iterator.next());
//        }
//        return lines;
//    }

    @Transactional(rollbackFor = Exception.class)
    public EwbDegradationResultVo analysis(String userId, String daId, Double xaxisStart, Double xaxisEnd, Double yaxisStart, Double yaxisEnd, Integer languageKey) throws Exception {
        // 获取设置
        EwbDaSetting daSetting = ewbDaSettingMapper.selectOne(new QueryWrapper<EwbDaSetting>().lambda().eq(EwbDaSetting::getDaId, daId));

        // 首先从redis中获取
//        String data = redisTemplate.opsForValue().get(daId + daSetting.getDaValue() + daSetting.getModelCode());
//        if (ValidateUtils.isNotEmpty(data)) {
//            EwbDegradationResultVo result = JSONObject.parseObject(data, EwbDegradationResultVo.class);
//            result.setDates(new String[]{DateTimeUtil.dateToStr(new Date(), "yyyy-MM-dd"), DateTimeUtil.dateToStr(new Date(), "HH:mm:ss")});
//            result.setModelStr(EwbDaModelCodeMnum.getName(daSetting.getModelCode()));
//            result.setDaValue(daSetting.getDaValue());
//            return result;
//        }

        // 然后从数据库获取
//        List<EwbDaChartData> ewbDaChartData = ewbDaChartDataMapper.selectList(new QueryWrapper<EwbDaChartData>().lambda().eq(EwbDaChartData::getDaId, daId).eq(EwbDaChartData::getModelCode, daSetting.getModelCode()).eq(EwbDaChartData::getDaValue, daSetting.getDaValue()));
//        if (ewbDaChartData != null && ewbDaChartData.size() > 0) {
//            List<EwbDaUnitResults> ewbDaUnitResults = ewbDaUnitResultsMapper.selectList(new QueryWrapper<EwbDaUnitResults>().lambda().eq(EwbDaUnitResults::getDaId, daId).eq(EwbDaUnitResults::getModel, daSetting.getModelCode()).eq(EwbDaUnitResults::getDaValue, daSetting.getDaValue()));
//            List<EwbDegradationDataVo> daDataDetailList = ewbDaDataDetailMapper.selectDaDetail(daId);
//            EwbDegradationResultVo result = new EwbDegradationResultVo();
//            result.setUnitResults(ewbDaUnitResults);
//            result.setPoints(daDataDetailList);
//            result.setLines(dataGroup(ewbDaChartData));
//            redisTemplate.opsForValue().set(daId + daSetting.getDaValue() + daSetting.getModelCode(), JSONObject.toJSONString(result));
//            return result;
//        }

        List<EwbDegradationDataVo> daDataDetailList = ewbDaDataDetailMapper.selectDaDetail(daId);
        //获取分析数据
        Map<String, Object> requestMap = getRequestData(daDataDetailList);
//        System.out.println(JSONObject.toJSONString(requestMap));

        URIBuilder uri = EwbConstants.SERVER_SYSTEM_WINDOWS.equals(allVariableConfig.getServerSystem()) ? new URIBuilder(allVariableConfig.getReliaQubeUrl() + "Degradation/Calculation/CalculateParameters") : new URIBuilder(allVariableConfig.getReliaQubeUrl() + "Degradation/Calculation");
        uri.setParameter(EwbConstants.MODEL, EwbDaModelCodeEnum.getName(daSetting.getModelCode()));
        uri.setParameter("CriticalValue", StringUtil.parseString(daSetting.getDaValue()));
        requestMap.put("data", new JSONObject());
        String resultsCallApi = rgaApiService.getResultsCallApi(new JSONObject(), uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1, JSONObject.toJSONString(requestMap));
        if (StringUtils.isBlank(resultsCallApi)) {
            throw new DescribeException(ReliaqubeParamDataCheckEnum.当前数据在该模型下无法进行分析, languageKey);
        }
        String exceptionMessage = JSONObject.parseObject(resultsCallApi).getString("ExceptionMessage");
        if (ValidateUtil.isNotEmpty(exceptionMessage)) {
            throw new DescribeException(ReliaqubeParamDataCheckEnum.当前数据在该模型下无法进行分析, languageKey);
        }
//        System.out.println(resultsCallApi);
        EwbDegradationResultVo result = JSONObject.parseObject(resultsCallApi.replaceAll("Infinity", ""), EwbDegradationResultVo.class);
        result.setXaxisStart(xaxisStart);
        result.setXaxisEnd(xaxisEnd);
        result.setYaxisStart(yaxisStart);
        result.setYaxisEnd(yaxisEnd);
        Optional<EwbDaUnitResults> max = result.getUnitResults().stream().max(Comparator.comparingDouble(EwbDaUnitResults::getPredictedFailureTime));
        Optional<EwbDaUnitResults> min = result.getUnitResults().stream().min(Comparator.comparingDouble(EwbDaUnitResults::getPredictedFailureTime));
        daSetting.setMaxFailureTime(isNaN(max.get().getPredictedFailureTime()));
        daSetting.setMinFailureTime(isNaN(min.get().getPredictedFailureTime()));
        ewbDaSettingMapper.updateById(daSetting);

        EwbDaData ewbDaData = ewbDaDataMapper.selectOne(new QueryWrapper<EwbDaData>().lambda().eq(EwbDaData::getDaId, daId));
        ewbDaData.setModel(result.getModel());
        ewbDaData.setTotalPho(isNaN(result.getTotalPho()));
        ewbDaData.setTotalSse(isNaN(result.getTotalSSE()));
        ewbDaDataMapper.updateById(ewbDaData);

        if (WeibullNumberUtil.isNull(result.getTotalPho())) {
            result.setTotalPho(null);
        }
        if (WeibullNumberUtil.isNull(result.getTotalSSE())) {
            result.setTotalSSE(null);
        }
        for (EwbDaUnitResults unitResult : result.getUnitResults()) {
            if (WeibullNumberUtil.isNull(unitResult.getA())) {
                unitResult.setA(null);
            }
            if (WeibullNumberUtil.isNull(unitResult.getB())) {
                unitResult.setB(null);
            }
            if (WeibullNumberUtil.isNull(unitResult.getC())) {
                unitResult.setC(null);
            }
            if (WeibullNumberUtil.isNull(unitResult.getVara())) {
                unitResult.setVara(null);
            }
            if (WeibullNumberUtil.isNull(unitResult.getVarb())) {
                unitResult.setVarb(null);
            }
            if (WeibullNumberUtil.isNull(unitResult.getVarc())) {
                unitResult.setVarc(null);
            }
            if (WeibullNumberUtil.isNull(unitResult.getPho())) {
                unitResult.setPho(null);
            }
            if (WeibullNumberUtil.isNull(unitResult.getSse())) {
                unitResult.setSse(null);
            }
            if (WeibullNumberUtil.isNull(unitResult.getPredictedFailureTime())) {
                unitResult.setPredictedFailureTime(null);
            }
            unitResult.setDaValue(daSetting.getDaValue());
            unitResult.setPredictedFailureTime(unitResult.getPredictedFailureTime() == null ? null : WeibullNumberUtil.round(unitResult.getPredictedFailureTime(), EwbConstants.NUM_DIGIT_9));
            unitResult.setModel(daSetting.getModelCode());
        }
        ewbDaUnitResultsMapper.batchInsert(daId, result.getUnitResults());

        List<List<EwbDaChartData>> lines = calcXy(daSetting.getModelCode(), ewbDaData.getMaxInspectTime(), result.getUnitResults(), daSetting.getDaValue(), result);
        List<String> lineNames = new ArrayList<>();
        daDataDetailList.forEach(dataDetail -> {
            if (!lineNames.contains(dataDetail.getUnitID())) {
                lineNames.add(dataDetail.getUnitID());
            }
        });
        List<List<EwbDaChartData>> linesResult = new ArrayList<>();
        lineNames.forEach(lineName -> lines.forEach(line -> {
            if (line.get(0).getUnitNo().equals(lineName)) {
                linesResult.add(line);
            }
        }));
        result.setLines(linesResult);
        result.setPoints(daDataDetailList);
        result.setDates(new String[]{DateTimeUtil.dateToStr(new Date(), "yyyy-MM-dd"), DateTimeUtil.dateToStr(new Date(), "HH:mm:ss")});
        result.setModelStr(EwbDaModelCodeEnum.getName(daSetting.getModelCode()));
        result.setDaValue(daSetting.getDaValue());
//        redisTemplate.opsForValue().set(daId + daSetting.getDaValue() + daSetting.getModelCode(), JSONObject.toJSONString(result));
        String wbId = ewbDaInfoMapper.selectWbIdByDaId(daId);
        ewbAnalysisInfoMapper.updateUpdateInfo(new Date(), userId, wbId);
        result.setModel(daSetting.getModelCode());
        return result;
    }

    public static Double isNaN(Double value) {
        return Double.isNaN(value) ? null : value;
    }

    /**
     * 计算作图线
     *
     * @param model
     * @param xmax
     * @return
     */
    public List<List<EwbDaChartData>> calcXy(String model, double xmax, List<EwbDaUnitResults> results, Double daValue, EwbDegradationResultVo resultVo) {
        // Step 1: Define the range for X.
        double xmin = resultVo.getXaxisStart() == null ? 0 : resultVo.getXaxisStart();
        List<Double> rangeX = new ArrayList<>();
        if (EwbDaModelCodeEnum.Logarithmic.getIndex().equals(model) || EwbDaModelCodeEnum.LogLogistic.getIndex().equals(model) || EwbDaModelCodeEnum.LloydLipow.getIndex().equals(model)) {
            xmin = 0.0001;
        }
        // Step 2: Define 21 points from Xmin and Xmax.
        xmax = resultVo.getXaxisEnd() == null ? xmax * 1.2 : resultVo.getXaxisEnd();
        resultVo.setXaxisEnd(Math.floor(xmax));
        double deltaX = (xmax - xmin) / 200;
        while (xmin <= xmax) {
            rangeX.add(xmax);
            xmax = xmax - deltaX;
        }
        rangeX.add(0.0001);
        DaFactory daFactory = new DaFactory();
        DaModelI daModelI = daFactory.getModel(model);
        // Step 3: Calculate Y value for each test unit
        List<List<EwbDaChartData>> lines = new ArrayList<>();
        for (EwbDaUnitResults result : results) {
            List<EwbDaChartData> degrade = daModelI.degrade(result, rangeX, daValue);
            lines.add(degrade);
//            ewbDaChartDataMapper.insertBatch(daId, degrade);
        }
        resultVo.setXaxisStart(xmin);
        return lines;
    }


    public Map<String, Object> getRequestData(List<EwbDegradationDataVo> daDataDetailList) {

        /*2、分组算法**/
        Map<String, List<EwbDegradationDataVo>> groupMap = new HashMap<>();
        for (EwbDegradationDataVo skuVo : daDataDetailList) {
            List<EwbDegradationDataVo> tempList = groupMap.get(skuVo.getUnitID());
            /*如果取不到数据,那么直接new一个空的ArrayList**/
            if (tempList == null) {
                tempList = new ArrayList<>();
                tempList.add(skuVo);
                groupMap.put(skuVo.getUnitID(), tempList);
            } else {
                /*某个sku之前已经存放过了,则直接追加数据到原来的List里**/
                tempList.add(skuVo);
            }
        }
        List<EwbDegradationVo> test = new ArrayList<>();
        /*3、请求数据组装**/
        for (String key : groupMap.keySet()) {
            List<EwbDegradationDataVo> ewbDaDataDetails = groupMap.get(key);
            EwbDegradationVo ewbDegradationVo = new EwbDegradationVo();
            ewbDegradationVo.setUnitID(key);
            ewbDegradationVo.setTotalNumofRows(ewbDaDataDetails.size());
            ewbDegradationVo.setDataRows(ewbDaDataDetails);
            ewbDegradationVo.setStress(new double[0]);
            test.add(ewbDegradationVo);
        }
        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.unitsData), test);
        requestMap.put(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.totalNumofUnits), test.size());
        return requestMap;
    }

    public void importTemplate(HttpServletResponse response) throws IOException {
        // 取得：Excel标题Map
        LinkedHashMap<String, String> excelTitleMap = this.buildExcelTitleMap();
        // 导出：Excel
        ExportExcelUtil.export("degradation", new ArrayList<>(100), excelTitleMap, response);
    }

    /**
     * 取得：Excel标题Map
     *
     * @return：Map
     */
    private LinkedHashMap<String, String> buildExcelTitleMap() {
        LinkedHashMap<String, String> excelTitleMap = new LinkedHashMap<>();
        excelTitleMap.put("UnitID", "UnitID");
        excelTitleMap.put("Time", "Time");
        excelTitleMap.put("Degradation", "Degradation");
        return excelTitleMap;
    }

    public List<EwbDaUnitResults> getResultList(String daId) {
        // 获取设置
        EwbDaSetting ewbDaSetting = ewbDaSettingMapper.selectOne(new QueryWrapper<EwbDaSetting>().lambda().eq(EwbDaSetting::getDaId, daId));
        return ewbDaUnitResultsMapper.selectList(new QueryWrapper<EwbDaUnitResults>().lambda().eq(EwbDaUnitResults::getDaId, daId).eq(EwbDaUnitResults::getModel, ewbDaSetting.getModelCode()));
    }

    public String getResultPlot(String daId) throws Exception {
        EwbDaPlot daPlot = ewbDaPlotMapper.selectOne(new QueryWrapper<EwbDaPlot>().lambda().eq(EwbDaPlot::getDaId, daId));
        if (daPlot != null) {
            return new String(daPlot.getPlotData());
        }

        List<EwbDaUnitResults> ewbDaUnitResults = ewbDaUnitResultsMapper.selectList(new QueryWrapper<EwbDaUnitResults>().lambda().eq(EwbDaUnitResults::getDaId, daId));
        EwbDaData ewbDaData = ewbDaDataMapper.selectOne(new QueryWrapper<EwbDaData>().lambda().eq(EwbDaData::getDaId, daId));
        EwbDaPlotRequestVo ewbDaPlotRequestVo = new EwbDaPlotRequestVo();
        EwbDaPlotDataVo ewbDaPlotDataVo = new EwbDaPlotDataVo();
        ewbDaPlotDataVo.setCritialDegradation(25.0);
        ewbDaPlotDataVo.setDegradationModel("Linear");
        List<EwbDegradationDataVo> daDataDetailList = ewbDaDataDetailMapper.selectDaDetail(daId);
        Map<String, Object> requestData = getRequestData(daDataDetailList);
        ewbDaPlotDataVo.setData(requestData);
        Map<String, Object> map = new HashMap<>();
        map.put("UnitResults", ewbDaUnitResults);
        map.put("TotalSSE", ewbDaData.getTotalSse());
        map.put("TotalPho", ewbDaData.getTotalPho());
        map.put(EwbConstants.MODEL, ewbDaData.getModel());
        ewbDaPlotDataVo.setResults(map);
        List<JSONObject> list = new ArrayList<>();
        list.add(JSONObject.parseObject("{'Settings':{'Title':{'Name':'A'}}}"));
        list.add(JSONObject.parseObject("{'Settings':{'Title':{'Name':'B'}}}"));
        ewbDaPlotDataVo.setSinglePlotSetting(list);
        ewbDaPlotRequestVo.setPlotData(ewbDaPlotDataVo);
        ewbDaPlotRequestVo.setSettings(JSONObject.parseObject("{'Resolution':'VeryLow','Title': {'Name':'Linear'}}"));
//        System.out.println(JSONObject.toJSONString(daPlotRequestVo));
        String uri = allVariableConfig.getReliaQubeUrl() + "/Degradation/Plot/DegradationPlots";
        String resultsCallApi = rgaApiService.getResultsCallApi(new JSONObject(), uri, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1, JSONObject.toJSONString(ewbDaPlotRequestVo));
        EwbDaPlot ewbDaPlot = new EwbDaPlot();
        ewbDaPlot.setDaId(daId);
        ewbDaPlot.setPlotData(resultsCallApi.getBytes());
        ewbDaPlotMapper.insert(ewbDaPlot);
        return resultsCallApi;
    }

    public EwbDaData getDataInfo(String daId) {
        return ewbDaDataMapper.selectOne(new QueryWrapper<EwbDaData>().lambda().eq(EwbDaData::getDaId, daId));
    }

    public List<EwbDegradationResultVo> calc(String daId) throws Exception {
        // 获取设置
        EwbDaSetting ewbDaSetting = ewbDaSettingMapper.selectOne(new QueryWrapper<EwbDaSetting>().lambda().eq(EwbDaSetting::getDaId, daId));
        List<EwbDegradationDataVo> daDataDetailList = ewbDaDataDetailMapper.selectDaDetail(daId);
        Map<String, Object> requestMap = getRequestData(daDataDetailList);
        List<EwbDegradationResultVo> list = new ArrayList<>();
        for (EwbDaModelCodeEnum value : EwbDaModelCodeEnum.values()) {
            URIBuilder uri = EwbConstants.SERVER_SYSTEM_WINDOWS.equals(allVariableConfig.getServerSystem()) ? new URIBuilder(allVariableConfig.getReliaQubeUrl() + "Degradation/Calculation/CalculateParameters") : new URIBuilder(allVariableConfig.getReliaQubeUrl() + "Degradation/Calculation");
            uri.setParameter(EwbConstants.MODEL, value.getName());
            uri.setParameter("CriticalValue", StringUtil.parseString(ewbDaSetting.getDaValue()));
            requestMap.put("data", new JSONObject());
            String resultsCallApi = rgaApiService.getResultsCallApi(new JSONObject(), uri.build().toString(), EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1, JSONObject.toJSONString(requestMap));
            EwbDegradationResultVo result = JSONObject.parseObject(resultsCallApi, EwbDegradationResultVo.class);
            if (result == null) {
                result = new EwbDegradationResultVo();
            }
            result.setModel(value.getIndex());
            list.add(result);
        }
        return list;
    }

    public void export(HttpServletResponse response, String dataId, Integer languageKey) {

        // 获取导出寿命数据
        List<JSONObject> expList = ewbDaDataDetailMapper.selectList(new QueryWrapper<EwbDaDataDetail>().lambda().eq(EwbDaDataDetail::getDaId, dataId)).stream().map(item -> {
            JSONObject js = new JSONObject();
            js.put("UnitID", item.getUnitNo());
            js.put("Time", item.getInspectTime());
            js.put("Degradation", item.getInspectValue());
            return js;
        }).collect(Collectors.toList());

        // 定义：sheet名称
        String sheetName = "_Degradation_data";
        // 取得：Excel标题Map
        LinkedHashMap<String, String> fieldMap = this.buildExcelTitleMap();
        // 导出：Excel
        ExportExcelUtil.export(sheetName, expList, fieldMap, response);

    }

    /**
     * @return List<String>
     * @Title: getCsvHeader
     * @Description: 生成相应的CSV列头
     */
    private List<String> getCsvHeader(Integer languageKey) {
        List<String> retList = new ArrayList<>();
        retList.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.单元编号, languageKey));
        retList.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.检测时间, languageKey));
        retList.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.测量值, languageKey));
        return retList;
    }

}
