package com.nationrel.modules.weibull.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.EwbDaModelCodeEnum;
import com.nationrel.modules.weibull.vo.EwbDaInfoSaveVo;
import com.nationrel.modules.weibull.util.ValidateUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @author Aimee on 2020/11/23.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbDaInfoService {

    private final EwbDaInfoMapper ewbDaInfoMapper;
    private final EwbDaSettingMapper ewbDaSettingMapper;
    private final EwbAnalysisInfoMapper ewbAnalysisInfoMapper;
    private final EwbDaDataMapper ewbDaDataMapper;
    private final EwbDaDataDetailMapper ewbDaDataDetailMapper;
    private final EwbDaChartDataMapper ewbDaChartDataMapper;
    private final EwbDaUnitResultsMapper ewbDaUnitResultsMapper;

    public String create(String name, String wbId, HeaderUserInfo userInfo){
        String daId = UUIDUtil.getUuid();
        EwbDaInfo ewbDaInfo = EwbDaInfo.builder()
                .daId(daId)
                .wbId(wbId)
                .daName(name)
                .delFlag("0")
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId().toString())
                .updateTime(new Date())
                .versionNo(1).build();
        ewbDaInfoMapper.insert(ewbDaInfo);

        EwbDaSetting ewbDaSetting = new EwbDaSetting();
        ewbDaSetting.setDaId(daId);
        ewbDaSetting.setDaValue(40.0);
        ewbDaSetting.setModelCode(EwbDaModelCodeEnum.Linear.getIndex());
        ewbDaSettingMapper.insert(ewbDaSetting);

        EwbDaData ewbDaData = new EwbDaData();
        ewbDaData.setDaId(daId);
        ewbDaData.setSource("01");
        ewbDaDataMapper.insert(ewbDaData);
        ewbAnalysisInfoMapper.updateUpdateInfo(new Date(), userInfo.getUserId(), wbId);
        return daId;
    }

    public List<EwbDaInfo> list(HeaderUserInfo userInfo, String wbId) {
        return ewbDaInfoMapper.selectList(
                new QueryWrapper<EwbDaInfo>().lambda()
                        .eq(EwbDaInfo::getInsertUserId, userInfo.getUserId())
                        .eq(EwbDaInfo::getDelFlag, "0")
                        .eq(EwbDaInfo::getWbId, wbId));
    }

    public void edit(HeaderUserInfo userInfo, EwbDaInfoSaveVo param) {
        EwbDaInfo old = ewbDaInfoMapper.selectOne(new QueryWrapper<EwbDaInfo>().lambda().eq(EwbDaInfo::getDaId, param.getDaId()));
        if (ValidateUtil.isNotEmpty(param.getDaName())) {
            old.setDaName(param.getDaName());
        }
        if (ValidateUtil.isNotEmpty(param.getDelFlag())) {
            old.setDelFlag(param.getDelFlag());
        }
        old.setUpdateTime(new Date());
        old.setUpdateUserId(userInfo.getUserId().toString());
        ewbDaInfoMapper.updateById(old);
    }

    @Transactional(rollbackFor = Exception.class)
    public String copy(String daId) {
        String newDaId = UUIDUtil.getUuid();
        ewbDaInfoMapper.copy(newDaId, daId);
        ewbDaDataMapper.copy(newDaId, daId);
        ewbDaSettingMapper.copy(newDaId, daId);
        ewbDaDataDetailMapper.copy(newDaId, daId);
        ewbDaChartDataMapper.copy(newDaId, daId);
        ewbDaUnitResultsMapper.copy(newDaId, daId);
        return newDaId;
    }
}
