package com.nationrel.modules.weibull.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisInfoMapper;
import com.nationrel.modules.weibull.mapper.EwbDaInfoMapper;
import com.nationrel.modules.weibull.mapper.EwbDaSettingMapper;
import com.nationrel.modules.weibull.entity.EwbDaSetting;
import com.nationrel.modules.weibull.vo.EwbDaSettingVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * @author Aimee on 2020/11/23.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbDaSettingService {
    private final EwbDaSettingMapper ewbDaSettingMapper;
    private final EwbDaInfoMapper ewbDaInfoMapper;
    private final EwbAnalysisInfoMapper ewbAnalysisInfoMapper;
    private final StringRedisTemplate redisTemplate;

    public EwbDaSetting getSetting(String daId) {
        return ewbDaSettingMapper.selectOne(new QueryWrapper<EwbDaSetting>().lambda().eq(EwbDaSetting::getDaId, daId));
    }

    @Transactional(rollbackFor = Exception.class)
    public void update(String userId, EwbDaSettingVo param) {
        EwbDaSetting ewbDaSetting = ewbDaSettingMapper.selectOne(new QueryWrapper<EwbDaSetting>().lambda().eq(EwbDaSetting::getDaId, param.getDaId()));
        ewbDaSetting.setModelCode(param.getModelCode());
        ewbDaSetting.setDaValue(param.getDaValue());
        ewbDaSettingMapper.updateById(ewbDaSetting);

        String wbId = ewbDaInfoMapper.selectWbIdByDaId(param.getDaId());
        ewbAnalysisInfoMapper.updateUpdateInfo(new Date(), userId, wbId);
        redisTemplate.delete(param.getDaId());
    }

    @Transactional(rollbackFor = Exception.class)
    public void selectModel(String daId, String model) {
        EwbDaSetting ewbDaSetting = ewbDaSettingMapper.selectOne(new QueryWrapper<EwbDaSetting>().lambda().eq(EwbDaSetting::getDaId, daId));
        ewbDaSetting.setModelCode(model);
        ewbDaSettingMapper.updateById(ewbDaSetting);
    }
}
