package com.nationrel.modules.weibull.service;

import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDataDetailMapper;
import com.nationrel.modules.weibull.dto.EwbCalcResultDetailDto;
import com.nationrel.modules.weibull.dto.Pagination;
import com.nationrel.modules.weibull.entity.EwbAnalysisDataDetail;
import com.nationrel.modules.weibull.entity.EwbAnalysisSetting;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @author Aimee on 2020/9/22.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbDataHandlerService {
    private final EwbAnalysisDataDetailMapper ewbAnalysisDataDetailMapper;

    public EwbCalcResultDetailDto handlerLd(EwbAnalysisSetting ewbAnalysisSetting) {
        EwbCalcResultDetailDto ewbCalcResultDetailDto = new EwbCalcResultDetailDto();
        String dataId = ewbAnalysisSetting.getDataId();

        int rowTotal = ewbAnalysisDataDetailMapper.countEwbDatas(dataId);

        int curPage = 1;
        int pageSize = EwbConstants.NUM_PAGESIZE_SEL;

        Pagination pagination = new Pagination();
        pagination.setPageSize(pageSize);
        pagination.setPageCurrent(curPage);
        pagination.setRowTotal(rowTotal);

        handlerLdForBelieve(dataId, pagination, ewbCalcResultDetailDto);

        return ewbCalcResultDetailDto;
    }
    /**
     * @Title: handlerLdForBelieve
     * @Description: 处理寿命数据
     * @param dataId 数据Id
     * @param pagination 分页参数
     * @param ldModel void
     */
    private void handlerLdForBelieve(String dataId, Pagination pagination, EwbCalcResultDetailDto ldModel) {
        // 准备数据
        List<EwbAnalysisDataDetail> lds = ewbAnalysisDataDetailMapper.selectDatasByDataId(dataId, pagination);
        if (CollectionUtils.isEmpty(lds)) {
            return ;
        }
        String type;
        int quantity;
        for (EwbAnalysisDataDetail ld : lds) {
            type = ld.getDataType();
            quantity = ld.getQuantity();
            switch (type) {
                case EwbConstants.COMPLETE_DATA_TYPE:
                    ldModel.addT1(ld.getFailureEndTime());
                    ldModel.addN1(quantity);
                    break;
                case EwbConstants.LEFT_DATA_TYPE:
                case EwbConstants.INTERVAL_DATA_TYPE:
                    ldModel.addTu(ld.getFailureEndTime());
                    ldModel.addTl(ld.getLastInspectTime());
                    ldModel.addNu(quantity);
                    ldModel.addNl(quantity);
                    break;
                case EwbConstants.RIGHT_DATA_TYPE:
                    ldModel.addT2(ld.getFailureEndTime());
                    ldModel.addN2(quantity);
                    break;
                default:
                    break;
            }
        }
        pagination.setPageCurrent(pagination.getPageCurrent() + 1);
        handlerLdForBelieve(dataId, pagination, ldModel);
    }
}
