package com.nationrel.modules.weibull.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDataDetailMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDataMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisInfoMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisMapper;
import com.nationrel.modules.weibull.bo.EwbAnalysisSettingSaveBo;
import com.nationrel.modules.weibull.bo.EwbGroupListSaveBo;
import com.nationrel.modules.weibull.entity.EwbAnalysis;
import com.nationrel.modules.weibull.entity.EwbAnalysisData;
import com.nationrel.modules.weibull.vo.EwbGroupListVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * @author Aimee on 2020/11/23.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbGroupService {

    private final EwbAnalysisDataDetailMapper ewbAnalysisDataDetailMapper;
    private final EwbAnalysisMapper ewbAnalysisMapper;
    private final EwbAnalysisInfoMapper ewbAnalysisInfoMapper;
    private final EwbAnalysisSettingService ewbAnalysisSettingService;
    private final SaveDataDetailService saveService;
    private final EwbAnalysisDataMapper ewbAnalysisDataMapper;

    public List<EwbGroupListVo> getGroupList(String dataId) {
        return ewbAnalysisDataDetailMapper.getGroupList(dataId);
    }

    @Transactional(rollbackFor = Exception.class)
    public void saveGroupList(HeaderUserInfo userInfo, EwbGroupListSaveBo param){
        // 拆分分组名称
        String[] names = param.getName().split(",");
        EwbAnalysis ewbAnalysis1 = ewbAnalysisMapper.selectOne(new QueryWrapper<EwbAnalysis>().lambda().eq(EwbAnalysis::getAnalysisId, param.getAnalysisId()));
        EwbAnalysisData ewbAnalysisData1 = ewbAnalysisDataMapper.selectOne(new QueryWrapper<EwbAnalysisData>().lambda().eq(EwbAnalysisData::getDataId, param.getDataId()));
        for (String name : names) {
            if (StringUtils.isBlank(name)) {
                continue;
            }
            String analysisId = UUIDUtil.getUuid();
            String settingId = UUIDUtil.getUuid();
            String dataId = UUIDUtil.getUuid();

            // 创建分析
            EwbAnalysis ewbAnalysis = EwbAnalysis.builder()
                    .analysisId(analysisId)
                    .analysisName(name.equals(param.getDataId()) ? "Others" : name)
                    .wbId(ewbAnalysis1.getWbId())
                    .parentId(param.getAnalysisId())
                    .objectCode(null)
                    .objectName(null)
                    .description(null)
                    .versionNo(0)
                    .insertTime(new Date())
                    .insertUserId(userInfo.getUserId())
                    .delFlag("0").build();
            ewbAnalysisMapper.insert(ewbAnalysis);

            // 创建设置信息
            EwbAnalysisSettingSaveBo ewbAnalysisSettingSaveBo = new EwbAnalysisSettingSaveBo();
            BeanUtils.copyProperties(param, ewbAnalysisSettingSaveBo);
            ewbAnalysisSettingSaveBo.setSettingId(settingId);
            ewbAnalysisSettingService.createAnalysisSetting(userInfo, ewbAnalysisSettingSaveBo, dataId);
            // 创建数据信息
            EwbAnalysisData ewbAnalysisData = new EwbAnalysisData();
            ewbAnalysisData.setDataId(dataId);
            ewbAnalysisData.setAnalysisId(analysisId);
            ewbAnalysisData.setRFlag(EwbConstants.FLAG_CHECKED);
            ewbAnalysisData.setTimeForRank(EwbConstants.FLAG_NO_CHECKED);
            ewbAnalysisData.setLFlag(ewbAnalysisData1.getLFlag());
            ewbAnalysisData.setGroupingFlag(ewbAnalysisData1.getGroupingFlag());
            ewbAnalysisData.setFailureModeFlag(EwbConstants.FLAG_NO_CHECKED);
            ewbAnalysisData.setEditFlag(EwbConstants.EDIT_FLAG);
            ewbAnalysisData.setInsertTime(new Date());
            ewbAnalysisData.setInsertUserId(userInfo.getUserId());
            ewbAnalysisData.setVersionNo(0);
            ewbAnalysisDataMapper.insert(ewbAnalysisData);

            // ==================
            // 5.保存分析明细数据
            // ==================
            // 页面选择"计算所选分组,其余分组不计算"和页面选择"计算所选分组,其余分组当作右删失数据计算"的不同处理逻辑
            if (EwbConstants.GROUPING_TYPE01.equals(param.getGroupingCode())) {
                // 复制分组明细表数据到分析明细数据表
                ewbAnalysisDataDetailMapper.copyGddToAdd(dataId, param.getDataId(), name.equals(param.getDataId()) ? "" : name, "=");
            } else {
                // 复制分组明细表数据到分析明细数据表(copy为S的数据)
                ewbAnalysisDataDetailMapper.copyGddToAddS(dataId, param.getDataId(), name.equals(param.getDataId()) ? "" : name, "=");
                // 复制分组明细表数据到分析明细数据表(copy为F的数据)
                ewbAnalysisDataDetailMapper.copyGddToAddF(dataId, param.getDataId(), name.equals(param.getDataId()) ? "" : name, "<>");
            }

            // ==================
            // 4.更新寿命数据分析数据表的各种类型的数据量
            // ==================
            saveService.updateAnalysisData(userInfo, dataId);
            ewbAnalysisInfoMapper.updateUpdateInfo(new Date(), userInfo.getUserId(), ewbAnalysis1.getWbId());
        }
    }
}
