package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.general.util.common.DateTimeUtil;
import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.controller.EwbChartController;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.entity.EwbAnalysisDataDetail;
import com.nationrel.modules.weibull.entity.EwbMultiChart;
import com.nationrel.modules.weibull.entity.EwbMultiChartDetail;
import com.nationrel.modules.weibull.enums.EwbConfidenceTypeCodeEnum;
import com.nationrel.modules.weibull.enums.EwbDistributingCodeEnum;
import com.nationrel.modules.weibull.vo.*;
import com.nationrel.modules.weibull.util.ValidateUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbMultiChartService {

    private final EwbMultiChartMapper ewbMultiChartMapper;
    private final EwbMultiChartDetailMapper ewbMultiChartDetailMapper;
    private final EwbAnalysisMapper ewbAnalysisMapper;
    private final EwbAnalysisDataMapper ewbAnalysisDataMapper;
    private final EwbAnalysisInfoMapper ewbAnalysisInfoMapper;
    private final EwbAnalysisDataDetailMapper ewbAnalysisDataDetailMapper;
    private final EwbPlotService ewbPlotService;

    @Transactional(rollbackFor = Exception.class)
    public ServerResponse createMultiChart(HeaderUserInfo userInfo, EwbMultiChartCreateVo param) {

        String multiChartId = UUIDUtil.getUuid();
        EwbMultiChart ewbMultiChart = EwbMultiChart.builder()
                .multiChartId(multiChartId)
                .multiChartName(param.getName())
                .wbId(param.getWbId())
                .delFlag("0")
                .description(param.getDescription())
                .lifeState(null)
                .objectCode(param.getObjectCode())
                .objectName(param.getObjectName())
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .updateTime(new Date())
                .versionNo(1).build();
        ewbMultiChartMapper.insert(ewbMultiChart);

        EwbMultiChartSaveListVo ewbMultiChartSaveListVo = new EwbMultiChartSaveListVo();
        ewbMultiChartSaveListVo.setMultiChartId(multiChartId);
        ewbMultiChartSaveListVo.setAnalysisIds(param.getAnalysisIds());
        saveList(ewbMultiChartSaveListVo, userInfo.getLanguageKey());
        ewbAnalysisInfoMapper.updateUpdateInfo(new Date(), userInfo.getUserId(), param.getWbId());
        return ServerResponse.createBySuccess(multiChartId);
    }

    public List<EwbMultiChart> list(String wbId) {
        return ewbMultiChartMapper.selectList(new QueryWrapper<EwbMultiChart>().lambda().eq(EwbMultiChart::getWbId, wbId).eq(EwbMultiChart::getDelFlag, "0"));
    }

    @Transactional(rollbackFor = Exception.class)
    public void edit(HeaderUserInfo userInfo, EwbMultiChart ewbMultiChart) {
        EwbMultiChart old = ewbMultiChartMapper.selectOne(new QueryWrapper<EwbMultiChart>().lambda().eq(EwbMultiChart::getMultiChartId, ewbMultiChart.getMultiChartId()));
        if (ValidateUtil.isNotEmpty(ewbMultiChart.getMultiChartName())) {
            old.setMultiChartName(ewbMultiChart.getMultiChartName());
        }
        if (ValidateUtil.isNotEmpty(ewbMultiChart.getDelFlag())) {
            old.setDelFlag(ewbMultiChart.getDelFlag());
        }
        old.setUpdateTime(new Date());
        old.setUpdateUserId(userInfo.getUserId());
        ewbMultiChartMapper.update(old, new QueryWrapper<EwbMultiChart>().lambda().eq(EwbMultiChart::getMultiChartId, ewbMultiChart.getMultiChartId()));
        ewbAnalysisInfoMapper.updateUpdateInfo(new Date(), userInfo.getUserId(), ewbMultiChart.getWbId());
    }

    /**
     * 查询可选择的weibull分析列表
     *
     * @param wbId
     * @param multiChartId
     * @param analysisType
     * @param userId
     * @return
     */
    public List<EwbMultiChartAnalysisListVo> selectAnalysisList(String wbId, String multiChartId, String analysisType, String userId) {
        List<EwbMultiChartAnalysisListVo> list = StringUtils.isNotBlank(analysisType) && "costOpti".equals(analysisType) ? ewbAnalysisMapper.selectAllAnalysisList(userId) : ewbAnalysisMapper.selectAnalysisList(wbId, multiChartId);
        list.forEach(entity -> entity.setDistributingCode(EwbDistributingCodeEnum.getName(entity.getDistributingCode())));
        return list;
    }

    /**
     * 更新多图的名称和选择的weibull分析案例
     *
     * @param param
     */
    public void saveList(EwbMultiChartSaveListVo param, Integer languageKey) {
        String[] analysisIds = param.getAnalysisIds().split(",");
        if (analysisIds.length < 1) {
            throw new DescribeException(EwbCustomPromptEnum.至少选择一条分析, languageKey);
        }
        ewbMultiChartMapper.updateNameById(param.getMultiChartName(), param.getMultiChartId());
        ewbMultiChartDetailMapper.delete(new QueryWrapper<EwbMultiChartDetail>().lambda().eq(EwbMultiChartDetail::getMultiChartId, param.getMultiChartId()));
        int sort = 1;
        for (String analysisId : analysisIds) {
            EwbMultiChartDetail ewbMultiChartDetail = EwbMultiChartDetail.builder()
                    .multiChartId(param.getMultiChartId())
                    .analysisId(analysisId)
                    .sortNo(sort++)
                    .build();
            ewbMultiChartDetailMapper.insert(ewbMultiChartDetail);
        }
    }

    /**
     * 多图画图信息
     *
     * @param userInfo
     * @param multiChartId
     * @return
     */
    public List<EwbProbabilityPlotVo> probabilityPlot(HeaderUserInfo userInfo, String multiChartId) {
        // 根据当前多图ID获取分析
        List<EwbMultiChartDetailVo> ewbMultiChartDetails = ewbMultiChartDetailMapper.listByMultiChartId(multiChartId);
        EwbMultiChart multiChart = ewbMultiChartMapper.selectById(multiChartId);
        List<EwbProbabilityPlotVo> resultList = new ArrayList<>();
        Double xaxisStart = Double.MAX_VALUE;
        Double xaxisEnd = Double.MIN_VALUE;
        double yaxisStart = Double.MAX_VALUE;
        double yaxisEnd = Double.MIN_VALUE;
        List<EwbAnalysisStartVo> params = new ArrayList<>(ewbMultiChartDetails.size());
        for (EwbMultiChartDetailVo ewbMultiChartDetail : ewbMultiChartDetails) {
            EwbAnalysisStartVo param = ewbAnalysisMapper.selectParamId(ewbMultiChartDetail.getAnalysisId());
            if (param.getXaxisStart() == null) {
                // 当前多图有未完成画图的分析，不能画图，在前端处理提示信息
                return null;
            }
            // 如果未打开百分比数据按钮则判断是否是无数据计算
            if (StringUtils.isBlank(ewbMultiChartDetail.getIsRankOrNodata()) || ewbMultiChartDetail.getIsRankOrNodata().equals(EwbConstants.FLAG_NO_CHECKED)) {
                Long count = ewbAnalysisDataDetailMapper.selectCount(new QueryWrapper<EwbAnalysisDataDetail>().lambda().eq(EwbAnalysisDataDetail::getDataId, param.getDataId()));
                ewbMultiChartDetail.setIsRankOrNodata(count == 0 ? EwbConstants.FLAG_CHECKED : EwbConstants.FLAG_NO_CHECKED);
            }
            params.add(param);
            xaxisStart = param.getXaxisStart() != null && param.getXaxisStart() < xaxisStart ? param.getXaxisStart() : xaxisStart;
            xaxisEnd = param.getXaxisEnd() != null && param.getXaxisEnd() > xaxisEnd ? param.getXaxisEnd() : xaxisEnd;
            yaxisStart = param.getYaxisStart() != null && param.getYaxisStart() < yaxisStart ? param.getYaxisStart() : yaxisStart;
            yaxisEnd = param.getYaxisEnd() != null && param.getYaxisEnd() > yaxisEnd ? param.getYaxisEnd() : yaxisEnd;
        }
        // 如果之前有设置过xy最大最小值，则使用原有设置
        if (multiChart.getXaxisStart() != null) {
            xaxisStart = multiChart.getXaxisStart();
            xaxisEnd = multiChart.getXaxisEnd();
        }
        for (int i = 0; i < ewbMultiChartDetails.size(); i++) {
            try {
                EwbMultiChartDetailVo ewbMultiChartDetail = ewbMultiChartDetails.get(i);
                EwbAnalysisStartVo param = params.get(i);
                param.setPlotType("01");
                param.setConfidenceLevel(Double.parseDouble(param.getConfidenceLevel()) / 100 + "");
                param.setBoundSided(EwbConfidenceTypeCodeEnum.双侧置信限.getIndex());
                param.setXaxisStart(xaxisStart);
                param.setXaxisEnd(xaxisEnd);
                EwbProbabilityPlotVo ewbProbabilityPlotVo = ewbPlotService.probabilityPlot(userInfo, param, true);
                ewbProbabilityPlotVo.setAnalysisName(param.getAnalysisName());
                ewbProbabilityPlotVo.setIsRankOrNodata(StringUtils.isNotBlank(ewbProbabilityPlotVo.getIsRankOrNodata()) && ewbProbabilityPlotVo.getIsRankOrNodata().equals(EwbConstants.FLAG_CHECKED) ? ewbProbabilityPlotVo.getIsRankOrNodata() : ewbMultiChartDetail.getIsRankOrNodata());
                ewbAnalysisDataMapper.updateIsRankOrNodata(param.getDataId(), ewbProbabilityPlotVo.getIsRankOrNodata());
                resultList.add(ewbProbabilityPlotVo);
            } catch (Exception e) {
                log.error(e.getMessage());
                throw new DescribeException(ReliaqubeParamDataCheckEnum.未知错误, userInfo.getLanguageKey());
            }

        }
        // 添加辅助线设置信息
        if (resultList.size() > 0) {
            resultList.get(0).setXaxisStart(multiChart.getXaxisStart() == null ? (xaxisStart == Double.MAX_VALUE ? null : xaxisStart) : multiChart.getXaxisStart());
            resultList.get(0).setXaxisEnd(multiChart.getXaxisEnd() == null ? (xaxisEnd == Double.MIN_VALUE ? null : xaxisEnd) : multiChart.getXaxisEnd());
            resultList.get(0).setYaxisStart(multiChart.getYaxisStart() == null ? (yaxisStart == Double.MAX_VALUE ? null : yaxisStart) : multiChart.getYaxisStart());
            resultList.get(0).setYaxisEnd(multiChart.getYaxisEnd() == null ? (yaxisEnd == Double.MIN_VALUE ? null : yaxisEnd) : multiChart.getYaxisEnd());
            resultList.get(0).setTime(multiChart.getTime());
            resultList.get(0).setUnReliability(multiChart.getUnReliability());
            resultList.get(0).setBetaMark(multiChart.getBetaMark());
            resultList.get(0).setEtaMark(multiChart.getEtaMark());
            resultList.get(0).setSigmaMark(multiChart.getSigmaMark());
            resultList.get(0).setMuMark(multiChart.getMuMark());
            resultList.get(0).setLambdaMark(multiChart.getLambdaMark());
            resultList.get(0).setTargetTime(multiChart.getTargetTime());
            resultList.get(0).setTargetReliability(multiChart.getTargetReliability());
        }
        return resultList;
    }

    public List<EwbProbabilityPlotVo> selectEchartDetail(HeaderUserInfo userInfo, String multiChartId, String plotType, Double xaxisStart, Double xaxisEnd, Double yaxisStart, Double yaxisEnd) {
        // 根据当前多图ID获取分析
        List<EwbMultiChartDetail> ewbMultiChartDetails = ewbMultiChartDetailMapper.selectList(new QueryWrapper<EwbMultiChartDetail>().lambda().eq(EwbMultiChartDetail::getMultiChartId, multiChartId));
        List<EwbProbabilityPlotVo> resultList = new ArrayList<>();
        if ((xaxisStart == null || xaxisStart == 0.0) && xaxisEnd != null) {
            xaxisStart = 0.00001;
        }
        this.handleMultiChart(userInfo, ewbMultiChartDetails, plotType, xaxisStart, xaxisEnd, resultList);
        if (xaxisStart == null) {
            xaxisStart = 0.0;
            xaxisEnd = 0.0;
            yaxisStart = 0.0;
            yaxisEnd = 0.0;
            // 未传入xy区间算出几个多图结果的x最大值再次画图
            for (EwbProbabilityPlotVo ewbProbabilityPlotVo : resultList) {
                JSONArray lineDataXy = ewbProbabilityPlotVo.getResponseResultPlot().getJSONArray("lineDataXY");
                for (Object o : lineDataXy) {
                    JSONArray dataXy = JSONArray.parseArray(o.toString());
                    xaxisEnd = dataXy.getDouble(0) > xaxisEnd ? dataXy.getDouble(0) : xaxisEnd;
                    xaxisEnd = Double.isInfinite(xaxisEnd) ? 0.0 : xaxisEnd;
                    yaxisEnd = dataXy.size() > 2 ? (dataXy.getDouble(3) > yaxisEnd ? dataXy.getDouble(3) : yaxisEnd) : (dataXy.getDouble(1) > yaxisEnd ? dataXy.getDouble(1) : yaxisEnd);
                    yaxisEnd = Double.isInfinite(yaxisEnd) ? 0.0 : yaxisEnd;
                }
            }
            xaxisEnd = "06".equals(plotType) ? xaxisEnd * 1.1 : xaxisEnd;
            resultList = new ArrayList<>();
            // 再次进行画图
            this.handleMultiChart(userInfo, ewbMultiChartDetails, plotType, 0.00001, xaxisEnd, resultList);
            if ("04".equals(plotType) || "05".equals(plotType) || "06".equals(plotType)) {
                resultList.get(0).setYaxisStart(yaxisStart);
                resultList.get(0).setYaxisEnd(yaxisEnd * 1.2);
            } else {
                resultList.get(0).setYaxisStart(0.0);
                resultList.get(0).setYaxisEnd(1.0);
            }
        } else {
            resultList.get(0).setYaxisStart(yaxisStart);
            resultList.get(0).setYaxisEnd(yaxisEnd);
        }
        // 返回xy区间
        resultList.get(0).setXaxisStart(xaxisStart);
        resultList.get(0).setXaxisEnd(xaxisEnd);
        return resultList;
    }

    /**
     * 处理多图除分布概率图外的其他画图
     *
     * @param userInfo
     * @param ewbMultiChartDetails
     * @param plotType
     * @param xaxisStart
     * @param xaxisEnd
     */
    private void handleMultiChart(HeaderUserInfo userInfo, List<EwbMultiChartDetail> ewbMultiChartDetails, String plotType, Double xaxisStart, Double xaxisEnd, List<EwbProbabilityPlotVo> resultList) {
        ewbMultiChartDetails.forEach(ewbMultiChartDetail -> {
            try {
                EwbAnalysisStartVo param = ewbAnalysisMapper.selectParamId(ewbMultiChartDetail.getAnalysisId());
                param.setPlotType(plotType);
                // 多图时传入的置信度是需要除100的
                if (StringUtils.isNotBlank(param.getBoundSided()) && Double.parseDouble(param.getConfidenceLevel()) > 1) {
                    param.setConfidenceLevel(Double.parseDouble(param.getConfidenceLevel()) / 100 + "");
                }
                param.setXaxisStart(xaxisStart);
                param.setXaxisEnd(xaxisEnd);
                EwbProbabilityPlotVo ewbProbabilityPlotVo = ewbPlotService.selectEchartDetail(userInfo, param);
                ewbProbabilityPlotVo.setAnalysisName(param.getAnalysisName());
                ewbProbabilityPlotVo.setIsRankOrNodata(param.getIsRankOrNodata());
                ewbProbabilityPlotVo.setDates(new String[]{DateTimeUtil.dateToStr(new Date(), "yyyy-MM-dd"), DateTimeUtil.dateToStr(new Date(), "HH:mm:ss")});
                resultList.add(ewbProbabilityPlotVo);
            } catch (Exception e) {
                log.error(e.getMessage());
            }
        });
    }

    /**
     * 更新画图的标记信息
     *
     * @param param
     * @return
     */
    public ServerResponse updateMarkInfo(EwbMultiChart param, Integer languageKey) {
        EwbMultiChart multiChart = ewbMultiChartMapper.selectById(param.getMultiChartId());
        // 判断前端传入的不可靠度是否合法
        if (param.getUnReliability() != null) {
            EwbChartController.calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.不可靠度, languageKey), String.valueOf(param.getUnReliability()), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        }
        // 判断前端传入的不可靠度是否合法
        if (param.getYaxisEnd() != null) {
            EwbChartController.calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.Y, languageKey), String.valueOf(param.getYaxisEnd()), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        }
        // 将前端传入的最新的设置更新至原有设置中
        multiChart.setXaxisStart(param.getXaxisStart());
        multiChart.setXaxisEnd(param.getXaxisEnd());
        multiChart.setYaxisStart(param.getYaxisStart());
        multiChart.setYaxisEnd(param.getYaxisEnd());
        multiChart.setTime(param.getTime());
        multiChart.setUnReliability(param.getUnReliability());
        multiChart.setBetaMark(param.getBetaMark());
        multiChart.setEtaMark(param.getEtaMark());
        multiChart.setSigmaMark(param.getSigmaMark());
        multiChart.setMuMark(param.getMuMark());
        multiChart.setLambdaMark(param.getLambdaMark());
        multiChart.setTargetTime(param.getTargetTime());
        multiChart.setTargetReliability(param.getTargetReliability());
        ewbMultiChartMapper.updateById(multiChart);
        return ServerResponse.createBySuccess();
    }
}
