package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.dto.EwbAnalysisNevadaPlotVo;
import com.nationrel.modules.weibull.dto.EwbNevadaCreateDto;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.EwbCalcParametersEnum;
import com.nationrel.modules.weibull.enums.EwbConfidenceMethodCodeEnum;
import com.nationrel.modules.weibull.enums.EwbConfidenceTypeCodeEnum;
import com.nationrel.modules.weibull.enums.EwbReliabilityGrowthPlotEnum;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.alta.AltaCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.general.domain.enums.reliaqube.ReliaqubeLanguageKeyEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.general.util.common.BasicDataUtil;
import com.nationrel.modules.weibull.general.util.common.DateTimeUtil;
import com.nationrel.modules.weibull.general.util.common.NumberUtil;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.vo.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * @author Hecs on 2020/9/22.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbNevadaService {

    private final RgaApiService rgaApiService;
    private final EwbAnalysisService analysisService;
    private final EwbNevadaDataMapper nevadaDataMapper;
    private final EwbNevadaKpiDataMapper nevadaKpiDataMapper;
    private final EwbNevadaSettingMapper nevadaSettingMapper;
    private final EwbNevadaKpiSettingMapper nevadaKpiSettingMapper;
    private final EwbNevadaResultDataMapper nevadaResultDataMapper;
    private final AllVariableConfig allVariableConfig;

    /**
     * 创建内华达数据
     *
     * @param param
     * @param userInfo
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse create(NevadaCreateVo param, HeaderUserInfo userInfo) {
        // 创建内华达设置数据
        EwbNevadaCreateDto createDto = createDto(param, userInfo);
        EwbNevadaSetting nevadaSetting = createDto.getNevadaSetting();
        EwbAnalysisReqVo analysis = EwbAnalysisReqVo.builder().dataId(BasicDataUtil.uuidGenerator()).settingId(BasicDataUtil.uuidGenerator()).build();
        nevadaSetting.setDataId(analysis.getDataId());
        nevadaSetting.setSettingId(analysis.getSettingId());
        nevadaSettingMapper.insert(nevadaSetting);
        if ("nevadaRI".equals(nevadaSetting.getNevadaType())) {
            nevadaDataMapper.insertBatch(createDto.getNevadaData());
        } else {
            nevadaKpiDataMapper.insertBatch(createDto.getKpiDatas());
            nevadaKpiSettingMapper.insertBatch(createDto.getKpiSettings());
        }
        return ServerResponse.createBySuccess(nevadaSetting);
    }


    public static EwbNevadaCreateDto createDto(NevadaCreateVo param, HeaderUserInfo userInfo) {
        String day = DateTimeUtil.dateToStr(new Date(), DateTimeUtil.STANDARD_FORMAT_NOH);
        // 创建内华达设置数据
        EwbNevadaSetting nevadaSetting = EwbNevadaSetting.builder().nevadaId(BasicDataUtil.uuidGenerator())
                .wbId(param.getWbId())
                .analysisName(param.getAnalysisName())
                .nevadaType(param.getNevadaType())
                .numberPeriod(6)
                .periodsIn("04")
                .startNumber(1)
                .claimsPer(1000)
                .startYear(Calendar.getInstance().get(Calendar.YEAR) + "")
                .startMounth(DateTimeUtil.dateToStr(new Date(), DateTimeUtil.STANDARD_FORMAT_NOD))
                .startDay(day)
                .futureStartNumber(2)
                .futureStartYear(Calendar.getInstance().get(Calendar.YEAR) + 1 + "")
                .futureStartDay(DateTimeUtil.dateToStr(DateTimeUtil.getBehindDay(new Date(), 1), DateTimeUtil.STANDARD_FORMAT_NOH))
                .futureStartMounth(DateTimeUtil.getBehindMonth(day, 1, DateTimeUtil.STANDARD_FORMAT_NOD, DateTimeUtil.STANDARD_FORMAT_NOD))
                .futureNumberPeriod(6)
                .forecast(6)
                .increment(1)
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .updateTime(new Date())
                .updateUserId(userInfo.getUserId())
                .versionNo(1)
                .build();
        // 创建内华达默认数据
        if ("nevadaRI".equals(param.getNevadaType())) {
            List<EwbNevadaData> nevadaDatas = new ArrayList<>(12);
            for (int i = 1; i <= 12; i++) {
                nevadaDatas.add(EwbNevadaData.builder().nevadaId(nevadaSetting.getNevadaId()).sort(i).datas(i + ",,,,,,,").build());
            }
            return EwbNevadaCreateDto.builder().nevadaSetting(nevadaSetting).nevadaData(nevadaDatas).build();
        } else {
            List<EwbNevadaKpiSetting> nevadaKpiSettings = new ArrayList<>(1);
            nevadaSetting.setFutureNumberPeriod(7);
            nevadaKpiSettings.add(EwbNevadaKpiSetting.builder().nevadaId(nevadaSetting.getNevadaId()).sort(1).itemName(userInfo.getLanguageKey().equals(ReliaqubeLanguageKeyEnum.中文.getIndex()) ? "数据1" : "MY1").startTime(1).endTime(7).build());
            List<EwbNevadaKpiData> nevadaKpiDatas = new ArrayList<>(7);
            for (int i = 1; i <= 7; i++) {
                nevadaKpiDatas.add(EwbNevadaKpiData.builder().nevadaId(nevadaSetting.getNevadaId()).sort(i).datas(i + ",").build());
            }
            return EwbNevadaCreateDto.builder().nevadaSetting(nevadaSetting).kpiDatas(nevadaKpiDatas).kpiSettings(nevadaKpiSettings).build();
        }
    }

    /**
     * 获取内华达列表信息
     *
     * @param wbId
     * @return
     */
    public ServerResponse<List<EwbNevadaSetting>> list(String wbId) {
        return ServerResponse.createBySuccess(nevadaSettingMapper.selectList(new QueryWrapper<EwbNevadaSetting>().lambda().eq(EwbNevadaSetting::getWbId, wbId)));
    }

    /**
     * 更新内华达数据以及设置
     *
     * @param nevada
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<NevadaDataVo> editNevadaData(NevadaSettingVo nevada, boolean init, HeaderUserInfo userInfo) {
        EwbNevadaSetting nevadaSetting = init ? nevadaSettingMapper.selectById(nevada.getNevadaId()) : new EwbNevadaSetting();
        if (init) {
            nevadaSetting = nevadaSettingMapper.selectById(nevada.getNevadaId());
        } else {
            BeanUtils.copyProperties(nevada, nevadaSetting);
        }
        // 查出原有的数据
        List<EwbNevadaData> nevadaDatas = nevadaDataMapper.selectList(new QueryWrapper<EwbNevadaData>().lambda().eq(EwbNevadaData::getNevadaId, nevadaSetting.getNevadaId()));
        // 生成新的数据
        List<String> nevadaDatasStr = new ArrayList<>();
        List<String> colHeader = new ArrayList<>();
        colHeader.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.时期, userInfo.getLanguageKey()));
        colHeader.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.数量, userInfo.getLanguageKey()));
        List<EwbNevadaData> nevadaDatasNew = new ArrayList<>();
        // 验证数据合法性
        validTotalBack(nevadaDatasNew, nevadaSetting, colHeader, nevadaDatasStr, nevadaDatas, init, userInfo.getLanguageKey());
        if (!init) {
            // 删除原有数据
            nevadaDataMapper.delete(new QueryWrapper<EwbNevadaData>().lambda().eq(EwbNevadaData::getNevadaId, nevadaSetting.getNevadaId()));
            // 插入新的数据
            nevadaDataMapper.insertBatch(nevadaDatasNew);
            // 更新设置数据
            nevadaSetting.setFutureStartDay(DateTimeUtil.getBehindDay(nevadaSetting.getStartDay(), nevadaSetting.getIncrement(), "yyyy/MM/dd"));
            nevadaSetting.setFutureStartMounth(DateTimeUtil.getBehindMonth(nevadaSetting.getStartMounth(), nevadaSetting.getIncrement(), DateTimeUtil.STANDARD_FORMAT_NOD, "yyyy/MM"));
            nevadaSetting.setFutureStartYear(Integer.parseInt(nevadaSetting.getStartYear()) + nevadaSetting.getIncrement() + "");
            nevadaSetting.setFutureStartNumber(nevadaSetting.getStartNumber() + nevadaSetting.getIncrement());
            nevadaSettingMapper.updateById(nevadaSetting);
        }
        return ServerResponse.createBySuccess(NevadaDataVo.builder().setting(nevadaSetting).nevadaDatas(nevadaDatasStr).colHeader(colHeader).build());
    }

    public void validTotalBack(List<EwbNevadaData> nevadaDatasNew, EwbNevadaSetting nevadaSetting, List<String> colHeader, List<String> nevadaDatasStr, List<EwbNevadaData> nevadaDatas, boolean init, Integer languageKey) {
        if (nevadaDatasNew.size() == 0) {
            for (int i = 0; i < nevadaSetting.getNumberPeriod() + nevadaSetting.getFutureNumberPeriod(); i++) {
                StringBuilder iData = new StringBuilder();
                String startFuture = EwbNonNevadaService.calculateDate(nevadaSetting.getPeriodsIn(), nevadaSetting.getStartMounth(), nevadaSetting.getStartDay(), nevadaSetting.getStartYear(), nevadaSetting.getStartNumber(), nevadaSetting.getIncrement(), i);
                for (int j = 0; j < nevadaSetting.getNumberPeriod() + 2; j++) {
                    String startSale = EwbNonNevadaService.calculateDate(nevadaSetting.getPeriodsIn(), nevadaSetting.getStartMounth(), nevadaSetting.getStartDay(), nevadaSetting.getStartYear(), nevadaSetting.getStartNumber(), nevadaSetting.getIncrement(), j + 1);
                    if (colHeader.size() < nevadaSetting.getNumberPeriod() + 2) {
                        colHeader.add(startSale);
                    }
                    if (init) {
                        continue;
                    }
                    if (j == 0) {
                        iData.append(startFuture);
                    } else if (i >= nevadaDatas.size() || j >= nevadaDatas.get(i).getDatas().split(",").length) {
                        iData.append(",");
                    } else {
                        String[] nevadaDataStr = nevadaDatas.get(i).getDatas().split(",");
                        iData.append(j >= nevadaDataStr.length ? "," : "," + (StringUtils.isBlank(nevadaDataStr[j]) ? "0" : nevadaDataStr[j]));
                    }

                }
                if (init) {
                    iData.append(nevadaDatas.get(i).getDatas());
                }
                nevadaDatasStr.add(iData.toString().replace(" ", ""));
                nevadaDatasNew.add(EwbNevadaData.builder().nevadaId(nevadaSetting.getNevadaId()).sort(i + 1).datas(iData.toString().replace(" ", "")).build());
            }
        }
        nevadaDatasNew.forEach(nevadaData -> {
            String[] datas = nevadaData.getDatas().split(",");
            if (datas.length > 1) {
                double sum = Double.parseDouble(datas[1]);
                double backSum = 0.0;
                for (int i = 2; i < datas.length; i++) {
                    backSum = StringUtils.isBlank(datas[i]) ? backSum : backSum + Double.parseDouble(datas[i]);
                }
                if (backSum > sum) {
                    throw new DescribeException(EwbCustomPromptEnum.返修数量的总和超过了销售数量, languageKey);
                }
            }
        });
    }


    /**
     * 保存内华达数据
     *
     * @param nevadaData
     * @return
     */
    public ServerResponse<NevadaResultVo> saveNevadaTable(HeaderUserInfo userInfo, NevadaSaveVo nevadaData) {
        EwbNevadaSetting nevadaSetting = nevadaSettingMapper.selectById(nevadaData.getNevadaId());
        // 处理前端传入的数据以满足后台的保存格式要求
        List<EwbNevadaData> nevadaDatasNew = IntStream.range(0, nevadaData.getNevadaDatas().size()).mapToObj(i -> EwbNevadaData.builder().nevadaId(nevadaData.getNevadaId()).sort(i + 1).datas(nevadaData.getNevadaDatas().get(i).toString().replace("[", "").replace("]", "")).build()).collect(Collectors.toList());
        // 校验数据的合法性并将其转为weibull标准数据
        List<EwbNevadaResultData> nevadaResultData = validNevadaNda(nevadaDatasNew, nevadaSetting.getNumberPeriod(), nevadaSettingMapper, userInfo.getLanguageKey());
        // 修改详细数据
        nevadaResultDataMapper.delete(new QueryWrapper<EwbNevadaResultData>().lambda().eq(EwbNevadaResultData::getNevadaId, nevadaData.getNevadaId()));
        nevadaResultDataMapper.insertBatch(nevadaResultData);
        // 删除原有数据
        nevadaDataMapper.delete(new QueryWrapper<EwbNevadaData>().lambda().eq(EwbNevadaData::getNevadaId, nevadaData.getNevadaId()));
        // 插入新的数据
        nevadaDataMapper.insertBatch(nevadaDatasNew);
        // 计算beta/eta/lambda
        validTotalBack(nevadaDatasNew, nevadaSetting, new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), false, userInfo.getLanguageKey());
        calcResult(nevadaSetting, nevadaResultData);
        nevadaSettingMapper.updateById(nevadaSetting);
        return ServerResponse.createBySuccess(NevadaResultVo.builder().resultDatas(nevadaResultDataMapper.selectList(new QueryWrapper<EwbNevadaResultData>().lambda().eq(EwbNevadaResultData::getNevadaId, nevadaData.getNevadaId()))).calcResults(calcResult(nevadaSetting.getClaimsPer(), nevadaSetting)).claimsPer(nevadaSetting.getClaimsPer()).periodsIn(nevadaSetting.getPeriodsIn()).build());
    }

    /**
     * 计算beta/eta/lambda
     *
     * @param nevadaSetting
     * @param nevadaResultData
     * @return
     */
    public EwbNevadaSetting calcResult(EwbNevadaSetting nevadaSetting, List<EwbNevadaResultData> nevadaResultData) {
        try {
            String results = rgaApiService.getResultsCallApi(rgaApiService.getRequestNevadaDataJson(nevadaResultData, nevadaSetting.getIncrement()).toString(), allVariableConfig.getReliaQubeUrl() + EwbConstants.API_RELIABILITYGROWTH_PARAMETERESTIMATE + "?analysisType=LeastSquares&betaBias=Unbiased", EwbConstants.RGA_API_CONTENTTYPE_1, null);
            JSONObject jsonObject = JSONObject.parseObject(results);
            nevadaSetting.setEta(jsonObject.getString("Eta"));
            nevadaSetting.setBeta(jsonObject.getString("Beta"));
            nevadaSetting.setLambda(jsonObject.getString("Lambda"));
            nevadaSetting.setCalcResult(results);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return nevadaSetting;
    }

    /**
     * 计算beta/eta/lambda
     *
     * @param nevadaSetting
     * @param nevadaResultData
     * @return
     */
    public EwbNevadaKpiSetting calcResult(EwbNevadaKpiSetting nevadaSetting, List<EwbNevadaResultData> nevadaResultData) {
        try {
            String results = rgaApiService.getResultsCallApi(rgaApiService.getRequestNevadaDataJson(nevadaResultData, 0).toString(), allVariableConfig.getReliaQubeUrl() + EwbConstants.API_RELIABILITYGROWTH_PARAMETERESTIMATE + "?analysisType=LeastSquares&betaBias=Biased", EwbConstants.RGA_API_CONTENTTYPE_1, null);
            JSONObject jsonObject = JSONObject.parseObject(results);
            nevadaSetting.setEta(WeibullNumberUtil.round(Double.parseDouble(jsonObject.getString("Eta")), 6).toString());
            nevadaSetting.setBeta(WeibullNumberUtil.round(Double.parseDouble(jsonObject.getString("Beta")), 6).toString());
            nevadaSetting.setLambda(WeibullNumberUtil.round(Double.parseDouble(jsonObject.getString("Lambda")), 6).toString());
            nevadaSetting.setCalcResult(results);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return nevadaSetting;
    }

    /**
     * 校验数据的合法性并将其转为weibull标准数据
     *
     * @param nevadaDatas
     * @return
     */
    public static List<EwbNevadaResultData> validNevadaNda(List<EwbNevadaData> nevadaDatas, Integer numberPeriod, EwbNevadaSettingMapper nevadaSettingMapper, Integer languageKey) {
        List<EwbNevadaResultData> result = new ArrayList<>();
        // 保存累计返修数据
        List<Integer> sumFailures = new ArrayList<>(numberPeriod);
        // 保存累计销售数据
        List<Integer> sumSales = new ArrayList<>(numberPeriod);
        try {
            // 初始化数据
            for (int dataSize = 0; dataSize < numberPeriod; dataSize++) {
                sumFailures.add(dataSize, 0);
                sumSales.add(dataSize, 0);
            }
            int total = 0;
            // 循环生成分析结果数据
            for (int i = 0; i < numberPeriod; i++) {
                String[] saleReturns = nevadaDatas.get(i).getDatas().replace(" ", "").split(",");
                total += Integer.parseInt(saleReturns[1]);
                sumSales.set(i, total);
                for (int j = 0; j < numberPeriod; j++) {
                    if (j + i + 2 >= saleReturns.length) {
                        break;
                    }
                    sumFailures.set(j, sumFailures.get(j) + Integer.parseInt(saleReturns[j + i + 2]));
                }
            }
            // 循环判断销售数据是否已经填入
            for (EwbNevadaData nevadaData : nevadaDatas) {
                nevadaData.getDatas().replace(" ", "").split(",");
                Integer.parseInt(nevadaData.getDatas().replace(" ", "").split(",")[1]);
            }
        } catch (NumberFormatException | ArrayIndexOutOfBoundsException e) {
            throw new DescribeException(EwbCustomPromptEnum.请填写合法且完整的数据, languageKey);
        }
        EwbNevadaSetting nevadaSetting = nevadaSettingMapper.selectById(nevadaDatas.get(0).getNevadaId());
        for (int i = 0; i < sumFailures.size(); i++) {
            result.add(EwbNevadaResultData.builder().sort((i + 1) * nevadaSetting.getIncrement()).data(NumberUtil.parseDecimal((double) sumFailures.get(i) / sumSales.get(sumSales.size() - i - 1) * 1000)).nevadaId(nevadaSetting.getNevadaId()).build());
        }
        return result;
    }

    /**
     * 删除内华达数据
     *
     * @param nevadaId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<NevadaDataVo> delNevadaData(String nevadaId) {
        nevadaSettingMapper.deleteById(nevadaId);
        nevadaDataMapper.delete(new QueryWrapper<EwbNevadaData>().lambda().eq(EwbNevadaData::getNevadaId, nevadaId));
        analysisService.delAnalysis(nevadaId);
        return ServerResponse.createBySuccess();
    }

    /**
     * 计算周期
     *
     * @param nevadaSetting
     * @param i
     * @return
     */
    private String calculateDate(EwbNevadaSetting nevadaSetting, int i) {
        // 计算周期数据（年、月、日、数值）
        switch (nevadaSetting.getPeriodsIn()) {
            case "01":
                return DateTimeUtil.getBehindMonth(nevadaSetting.getStartMounth(), nevadaSetting.getIncrement() * i, DateTimeUtil.STANDARD_FORMAT_NOD, "yyyy/MM");
            case "02":
                return DateTimeUtil.getBehindDay(nevadaSetting.getStartDay(), nevadaSetting.getIncrement() * i, "yyyy/MM/dd");
            case "03":
                return Integer.parseInt(nevadaSetting.getStartYear()) + nevadaSetting.getIncrement() * i + "";
            default:
                return nevadaSetting.getStartNumber() + nevadaSetting.getIncrement() * i + "";
        }
    }

    /**
     * 复制：内华达数据
     *
     * @param userInfo 用户信息
     * @param nevadaId 内华达ID
     * @return void
     */
    @Transactional(rollbackFor = Exception.class)
    public void copyNevada(HeaderUserInfo userInfo, String nevadaId) {
        // 复制：内华达分析设定表 Start
        EwbNevadaSetting nevadaSetting = nevadaSettingMapper.selectById(nevadaId);
        nevadaSetting.setNevadaId(BasicDataUtil.uuidGenerator());
        EwbAnalysisReqVo analysis = analysisService.createAnalysis(userInfo, nevadaSetting.getWbId(),
                nevadaSetting.getNevadaId(), "", null, null, null);
        nevadaSetting.setAnalysisName(nevadaSetting.getAnalysisName() + "_copy");
        nevadaSetting.setDataId(analysis.getDataId());
        nevadaSetting.setSettingId(analysis.getSettingId());
        nevadaSetting.setInsertTime(new Date());
        nevadaSetting.setUpdateTime(new Date());
        nevadaSetting.setVersionNo(EwbConstants.NUM_ONE); // 默认版本号1
        nevadaSettingMapper.insert(nevadaSetting);
        // 复制：内华达分析设定表 End
        // 构建：内华达分析数据
        List<List<Object>> nevadaData = this.buildNevadaData(nevadaId);
        NevadaSaveVo nevadaSaveVo = NevadaSaveVo.builder()
                .nevadaId(nevadaSetting.getNevadaId())
                .nevadaDatas(nevadaData)
                .build();
        // 保存：内华达数据
        this.saveNevadaTable(userInfo, nevadaSaveVo);
    }

    /**
     * 构建：内华达分析数据
     *
     * @param nevadaId 内华达ID
     * @return List<List < Object>>
     */
    private List<List<Object>> buildNevadaData(String nevadaId) {
        // 取得：内华达分析数据
        List<EwbNevadaData> ewbNonNevadaDataList = nevadaDataMapper.selectList(new QueryWrapper<EwbNevadaData>()
                .lambda().eq(EwbNevadaData::getNevadaId, nevadaId));
        // 为空，直接return
        if (CollectionUtils.isEmpty(ewbNonNevadaDataList)) {
            return new ArrayList<>();
        }
        // 组装数据
        List<List<Object>> nevadaData = new ArrayList<>(ewbNonNevadaDataList.size());
        ewbNonNevadaDataList.forEach(item -> {
            if (StringUtils.isNoneBlank(item.getDatas())) {
                nevadaData.add(Arrays.asList(item.getDatas().split(EwbConstants.COMMA)));
            }
        });
        // 返回结果
        return nevadaData;
    }

    public List<EwbCalcResultKeyValueVo> calcResult(Integer claimsPer, EwbNevadaSetting setting) {
        if (claimsPer != 0 && !claimsPer.equals(setting.getClaimsPer())) {
            // 当claimsPer发生变化，则修改数据并重新计算beta/eta/lambda
            List<EwbNevadaResultData> nevadaResultDatas = nevadaResultDataMapper.selectList(new QueryWrapper<EwbNevadaResultData>().lambda().eq(EwbNevadaResultData::getNevadaId, setting.getNevadaId()).orderByAsc(EwbNevadaResultData::getData));
            for (EwbNevadaResultData nevadaResultData : nevadaResultDatas) {
                nevadaResultData.setData(nevadaResultData.getData() * claimsPer / setting.getClaimsPer());
            }
            nevadaResultDataMapper.delete(new QueryWrapper<EwbNevadaResultData>().lambda().eq(EwbNevadaResultData::getNevadaId, setting.getNevadaId()));
            nevadaResultDataMapper.insertBatch(nevadaResultDatas);
            // 计算beta/eta/lambda
            calcResult(setting, nevadaResultDatas);
            setting.setClaimsPer(claimsPer);
            nevadaSettingMapper.updateById(setting);
        }
        // 生成前端需要的计算结果信息
        List<EwbCalcResultKeyValueVo> calcResults = new ArrayList<>();
        if (StringUtils.isNotBlank(setting.getBeta())) {
            calcResults.add(new EwbCalcResultKeyValueVo("β", WeibullNumberUtil.round(Double.valueOf(setting.getBeta()), EwbConstants.NUM_DIGIT_6)));
        }
        if (StringUtils.isNotBlank(setting.getEta())) {
            calcResults.add(new EwbCalcResultKeyValueVo("η", WeibullNumberUtil.round(Double.valueOf(setting.getEta()), EwbConstants.NUM_DIGIT_6)));
        }
        if (StringUtils.isNotBlank(setting.getLambda())) {
            calcResults.add(new EwbCalcResultKeyValueVo("λ", WeibullNumberUtil.round(Double.valueOf(setting.getLambda()), EwbConstants.NUM_DIGIT_6)));
        }
        return calcResults;
    }

    /**
     * 内华达table数据
     *
     * @param nevadaId
     * @return
     */
    public ServerResponse getNevadaTableData(String nevadaId, Integer claimsPer) {
        EwbNevadaSetting setting = nevadaSettingMapper.selectById(nevadaId);
        return ServerResponse.createBySuccess(NevadaResultVo.builder().calcResults(calcResult(claimsPer, setting)).claimsPer(claimsPer == 0 ? setting.getClaimsPer() : claimsPer).periodsIn(setting.getPeriodsIn()).resultDatas(nevadaResultDataMapper.selectList(new QueryWrapper<EwbNevadaResultData>().lambda().eq(EwbNevadaResultData::getNevadaId, nevadaId))).build());
    }

    /**
     * 内华达table数据
     *
     * @param nevadaId
     * @return
     */
    public ServerResponse getKpiNevadaTableData(String nevadaId) {
        EwbNevadaSetting setting = nevadaSettingMapper.selectById(nevadaId);
        List<EwbNevadaKpiData> kpiDatas = nevadaKpiDataMapper.selectList(new QueryWrapper<EwbNevadaKpiData>().lambda().eq(EwbNevadaKpiData::getNevadaId, nevadaId));
        List<String> nevadaDatasStr = new ArrayList<>();
        kpiDatas.forEach(kpiData -> nevadaDatasStr.add(kpiData.getDatas().replace(" ", "")));
        List<EwbNevadaKpiSetting> kpiSettings = nevadaKpiSettingMapper.selectList(new QueryWrapper<EwbNevadaKpiSetting>().lambda().eq(EwbNevadaKpiSetting::getNevadaId, nevadaId));
        return ServerResponse.createBySuccess(NevadaKpiResultVo.builder().setting(setting).nevadaDatasStr(nevadaDatasStr).kpiSettings(kpiSettings).dates(new String[]{DateTimeUtil.dateToStr(new Date(), DateTimeUtil.STANDARD_FORMAT_NOH), DateTimeUtil.dateToStr(new Date(), "HH:mm:ss")}).build());
    }

    /**
     * 生成内华达图的echart信息
     *
     * @param user
     * @param param
     * @return
     */
    public EwbProbabilityPlotVo probabilityPlot(HeaderUserInfo user, EwbAnalysisNevadaPlotVo param) {
        EwbNevadaSetting setting = nevadaSettingMapper.selectById(param.getAnalysisId());
        List<EwbNevadaResultData> resultData = nevadaResultDataMapper.selectList(new QueryWrapper<EwbNevadaResultData>().lambda().eq(EwbNevadaResultData::getNevadaId, param.getAnalysisId()).orderByAsc(EwbNevadaResultData::getData));
        JSONObject data = new JSONObject();
        JSONArray datas = new JSONArray();
        datas.add(rgaApiService.getRequestNevadaDataJson(resultData, setting.getIncrement()));
        String serverSystem = allVariableConfig.getServerSystem();
        data.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataList), datas);
        data.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data), data);
        data.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.totalNumofSystems), 0);
        data.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.totalNumFailures), 0);
        data.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.terminationType), "FailureTerminated");
        JSONObject requestNevadaPlot = rgaApiService.getApiPlotArgs(param.getBoundSided(), param.getMethod(), Double.parseDouble(param.getConfidenceLevel()) * 100 + "", setting.getCalcResult(), data);
        String results = null;
        try {
            // 添加是否自动计算x区间的设置
            boolean autoscaleX = param.getXaxisStart() == null;
            double xmin = autoscaleX ? 0 : param.getXaxisStart();
            double xmax = autoscaleX ? Math.ceil(resultData.get(resultData.size() - 1).getData() + 1) : param.getXaxisEnd();
            // uri待修改为最新
            results = rgaApiService.getResultsCallApi(requestNevadaPlot.toString(), allVariableConfig.getReliaQubeUrl() + EwbConstants.API_RELIABILITYGROWTH_PLOT + "/" + EwbReliabilityGrowthPlotEnum.getUrl(param.getPlotType()) + "?autoscaleX=" + autoscaleX + "&xmin=" + xmin + "&xmax=" + xmax + "&NumofPoints=100&DataType=Grouped", EwbConstants.RGA_API_CONTENTTYPE_1, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
        EwbProbabilityPlotVo output = new EwbProbabilityPlotVo();
        BeanUtils.copyProperties(param, output);
        List<EwbCalcResultKeyValueVo> calcResults = new ArrayList<>();
        calcResults.add(new EwbCalcResultKeyValueVo("β", WeibullNumberUtil.round(Double.valueOf(setting.getBeta()), EwbConstants.NUM_DIGIT_6)));
        calcResults.add(new EwbCalcResultKeyValueVo("η", WeibullNumberUtil.round(Double.valueOf(setting.getEta()), EwbConstants.NUM_DIGIT_6)));
        calcResults.add(new EwbCalcResultKeyValueVo("λ", WeibullNumberUtil.round(Double.valueOf(setting.getLambda()), EwbConstants.NUM_DIGIT_6)));
        calcResults.add(new EwbCalcResultKeyValueVo(AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.置信类型, user.getLanguageKey()), EwbConfidenceTypeCodeEnum.getName(param.getBoundSided())));
        calcResults.add(new EwbCalcResultKeyValueVo(AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.置信度, user.getLanguageKey()), Double.parseDouble(param.getConfidenceLevel()) * 100 + ""));
        calcResults.add(new EwbCalcResultKeyValueVo(AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.置信法, user.getLanguageKey()), EwbConfidenceMethodCodeEnum.getByLaguageKey(param.getMethod(), user.getLanguageKey())));
        List<EwbCalcResultKeyValueVo> settings = new ArrayList<>();
        settings.add(new EwbCalcResultKeyValueVo("key", EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.每, user.getLanguageKey())));
        settings.add(new EwbCalcResultKeyValueVo("key", setting.getClaimsPer() + EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.台索赔数, user.getLanguageKey())));
        output.setBelieveCode(param.getBoundSided());
        output.setResult(calcResults);
        output.setSetting(settings);
        output.setDates(new String[]{DateTimeUtil.dateToStr(new Date(), DateTimeUtil.STANDARD_FORMAT_NOH), DateTimeUtil.dateToStr(new Date(), "HH:mm:ss")});
        output.setResponseResultPlot(JSONObject.parseObject(results));

        // 如果是第一次进入页面没有xy区间信息则需要计算xy区间信息
        if (param.getXaxisStart() == null) {
            output.setXaxisStart(0.0);
            output.setXaxisEnd(0.0);
            output.setYaxisStart(0.0);
            output.setYaxisEnd(0.0);
            JSONArray lineDataXy = output.getResponseResultPlot().getJSONArray(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.lineDataXY));
            for (Object o : lineDataXy) {
                JSONArray dataXy = JSONArray.parseArray(o.toString());
                output.setXaxisEnd(dataXy.getDouble(0) > output.getXaxisEnd() ? NumberUtil.round(dataXy.getDouble(0), 2) : output.getXaxisEnd());
                output.setYaxisEnd(dataXy.getDouble(1) > output.getYaxisEnd() ? NumberUtil.round(dataXy.getDouble(1), 2) : output.getYaxisEnd());
            }
        }
        return output;
    }

    /**
     * 保存kpi内华达设置数据
     *
     * @param resultVo
     * @return
     */
    public ServerResponse saveKpiNevadaSetting(NevadaKpiResultVo resultVo) {
        // 更新主信息设置
        EwbNevadaSetting setting = nevadaSettingMapper.selectById(resultVo.getSetting().getNevadaId());
        setting.setFutureNumberPeriod(resultVo.getSetting().getFutureNumberPeriod());
        setting.setPeriodsIn(resultVo.getSetting().getPeriodsIn());
        nevadaSettingMapper.updateById(setting);
        // 删除原有的kpiSetting数据
        nevadaKpiSettingMapper.delete(new QueryWrapper<EwbNevadaKpiSetting>().lambda().eq(EwbNevadaKpiSetting::getNevadaId, resultVo.getSetting().getNevadaId()));
        // 给传入的数据编sort
        for (int i = 0; i < resultVo.getKpiSettings().size(); i++) {
            resultVo.getKpiSettings().get(i).setSort(i);
            resultVo.getKpiSettings().get(i).setNevadaId(setting.getNevadaId());
        }
        // 添加新的kpiSetting数据
        nevadaKpiSettingMapper.insertBatch(resultVo.getKpiSettings());
        return ServerResponse.createBySuccess();
    }

    /**
     * 保存kpi内华达数据
     *
     * @param nevadaData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse saveKpiNevadaTable(HeaderUserInfo userInfo, NevadaKpiResultVo nevadaData) {
        nevadaData.setKpiSettings(nevadaData.getPreKpiSettings());
        saveKpiNevadaSetting(nevadaData);
        List<EwbNevadaKpiSetting> kpiSettings = nevadaData.getPreKpiSettings();
        // 先循环一遍数据，读出最大的填写行数
        int totalMis = 0;
        for (int i = 0; i < nevadaData.getNevadaDatas().size(); i++) {
            if (nevadaData.getNevadaDatas().get(i).get(0) != null && StringUtils.isNotBlank(nevadaData.getNevadaDatas().get(i).get(0).toString())) {
                totalMis = i;
            }
            System.out.println(nevadaData.getNevadaDatas().get(i).get(0));
            if (nevadaData.getNevadaDatas().get(i).get(0) != null && StringUtils.isNotBlank(nevadaData.getNevadaDatas().get(i).get(0).toString()) && nevadaData.getNevadaDatas().get(i).size() < nevadaData.getKpiSettings().size() + 1) {
                for (int j = nevadaData.getNevadaDatas().get(i).size(); j < nevadaData.getKpiSettings().size() + 1; j++) {
                    nevadaData.getNevadaDatas().get(i).add(BasicDataUtil.EMPTY_STR);
                }
            }
        }
        validKpiTableData(totalMis, nevadaData, userInfo.getLanguageKey());
        // 根据现有的配置，去计算beta。eta等值
        for (int i = 0; i < kpiSettings.size(); i++) {
            List<EwbNevadaResultData> nevadaResultDatas = new ArrayList<>();
            if (kpiSettings.get(i).getStartTime() != null && kpiSettings.get(i).getEndTime() != null) {
                for (int j = kpiSettings.get(i).getStartTime() - 1; j < kpiSettings.get(i).getEndTime(); j++) {
                    if (nevadaData.getNevadaDatas().get(j).get(i + 1) != null && StringUtils.isNotBlank(nevadaData.getNevadaDatas().get(j).get(i + 1).toString())) {
                        nevadaResultDatas.add(EwbNevadaResultData.builder().sort(Integer.parseInt(nevadaData.getNevadaDatas().get(j).get(0).toString())).data(Double.parseDouble(nevadaData.getNevadaDatas().get(j).get(i + 1).toString())).build());
                    }
                }
            } else {
                for (int j = 0; j <= totalMis; j++) {
                    if (nevadaData.getNevadaDatas().get(j).get(i + 1) != null && StringUtils.isNotBlank(nevadaData.getNevadaDatas().get(j).get(i + 1).toString())) {
                        nevadaResultDatas.add(EwbNevadaResultData.builder().sort(Integer.parseInt(nevadaData.getNevadaDatas().get(j).get(0).toString())).data(Double.parseDouble(nevadaData.getNevadaDatas().get(j).get(i + 1).toString())).build());
                    } else {
                        break;
                    }
                }
            }
            calcResult(kpiSettings.get(i), nevadaResultDatas);
            nevadaKpiSettingMapper.updateCalcResult(kpiSettings.get(i));
        }
        // 删除原有的数据
        nevadaKpiDataMapper.delete(new QueryWrapper<EwbNevadaKpiData>().lambda().eq(EwbNevadaKpiData::getNevadaId, nevadaData.getSetting().getNevadaId()));
        // 添加新的数据
        List<EwbNevadaKpiData> kpiDatas = new ArrayList<>(totalMis);
        for (int j = 0; j <= totalMis; j++) {
            kpiDatas.add(EwbNevadaKpiData.builder().nevadaId(nevadaData.getSetting().getNevadaId()).sort(j).datas(nevadaData.getNevadaDatas().get(j).toString().replace("[", "").replace("]", "").replace("null", "")).build());
        }
        nevadaKpiDataMapper.insertBatch(kpiDatas);
        return ServerResponse.createBySuccess(kpiSettings);
    }

    /**
     * 判断当前传入的数据是否合法
     *
     * @param totalMis
     * @param nevadaData
     */
    private void validKpiTableData(Integer totalMis, NevadaKpiResultVo nevadaData, Integer languageKey) {
        List<Integer> itemIndex = IntStream.range(0, nevadaData.getKpiSettings().size()).mapToObj(j -> 0).collect(Collectors.toCollection(() -> new ArrayList<>(nevadaData.getKpiSettings().size())));
        for (int i = 0; i < nevadaData.getKpiSettings().size(); i++) {
            for (int j = 0; j < totalMis; j++) {
                if (nevadaData.getNevadaDatas().get(j).get(i + 1) != null && StringUtils.isNotBlank(nevadaData.getNevadaDatas().get(j).get(i + 1).toString())) {
                    itemIndex.set(i, j);
                }
            }
        }
        for (int i = 0; i < nevadaData.getKpiSettings().size(); i++) {
            for (int j = 0; j < totalMis; j++) {
                if (j < itemIndex.get(i) && nevadaData.getNevadaDatas().get(j).get(i + 1) != null && StringUtils.isBlank(nevadaData.getNevadaDatas().get(j).get(i + 1).toString())) {
                    throw new DescribeException(EwbCustomPromptEnum.请将kpi内华达数据填写完整, languageKey);
                }
            }
        }
    }

}

