package com.nationrel.modules.weibull.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.general.domain.enums.common.WhetherFlagEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.general.util.common.BasicDataUtil;
import com.nationrel.modules.weibull.general.util.common.DateTimeUtil;
import com.nationrel.modules.weibull.general.util.common.ObjectActionUtil;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.bo.EwbCalcParamBo;
import com.nationrel.modules.weibull.dto.EwbNevadaCreateDto;
import com.nationrel.modules.weibull.dto.EwbNevadaSettingDto;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.*;
import com.nationrel.modules.weibull.vo.*;
import com.nationrel.modules.weibull.util.NevadaExportExcelUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import jakarta.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * @author Hecs on 2020/9/22.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbNonNevadaService {

    private final EwbCalcService calcService;
    private final EwbAnalysisService analysisService;
    private final EwbNevadaDataMapper nevadaDataMapper;
    private final EwbNonNevadaDataMapper nonNevadaDataMapper;
    private final EwbNonNevadaSettingMapper nonNevadaSettingMapper;
    private final EwbNevadaSettingMapper nevadaSettingMapper;
    private final EwbAnalysisResultMapper analysisResultMapper;
    private final EwbAnalysisDataDetailService analysisDataDetailService;

    /**
     * 创建内华达数据
     *
     * @param param
     * @param userInfo
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse create(NevadaCreateVo param, HeaderUserInfo userInfo) {
        EwbNevadaCreateDto createDto = createDto(param, userInfo);
        EwbNonNevadaSetting nevadaSetting = createDto.getNonNevadaSetting();
        EwbAnalysisReqVo analysis = analysisService.createAnalysis(userInfo, param.getWbId(), nevadaSetting.getNevadaId(), "", null, null, null);
        nevadaSetting.setDataId(analysis.getDataId());
        nevadaSetting.setSettingId(analysis.getSettingId());
        nevadaSetting.setForecast(6);
        nevadaSetting.setForecastType(EwbNevadaForecastTypeEnum.未设置.getIndex());
        nevadaSetting.setConfidenceBound(EwbConfidenceBoundEnum.双侧.getIndex());
        nevadaSetting.setConfidenceLevel(EwbConfidenceBoundEnum.双侧.getConfidenceLevel());
        nevadaSetting.setBoundSided(EwbConfidenceTypeCodeEnum.无.getIndex());
        nevadaSetting.setSettingId(analysis.getSettingId());
        nevadaSetting.setIsComplete(WhetherFlagEnum.NO.getIndex());
        nonNevadaSettingMapper.insert(nevadaSetting);
        nonNevadaDataMapper.insertBatch(createDto.getNonNevadaData());
        return ServerResponse.createBySuccess(nevadaSetting);
    }

    public static EwbNevadaCreateDto createDto(NevadaCreateVo param, HeaderUserInfo userInfo) {
        String day = DateTimeUtil.dateToStr(new Date(), "yyyy-MM-dd");
        // 创建内华达设置数据
        EwbNonNevadaSetting nevadaSetting = EwbNonNevadaSetting.builder().nevadaId(BasicDataUtil.uuidGenerator())
                .wbId(param.getWbId())
                .analysisName(param.getAnalysisName())
                .numberPeriod(6)
                .periodsIn("04")
                .startNumber(1)
                .startYear(Calendar.getInstance().get(Calendar.YEAR) + BasicDataUtil.EMPTY_STR)
                .startMounth(DateTimeUtil.dateToStr(new Date(), DateTimeUtil.STANDARD_FORMAT_NOD))
                .startDay(day)
                .futureStartNumber(2)
                .futureStartYear(Calendar.getInstance().get(Calendar.YEAR) + 1 + BasicDataUtil.EMPTY_STR)
                .futureStartDay(DateTimeUtil.dateToStr(DateTimeUtil.getBehindDay(new Date(), 1), DateTimeUtil.STANDARD_FORMAT_NOH))
                .futureStartMounth(DateTimeUtil.getBehindMonth(day, 1, DateTimeUtil.STANDARD_FORMAT_NOD, DateTimeUtil.STANDARD_FORMAT_NOD))
                .futureNumberPeriod(6)
                .forecast(6)
                .increment(1)
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .updateTime(new Date())
                .updateUserId(userInfo.getUserId())
                .versionNo(1)
                .build();
        // 创建内华达默认数据
        List<EwbNonNevadaData> nevadaDatas = new ArrayList<>(12);
        for (int i = 1; i <= 12; i++) {
            nevadaDatas.add(EwbNonNevadaData.builder().nevadaId(nevadaSetting.getNevadaId()).sort(i).datas(i + ",,,,,,,").build());
        }
        return EwbNevadaCreateDto.builder().nonNevadaSetting(nevadaSetting).nonNevadaData(nevadaDatas).build();
    }

    /**
     * 获取内华达列表信息
     *
     * @param wbId
     * @return
     */
    public ServerResponse<NevadaListVo> list(String wbId) {
        return ServerResponse.createBySuccess(NevadaListVo.builder().nonNevadaSettings(nonNevadaSettingMapper.selectList(new QueryWrapper<EwbNonNevadaSetting>().lambda().eq(EwbNonNevadaSetting::getWbId, wbId))).nevadaSettings(nevadaSettingMapper.selectList(new QueryWrapper<EwbNevadaSetting>().lambda().eq(EwbNevadaSetting::getWbId, wbId))).build());
    }

    /**
     * 更新内华达图名称
     *
     * @param nevadaSetting
     * @return
     */
    public ServerResponse editNevada(EwbNevadaSettingDto nevadaSetting) {
        EwbNonNevadaSetting preNevadaSetting = nonNevadaSettingMapper.selectById(nevadaSetting.getNevadaId());
        if (ObjectActionUtil.isEmpty(preNevadaSetting)) {
            EwbNevadaSetting preSetting = nevadaSettingMapper.selectById(nevadaSetting.getNevadaId());
            preSetting.setAnalysisName(nevadaSetting.getAnalysisName());
            nevadaSettingMapper.updateById(preSetting);
        } else {
            preNevadaSetting.setAnalysisName(nevadaSetting.getAnalysisName());
            nonNevadaSettingMapper.updateById(preNevadaSetting);
        }
        return ServerResponse.createBySuccess();
    }

    /**
     * 更新内华达数据以及设置
     *
     * @param nevada
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<NevadaDataVo> editNevadaData(NevadaSettingVo nevada, boolean init, HeaderUserInfo userInfo) {
        EwbNonNevadaSetting nevadaSetting = init ? nonNevadaSettingMapper.selectById(nevada.getNevadaId()) : new EwbNonNevadaSetting();
        if (init) {
            nevadaSetting = nonNevadaSettingMapper.selectById(nevada.getNevadaId());
        } else {
            BeanUtils.copyProperties(nevada, nevadaSetting);
        }
        // 查出原有的数据
        List<EwbNonNevadaData> nevadaDatas = nonNevadaDataMapper.selectList(new QueryWrapper<EwbNonNevadaData>().lambda().eq(EwbNonNevadaData::getNevadaId, nevadaSetting.getNevadaId()));
        // 生成新的数据
        List<String> nevadaDatasStr = new ArrayList<>();
        List<String> colHeader = new ArrayList<>();
        colHeader.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.时期, userInfo.getLanguageKey()));
        colHeader.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.数量, userInfo.getLanguageKey()));
        List<EwbNonNevadaData> nevadaDatasNew = new ArrayList<>();
        // 验证数据合法性
        validTotalBack(nevadaDatasNew, nevadaSetting, colHeader, nevadaDatasStr, nevadaDatas, init, userInfo.getLanguageKey());
        if (!init) {
            // 删除原有数据
            nonNevadaDataMapper.delete(new QueryWrapper<EwbNonNevadaData>().lambda().eq(EwbNonNevadaData::getNevadaId, nevadaSetting.getNevadaId()));
            // 插入新的数据
            nonNevadaDataMapper.insertBatch(nevadaDatasNew);
            // 更新设置数据
            nevadaSetting.setFutureStartDay(DateTimeUtil.getBehindDay(nevadaSetting.getStartDay(), nevadaSetting.getIncrement(), "yyyy/MM/dd"));
            nevadaSetting.setFutureStartMounth(DateTimeUtil.getBehindMonth(nevadaSetting.getStartMounth(), nevadaSetting.getIncrement(), DateTimeUtil.STANDARD_FORMAT_NOD, "yyyy/MM"));
            nevadaSetting.setFutureStartYear(Integer.parseInt(nevadaSetting.getStartYear()) + nevadaSetting.getIncrement() + "");
            nevadaSetting.setFutureStartNumber(nevadaSetting.getStartNumber() + nevadaSetting.getIncrement());
            nevadaSetting.setIsComplete(WhetherFlagEnum.NO.getIndex());
            nonNevadaSettingMapper.updateById(nevadaSetting);
        }
        return ServerResponse.createBySuccess(NevadaDataVo.builder().nevadaSetting(nevadaSetting).nevadaDatas(nevadaDatasStr).colHeader(colHeader).build());
    }

    public void validTotalBack(List<EwbNonNevadaData> nevadaDatasNew, EwbNonNevadaSetting nevadaSetting, List<String> colHeader, List<String> nevadaDatasStr, List<EwbNonNevadaData> nevadaDatas, boolean init, Integer languageKey) {
        if (nevadaDatasNew.size() == 0) {
            for (int i = 0; i < nevadaSetting.getNumberPeriod() + nevadaSetting.getFutureNumberPeriod(); i++) {
                StringBuilder iData = new StringBuilder();
                String startFuture = calculateDate(nevadaSetting.getPeriodsIn(), nevadaSetting.getStartMounth(), nevadaSetting.getStartDay(), nevadaSetting.getStartYear(), nevadaSetting.getStartNumber(), nevadaSetting.getIncrement(), i);
                for (int j = 0; j < nevadaSetting.getNumberPeriod() + 2; j++) {
                    String startSale = calculateDate(nevadaSetting.getPeriodsIn(), nevadaSetting.getStartMounth(), nevadaSetting.getStartDay(), nevadaSetting.getStartYear(), nevadaSetting.getStartNumber(), nevadaSetting.getIncrement(), j + 1);
                    if (colHeader.size() < nevadaSetting.getNumberPeriod() + 2) {
                        colHeader.add(startSale);
                    }
                    if (init) {
                        continue;
                    }
                    if (j == 0) {
                        iData.append(startFuture);
                    } else if (i >= nevadaDatas.size() || j >= nevadaDatas.get(i).getDatas().split(",").length) {
                        iData.append(",");
                    } else {
                        String[] nevadaDataStr = nevadaDatas.get(i).getDatas().split(",");
                        iData.append(j >= nevadaDataStr.length ? "," : "," + (StringUtils.isBlank(nevadaDataStr[j]) ? "0" : nevadaDataStr[j]));
                    }

                }
                if (init) {
                    iData.append(nevadaDatas.get(i).getDatas());
                }
                nevadaDatasStr.add(iData.toString().replace(" ", ""));
                nevadaDatasNew.add(EwbNonNevadaData.builder().nevadaId(nevadaSetting.getNevadaId()).sort(i + 1).datas(iData.toString().replace(" ", "")).build());
            }
        }
        nevadaDatasNew.forEach(nevadaData -> {
            String[] datas = nevadaData.getDatas().split(",");
            if (datas.length > 1) {
                double sum = Double.parseDouble(datas[1]);
                double backSum = 0.0;
                for (int i = 2; i < datas.length; i++) {
                    backSum = StringUtils.isBlank(datas[i]) ? backSum : backSum + Double.parseDouble(datas[i]);
                }
                if (backSum > sum) {
                    throw new DescribeException(EwbCustomPromptEnum.返修数量的总和超过了销售数量, languageKey);
                }
            }
        });
    }

    /**
     * 保存内华达数据
     *
     * @param nevadaData
     * @return
     */
    public ServerResponse<String> saveNevadaTable(HeaderUserInfo userInfo, NevadaSaveVo nevadaData) {
        EwbNonNevadaSetting nevadaSetting = nonNevadaSettingMapper.selectById(nevadaData.getNevadaId());
        // 处理前端传入的数据以满足后台的保存格式要求
        List<EwbNonNevadaData> nevadaDatasNew = IntStream.range(0, nevadaData.getNevadaDatas().size()).mapToObj(i -> EwbNonNevadaData.builder().nevadaId(nevadaData.getNevadaId()).sort(i + 1).datas(nevadaData.getNevadaDatas().get(i).toString().replace("[", "").replace("]", "")).build()).collect(Collectors.toList());
        // 校验数据的合法性并将其转为weibull标准数据
        List<EwbAnalysisDataDetail> analysisDataDetails = validNevadaNda(nevadaDatasNew, nevadaSetting.getNumberPeriod(), userInfo.getLanguageKey());
        // 修改详细数据
        analysisDataDetailService.saveData(userInfo, EwbAnalysisDataSaveVo.builder().analysisId(nevadaData.getNevadaId()).analysisDataDetails(analysisDataDetails).leftFlag(EwbConstants.FLAG_NO_CHECKED).rightFlag(EwbConstants.FLAG_CHECKED).groupingFlag(EwbConstants.FLAG_CHECKED).startDataNo(1).endDataNo(analysisDataDetails.size()).dataId(analysisDataDetails.get(0).getDataId()).hotDataChange(2).hotDataRankChange(0).timeForRank(EwbConstants.FLAG_NO_CHECKED).build(), true);
        // 删除原有数据
        nonNevadaDataMapper.delete(new QueryWrapper<EwbNonNevadaData>().lambda().eq(EwbNonNevadaData::getNevadaId, nevadaData.getNevadaId()));
        // 插入新的数据
        validTotalBack(nevadaDatasNew, nevadaSetting, new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), false, userInfo.getLanguageKey());
        nonNevadaDataMapper.insertBatch(nevadaDatasNew);
        return ServerResponse.createBySuccess();
    }

    /**
     * 校验数据的合法性并将其转为weibull标准数据
     *
     * @param nevadaDatas
     * @return
     */
    private List<EwbAnalysisDataDetail> validNevadaNda(List<EwbNonNevadaData> nevadaDatas, Integer numberPeriod, Integer languageKey) {
        List<EwbAnalysisDataDetail> result = new ArrayList<>();
        // 保存s数据
        List<Integer> suspensions = new ArrayList<>(numberPeriod);
        // 保存f数据
        List<Integer> failures = new ArrayList<>(numberPeriod);
        // 保存销售数据
        List<Integer> sales = new ArrayList<>(numberPeriod);
        try {
            // 初始化数据
            for (int dataSize = 0; dataSize < numberPeriod; dataSize++) {
                suspensions.add(dataSize, 0);
                failures.add(dataSize, 0);
            }

            // 循环生成weibull标准数据
            for (int i = 0; i < numberPeriod; i++) {
                String[] saleReturns = nevadaDatas.get(i).getDatas().replace(" ", "").split(",");
                sales.add(Integer.parseInt(saleReturns[1]));
                for (int j = 0; j < numberPeriod; j++) {
                    if (j + 2 >= saleReturns.length) {
                        failures.set(i, Integer.parseInt(""));
                    }
                    failures.set(i, StringUtils.isBlank(saleReturns[j + 2]) || j < i ? failures.get(i) : failures.get(i) + Integer.parseInt(saleReturns[j + 2]));
                    if (j - i < 0) {
                        continue;
                    }
                    suspensions.set(j - i, StringUtils.isBlank(saleReturns[j + 2]) ? suspensions.get(j - i) : suspensions.get(j - i) + Integer.parseInt(saleReturns[j + 2]));
                }
            }
            // 循环判断销售数据是否已经填入
            for (EwbNonNevadaData nevadaData : nevadaDatas) {
                Integer.parseInt(nevadaData.getDatas().replace(" ", "").split(",")[1]);
            }
        } catch (NumberFormatException | ArrayIndexOutOfBoundsException e) {
            throw new DescribeException(EwbCustomPromptEnum.请填写合法且完整的数据, languageKey);
        }
        EwbNonNevadaSetting nevadaSetting = nonNevadaSettingMapper.selectById(nevadaDatas.get(0).getNevadaId());
        // 拼接最终需要生成的weibull数据
        for (int i = 0; i < numberPeriod; i++) {
            result.add(i, EwbAnalysisDataDetail.builder().dataId(nevadaSetting.getDataId()).dataNo(i * 2 + 1).quantity(suspensions.get(i)).failureSuspension(EwbConstants.STATE_FAILURE).failureEndTime(Double.parseDouble((i + 1) * nevadaSetting.getIncrement() + "")).lastInspectTime(Double.parseDouble((i + 1) * nevadaSetting.getIncrement() + "")).failureTime(Double.parseDouble((i + 1) * nevadaSetting.getIncrement() + "")).build());
            result.add(i, EwbAnalysisDataDetail.builder().dataId(nevadaSetting.getDataId()).dataNo(i * 2 + 2).quantity(sales.get(i) - failures.get(i)).failureSuspension(EwbConstants.STATE_SUSPENSION).failureEndTime(Double.parseDouble((numberPeriod - i) * nevadaSetting.getIncrement() + "")).lastInspectTime(Double.parseDouble((numberPeriod - i) * nevadaSetting.getIncrement() + "")).failureTime(Double.parseDouble((numberPeriod - i) * nevadaSetting.getIncrement() + "")).build());
        }
        return result;
    }

    /**
     * 内华达预测销售数据
     *
     * @param nevadaId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<NevadaDataVo> forecastData(String whichTree, String nevadaId, Integer forecast, Integer forecastType, String boundSided, Double confidenceLevel, Integer confidenceBound, HeaderUserInfo userInfo) throws Exception {
        Integer languageKey = userInfo.getLanguageKey();
        // 获取设置数据
        EwbNevadaSetting nevadaSetting = new EwbNevadaSetting();
        EwbNonNevadaSetting nonNevadaSetting = new EwbNonNevadaSetting();
        EwbAnalysisSetting analysisSetting = new EwbAnalysisSetting();
        List<EwbNonNevadaData> nevadaDatas = new ArrayList<>();
        // 获取对应的标准数据和内华达数据
        List<EwbAnalysisDataDetail> analysisDataDetails = new ArrayList<>();
        if (EwbConstants.NON_NEVADA.equals(whichTree)) {
            nonNevadaSetting = nonNevadaSettingMapper.selectById(nevadaId);
            if (!WhetherFlagEnum.YES.getIndex().equals(nonNevadaSetting.getIsComplete())) {
                return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.当前数据所关联weibull未完成分析, languageKey);
            }
            nonNevadaSetting.setForecast(forecast);
            nonNevadaSetting.setForecastType(forecastType);
            nonNevadaSetting.setBoundSided(boundSided);
            nonNevadaSetting.setConfidenceLevel(confidenceLevel);
            nonNevadaSetting.setConfidenceBound(confidenceBound);
            nonNevadaSettingMapper.updateById(nonNevadaSetting);
            analysisSetting = analysisDataDetailService.getAnalysisSetting(nonNevadaSetting.getSettingId());
            analysisDataDetails = analysisDataDetailService.listDataDetail(nonNevadaSetting.getDataId());
            nevadaDatas = nonNevadaDataMapper.selectList(new QueryWrapper<EwbNonNevadaData>().lambda().eq(EwbNonNevadaData::getNevadaId, nevadaId));
            validNevadaNda(nevadaDatas, nonNevadaSetting.getNumberPeriod(), languageKey);
        } else {
            nevadaSetting = nevadaSettingMapper.selectById(nevadaId);
            List<EwbNevadaData> nevadaData = nevadaDataMapper.selectList(new QueryWrapper<EwbNevadaData>().lambda().eq(EwbNevadaData::getNevadaId, nevadaId));
            for (EwbNevadaData nevadaData1 : nevadaData) {
                EwbNonNevadaData nonNevadaData = new EwbNonNevadaData();
                BeanUtils.copyProperties(nevadaData1, nonNevadaData);
                nevadaDatas.add(nonNevadaData);
            }
        }
        // 获取计算需要的数据如：beta、eta、lambada...
        EwbAnalysisResult analysisResult = EwbConstants.NON_NEVADA.equals(whichTree) ? analysisResultMapper.selectById(nonNevadaSetting.getSettingId()) : EwbAnalysisResult.builder().p1(Double.parseDouble(nevadaSetting.getBeta())).p2(Double.parseDouble(nevadaSetting.getLambda())).build();
        // 存放最终生成的表头以及列表数据
        List<List<String>> result = new ArrayList<>(4);
        List<List<String>> resultUp = new ArrayList<>(4);
        List<List<String>> resultDown = new ArrayList<>(4);
        List<String> forecastDatas = new ArrayList<>();
        Integer numberPeriod = EwbConstants.NON_NEVADA.equals(whichTree) ? nonNevadaSetting.getNumberPeriod() : nevadaSetting.getNumberPeriod();
        forecast = forecast == 0 ? (EwbConstants.NON_NEVADA.equals(whichTree) ? nonNevadaSetting.getFutureNumberPeriod() : nevadaSetting.getFutureNumberPeriod()) : forecast;
        if (EwbConstants.NON_NEVADA.equals(whichTree)) {
            nonNevadaSetting.setForecast(forecast);
            nonNevadaSetting.setForecastType(forecastType);
            nonNevadaSettingMapper.updateById(nonNevadaSetting);
        } else {
            nevadaSetting.setForecast(forecast);
            nevadaSetting.setForecastType(forecastType);
            nevadaSettingMapper.updateById(nevadaSetting);
        }
        List<Double> multiples = new ArrayList<>();
        List<Double> multipleUps = new ArrayList<>();
        List<Double> multipleDowns = new ArrayList<>();
        for (int k = 0; k < numberPeriod + forecast; k++) {
            String calcResult = calcService.calc(userInfo.getUserId(),
                    EwbCalcParamBo.builder()
                            .dataId(analysisSetting.getDataId())
                            .settingId(analysisSetting.getSettingId())
                            .confidenceType(EwbConfidenceTypeCodeEnum.双侧置信限.getIndex())
                            .distributingCode(analysisSetting.getDistributingCode())
                            .confidenceMethod(EwbConfidenceMethodCodeEnum.fishermatrix.getIndex())
                            .confidencePercent(calcConfidenceLevel(confidenceLevel, confidenceBound))
                            .calcType(EwbConstants.CALC_TYPE_CONDITIONALUNRELIABILITY)
                            .parameterCode(analysisSetting.getParameterCode())
                            .startTime(String.valueOf(k * nonNevadaSetting.getIncrement()))
                            .additionalTime(String.valueOf(nonNevadaSetting.getIncrement()))
                            .endTime(nonNevadaSetting.getIncrement().toString()).build(), userInfo, true).getData().toString();
            multiples.add(Double.parseDouble(calcResult.split(",")[0]));
            multipleUps.add(Double.parseDouble(calcResult.split(",")[1]));
            multipleDowns.add(Double.parseDouble(calcResult.split(",")[2]));
        }
        List<String> colHeader = new ArrayList<>();
        if (EwbConstants.NON_NEVADA.equals(whichTree)) {
            calcAllData(nevadaDatas, languageKey, numberPeriod, whichTree, analysisDataDetails, nonNevadaSetting, forecast, colHeader, analysisSetting, nevadaSetting, analysisResult, forecastType, multiples, result, new ArrayList<>());
            calcAllData(nevadaDatas, languageKey, numberPeriod, whichTree, analysisDataDetails, nonNevadaSetting, forecast, colHeader, analysisSetting, nevadaSetting, analysisResult, forecastType, multipleUps, resultUp, new ArrayList<>());
            colHeader = calcAllData(nevadaDatas, languageKey, numberPeriod, whichTree, analysisDataDetails, nonNevadaSetting, forecast, colHeader, analysisSetting, nevadaSetting, analysisResult, forecastType, multipleDowns, resultDown, new ArrayList<>());
        } else {
            colHeader = calcAllData(nevadaDatas, languageKey, numberPeriod, whichTree, analysisDataDetails, nonNevadaSetting, forecast, colHeader, analysisSetting, nevadaSetting, analysisResult, forecastType, multiples, result, forecastDatas);
        }
        return ServerResponse.createBySuccess(NevadaDataVo.builder().colHeader(colHeader).nevadaDataNss(result).nevadaDataDowns(resultDown).nevadaDataUps(resultUp).nevadaSetting(nonNevadaSetting).nevadaDatas(forecastDatas).setting(nevadaSetting).build());
    }

    /**
     * 内华达预测销售数据导出
     *
     * @param nevadaDataVo
     * @param response
     */
    @Transactional(rollbackFor = Exception.class)
    public void forecastExport(NevadaDataVo nevadaDataVo, HttpServletResponse response) {
        // 导出：Excel
        NevadaExportExcelUtil.export(response, nevadaDataVo.getColHeader(), !CollectionUtils.isEmpty(nevadaDataVo.getNevadaDataNss()) ? nevadaDataVo.getNevadaDataNss().get(0) : null, !CollectionUtils.isEmpty(nevadaDataVo.getNevadaDataUps()) ? nevadaDataVo.getNevadaDataUps().get(0) : null, !CollectionUtils.isEmpty(nevadaDataVo.getNevadaDataDowns()) ? nevadaDataVo.getNevadaDataDowns().get(0) : null);
    }

    /**
     * 计算ConfidenceLevel
     * 双侧无需计算，单侧转为双侧计算，单侧转换双侧公式：50 + confidenceLevel/2
     *
     * @param confidenceLevel
     * @param confidenceBound
     * @return
     */
    private String calcConfidenceLevel(Double confidenceLevel, Integer confidenceBound) {
        if (confidenceLevel == null) {
            return EwbConfidenceBoundEnum.getConfidenceLevel(confidenceBound).toString();
        }
        return EwbConfidenceBoundEnum.双侧.getIndex().equals(confidenceBound) ? confidenceLevel.toString() : String.valueOf(confidenceLevel / 2 + 50);
    }

    /**
     * 生成计算结果（中值、上下限）
     *
     * @param nevadaDatas
     * @param languageKey
     * @param numberPeriod
     * @param whichTree
     * @param analysisDataDetails
     * @param nonNevadaSetting
     * @param forecast
     * @param colHeader
     * @param analysisSetting
     * @param nevadaSetting
     * @param analysisResult
     * @param forecastType
     * @param multiples
     * @param result
     */
    public List<String> calcAllData(List<EwbNonNevadaData> nevadaDatas, Integer languageKey, Integer numberPeriod, String whichTree, List<EwbAnalysisDataDetail> analysisDataDetails, EwbNonNevadaSetting nonNevadaSetting, Integer forecast, List<String> colHeader, EwbAnalysisSetting analysisSetting, EwbNevadaSetting nevadaSetting, EwbAnalysisResult analysisResult, Integer forecastType, List<Double> multiples, List<List<String>> resultAll, List<String> result) {
        colHeader = new ArrayList<>();
        colHeader.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.时期, languageKey));
        colHeader.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.数量, languageKey));
        // 初始化内华达中间数据
        List<String> resultNs = new ArrayList<>(nevadaDatas.size() + 1), resultRu = new ArrayList<>(nevadaDatas.size() + 1), resultRd = new ArrayList<>(nevadaDatas.size() + 1), resultR = new ArrayList<>(nevadaDatas.size() + 1);
        // 初始化内华达总和数据
        List<String> lastDataNs = new ArrayList<>(), lastDataRu = new ArrayList<>(), lastDataRd = new ArrayList<>(), lastDataR = new ArrayList<>();
        for (int i = 0; i < nevadaDatas.size(); i++) {
            StringBuilder iDataNs = new StringBuilder(), iDataRu = new StringBuilder(), iDataRd = new StringBuilder(), iDataR = new StringBuilder();
            String[] split = nevadaDatas.get(i).getDatas().split(",");
            // 拼接列表的第一列和第二列数据
            if (i == 0) {
                lastDataNs.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.总计, languageKey));
                lastDataNs.add("0");
                lastDataRu.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.总计, languageKey));
                lastDataRu.add("0");
                lastDataRd.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.总计, languageKey));
                lastDataRd.add("0");
                lastDataR.add(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.总计, languageKey));
                lastDataR.add("0");
            }
            iDataNs.append(split[0]).append(",").append(split[1].trim());
            iDataRu.append(split[0]).append(",").append(split[1].trim());
            iDataRd.append(split[0]).append(",").append(split[1].trim());
            iDataR.append(split[0]).append(",").append(split[1].trim());
            lastDataNs.set(1, String.valueOf(Double.parseDouble(lastDataNs.get(1)) + Double.parseDouble(split[1].trim())));
            lastDataRu.set(1, String.valueOf(Double.parseDouble(lastDataRu.get(1)) + Double.parseDouble(split[1].trim())));
            lastDataRd.set(1, String.valueOf(Double.parseDouble(lastDataRd.get(1)) + Double.parseDouble(split[1].trim())));
            lastDataR.set(1, String.valueOf(Double.parseDouble(lastDataR.get(1)) + Double.parseDouble(split[1].trim())));
            if (i < numberPeriod) {
                // 拼接已销售的月份的预测返修数据
                int suspension = EwbConstants.NON_NEVADA.equals(whichTree) ? analysisDataDetails.get(nonNevadaSetting.getNumberPeriod() - i - 1).getQuantity() : Integer.parseInt(split[1].trim());
                // 记录实时的剩余数量
                double suspensionCurNs = suspension;
                double suspensionCurRu = suspension;
                double suspensionCurRd = suspension;
                double suspensionCurR = suspension;
                // double failurePre = 0.0;
                for (int j = 0; j < forecast; j++) {
                    // 拼接表头数据
                    if (colHeader.size() < forecast + 2) {
                        colHeader.add(EwbConstants.NON_NEVADA.equals(whichTree) ? calculateDate(nonNevadaSetting.getPeriodsIn(), nonNevadaSetting.getStartMounth(), nonNevadaSetting.getStartDay(), nonNevadaSetting.getStartYear(), nonNevadaSetting.getStartNumber(), nonNevadaSetting.getIncrement(), nonNevadaSetting.getNumberPeriod() + j + 1) + "" : calculateDate(nevadaSetting.getPeriodsIn(), nevadaSetting.getStartMounth(), nevadaSetting.getStartDay(), nevadaSetting.getStartYear(), nevadaSetting.getStartNumber(), nevadaSetting.getIncrement(), nevadaSetting.getNumberPeriod() + j + 1) + "");
                    }
                    // 计算内华达数据，以weibull为例：第一行数据为1-已销售的数据的最大周期的可靠度/以j+1为基础的可靠度*S
                    double multiple = EwbConstants.NON_NEVADA.equals(whichTree)
                            //? calculateMultiple(analysisSetting, numberPeriod, nonNevadaSetting, i, j, analysisResult, false, !CollectionUtils.isEmpty(multiples) ? multiples.get(numberPeriod + j - i) : null)
                            ? multiples.get(numberPeriod + j - i)
                            : getDifferYt(nevadaSetting.getStartNumber() + nevadaSetting.getIncrement() * (nevadaSetting.getNumberPeriod() + j - i), nevadaSetting.getStartNumber() + nevadaSetting.getIncrement() * (nevadaSetting.getNumberPeriod() + 1 + j - i), Double.parseDouble(nevadaSetting.getBeta()), Double.parseDouble(nevadaSetting.getLambda())) / nevadaSetting.getClaimsPer();
                    // double failure = calculateByType((nonNevadaSetting.getBoundSided().equals(EwbConfidenceTypeCodeEnum.无.getIndex()) ? suspension : suspensionCurNs) * multiple, forecastType);
                    double failureNs = calculateByType(suspensionCurNs * multiple, EwbNevadaForecastTypeEnum.未设置.getIndex());
                    double failureRu = calculateByType(suspensionCurRu * multiple, EwbNevadaForecastTypeEnum.向上取整.getIndex());
                    double failureRd = calculateByType(suspensionCurRd * multiple, EwbNevadaForecastTypeEnum.向下取整.getIndex());
                    double failureR = calculateByType(suspensionCurR * multiple, EwbNevadaForecastTypeEnum.四舍五入.getIndex());
                    // 若计算上下限了，则直接相乘得出结果
                    // double value = nonNevadaSetting.getBoundSided().equals(EwbConfidenceTypeCodeEnum.无.getIndex()) ? failure - failurePre : failure;
                    iDataNs.append(",").append(String.format("%.4f", failureNs));
                    iDataRu.append(",").append(String.format("%.0f", failureRu));
                    iDataRd.append(",").append(String.format("%.0f", failureRd));
                    iDataR.append(",").append(String.format("%.0f", failureR));
                    // 初始化集合，若无数据则添加0
                    if (j + 2 >= lastDataNs.size()) {
                        lastDataNs.add("0");
                        lastDataRu.add("0");
                        lastDataRd.add("0");
                        lastDataR.add("0");
                    }
                    lastDataNs.set(j + 2, String.valueOf(Double.parseDouble(lastDataNs.get(j + 2)) + failureNs));
                    lastDataRu.set(j + 2, String.valueOf(Double.parseDouble(lastDataRu.get(j + 2)) + failureRu));
                    lastDataRd.set(j + 2, String.valueOf(Double.parseDouble(lastDataRd.get(j + 2)) + failureRd));
                    lastDataR.set(j + 2, String.valueOf(Double.parseDouble(lastDataR.get(j + 2)) + failureR));
                    suspensionCurNs = suspensionCurNs - failureNs;
                    suspensionCurRu = suspensionCurRu - failureRu;
                    suspensionCurRd = suspensionCurRd - failureRd;
                    suspensionCurR = suspensionCurR - failureR;
                    // failurePre = EwbConstants.NON_NEVADA.equals(whichTree) ? failure : 0.0;
                }
                resultNs.add(iDataNs.toString());
                resultRu.add(iDataRu.toString());
                resultRd.add(iDataRd.toString());
                resultR.add(iDataR.toString());
            } else {
                // 拼接预测月份的预测返修数据
                double failurePre = 0.0;
                // 记录实时的剩余数量
                double suspension = Double.parseDouble(split[1].trim());
                double suspensionCurNs = suspension;
                double suspensionCurRu = suspension;
                double suspensionCurRd = suspension;
                double suspensionCurR = suspension;
                for (int j = 0; j < forecast; j++) {
                    if (i - numberPeriod - j > 0) {
                        // 该处是预测数据的销售时间，所以并不存在返修数据
                        iDataNs.append(",");
                        iDataRu.append(",");
                        iDataRd.append(",");
                        iDataR.append(",");
                    } else {
                        double multiple = EwbConstants.NON_NEVADA.equals(whichTree)
                                //? calculateMultiple(analysisSetting, numberPeriod, nonNevadaSetting, i, j, analysisResult, true, !CollectionUtils.isEmpty(multiples) ? multiples.get(numberPeriod + j - i) : null)
                                ? multiples.get(numberPeriod + j - i)
                                : getDifferYt(nevadaSetting.getStartNumber() + nevadaSetting.getIncrement() * (nevadaSetting.getNumberPeriod() + j - i), nevadaSetting.getStartNumber() + nevadaSetting.getIncrement() * (nevadaSetting.getNumberPeriod() + 1 + j - i), Double.parseDouble(nevadaSetting.getBeta()), Double.parseDouble(nevadaSetting.getLambda()) / nevadaSetting.getClaimsPer());

                        // 计算内华达数据，以weibull为例：第一行数据为1-已销售的数据的最大周期的可靠度/以j+1为基础的可靠度*S
                        // double failure = calculateByType((nonNevadaSetting.getBoundSided().equals(EwbConfidenceTypeCodeEnum.无.getIndex()) ? Integer.parseInt(split[1].trim()) : suspensionCur) * multiple, forecastType);
                        double failureNs = calculateByType(suspensionCurNs * multiple, EwbNevadaForecastTypeEnum.未设置.getIndex());
                        double failureRu = calculateByType(suspensionCurRu * multiple, EwbNevadaForecastTypeEnum.向上取整.getIndex());
                        double failureRd = calculateByType(suspensionCurRd * multiple, EwbNevadaForecastTypeEnum.向下取整.getIndex());
                        double failureR = calculateByType(suspensionCurR * multiple, EwbNevadaForecastTypeEnum.四舍五入.getIndex());
                        // 若计算上下限了，则直接相乘得出结果
                        // double value = nonNevadaSetting.getBoundSided().equals(EwbConfidenceTypeCodeEnum.无.getIndex()) ? failure - failurePre : failure;
                        iDataNs.append(",").append(String.format("%.4f", failureNs));
                        iDataRu.append(",").append(String.format("%.0f", failureRu));
                        iDataRd.append(",").append(String.format("%.0f", failureRd));
                        iDataR.append(",").append(String.format("%.0f", failureR));
                        lastDataNs.set(j + 2, String.valueOf(Double.parseDouble(lastDataNs.get(j + 2)) + failureNs));
                        lastDataRu.set(j + 2, String.valueOf(Double.parseDouble(lastDataRu.get(j + 2)) + failureRu));
                        lastDataRd.set(j + 2, String.valueOf(Double.parseDouble(lastDataRd.get(j + 2)) + failureRd));
                        lastDataR.set(j + 2, String.valueOf(Double.parseDouble(lastDataR.get(j + 2)) + failureR));
                        suspensionCurNs = suspensionCurNs - failureNs;
                        suspensionCurRu = suspensionCurRu - failureRu;
                        suspensionCurRd = suspensionCurRd - failureRd;
                        suspensionCurR = suspensionCurR - failureR;
                    }
                }
                resultNs.add(iDataNs.toString());
                resultRu.add(iDataRu.toString());
                resultRd.add(iDataRd.toString());
                resultR.add(iDataR.toString());
            }
        }
        for (int k = 2; k < lastDataNs.size(); k++) {
            lastDataNs.set(k, String.format("%.4f", Double.parseDouble(lastDataNs.get(k))));
            lastDataRu.set(k, String.format("%.0f", Double.parseDouble(lastDataRu.get(k))));
            lastDataRd.set(k, String.format("%.0f", Double.parseDouble(lastDataRd.get(k))));
            lastDataR.set(k, String.format("%.0f", Double.parseDouble(lastDataR.get(k))));
        }
        resultNs.add(BasicDataUtil.listToString(lastDataNs));
        resultRu.add(BasicDataUtil.listToString(lastDataRu));
        resultRd.add(BasicDataUtil.listToString(lastDataRd));
        resultR.add(BasicDataUtil.listToString(lastDataR));
        resultAll.add(resultNs);
        resultAll.add(resultRu);
        resultAll.add(resultRd);
        resultAll.add(resultR);
        result.add(BasicDataUtil.listToString(lastDataNs));
        return colHeader;
    }

    /**
     * 验证数据是否合法
     *
     * @param nevadaId
     * @return
     */
    public ServerResponse validData(String nevadaId, Integer action, Integer languageKey) {
        EwbNonNevadaSetting nonNevadaSetting = nonNevadaSettingMapper.selectById(nevadaId);
        if (ObjectActionUtil.isNotEmpty(nonNevadaSetting)) {
            if (!WhetherFlagEnum.YES.getIndex().equals(nonNevadaSetting.getIsComplete()) && action.equals(EwbConstants.FORECAST)) {
                return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.当前数据所关联weibull未完成分析, languageKey);
            }
            List<EwbNonNevadaData> nevadaDatas = nonNevadaDataMapper.selectList(new QueryWrapper<EwbNonNevadaData>().lambda().eq(EwbNonNevadaData::getNevadaId, nevadaId));
            validNevadaNda(nevadaDatas, nonNevadaSetting.getNumberPeriod(), languageKey);
        } else {
            EwbNevadaSetting nevadaSetting = nevadaSettingMapper.selectById(nevadaId);
            List<EwbNevadaData> nevadaDatas = nevadaDataMapper.selectList(new QueryWrapper<EwbNevadaData>().lambda().eq(EwbNevadaData::getNevadaId, nevadaId));
            EwbNevadaService.validNevadaNda(nevadaDatas, nevadaSetting.getNumberPeriod(), nevadaSettingMapper, languageKey);
        }
        return ServerResponse.createBySuccess();
    }

    /**
     * 进行取整操作
     *
     * @param v
     * @param forecastType
     * @return
     */
    private double calculateByType(double v, Integer forecastType) {
        if (forecastType == 1) {
            return Math.ceil(v);
        } else if (forecastType == 2) {
            return Math.floor(v);
        } else if (forecastType == 3) {
            return Math.round(v);
        }
        return v;
    }

    /**
     * 计算倍率
     *
     * @param analysisSetting
     * @param numberPeriod
     * @param nonNevadaSetting
     * @param i
     * @param j
     * @param analysisResult
     * @param isFuture
     * @return
     */
    public Double calculateMultiple(EwbAnalysisSetting analysisSetting, Integer numberPeriod, EwbNonNevadaSetting nonNevadaSetting, Integer i, Integer j, EwbAnalysisResult analysisResult, boolean isFuture, Double multiple) {
        int forecast = i > numberPeriod ? 0 : (numberPeriod - i);
        int increment = nonNevadaSetting.getIncrement();
        int molecule = (numberPeriod + j + 1 - i) * increment;
        // 新增上下限计算，如果置信类型是上下限，则需要重新计算
        if (!nonNevadaSetting.getBoundSided().equals(EwbConfidenceTypeCodeEnum.无.getIndex())) {
            return multiple;
        }
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(analysisSetting.getDistributingCode())) {
            if (EwbDistributionTypeEnum.Mixed2Pop.getIndex().equals(analysisSetting.getParameterCode())) {
                return 1 - (analysisResult.getInterval1() / (Math.pow(Math.E, Math.pow(molecule / analysisResult.getP2(), analysisResult.getP1()))) + analysisResult.getInterval2() / (Math.pow(Math.E, Math.pow(molecule / analysisResult.getDb(), analysisResult.getP3())))) / (analysisResult.getInterval1() / (Math.pow(Math.E, Math.pow(forecast / analysisResult.getP2(), analysisResult.getP1()))) + analysisResult.getInterval2() / (Math.pow(Math.E, Math.pow(forecast / analysisResult.getDb(), analysisResult.getP3()))));
            } else if (EwbDistributionTypeEnum.Mixed3Pop.getIndex().equals(analysisSetting.getParameterCode())) {
                return 1 - (analysisResult.getInterval1() / (Math.pow(Math.E, Math.pow((numberPeriod + j + 1 - i) / analysisResult.getP2(), analysisResult.getP1()))) + analysisResult.getInterval2() / (Math.pow(Math.E, Math.pow((numberPeriod + j + 1 - i) / analysisResult.getDb(), analysisResult.getP3()))) + analysisResult.getInterval3() / (Math.pow(Math.E, Math.pow((numberPeriod + j + 1 - i) / analysisResult.getDbe(), analysisResult.getDe())))) / (analysisResult.getInterval1() / (Math.pow(Math.E, Math.pow(forecast / analysisResult.getP2(), analysisResult.getP1()))) + analysisResult.getInterval2() / (Math.pow(Math.E, Math.pow(forecast / analysisResult.getDb(), analysisResult.getP3()))) + analysisResult.getInterval3() / (Math.pow(Math.E, Math.pow(forecast / analysisResult.getDbe(), analysisResult.getDe()))));
            }
            return 1 - (Math.pow(Math.E, Math.pow(isFuture ? 0 : ((numberPeriod - i) * increment - analysisResult.getP3()) / analysisResult.getP2(), analysisResult.getP1()))) / (Math.pow(Math.E, Math.pow(((numberPeriod - i + j + 1) * increment - analysisResult.getP3()) / (analysisResult.getP2()), analysisResult.getP1())));
        } else if (EwbDistributingCodeEnum.Exponential.getIndex().equals(analysisSetting.getDistributingCode())) {
            return 1 - (Math.exp(-1 * analysisResult.getP2() * (i > numberPeriod ? (numberPeriod * 2 + j - i) * increment - numberPeriod : (numberPeriod + j) * increment - i)) / Math.exp(-1 * analysisResult.getP2() * (i > numberPeriod ? (numberPeriod - 1) * increment - numberPeriod : (numberPeriod - 1) * increment - i)));
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(analysisSetting.getDistributingCode())) {
            return 1 - (1 - normdist(molecule, analysisResult.getP1(), analysisResult.getP2())) / ((1 - normdist(forecast * increment, analysisResult.getP1(), analysisResult.getP2())));
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(analysisSetting.getDistributingCode())) {
            return 1 - (1 - normdist(Math.log(molecule), analysisResult.getP1(), analysisResult.getP2())) / ((1 - normdist(Math.log(forecast * increment), analysisResult.getP1(), analysisResult.getP2())));
        }
        return null;
    }

    /**
     * 删除内华达数据
     *
     * @param nevadaId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<NevadaDataVo> delNevadaData(String nevadaId) {
        int i = nevadaSettingMapper.deleteById(nevadaId);
        if (i > 0) {
            nevadaDataMapper.delete(new QueryWrapper<EwbNevadaData>().lambda().eq(EwbNevadaData::getNevadaId, nevadaId));
        } else {
            nonNevadaSettingMapper.deleteById(nevadaId);
            nonNevadaDataMapper.delete(new QueryWrapper<EwbNonNevadaData>().lambda().eq(EwbNonNevadaData::getNevadaId, nevadaId));
            analysisService.delAnalysis(nevadaId);
        }
        return ServerResponse.createBySuccess();
    }

    /**
     * 计算周期
     *
     * @param periodsIn
     * @param startMounth
     * @param startDay
     * @param startYear
     * @param startNumber
     * @param increment
     * @param i
     * @return
     */
    public static String calculateDate(String periodsIn, String startMounth, String startDay, String startYear, Integer startNumber, Integer increment, int i) {
        // 计算周期数据（年、月、日、数值）
        switch (periodsIn) {
            case "01":
                return DateTimeUtil.getBehindMonth(startMounth, increment * i, DateTimeUtil.STANDARD_FORMAT_NOD, "yyyy/MM");
            case "02":
                return DateTimeUtil.getBehindDay(startDay, increment * i, "yyyy/MM/dd");
            case "03":
                return Integer.parseInt(startYear) + increment * i + "";
            default:
                return startNumber + increment * i + "";
        }
    }


    /**
     * excel中normdist的java实现
     *
     * @param x
     * @param mean
     * @param standardDev
     * @return
     */
    public static double normdist(double x, double mean, double standardDev) {
        double res;
        x = (x - mean) / standardDev;
        if (x == 0) {
            res = 0.5;
        } else {
            double oor2pi = 1 / (Math.sqrt(2 * 3.14159265358979323846));
            double t = 1 / (1 + 0.2316419 * Math.abs(x));
            t *= oor2pi * Math.exp(-0.5 * x * x) * (0.31938153 + t * (-0.356563782 + t * (1.781477937 + t * (-1.821255978 + t * 1.330274429))));
            if (x >= 0) {
                res = 1 - t;
            } else {
                res = t;
            }
        }
        return res;
    }

    /**
     * 复制：内华达数据
     *
     * @param userInfo 用户信息
     * @param nevadaId 内华达ID
     * @return void
     */
    @Transactional(rollbackFor = Exception.class)
    public void copyNevada(HeaderUserInfo userInfo, String nevadaId) {
        // 复制：内华达分析设定表 Start
        EwbNonNevadaSetting nevadaSetting = nonNevadaSettingMapper.selectById(nevadaId);
        nevadaSetting.setNevadaId(BasicDataUtil.uuidGenerator());
        EwbAnalysisReqVo analysis = analysisService.createAnalysis(userInfo, nevadaSetting.getWbId(),
                nevadaSetting.getNevadaId(), "", null, null, null);
        nevadaSetting.setAnalysisName(nevadaSetting.getAnalysisName() + "_copy");
        nevadaSetting.setDataId(analysis.getDataId());
        nevadaSetting.setSettingId(analysis.getSettingId());
        nevadaSetting.setInsertTime(new Date());
        nevadaSetting.setUpdateTime(new Date());
        nevadaSetting.setForecastType(EwbNevadaForecastTypeEnum.未设置.getIndex());
        nevadaSetting.setConfidenceBound(EwbConfidenceBoundEnum.双侧.getIndex());
        nevadaSetting.setConfidenceLevel(EwbConfidenceBoundEnum.双侧.getConfidenceLevel());
        nevadaSetting.setBoundSided(EwbConfidenceTypeCodeEnum.无.getIndex());
        nevadaSetting.setIsComplete(WhetherFlagEnum.NO.getIndex());
        nevadaSetting.setVersionNo(EwbConstants.NUM_ONE); // 默认版本号1
        nonNevadaSettingMapper.insert(nevadaSetting);
        // 复制：内华达分析设定表 End
        // 构建：内华达分析数据
        List<List<Object>> nevadaData = this.buildNevadaData(nevadaId);
        NevadaSaveVo nevadaSaveVo = NevadaSaveVo.builder()
                .nevadaId(nevadaSetting.getNevadaId())
                .nevadaDatas(nevadaData)
                .build();
        // 保存：内华达数据
        this.saveNevadaTable(userInfo, nevadaSaveVo);
    }

    /**
     * 构建：内华达分析数据
     *
     * @param nevadaId 内华达ID
     * @return List<List < Object>>
     */
    private List<List<Object>> buildNevadaData(String nevadaId) {
        // 取得：内华达分析数据
        List<EwbNonNevadaData> ewbNonNevadaDataList = nonNevadaDataMapper.selectList(new QueryWrapper<EwbNonNevadaData>()
                .lambda().eq(EwbNonNevadaData::getNevadaId, nevadaId));
        // 为空，直接return
        if (CollectionUtils.isEmpty(ewbNonNevadaDataList)) {
            return new ArrayList<>();
        }
        // 组装数据
        List<List<Object>> nevadaData = new ArrayList<>(ewbNonNevadaDataList.size());
        ewbNonNevadaDataList.forEach(item -> {
            if (StringUtils.isNoneBlank(item.getDatas())) {
                nevadaData.add(Arrays.asList(item.getDatas().split(EwbConstants.COMMA)));
            }
        });
        // 返回结果
        return nevadaData;
    }

    /**
     * 获取所有的yt值
     *
     * @param count
     * @param beta
     * @param lambda
     * @return
     */
    public static List<Double> getAllYt(int count, Double beta, Double lambda) {
        List<Double> yt = new ArrayList<>();
        for (int i = 0; i < count; i++) {
            yt.add(lambda * Math.pow(i + 1, beta));
        }
        return yt;
    }

    /**
     * 获取某个位置的yt值
     *
     * @param count
     * @param yt
     * @return
     */
    public static double getOneYt(int count, List<Double> yt) {
        double result = 0.0;
        for (int i = count; i > 0; i--) {
            if (i % 2 == 1) {
                result += yt.get(i - 1);
            } else {
                result -= yt.get(i - 1);
            }
        }
        return Math.abs(result);
    }

    /**
     * 获取某个位置的yt值
     *
     * @param count1
     * @param count2
     * @return
     */
    public static double getDifferYt(int count1, int count2, Double beta, Double lambda) {
        return lambda * (Math.pow(count2, beta) - Math.pow(count1, beta));
    }
}


