package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.general.util.common.DateTimeUtil;
import com.nationrel.modules.weibull.general.util.common.NumberUtil;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.controller.EwbChartController;
import com.nationrel.modules.weibull.mapper.*;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.*;
import com.nationrel.modules.weibull.vo.*;
import com.nationrel.modules.weibull.util.ValidateUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author Aimee on 2020/9/22.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbPlotService {

    private final RgaApiService rgaApiService;
    private final EwbAnalysisMapper analysisMapper;
    private final StringRedisTemplate redisTemplate;
    private final EwbAnalysisService analysisService;
    private final EwbChartDataService chartDataService;
    private final EwbAnalysisInfoMapper analysisInfoMapper;
    private final EwbReplaceIntervalMapper replaceIntervalMapper;
    private final EwbInspectIntervalMapper inspectIntervalMapper;
    private final EwbAnalysisResultMapper ldAnalysisResultMapper;
    private final EwbStressStrengthMapper stressStrengthMapper;
    private final EwbAnalysisSettingMapper ldAnalysisSettingMapper;
    private final EwbAnalysisDataDetailMapper analysisDataDetailMapper;
    private final AllVariableConfig allVariableConfig;

    @Transactional(rollbackFor = Exception.class)
    public EwbProbabilityPlotVo probabilityPlot(HeaderUserInfo userInfo, EwbAnalysisStartVo param, boolean isMulti) throws Exception {
        EwbAnalysisSetting ldAnalysisSetting = ldAnalysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select("setting_id", "data_id", "distributing_code", "parameter_code", "estimation_method_code", "rank_method_code", "believe_code", "believe_method_code", "paramter", "believe_percent", "beta", "sigma", "logsigma", "xaxis_end", "xaxis_start", "yaxis_end", "yaxis_start", "time", "un_Reliability", "beta_mark", "eta_mark", "mu_mark", "sigma_mark", "lambda_mark", "target_time", "target_reliability", "reliability_plot_name").lambda().eq(EwbAnalysisSetting::getSettingId, param.getSettingId()));

        String serverSystem = allVariableConfig.getServerSystem();
        String data = EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data);
        String dataRow = EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataRow);
        String plotData = EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.plotData);
        String failureRank = EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank);
        String dataPointXY = EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataPointXY);
        EwbProbabilityPlotVo output = new EwbProbabilityPlotVo();
        BeanUtils.copyProperties(ldAnalysisSetting, output);
        output.setFailureQuantity(analysisDataDetailMapper.selectCount(new QueryWrapper<EwbAnalysisDataDetail>().lambda().eq(EwbAnalysisDataDetail::getDataId, param.getDataId()).eq(EwbAnalysisDataDetail::getFailureSuspension, EwbConstants.STATE_F)));
        output.setDates(new String[]{DateTimeUtil.dateToStr(new Date(), DateTimeUtil.STANDARD_FORMAT_NOH), DateTimeUtil.dateToStr(new Date(), DateTimeUtil.STANDARD_FORMAT_HMS)});

        // 作图结果
        EwbChartDataModel ldChartDataModel = chartDataService.getProbabilityChartData(param.getAnalysisId(), EwbConstants.PLOT_TYPE_PROBABLITY).get(0);
        ldChartDataModel.setBelieveCode(param.getBoundSided());
        output.setLdChartDataModel(ldChartDataModel);

        // 获取评估结果值
        getChartParam(param, output, ldAnalysisSetting, userInfo.getLanguageKey());
        // 常规数据的画图
        JSONObject requestPlotData = analysisService.getApiResult(param);
        requestPlotData.remove(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.settings));

        //************For getting Confidence bound data - Start**************//
        requestPlotData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.settings), rgaApiService.getApiPlotBoundsArgs(EwbConfidenceTypeCodeEnum.getValue(param.getBoundSided()), EwbConfidenceMethodCodeEnum.getValue(param.getMethod()), param.getConfidenceLevel()));

        EwbAnalysis analysis = analysisService.getAnalysis(param.getAnalysisId());
        output.setEndTimeName(analysis.getEndTimeName());
        output.setAnalysisName(analysis.getAnalysisName());
        if (StringUtils.isNotBlank(param.getPlotType()) && (param.getPlotType().equals(EwbConstants.PLOT_TYPE_REPLACECOST) || param.getPlotType().equals(EwbConstants.PLOT_TYPE_INSPECTCOST))) {
            JSONObject plotParam;
            // redis存在则直接获取
            String plotStr = redisTemplate.opsForValue().get(param.getAnalysisId());
            if (ValidateUtil.isNotEmpty(plotStr)) {
                plotParam = JSONObject.parseObject(plotStr);
            } else {
                // redis无数据则从数据库取数据
                plotStr = ldAnalysisSettingMapper.selectById(param.getSettingId()).getCalcResultAll();
                plotParam = ValidateUtil.isNotEmpty(plotStr) ? JSONObject.parseObject(plotStr) : null;
            }
            // 若了两边都没有数据，则该weibull数据未完成分析，返回提示信息
            if (plotParam == null) {
                throw new DescribeException(EwbCustomPromptEnum.当前数据所关联weibull未完成分析, userInfo.getLanguageKey());
            }

            if (ldAnalysisSetting.getDistributingCode().equals(EwbDistributingCodeEnum.Weibull.getIndex()) && Double.parseDouble(JSONObject.parseObject(plotParam.get(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results)).toString()).get(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.beta)).toString()) < 1) {
                throw new DescribeException(EwbCustomPromptEnum.当前选择的weibull分布的β参数, userInfo.getLanguageKey());
            }
            JSONObject datas = new JSONObject();
            datas.put(data, plotParam.get(data));
            output.setRequestFailureData(datas);
            // 生成需要画图的数据
            plotParam.remove(data);
            plotParam.remove(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank));
            JSONObject plotParamData = new JSONObject();
            if (param.getPlotType().equals(EwbConstants.PLOT_TYPE_REPLACECOST)) {
                plotParamData.put(plotData, plotParam);
            } else {
                plotParam.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.cp), param.getPlanReplace());
                plotParam.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.cf), param.getUnPlanReplace());
                plotParam.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.ci), param.getInspectTask());
                plotParam.put("p", param.getFailDetect());
                plotParam.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.inspectionType), param.getMaintenanceType());
                JSONObject inputsData = new JSONObject();
                inputsData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.inputs), plotParam);
                plotParamData.put(plotData, inputsData);
            }
            // 更新前端传入的数值信息
            if (param.getPlotType().equals(EwbConstants.PLOT_TYPE_REPLACECOST)) {
                EwbReplaceInterval replaceInterval = replaceIntervalMapper.selectById(param.getReplaceIntervalId());
                replaceInterval.setPlanReplace(param.getPlanReplace());
                replaceInterval.setUnPlanReplace(param.getUnPlanReplace());
                replaceInterval.setMaintenanceType(param.getMaintenanceType());
                replaceIntervalMapper.updateById(replaceInterval);
            } else {
                EwbInspectInterval inspectInterval = inspectIntervalMapper.selectById(param.getInspectIntervalId());
                inspectInterval.setPlanReplace(param.getPlanReplace());
                inspectInterval.setUnPlanReplace(param.getUnPlanReplace());
                inspectInterval.setInspectTask(param.getInspectTask());
                inspectInterval.setFailDetect(param.getFailDetect());
                inspectInterval.setMaintenanceType(param.getMaintenanceType());
                inspectIntervalMapper.updateById(inspectInterval);
            }
            // 生成请求画图的url
            String apiPlotUrl = rgaApiService.getApiPlotUrl(null, null, param.getPlotType(), null);
            apiPlotUrl = param.getPlotType().equals(EwbConstants.PLOT_TYPE_REPLACECOST) ? apiPlotUrl + "?Cp=" + param.getPlanReplace() + "&Cu=" + param.getUnPlanReplace() + "&maintenanceType=" + param.getMaintenanceType() : apiPlotUrl;

            String responseResultPlot = rgaApiService.getResultsCallApi(plotParamData.toString(), apiPlotUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
            if (StringUtils.isBlank(responseResultPlot)) {
                throw new DescribeException(EwbCustomPromptEnum.当前所选数据无法计算所选周期的最优区间, userInfo.getLanguageKey());
            }
            String exceptionMessage = JSONObject.parseObject(responseResultPlot).getString(EwbConstants.REQUEST_PARAM_EXCEPTIONMESSAGE);
            if (ValidateUtil.isNotEmpty(exceptionMessage)) {
                throw new DescribeException(EwbCustomPromptEnum.当前所选数据无法计算所选周期的最优区间, userInfo.getLanguageKey());
            }
            output.setResponseResultPlot(JSONObject.parseObject(responseResultPlot));
            output.getLdChartDataModel().setDistributingCode(param.getPlotType());
        } else {
            long maxTime = requestPlotData.toString().contains("maxTime") ? requestPlotData.getLong("maxTime") : JSONObject.parseObject(requestPlotData.get(data).toString()).getLong(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.endTime));
            int maxTimeLength = String.valueOf(maxTime).length();
            long increase = new BigDecimal("15e+" + maxTimeLength).longValue(); // To increase lower bound to Xaxis length Ref:Rb0427140.jsp
            long finalMaxTime = maxTime + increase;
            //************For getting Confidence bound data - End****************//

            double xmin = 1;
            int numOfPoints = 100;

            if (EwbDistributingCodeEnum.Weibull.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
                if ((EwbParameterCodeEnum.二参数.getIndex().equals(ldAnalysisSetting.getParameterCode())) || (EwbParameterCodeEnum.混合2参数.getIndex().equals(ldAnalysisSetting.getParameterCode())) || (EwbParameterCodeEnum.混合3参数.getIndex().equals(ldAnalysisSetting.getParameterCode()))) {
                    xmin = 0.001;
                } else if (EwbParameterCodeEnum.三参数.getIndex().equals(ldAnalysisSetting.getParameterCode())) {
                    xmin = Math.ceil(ldChartDataModel.getP3());
                }
            } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
                xmin = 0.001;
            }
            if (EwbDistributingCodeEnum.Normal.getIndex().equals(ldAnalysisSetting.getDistributingCode())
                    || EwbDistributingCodeEnum.Exponential.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
                xmin = 0.001;
                finalMaxTime = maxTime * 2;
            }

            // 处理x的最大最小值
//            calcXMinMax(requestPlotData.getJSONObject("results").toJSONString(), ldAnalysisSetting.getDistributingCode(), ldAnalysisSetting.getParameterCode(), xmin, (Double) finalMaxTime, userInfo.getLanguageKey());

            boolean autoScaleX = isMulti ? (param.getXaxisStart() == null) : (ldAnalysisSetting.getXaxisStart() == null && maxTime == 0);
            xmin = isMulti ? (param.getXaxisStart() == null ? 0 : param.getXaxisStart()) : (ldAnalysisSetting.getXaxisStart() == null ? xmin : ldAnalysisSetting.getXaxisStart());
            finalMaxTime = isMulti ? Double.valueOf(String.valueOf((param.getXaxisEnd() == null ? 0 : param.getXaxisEnd()))).longValue() : Double.valueOf((String.valueOf(ldAnalysisSetting.getXaxisEnd() == null ? finalMaxTime : ldAnalysisSetting.getXaxisEnd()))).longValue();
            String moreParameter = "?autoscaleX=" + autoScaleX + "&xmin=" + xmin + "&xmax=" + finalMaxTime + "&NumofPoints=" + numOfPoints;
            String apiPlotUrl = rgaApiService.getApiPlotUrl(ldAnalysisSetting.getDistributingCode(), ldAnalysisSetting.getParameterCode(), EwbConstants.PLOT_TYPE_PROBABLITY, moreParameter);
            // 记录并处理DataRow、FailureRank
            JSONArray dataRowPres = requestPlotData.getJSONArray(dataRow);
            dataRowPres = CollectionUtils.isEmpty(dataRowPres) ? requestPlotData.getJSONObject(data).getJSONArray(
                    dataRow) : dataRowPres;
            // 已有处理好的点数据，直接请求线数据
            if (!CollectionUtils.isEmpty(dataRowPres) && dataRowPres.get(0).toString().contains("[")) {
                JSONArray dataRowSpecial = requestPlotData.getJSONObject(data).getJSONArray(dataRow);
                // 删除这一次画图不需要的数据
                RgaApiService.deleteUselessData(serverSystem, requestPlotData);
                // 计算似然比需要传入全部的FailureRank和DataRow数据
                if (param.getMethod().equals(EwbConfidenceMethodCodeEnum.likelihoodradio.getIndex()) || param.getPlotType().equals(EwbConstants.PLOT_TYPE_CONTOUR)) {
                    rgaApiService.addFailureRankAndRow(ldAnalysisSettingMapper, requestPlotData, param);
                }
                boolean hasUclLclInterval = judgeUclLclInterval(serverSystem, requestPlotData, ldAnalysisSetting.getDistributingCode());
                output.setIsRankOrNodata(hasUclLclInterval ? EwbConstants.FLAG_CHECKED : EwbConstants.FLAG_NO_CHECKED);
                String responseResultPlot = rgaApiService.getResultsCallApi(requestPlotData.toString(), apiPlotUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
                JSONObject resPlot = JSONObject.parseObject(responseResultPlot);
                resPlot.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataPointXY), dataRowPres);
                output.setResponseResultPlot(resPlot);
                requestPlotData.getJSONObject(data).put(dataRow, dataRowSpecial);
                output.setRequestFailureData(requestPlotData);
                return output;
            }
            JSONArray dataRows = new JSONArray();
            if (dataRowPres != null) {
                for (Object dataRowPre : dataRowPres) {
                    if (JSONObject.parseObject(dataRowPre.toString()).get(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.fSstatus)).toString().equals(EwbConstants.STATE_FH)) {
                        dataRows.add(dataRowPre);
                    }
                }
            }
            JSONArray failureRanks = requestPlotData.getJSONObject(data).getJSONArray(failureRank);
            requestPlotData.remove(dataRow);
            requestPlotData.remove(failureRank);
            // 清除画图数据的点数据，该数据已记录，无需再传
            requestPlotData.getJSONObject(data).put(dataRow, new String[]{});
            requestPlotData.getJSONObject(data).put(failureRank, new String[]{});
            // 计算似然比需要传入全部的FailureRank和DataRow数据
            if (param.getMethod().equals(EwbConfidenceMethodCodeEnum.likelihoodradio.getIndex()) || param.getPlotType().equals(EwbConstants.PLOT_TYPE_CONTOUR)) {
                rgaApiService.addFailureRankAndRow(ldAnalysisSettingMapper, requestPlotData, param);
            }
            String responseResultPlot = rgaApiService.getResultsCallApi(requestPlotData.toString(), apiPlotUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
            // 将点数据再存入requestPlotData，后面筛选点数据需要用到
            requestPlotData.getJSONObject(data).put(dataRow, dataRows);
            JSONObject resPlot = JSONObject.parseObject(responseResultPlot);
            JSONArray dataRowPoints = new JSONArray();
            if (param.getPlotType().equals(EwbConstants.PLOT_TYPE_PROBABLITY)) {
                for (int i = 0; i < dataRows.size(); i++) {
                    JSONObject dataRowJson = JSONObject.parseObject(dataRows.get(i).toString());
                    dataRowPoints.add(new String[]{dataRowJson.get(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lastInspectTime)).toString(), failureRanks.get(i).toString()});
                }
            }
            resPlot.put(dataPointXY, dataRowPoints);
            responseResultPlot = resPlot.toString();
            requestPlotData.put(dataRow, dataRows);
            responseResultPlot = mapDataQuantity(responseResultPlot, requestPlotData);
            String exceptionMessage = JSONObject.parseObject(responseResultPlot).getString(EwbConstants.REQUEST_PARAM_EXCEPTIONMESSAGE);
            if (ValidateUtil.isNotEmpty(exceptionMessage)) {
                log.error("[外部API异常-EwbPlotService-196]{}", exceptionMessage);
                throw new DescribeException(ReliaqubeParamDataCheckEnum.未知错误, userInfo.getLanguageKey());
            }
            requestPlotData.getJSONObject(data).put(failureRank, new double[]{});
            requestPlotData.getJSONObject(data).put(dataRow, new double[]{});

//            saveDataDetailService.deleteEwbApiAnalysisData(param.getDataId());
//            saveDataDetailService.saveEwbApiAnalysisData(userInfo, param.getAnalysisId(), param.getDataId(), responseResultPlot, EwbConstants.PLOT_TYPE_PROBABLITY, param);

            //将所有类型的图表结果保存到表中
            EwbAnalysisInfoService.updateUpdateInfo(userInfo.getUserName(), ldChartDataModel.getAnalysisId(), analysisInfoMapper, analysisMapper);
            handRowDatas(requestPlotData, JSONObject.parseObject(responseResultPlot), output);
            System.out.println(JSONObject.parseObject(responseResultPlot).getJSONArray(dataPointXY));
            requestPlotData.put(dataRow, output.getResponseResultPlot().getJSONArray(dataPointXY));

            // 无数据画图且画图时weibull-3/Mix2/Mix3且是第一次画图时，强制将y最大最小值置成99\0.1
            if (ldAnalysisSetting.getXaxisStart() == null && output.getResponseResultPlot().getJSONArray(dataPointXY).size() == 0) {
                if ((EwbDistributingCodeEnum.Weibull.getIndex().equals(ldAnalysisSetting.getDistributingCode()) && (EwbParameterCodeEnum.三参数.getIndex().equals(ldAnalysisSetting.getParameterCode()) || EwbParameterCodeEnum.混合2参数.getIndex().equals(ldAnalysisSetting.getParameterCode()) || EwbParameterCodeEnum.混合3参数.getIndex().equals(ldAnalysisSetting.getParameterCode()))) || (EwbDistributingCodeEnum.Exponential.getIndex().equals(ldAnalysisSetting.getDistributingCode()) && EwbParameterCodeEnum.二参数.getIndex().equals(ldAnalysisSetting.getParameterCode()))) {
                    output.setYaxisStart(0.1);
                    output.setYaxisEnd(99.0);
                }
            }
            ldAnalysisSettingMapper.updateCalcResultAll(param.getSettingId(), output.getRequestFailureData().toJSONString());
            redisTemplate.opsForValue().set(param.getAnalysisId(), requestPlotData.toJSONString(), EwbConstants.REDIS_TIMEOUT, TimeUnit.HOURS);
        }

        return output;
    }

//    /**
//     * 计算x的最大最小值
//     *
//     * @param languageKey
//     * @param requestData
//     * @param distributingCode
//     * @param parameterCode
//     * @param xmin
//     * @param xmax
//     */
//    private void calcXMinMax(String requestData, String distributingCode, String parameterCode, double xmin, double xmax, Integer languageKey) {
//        try {
//            String apiCalcFunctionUrl = rgaApiService.getApiCalcFunctionUrl(EwbConstants.CALC_TYPE_BLIFE, BasicDataUtil.emptyStr, BasicDataUtil.emptyStr, distributingCode, BasicDataUtil.emptyStr, String.valueOf(EwbConstants.YAXIS_MIN_IMUM), parameterCode);
//            String results = rgaApiService.getResultsCallApi(requestData, apiCalcFunctionUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
//            if (results.indexOf("error") > 0 || results.indexOf("Exception") > 0)
//                throw new CannotPlotException(languageKey);
//            xmin = Double.parseDouble(results);
//            apiCalcFunctionUrl = rgaApiService.getApiCalcFunctionUrl(EwbConstants.CALC_TYPE_BLIFE, BasicDataUtil.emptyStr, BasicDataUtil.emptyStr, distributingCode, BasicDataUtil.emptyStr, String.valueOf(EwbConstants.YAXIS_MAX_IMUM), parameterCode);
//            results = rgaApiService.getResultsCallApi(requestData, apiCalcFunctionUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
//            if (results.indexOf("error") > 0 || results.indexOf("Exception") > 0)
//                throw new CannotPlotException(languageKey);
//            xmax = Double.parseDouble(results);
//        } catch (Exception e) {
//            throw new CannotPlotException(languageKey);
//        }
//    }

    /**
     * 将最近检查时间与结束时间不一致的数据返回给前端
     *
     * @param requestPlotData
     */
    private void handRowDatas(JSONObject requestPlotData, JSONObject responseResultPlot, EwbProbabilityPlotVo output) {
        String serverSystem = allVariableConfig.getServerSystem();
        String dataRow = EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataRow);
        JSONArray dataRowJson = requestPlotData.getJSONArray(dataRow);
        List<String[]> datas = new ArrayList<>(dataRowJson.size());
        for (int i = 0; i < dataRowJson.size(); i++) {
            String time = dataRowJson.getJSONObject(i).getString(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.time));
            String lastInspectTime = dataRowJson.getJSONObject(i).getString(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lastInspectTime));
            // 将最近检查时间与结束时间不一致的数据返回给前端
            // 处理一条数据都不存在的情况，加入首尾数据确保前端可以计算出x/y的最大最小值
            if (!time.equals(lastInspectTime) || i == 0 || i == dataRowJson.size() - 1) {
                datas.add(new String[]{time, lastInspectTime, dataRowJson.getJSONObject(i).getString(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.fSstatus)), String.valueOf(i)});
                if (!time.equals(lastInspectTime)) {
                    responseResultPlot.getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataPointXY)).getJSONArray(i).set(0, (Double.parseDouble(time) + Double.parseDouble(lastInspectTime)) / 2);
                }

            }
        }
        requestPlotData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data)).put(dataRow, datas);
        requestPlotData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), null);
        requestPlotData.remove(dataRow);
        output.setRequestFailureData(requestPlotData);
        output.setResponseResultPlot(responseResultPlot);
    }

    /**
     * @param responseResultPlot
     * @param requestPlotData
     * @Title: Map Quantity from request to response object
     * @Description:保存来自API的分布分析数据
     */
    private String mapDataQuantity(String responseResultPlot, JSONObject requestPlotData) {
        String serverSystem = allVariableConfig.getServerSystem();
        JSONObject originalResponse = JSONObject.parseObject(responseResultPlot);
        JSONArray originalRequest = requestPlotData.getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataRow));
        //JSONArray tempRequest = requestPlotData.getJSONObject("Data").getJSONArray("dataRow");
        JSONArray tempRequest = new JSONArray();
        JSONArray response = JSONObject.parseObject(responseResultPlot).getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataPointXY));

        for (int i = 0; i < originalRequest.size(); i++) {
            String fsStatus = originalRequest.getJSONObject(i).getString(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.fSstatus));
            if (Integer.parseInt(fsStatus) == 0) {
                tempRequest.add(originalRequest.getJSONObject(i));
            }
        }
        JSONArray response1 = new JSONArray();
        JSONArray tempRequest1 = new JSONArray();
        if (tempRequest.size() == response.size()) {
            if (response.size() > 0) {
                // 计算出最小最大值的xy值之间的平均区间值
                double minY = response.getJSONArray(0).toJavaObject(Double[].class)[1];
                double maxY = response.getJSONArray(response.size() - 1).toJavaObject(Double[].class)[1];
                double averageY = (maxY - minY) / 100;
                double minX = Math.log(response.getJSONArray(0).toJavaObject(Double[].class)[0]);
                double maxX = Math.log(response.getJSONArray(response.size() - 1).toJavaObject(Double[].class)[0]);
                double averageX = (maxX - minX) / 100;
                int x = 0;
                int y = 0;
                // 基于xy的平均区间进行点的筛选
                JSONArray response2 = new JSONArray();
                for (int j = 0; j < tempRequest.size() - 1; j++) {
                    String numofForS = tempRequest.getJSONObject(j).getString(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.numofForS));
                    String lastInspectTime = tempRequest.getJSONObject(j).getString(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lastInspectTime));
                    String time = tempRequest.getJSONObject(j).getString(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.time));
                    response.getJSONArray(j).add(Integer.parseInt(numofForS));
                    if (tempRequest.size() > 400) {
                        if (!lastInspectTime.equals(time) || !"1".equals(numofForS)) {
                            response1.add(response.getJSONArray(j));
                            tempRequest1.add(tempRequest.get(j));
                        } else if (j < 30 || (response.getJSONArray(j).toJavaObject(Double[].class)[1] <= minY + averageY * y && response.getJSONArray(j + 1).toJavaObject(Double[].class)[1] > minY + averageY * y)) {
                            y++;
                            response1.add(response.getJSONArray(j));
                            tempRequest1.add(tempRequest.get(j));
                        } else if ((Math.log(response.getJSONArray(j).toJavaObject(Double[].class)[0]) <= minX + averageX * x && Math.log(response.getJSONArray(j + 1).toJavaObject(Double[].class)[0]) > minX + averageX * x) || (minX + averageX * x < Math.log(response.getJSONArray(j).toJavaObject(Double[].class)[0]))) {
                            x++;
                            if ((minX + averageX * x >= Math.log(response.getJSONArray(j).toJavaObject(Double[].class)[0])) || Math.abs(x - y) <= 100) {
                                response1.add(response.getJSONArray(j));
                                tempRequest1.add(tempRequest.get(j));
                            }
                        } else {
                            response2.add(response.getJSONArray(j));
                        }
                    }
                }
                response.getJSONArray(tempRequest.size() - 1).add(Integer.parseInt(tempRequest.getJSONObject(tempRequest.size() - 1).getString(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.numofForS))));
                response1.add(response.getJSONArray(tempRequest.size() - 1));
                tempRequest1.add(tempRequest.get(tempRequest.size() - 1));
                // 进行第2轮筛选，将第一轮未选中的数据再筛选一遍，补100个点使点的曲线更加平滑
//                int average100 = response2.size() / 150;
//                for (int j = 0; j < response2.size(); j++) {
//                    if (j % average100 == 0) {
//                        response1.add(response2.getJSONArray(j));
//                    } else {
//                        response3.add(response2.getJSONArray(j));
//                    }
//                }
//                // 进行第3轮筛选，将第一轮未选中的数据再筛选一遍，补100个点使点的曲线更加平滑
//                int average2100 = response3.size() / 250;
//                for (int j = 0; j < response3.size(); j++) {
//                    if (j % average2100 == 0) {
//                        response1.add(response3.getJSONArray(j));
//                    }
//                }
//                if (response2.size() > 0) {
//                    for (int j = 0; j < 10; j++) {
//                        JSONArray responseBack = circularAccess(response2, new JSONArray(), response1, (j + 1) * 10);
//                        response2 = responseBack;
//                    }
//                }
            }
        }
        originalResponse.remove(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataPointXY));
        originalResponse.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataPointXY), tempRequest.size() > 400 ? response1 : response);
        requestPlotData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataRow), tempRequest.size() > 400 ? tempRequest1 :
                tempRequest);
        requestPlotData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataRow), tempRequest.size() > 400 ? tempRequest1 : tempRequest);
        return originalResponse.toString();
    }

    /**
     * 生成除分布概率图外的其他模型的图形数据
     *
     * @param userInfo
     * @param param
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public EwbProbabilityPlotVo selectEchartDetail(HeaderUserInfo userInfo, EwbAnalysisStartVo param) throws Exception {
        String serverSystem = allVariableConfig.getServerSystem();
        // 校验数据的正确性
        EwbProbabilityPlotVo output = new EwbProbabilityPlotVo();
        BeanUtils.copyProperties(param, output);

        // 用于生成图形的右下角的时间和图形的横坐标信息
        output.setEndTimeName(analysisService.getAnalysis(param.getAnalysisId()).getEndTimeName());
        output.setDates(new String[]{DateTimeUtil.dateToStr(new Date(), DateTimeUtil.STANDARD_FORMAT_NOH), DateTimeUtil.dateToStr(new Date(), "HH:mm:ss")});

        // 查询该模型现在使用的设置并将有效信息复制到返回结果中
        EwbAnalysisSetting ldAnalysisSetting = ldAnalysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select("setting_id", "data_id", "distributing_code", "parameter_code", "estimation_method_code", "rank_method_code", "believe_code", "believe_method_code", "paramter", "believe_percent", "beta", "sigma", "logsigma").lambda().eq(EwbAnalysisSetting::getSettingId, param.getSettingId()));

        // 获取作图结果信息和作图参数信息，可用于前端的各类显示
        getChartResult(param, output, param.getPlotType());
        getChartParam(param, output, ldAnalysisSetting, userInfo.getLanguageKey());

        // 获取画图输入数据
        JSONObject requestPlotData = JSONObject.parseObject(ldAnalysisSettingMapper.selectById(param.getSettingId()).getCalcResultAll());
        requestPlotData.remove(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank));
        if (!EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getBoundSided())) {
            requestPlotData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), rgaApiService.getApiPlotBoundsArgs(EwbConfidenceTypeCodeEnum.getValue(param.getBoundSided()), EwbConfidenceMethodCodeEnum.getValue(param.getMethod()), param.getConfidenceLevel()));
        }
        // 暂存画点数据
        JSONArray dataRows = requestPlotData.getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataRow));
        // 删除这一次画图不需要的数据
        RgaApiService.deleteUselessData(serverSystem, requestPlotData);
        // 添加画图接口需要的必要参数
        String moreParameter = param.getXaxisStart() == null ? "?autoscaleX=true&xmin=1&xmax=5&NumofPoints=100" : "?autoscaleX=false&xmin=" + param.getXaxisStart() + "&xmax=" + param.getXaxisEnd() + "&NumofPoints=100";
        if (EwbConstants.PLOT_TYPE_CONTOUR.equals(param.getPlotType())) {
            moreParameter = "?NumofPoints=100";
        } else if (EwbConstants.PLOT_TYPE_HISTOGRAM.equals(param.getPlotType())) {
            moreParameter = "?autoscaleX=true&xmin=1&xmax=5&NumofPdfPoints=100&autoBinSize=true&BinSize=5";
        }
        // 获取完整的画图请求url
        String apiPlotUrl = rgaApiService.getApiPlotUrl(ldAnalysisSetting.getDistributingCode(), ldAnalysisSetting.getParameterCode(), param.getPlotType(), moreParameter);
        // 计算似然比需要传入全部的FailureRank和DataRow数据
        if (param.getMethod().equals(EwbConfidenceMethodCodeEnum.likelihoodradio.getIndex()) || param.getPlotType().equals(EwbConstants.PLOT_TYPE_CONTOUR)) {
            rgaApiService.addFailureRankAndRow(ldAnalysisSettingMapper, requestPlotData, param);
        }
        if (EwbConstants.SERVER_SYSTEM_LINUX.equals(serverSystem) && requestPlotData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data)).get(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank)) == null) {
            requestPlotData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), new double[0]);
        }
        // 获取画图数据
        String responseResultPlot = rgaApiService.getResultsCallApi(requestPlotData.toString(), apiPlotUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        // 判读请求是否出错
        String exceptionMessage = JSONObject.parseObject(responseResultPlot).getString(EwbConstants.REQUEST_PARAM_EXCEPTIONMESSAGE);
        if (ValidateUtil.isNotEmpty(exceptionMessage)) {
            log.error("请求接口：" + apiPlotUrl + "，请求数据：" + requestPlotData + "，错误信息：" + exceptionMessage);
            throw new DescribeException(EwbCustomPromptEnum.外部API异常, userInfo.getLanguageKey());
        }
        // 解析结果数据
        JSONObject jsonObject = JSONObject.parseObject(responseResultPlot);
        // 如果计算的是可靠性数据则需要将之前不可靠数据的y变为1-y
        if (param.getPlotType().equals(EwbConstants.PLOT_TYPE_RELIABILITY)) {
            for (int i = 0; i < dataRows.size(); i++) {
                dataRows.getJSONArray(i).set(1, 1 - dataRows.getJSONArray(i).getDouble(1));
            }
        }
        // 画可靠性图和不可靠性的图需要添加点的数据
        if (param.getPlotType().equals(EwbConstants.PLOT_TYPE_RELIABILITY) || param.getPlotType().equals(EwbConstants.PLOT_TYPE_UNRELIABILITY)) {
            jsonObject.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataPointXY), dataRows);
        }

        // 返回结果数据
        output.setResponseResultPlot(jsonObject);
        output.getLdChartDataModel().setBelieveCode(param.getBoundSided());
        EwbAnalysisInfoService.updateUpdateInfo(userInfo.getUserName(), param.getAnalysisId(), analysisInfoMapper, analysisMapper);
        return output;
    }

    /**
     * 依据数据方差判断是否可以画出上下区间
     *
     * @param requestPlotData
     * @param distributingCode
     */
    public static boolean judgeUclLclInterval(String serverSystem, JSONObject requestPlotData, String distributingCode) {
        return (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode) && isLessThanZero(serverSystem, requestPlotData, new String[]{EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varBeta), EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varEta)})) || (EwbDistributingCodeEnum.Exponential.getIndex().equals(distributingCode) && isLessThanZero(serverSystem, requestPlotData, new String[]{EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varLambda)})) || (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode) && isLessThanZero(serverSystem, requestPlotData, new String[]{EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varLogSigma), EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varLogMu)})) || (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode) && isLessThanZero(serverSystem, requestPlotData, new String[]{EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varSigma), EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.varMu)}));
    }

    /**
     * 判断传入的key是否小于0
     *
     * @param requestPlotData
     * @param keys
     * @return
     */
    public static boolean isLessThanZero(String serverSystem, JSONObject requestPlotData, String[] keys) {
        String bounds = EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.bounds);
        String results = EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results);
        String settings = EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.settings);
        boolean isLessThanZero = Arrays.stream(keys).findFirst().filter(key -> requestPlotData.getJSONObject(results).getString(key) != null && NumberUtil.parseDouble(requestPlotData.getJSONObject(results).getString(key)) < 0.0).isPresent();
        if (isLessThanZero && requestPlotData.getJSONObject(settings) != null) {
            requestPlotData.getJSONObject(settings).getJSONObject(bounds).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.sided), EwbConfidenceTypeCodeEnum.getValue("00"));
            requestPlotData.getJSONObject(settings).getJSONObject(bounds).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.on), "Time");
        }
        return isLessThanZero;
    }

    /**
     * 生成需要返回的计算数据
     *
     * @param param
     * @param output
     * @param ldAnalysisSetting
     */
    private void getChartParam(EwbAnalysisStartVo param, EwbReliabilityPlotVo output, EwbAnalysisSetting ldAnalysisSetting, Integer languageKey) {
        EwbAnalysisResult ldAnalysisResult = ldAnalysisResultMapper.selectById(param.getSettingId());
        // 作图参数
        List<EwbCalcResultKeyValueVo> setting = new ArrayList<>();
        setting.add(new EwbCalcResultKeyValueVo(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.分析,
                languageKey), EwbDistributingCodeEnum.getByLaguageKey(ldAnalysisSetting.getDistributingCode(),
                languageKey) + "-" + EwbParameterCodeEnum.getByLaguageKey(ldAnalysisSetting.getParameterCode(), languageKey)));
        setting.add(new EwbCalcResultKeyValueVo(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.估计方法, languageKey), EwbEvaluateCodeEnum.getByLaguageKey(ldAnalysisSetting.getEstimationMethodCode(), languageKey)));
        output.setSetting(setting);

        List<EwbCalcResultKeyValueVo> result = new ArrayList<>();
        // weibull2p
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
            result.add(new EwbCalcResultKeyValueVo("β", output.getLdChartDataModel().getP1()));
            result.add(new EwbCalcResultKeyValueVo("η", output.getLdChartDataModel().getP2()));
            // weibull3p
            if (EwbParameterCodeEnum.三参数.getIndex().equals(output.getLdChartDataModel().getParamCode())) {
                result.add(new EwbCalcResultKeyValueVo("γ", output.getLdChartDataModel().getP3()));
            } else if (EwbParameterCodeEnum.混合2参数.getIndex().equals(output.getLdChartDataModel().getParamCode())) {
                result.remove(0);
                result.remove(0);
                // weibull混合2p
                result.add(new EwbCalcResultKeyValueVo("β1", output.getLdChartDataModel().getP1()));
                result.add(new EwbCalcResultKeyValueVo("η1", output.getLdChartDataModel().getP2()));
                result.add(new EwbCalcResultKeyValueVo("P1", output.getLdChartDataModel().getP7()));
                result.add(new EwbCalcResultKeyValueVo("β2", output.getLdChartDataModel().getP3()));
                result.add(new EwbCalcResultKeyValueVo("η2", output.getLdChartDataModel().getP4()));
                result.add(new EwbCalcResultKeyValueVo("P2", output.getLdChartDataModel().getP8()));
            } else if (EwbParameterCodeEnum.混合3参数.getIndex().equals(output.getLdChartDataModel().getParamCode())) {
                result.remove(0);
                result.remove(0);
                // weibull混合3p
                result.add(new EwbCalcResultKeyValueVo("β1", output.getLdChartDataModel().getP1()));
                result.add(new EwbCalcResultKeyValueVo("η1", output.getLdChartDataModel().getP2()));
                result.add(new EwbCalcResultKeyValueVo("P1", output.getLdChartDataModel().getP7()));
                result.add(new EwbCalcResultKeyValueVo("β2", output.getLdChartDataModel().getP3()));
                result.add(new EwbCalcResultKeyValueVo("η2", output.getLdChartDataModel().getP4()));
                result.add(new EwbCalcResultKeyValueVo("P2", output.getLdChartDataModel().getP8()));
                result.add(new EwbCalcResultKeyValueVo("β3", output.getLdChartDataModel().getP5()));
                result.add(new EwbCalcResultKeyValueVo("η3", output.getLdChartDataModel().getP6()));
                result.add(new EwbCalcResultKeyValueVo("P3", output.getLdChartDataModel().getP9()));
            }
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
            result.add(new EwbCalcResultKeyValueVo("u'", output.getLdChartDataModel().getP1()));
            result.add(new EwbCalcResultKeyValueVo("σ'", output.getLdChartDataModel().getP2()));
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
            result.add(new EwbCalcResultKeyValueVo("u", output.getLdChartDataModel().getP1()));
            result.add(new EwbCalcResultKeyValueVo("σ", output.getLdChartDataModel().getP2()));
        } else if (EwbDistributingCodeEnum.Exponential.getIndex().equals(ldAnalysisSetting.getDistributingCode())) {
            result.add(new EwbCalcResultKeyValueVo("λ", output.getLdChartDataModel().getP2()));
            if (EwbParameterCodeEnum.二参数.getIndex().equals(output.getLdChartDataModel().getParamCode())) {
                result.add(new EwbCalcResultKeyValueVo("γ", output.getLdChartDataModel().getP3()));
            }
        }
        result.add(new EwbCalcResultKeyValueVo("LK", ldAnalysisResult.getLk() == 0.0 ? "N/A" : ldAnalysisResult.getLk()));
        if (EwbEvaluateCodeEnum.rrx.getIndex().equals(ldAnalysisSetting.getEstimationMethodCode()) || EwbEvaluateCodeEnum.rry.getIndex().equals(ldAnalysisSetting.getEstimationMethodCode())) {
            result.add(new EwbCalcResultKeyValueVo("ρ", ldAnalysisResult.getRao()));
        }
        output.setResult(result);
        List<EwbCalcResultKeyValueVo> confidenceBounds = new ArrayList<>();
        confidenceBounds.add(new EwbCalcResultKeyValueVo(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信类型, languageKey), EwbConfidenceTypeCodeEnum.getName(param.getBoundSided())));
        String confidenceLevel = StringUtils.isBlank(param.getBoundSided()) || EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getBoundSided()) ? EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey) + "；" : StringUtils.isBlank(param.getConfidenceLevel()) ? EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey) + "；" : Double.parseDouble(param.getConfidenceLevel()) * 100 + "%";
        confidenceBounds.add(new EwbCalcResultKeyValueVo(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), confidenceLevel));
        confidenceBounds.add(new EwbCalcResultKeyValueVo(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信法, languageKey), EwbConfidenceMethodCodeEnum.getByLaguageKey(param.getMethod(), languageKey)));
        output.setConfidenceBounds(confidenceBounds);
    }

    private void getChartResult(EwbAnalysisStartVo param, EwbReliabilityPlotVo output, String ploryType) {
        // 作图结果
        EwbChartDataModel ldChartDataModel = chartDataService.getContourMapChartData(param.getAnalysisId(), ploryType).get(0);
        output.setLdChartDataModel(ldChartDataModel);
    }

    /**
     * 生成ss曲线图数据
     *
     * @param param
     * @return
     */
    public ServerResponse<HashMap<String, Object>> stressStrengthPlot(EwbAnalysisStressStrengthVo param) throws Exception {
        String serverSystem = allVariableConfig.getServerSystem();
        EwbStressStrength stressStrength = stressStrengthMapper.selectById(param.getStressStrengthId());
        List<HashMap<String, Object>> calResult = new ArrayList<>();
        HashMap<String, Object> cal = new HashMap<>();
        HashMap<String, Object> compareCal = new HashMap<>();
        if (param.getBeta() != null && param.getEta() != null && EwbDistributingCodeEnum.Weibull.getIndex().equals(param.getDistributingCode())) {
            cal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.beta), param.getBeta());
            cal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.eta), param.getEta());
            stressStrength.setBeta(param.getBeta());
            stressStrength.setEta(param.getEta());
        } else if (param.getMiu() != null && param.getSigma() != null && EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getDistributingCode()) || EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode())) {
            if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getDistributingCode())) {
                cal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logMu), param.getLogMiu());
                cal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logSigma), param.getLogSigma());
                stressStrength.setLogMiu(param.getLogMiu());
                stressStrength.setLogSigma(param.getLogSigma());
            } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode())) {
                cal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mu), param.getMiu());
                cal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.sigma), param.getSigma());
                stressStrength.setMiu(param.getMiu());
                stressStrength.setSigma(param.getSigma());
            }
        } else if (param.getLambda() != null && EwbDistributingCodeEnum.Exponential.getIndex().equals(param.getDistributingCode())) {
            cal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.beta), 1);
            cal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.eta), 1 / param.getLambda());
            stressStrength.setLambda(param.getLambda());
        }
        if (param.getCompareBeta() != null && param.getCompareEta() != null && EwbDistributingCodeEnum.Weibull.getIndex().equals(param.getCompareDistributingCode())) {
            compareCal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.beta), param.getCompareBeta());
            compareCal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.eta), param.getCompareEta());
            stressStrength.setCompareBeta(param.getCompareBeta());
            stressStrength.setCompareEta(param.getCompareEta());
        } else if (param.getCompareMiu() != null && param.getCompareSigma() != null && EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getCompareDistributingCode()) || EwbDistributingCodeEnum.Normal.getIndex().equals(param.getCompareDistributingCode())) {
            if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getCompareDistributingCode())) {
                compareCal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logMu), param.getCompareLogMiu());
                compareCal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logSigma), param.getCompareLogSigma());
                stressStrength.setCompareLogMiu(param.getCompareLogMiu());
                stressStrength.setLogSigma(param.getCompareLogSigma());
            } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode())) {
                compareCal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mu), param.getCompareMiu());
                compareCal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.sigma), param.getCompareSigma());
                stressStrength.setCompareMiu(param.getCompareMiu());
                stressStrength.setCompareSigma(param.getCompareSigma());
            }
        } else if (param.getCompareLambda() != null && EwbDistributingCodeEnum.Exponential.getIndex().equals(param.getCompareDistributingCode())) {
            compareCal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.beta), 1);
            compareCal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.eta), 1 / param.getCompareLambda());
            stressStrength.setCompareLambda(param.getCompareLambda());
        }
        stressStrengthMapper.updateById(stressStrength);
        cal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.analysisType), param.getAnalysisCode());
        cal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.distributionType), EwbDistributingCodeEnum.Exponential.getIndex().equals(param.getDistributingCode()) ? EwbDistributingCodeEnum.Weibull.getName() : EwbDistributingCodeEnum.getName(param.getDistributingCode()));
        compareCal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.analysisType), param.getCompareAnalysisCode());
        compareCal.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.distributionType), EwbDistributingCodeEnum.Exponential.getIndex().equals(param.getCompareDistributingCode()) ? EwbDistributingCodeEnum.Weibull.getName() : EwbDistributingCodeEnum.getName(param.getCompareDistributingCode()));
        HashMap<String, Object> dataSetOne = new HashMap<>();
        HashMap<String, Object> dataSetTwo = new HashMap<>();
        dataSetOne.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results), cal);
        dataSetTwo.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results), compareCal);
        if (EwbConstants.SERVER_SYSTEM_LINUX.equals(serverSystem)) {
            JSONObject data = new JSONObject();
            data.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), new double[0]);
            JSONObject data1 = new JSONObject();
            data1.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), new double[0]);
            dataSetOne.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data), data);
            dataSetTwo.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data), data1);
        }
        List<HashMap<String, Object>> dataSetList = new ArrayList<>();
        dataSetList.add(dataSetOne);
        dataSetList.add(dataSetTwo);
        HashMap<String, Object> dataSets = new HashMap<>();
        dataSets.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataSets), dataSetList);
        HashMap<String, Object> plotData = new HashMap<>();
        plotData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.plotData), dataSets);
        plotData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.totalNumofDataSets), 2);
        if (EwbConstants.SERVER_SYSTEM_LINUX.equals(serverSystem)) {
            plotData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data), new JSONObject());
        }
        calResult.add(cal);
        calResult.add(compareCal);
        String url = rgaApiService.getApiPlotUrl(null, null, EwbConstants.PLOT_TYPE_STRESSSTRENGTH, null);
        String calPlotUrl = url + (EwbConstants.SERVER_SYSTEM_LINUX.equals(serverSystem) ? "StressStrength/StressStrengthPlotData" : "/StressStrengthPlot/StressStrengthPlotData");
        String calResultUrl = url + "StressStrength/ProbStrengthGreaterThanStress";

        String responseResultPlot;
        String responseResultCal = rgaApiService.getResultsCallApi(JSON.toJSONString(calResult), calResultUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);

        double xmin = 0.0;
        double xmax = 0.0;
        // 未传入xy区间，计算x最大最小值
        if (param.getXaxisStart() == null) {
            responseResultPlot = rgaApiService.getResultsCallApi(JSON.toJSONString(plotData), calPlotUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
            JSONArray lineDataXys = JSONObject.parseObject(responseResultPlot).getJSONArray(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lineDataXY));
            for (Object dataXy : lineDataXys) {
                JSONArray lineDatas = JSONArray.parseArray(JSONObject.parseObject(dataXy.toString()).get(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.line)).toString());
                for (Object lineData : lineDatas) {
                    double x = Double.parseDouble(JSONObject.parseObject(lineData.toString()).get(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.x)).toString());
                    xmax = xmax < x ? x : xmax;
                }
            }
            xmax = xmax * 1.2;
        } else {
            xmin = param.getXaxisStart();
            xmax = param.getXaxisEnd();
        }
        HashMap<String, Object> xAxis = new HashMap<>();
        xAxis.put("AutoScale", false);
        xAxis.put("Minimum", xmin);
        xAxis.put("Maximum", Math.round(xmax));
        HashMap<String, Object> settings = new HashMap<>();
        settings.put("XAxis", xAxis);
        settings.put("Resolution", "VeryHigh");
        plotData.put("Settings", settings);
        responseResultPlot = rgaApiService.getResultsCallApi(JSON.toJSONString(plotData), calPlotUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        HashMap<String, Object> result = new HashMap<>();
        result.put("calResult", responseResultCal);
        result.put("plotResult", JSONObject.parseObject(responseResultPlot));
        result.put("dates", new String[]{DateTimeUtil.dateToStr(new Date(), DateTimeUtil.STANDARD_FORMAT_NOH), DateTimeUtil.dateToStr(new Date(), "HH:mm:ss")});
        HashMap<String, Object> xyInterval = new HashMap<>();
        xyInterval.put("xaxisStart", Math.floor(xmin));
        xyInterval.put("xaxisEnd", Math.floor(xmax));
        xyInterval.put("yaxisStart", param.getYaxisStart());
        xyInterval.put("yaxisEnd", param.getYaxisEnd());
        result.put("xyInterval", xyInterval);
        return ServerResponse.createBySuccess(result);
    }

    /**
     * 更新画图的标记信息
     *
     * @param param
     * @return
     */
    public ServerResponse updateMarkInfo(EwbAnalysisSetting param, Integer languageKey) {
        EwbAnalysisSetting analysisSetting = ldAnalysisSettingMapper.selectById(param.getSettingId());
        // 判断前端传入的不可靠度是否合法
        if (param.getUnReliability() != null) {
            EwbChartController.calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.不可靠度, languageKey), String.valueOf(param.getUnReliability()), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        }
        // 判断前端传入的不可靠度是否合法
        if (param.getYaxisEnd() != null) {
            EwbChartController.calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.Y, languageKey), String.valueOf(param.getYaxisEnd()), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
        }
        // 将前端传入的最新的设置更新至原有设置中
        analysisSetting.setXaxisStart(param.getXaxisStart());
        analysisSetting.setXaxisEnd(param.getXaxisEnd());
        analysisSetting.setYaxisStart(param.getYaxisStart());
        analysisSetting.setYaxisEnd(param.getYaxisEnd());
        analysisSetting.setTime(param.getTime());
        analysisSetting.setUnReliability(param.getUnReliability());
        analysisSetting.setBetaMark(param.getBetaMark());
        analysisSetting.setEtaMark(param.getEtaMark());
        analysisSetting.setSigmaMark(param.getSigmaMark());
        analysisSetting.setMuMark(param.getMuMark());
        analysisSetting.setLambdaMark(param.getLambdaMark());
        analysisSetting.setTargetTime(param.getTargetTime());
        analysisSetting.setTargetReliability(param.getTargetReliability());
        ldAnalysisSettingMapper.updateById(analysisSetting);
        return ServerResponse.createBySuccess();
    }

    /**
     * 更新分布概率图的图形的名称
     *
     * @param param
     * @return
     */
    public ServerResponse updateReliabilityPlotName(EwbAnalysisSetting param) {
        EwbAnalysisSetting analysisSetting = ldAnalysisSettingMapper.selectById(param.getSettingId());
        analysisSetting.setReliabilityPlotName(param.getReliabilityPlotName());
        ldAnalysisSettingMapper.updateById(analysisSetting);
        return ServerResponse.createBySuccess();
    }
}
