package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.bo.EwbCalcRandomBo;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.dto.EwbCalcRandomDto;
import com.nationrel.modules.weibull.entity.EwbAnalysisData;
import com.nationrel.modules.weibull.entity.EwbAnalysisDataDetail;
import com.nationrel.modules.weibull.entity.EwbAnalysisSetting;
import com.nationrel.modules.weibull.enums.EwbDistributingCodeEnum;
import com.nationrel.modules.weibull.enums.EwbEvaluateCodeEnum;
import com.nationrel.modules.weibull.enums.EwbParameterCodeEnum;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDataDetailMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDataMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisSettingMapper;
import com.nationrel.modules.weibull.util.ValidateUtil;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aimee on 2020/9/24.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbRandomService {

    private final SaveDataDetailService saveService;
    private final RgaApiService rgaApiService;
    private final EwbAnalysisSettingMapper analysisSettingMapper;
    private final EwbAnalysisDataDetailMapper ldAnalysisDataDetailMapper;
    private final EwbAnalysisService ewbAnalysisService;
    private final EwbAnalysisDataMapper ewbAnalysisDataMapper;
    private final AllVariableConfig allVariableConfig;

    @Transactional(rollbackFor = Exception.class)
    public ServerResponse random(HeaderUserInfo userInfo, EwbCalcRandomBo param) {
        // 数据ID
        String dataId = param.getDataId();
        // 设定ID
        String settingId = param.getSettingId();
        // 删除寿命数据分析后产生的其他相关数据
        ldAnalysisDataDetailMapper.deleteByDataId(dataId);
        EwbAnalysisData ewbAnalysisData = ewbAnalysisDataMapper.selectOne(new QueryWrapper<EwbAnalysisData>().lambda().eq(EwbAnalysisData::getAnalysisId, param.getAnalysisId()));
//        deleteData(dataId, settingId);
        if (EwbConstants.RIGHT_CENSORED.equals(param.getCensoring())) {
            ewbAnalysisData.setRFlag(EwbConstants.FLAG_CHECKED);
            ewbAnalysisDataMapper.updateByPrimaryKey(ewbAnalysisData);
        }
        JSONObject queryData = new JSONObject();
        // 必填的两个参数
        queryData.put("AnalysisType", EwbEvaluateCodeEnum.getCode(param.getEstimationMethodCode()));
        queryData.put("DistributionType", EwbDistributingCodeEnum.getName(param.getDistributingCode()));
        EwbAnalysisSetting ewbAnalysisSetting = analysisSettingMapper.selectById(settingId);
        // 选择weibull所需参数转换
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(param.getDistributingCode())) {
            queryData.put("Beta", WeibullNumberUtil.parseDouble(param.getBeta()));
            queryData.put("Eta", WeibullNumberUtil.parseDouble(param.getEta()));
            ewbAnalysisSetting.setDistributingCode(EwbDistributingCodeEnum.Weibull.getIndex());
            ewbAnalysisSetting.setParameterCode(EwbParameterCodeEnum.二参数.getIndex());
        }

        // 选择lognormal所需参数转换
        if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getDistributingCode())) {
            queryData.put("LogSigma", WeibullNumberUtil.parseDouble(param.getLogSigma()));
            queryData.put("LogMu", WeibullNumberUtil.parseDouble(param.getLogMean()));
            ewbAnalysisSetting.setDistributingCode(EwbDistributingCodeEnum.Lognormal.getIndex());
            ewbAnalysisSetting.setParameterCode(EwbParameterCodeEnum.二参数.getIndex());
        }

        // 选择normal所需参数转换
        if (EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode())) {
            queryData.put("Sigma", WeibullNumberUtil.parseDouble(param.getSigma()));
            queryData.put("Mu", WeibullNumberUtil.parseDouble(param.getMean()));
            ewbAnalysisSetting.setDistributingCode(EwbDistributingCodeEnum.Normal.getIndex());
            ewbAnalysisSetting.setParameterCode(EwbParameterCodeEnum.二参数.getIndex());
        }

        // 选择exponential所需参数转换
        if (EwbDistributingCodeEnum.Exponential.getIndex().equals(param.getDistributingCode())) {
            queryData.put("Lambda", WeibullNumberUtil.parseDouble(param.getLamda()));
            ewbAnalysisSetting.setDistributingCode(EwbDistributingCodeEnum.Exponential.getIndex());
            ewbAnalysisSetting.setParameterCode(EwbParameterCodeEnum.一参数.getIndex());
        }

        // 失效数
        int failureQuantity = 0;
        if (EwbConstants.RIGHT_CENSORED.equals(param.getCensoring()) && ValidateUtil.isNotEmpty(param.getFailureQuantity())) {
            failureQuantity = WeibullNumberUtil.parseInteger(param.getFailureQuantity());
        }

        // 生成点个数
        int produceQuantity = WeibullNumberUtil.parseInteger(param.getProduceQuantity());
        // 获取请求生成随机数接口的路径
        String url = allVariableConfig.getReliaQubeUrl() + RgaApiService.getApiRadomUrl(failureQuantity, produceQuantity);
        String resultsRandomDatas;
        try {
            resultsRandomDatas = rgaApiService.getResultsCallApi(queryData.toString(), url, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
        } catch (Exception e){
            throw new DescribeException(EwbCustomPromptEnum.请求生成随机数接口出错, userInfo.getLanguageKey());
        }
        List<EwbCalcRandomDto> randomDtos = JSONArray.parseArray(resultsRandomDatas, EwbCalcRandomDto.class);

        // 分组数据集合
        List<EwbAnalysisDataDetail> listGroup = new ArrayList<>();

        int dataNo = 1;
        int lastIndex;
        for (int i = 0; i < produceQuantity; i++) {
            EwbAnalysisDataDetail ld = new EwbAnalysisDataDetail();
            ld.setDataId(dataId);
            ld.setDataNo(dataNo++);
            ld.setQuantity(EwbConstants.LIFEDATA_QUANTITY_DEFAULT);
            lastIndex = i >= randomDtos.size() ? randomDtos.size() - 1 : i;
            // 结束时间
            ld.setFailureEndTime(randomDtos.get(lastIndex).getTime());
            // 最近检查时间
            ld.setLastInspectTime(randomDtos.get(lastIndex).getTime());
            // 失效时间
            ld.setFailureTime(randomDtos.get(lastIndex).getTime());
            // F/S
            ld.setFailureSuspension("F".equals(randomDtos.get(lastIndex).getFs()) ? EwbConstants.STATE_F : EwbConstants.STATE_S);
            // 添加随机数到集合
            listGroup.add(ld);
            if (listGroup.size() > EwbConstants.MAX_BATCH_NUM) {
                // 批量插入分组明细数据
                ldAnalysisDataDetailMapper.batchInsert(userInfo, listGroup);
                listGroup.clear();
            }
        }
        if (listGroup.size() > 0) {
            // 批量插入分组明细数据
            ldAnalysisDataDetailMapper.batchInsert(userInfo, listGroup);
            listGroup.clear();
        }
        // 若time不为空则将之前新增的failureEndTime>time的数据f/s置为s
        if (EwbConstants.TIME_CENSORED.equals(param.getCensoring()) && ValidateUtil.isNotEmpty(param.getTime())) {
            int updateNum = ldAnalysisDataDetailMapper.updateSusByTime(dataId, WeibullNumberUtil.parseDouble(param.getTime()));
            if (updateNum > 0) {
                ewbAnalysisData.setRFlag(EwbConstants.FLAG_CHECKED);
                ewbAnalysisDataMapper.updateByPrimaryKey(ewbAnalysisData);
            }
        }
        // ==================
        // 更新寿命数据分析数据明细表的各种类型的数据量
        // ==================
        saveService.updateAnalysisData(userInfo, dataId);
        ewbAnalysisData.setTimeForRank(EwbConstants.FLAG_NO_CHECKED);
        ewbAnalysisDataMapper.updateById(ewbAnalysisData);
        analysisSettingMapper.updateById(ewbAnalysisSetting);
        // 生成之后清除分析结果
        ewbAnalysisService.deleteAnalysisResultData(dataId, settingId, param.getAnalysisId());
        // 返回结果
        return ServerResponse.createBySuccess();
    }
//
//    /**
//     * @Title: deleteData
//     * @Description:删除寿命数据分析后产生的其他相关数据
//     * @param dataId 数据ID
//     * @param settingId 设定ID
//     */
//    public void deleteData(String dataId, String settingId) {
//        // 删除编辑分析明细数据
//        ldAnalysisDataDetailMapper.deleteByDataId(dataId);
//        // 删除寿命数据分析秩表数据
//        ldAnalysisRankMapper.deleteByDataId(dataId);
//        // 删除寿命数据分析结果表数据
//        ldAnalysisResultMapper.deleteById(settingId);
//        // 删除寿命数据分析结果明细表数据
//        ldAnalysisResultDetailMapper.deleteAnalysisResultDetail(settingId);
//        // 删除寿命数据分析概率图置信数据表数据
//        ldAnalysisPrBelieveMapper.deleteBySettingId(settingId);
//        // 删除寿命数据分析失效率图置信数据表数据
//        ldAnalysisFrBelieveMapper.deleteBySettingId(settingId);
//        // 删除寿命数据分析设定表数据
////        ldAnalysisSettingMapper.deleteById(settingId);
//        // 删除寿命数据分布向导表数据
//        ldAnalysisDisWizardMapper.deleteByDataId(dataId);
//    }

    // 创建寿命数据分析数据明细实体
//    EwbAnalysisDataDetail ld = new EwbAnalysisDataDetail();
//
//    // weibull
//            if (EwbDistributingCodeEnum.Weibull.getIndex().equals(param.getDistributingCode())) {
//        // 2参数
//        if (EwbParameterCodeEnum.二参数.getIndex().equals(param.getEstimationMethodCode())) {
//            // 生成随机数
//            failureEndTime = random.nextWeibull(beta, eta);
//            // 3 参数
//        } else if (EwbParameterCodeEnum.三参数.getIndex().equals(param.getEstimationMethodCode())) {
//            // 生成随机数
//            // failureEndTime = random.nextWeibull(gamma, sigma, beta);
//        }
//        // Exponential
//    } else if (EwbDistributingCodeEnum.Exponential.getIndex().equals(param.getDistributingCode())) {
//        // 1参数
//        if (EwbParameterCodeEnum.一参数.getIndex().equals(param.getEstimationMethodCode())) {
//            // 生成随机数
//            failureEndTime = random.nextExponential(lamda);
//            // 2 参数
//        } else if (EwbParameterCodeEnum.二参数.getIndex().equals(param.getEstimationMethodCode())) {
//            // 生成随机数
//            // failureEndTime = random.nextExponential(lamda, gamma);
//        }
//        // Normal
//    } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode())) {
//        // 生成随机数
//        failureEndTime = random.nextGaussian(mean, sigma) ;
//        // Lognormal
//    } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getDistributingCode())) {
//        // 生成随机数
//        failureEndTime = Math.exp(random.nextGaussian(mean, sigma));
//    }
//    failureEndTime = NumberUtils.round(failureEndTime, EwbConstants.NUM_DIGIT_4);
//    // 随机数为负数则重新生成
//            if (failureEndTime <= 0 || failureEndTime > EwbConstants.TAB_FAILURE_END_TIME_MAX.doubleValue()) {
//        ++produceQuantity;
//        ++produceNegativeQuantity;
//        continue;
//    }
//
//    // dataId
//            ld.setDataId(dataId);
//
//    // dataNo
//            ld.setDataNo(dataNo++);
//
//    // 数量
//            ld.setQuantity(EwbConstants.LIFEDATA_QUANTITY_DEFAULT);
//
//    // 无删失数据
//            if (EwbConstants.NONE_CENSORED.equals(param.getCensoring())) {
//        // F或S状态
//        ld.setFailureSuspension(EwbConstants.STATE_F);
//        // 数据类型
//        ld.setDataType(EwbConstants.COMPLETE_DATA_TYPE);
//
//        // 生成指定数量的失效数据，其余为右删失数据
//    } else if (EwbConstants.RIGHT_CENSORED.equals(param.getCensoring())) {
//        if (tempFailureEndTime <= failureEndTime) {
//            tempFailureEndTime = failureEndTime;
//        }
//        // 如果页面选择了"指定数量失效后的数据为右删失数据 "则，取数规则为: 例如 失效数为5， 则从第6个开始时删失，在指定时间后的删失 和 第五个一样，event为s
//        if (i + 1 > failureQuantity + produceNegativeQuantity) {
//            // F或S状态
//            ld.setFailureSuspension(EwbConstants.STATE_S);
//            failureEndTime = tempFailureEndTime;
//        } else {
//            // F或S状态
//            ld.setFailureSuspension(EwbConstants.STATE_F);
//        }
//        // 数据类型
//        ld.setDataType(EwbConstants.RIGHT_DATA_TYPE);
//
//        // 指定失效时间，大于该时间的为右删失数据
//    } else if (EwbConstants.TIME_CENSORED.equals(param.getCensoring())) {
//        if (failureEndTime > time) {
//            if (tempFailureEndTime == 0) {
//                tempFailureEndTime = failureEndTime;
//            }
//            failureEndTime = tempFailureEndTime;
//            // F或S状态
//            ld.setFailureSuspension(EwbConstants.STATE_S);
//        } else {
//            // F或S状态
//            ld.setFailureSuspension(EwbConstants.STATE_F);
//        }
//        // 数据类型
//        ld.setDataType(EwbConstants.RIGHT_DATA_TYPE);
//    }
//
//    // 结束时间
//            ld.setFailureEndTime(failureEndTime);
//
//    // 最近检查时间
//            ld.setLastInspectTime(failureEndTime);
//
//    // 失效时间
//            ld.setFailureTime(failureEndTime);
}
