package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.weibull.general.domain.enums.reliaqube.ReliaqubeProgramExceptionEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDataDetailMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisSettingMapper;
import com.nationrel.modules.weibull.entity.EwbAnalysisDataDetail;
import com.nationrel.modules.weibull.entity.EwbAnalysisSetting;
import com.nationrel.modules.weibull.enums.EwbDistributingCodeEnum;
import com.nationrel.modules.weibull.enums.EwbEvaluateCodeEnum;
import com.nationrel.modules.weibull.enums.EwbParameterCodeEnum;
import com.nationrel.modules.weibull.vo.EwbAnalysisDataDetailVo;
import com.nationrel.modules.weibull.vo.EwbAnalysisDataMaxMinSVo;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.util.ValidateUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Aimee on 2020/10/28.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbValidatorService {

    private final EwbAnalysisDataDetailMapper dataDetailMapper;
    private final EwbAnalysisSettingMapper settingMapper;
    private final EwbAnalysisDataDetailMapper ldAnalysisDataDetailMapper;

    /**
     * @param dataId    数据Id
     * @param settingId 设定Id
     * @Title: validatorDataDetail
     * @Description: 明细数据业务校验
     * 消息处理辅助类
     */
    public void validatorDataDetail(String dataId, String settingId, String parameterCode, String distributingCode, String estimationMethodCode, String rightFlag, HeaderUserInfo userInfo) {
        // ==================
        // 1.获取寿命数据-数据量信息
        // ==================
        Integer sumQuantity = dataDetailMapper.sumQuantity(dataId);
        if (sumQuantity == null) {
            throw new DescribeException(ReliaqubeParamDataCheckEnum.暂未添加数据, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
        }

        // ==================
        // 2.关于"完整数据"校验的警告提示信息
        // ==================
        if (StringUtils.isBlank(distributingCode)) {

            EwbAnalysisSetting ewbAnalysisSetting = settingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select("distributing_code", "estimation_method_code", "parameter_code").lambda().eq(EwbAnalysisSetting::getSettingId, settingId));
            // 分布类型
            distributingCode = ewbAnalysisSetting.getDistributingCode();
            // 参数估计方法
            estimationMethodCode = ewbAnalysisSetting.getEstimationMethodCode();
            // 参数类型
            parameterCode = ewbAnalysisSetting.getParameterCode();
        }

        // 混合3参数必须要有12条数据
        if (parameterCode.equals(EwbParameterCodeEnum.混合3参数.getIndex())) {
            int dataCount = dataDetailMapper.selectDiffPointCount(dataId).size();
            if (dataCount < 15) {
                throw new DescribeException(EwbCustomPromptEnum.至少需要15条F数据才能计算, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
        } else if (parameterCode.equals(EwbParameterCodeEnum.混合2参数.getIndex())) {
            int dataCount = dataDetailMapper.selectDiffPointCount(dataId).size();
            if (dataCount < 6) {
                throw new DescribeException(EwbCustomPromptEnum.必须至少具有6个不同的故障点才能计算, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
        }

        // ==================
        // 3.关于mle方法的数据校验
        // ==================
        // 如果页面选择的MLE方法
        int countLeftByFtEqu = dataDetailMapper.countfCompleteByFtEqu(dataId);
        int countRightByFtEqu = dataDetailMapper.countsCompleteByFtEqu(dataId);
        if (EwbEvaluateCodeEnum.mle.getIndex().equals(estimationMethodCode)) {
            if (countLeftByFtEqu == 1) {
                if (countRightByFtEqu == 0 && (!EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode) || (!EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode) && !EwbDistributingCodeEnum.Exponential.getIndex().equals(distributingCode)))) {
                    throw new DescribeException(EwbCustomPromptEnum.只有一个失效数据或者所有失效数据相等则只有1参数的weibull分布和1参数的指数分布适合, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataWeibullMleException.getIndex());
                } else {
                    double maxF = dataDetailMapper.searchMaxFtime(dataId);
                    EwbAnalysisDataMaxMinSVo minAndMaxS = dataDetailMapper.searchMaxAndMinStime(dataId);
                    if (null != minAndMaxS && maxF > minAndMaxS.getMaxSuspensionTime() && !EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                        // 仅有一个失效且失效时间大于最大右删失数据且分布不为weibull和Exponential且不是一参数
                        throw new DescribeException(EwbCustomPromptEnum.只有一个失效数据或者所有失效数据相等且失效时间大于最大右删失数据可选择MLE方法, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataWeibullMleException.getIndex());
                    }
                }
            } else {
                this.zerofValid(countLeftByFtEqu, parameterCode, distributingCode, userInfo);
            }
        } else {
            // 仅有一个失效时
            if (countLeftByFtEqu == 1 && countRightByFtEqu > 0) {
                double maxF = dataDetailMapper.searchMaxFtime(dataId);
                EwbAnalysisDataMaxMinSVo minAndMaxS = dataDetailMapper.searchMaxAndMinStime(dataId);
                if (maxF < minAndMaxS.getMinSuspensionTime()) {
                    // 仅有一个失效且失效时间小于最小右删失数据
                    throw new DescribeException(EwbCustomPromptEnum.只有一个失效数据或者所有失效数据相等且失效时间小于最小右删失数据可选择MLE方法, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataMleException.getIndex());
                } else if (maxF > minAndMaxS.getMaxSuspensionTime() && !EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode) && !EwbDistributingCodeEnum.Exponential.getIndex().equals(distributingCode)) {
                    // 仅有一个失效且失效时间大于最大右删失数据且分布不为weibull和Exponential
                    throw new DescribeException(EwbCustomPromptEnum.只有一个失效数据或者所有失效数据相等且失效时间大于最大右删失数据可选择MLE方法, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataWeibullMleException.getIndex());
                }
            }
            this.zerofValid(countLeftByFtEqu, parameterCode, distributingCode, userInfo);
            if (!EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode) && countLeftByFtEqu < 2) {
                throw new DescribeException(EwbCustomPromptEnum.只有一个失效数据则只有1参数MLE方法适合, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataWeibullMleException.getIndex());
            }
        }
        // 如果是两参数或者三参数且计算区间数据，需要两个不完全包含的区间信息
        if ((EwbParameterCodeEnum.二参数.getIndex().equals(parameterCode) || EwbParameterCodeEnum.三参数.getIndex().equals(parameterCode)) && EwbConstants.FLAG_CHECKED.equals(rightFlag)) {
            List<EwbAnalysisDataDetail> analysisDataDetails = ldAnalysisDataDetailMapper.selectList(new QueryWrapper<EwbAnalysisDataDetail>().lambda().eq(EwbAnalysisDataDetail::getDataId, dataId));
            if (seperatedInterval(analysisDataDetails).size() < 2) {
                throw new DescribeException(EwbCustomPromptEnum.P2P3的区间数据计算至少需要两个不完全重合的区间, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
        }
        ServerResponse.createBySuccess();
    }

    /**
     * 0F验证
     * @param countLeftByFtEqu
     * @param parameterCode
     * @param distributingCode
     * @param userInfo
     */
    private void zerofValid(int countLeftByFtEqu, String parameterCode, String distributingCode, HeaderUserInfo userInfo) {
        if (countLeftByFtEqu == 0) {
            if (!EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                throw new DescribeException(EwbCustomPromptEnum.当前寿命数据都为右删失只能选择1参数weibull分布MLE方法或者1参数指数分布MLE方法, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataMleBoundException.getIndex());
            } else if (!EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode) && !EwbDistributingCodeEnum.Exponential.getIndex().equals(distributingCode)) {
                throw new DescribeException(EwbCustomPromptEnum.当前寿命数据都为右删失只能选择1参数weibull分布MLE方法或者1参数指数分布MLE方法, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataMleBoundException.getIndex());
            }
        }
    }
    /**
     * 判断是否存在完全不想交的区间
     *
     * @param analysisDataDetails
     * @return
     */
    public List<EwbAnalysisDataDetail> seperatedInterval(List<EwbAnalysisDataDetail> analysisDataDetails) {
        boolean hasIntersection = false;
        List<EwbAnalysisDataDetail> updatedList = new ArrayList<>();
        for (int i = 0; i < analysisDataDetails.size(); i++) {
            for (int j = i + 1; j < analysisDataDetails.size(); j++) {
                // 此处两个区间肯定不相交，则无需判断
                if (analysisDataDetails.get(i).getLastInspectTime() >= analysisDataDetails.get(j).getFailureEndTime()) {
                    continue;
                }
                // 判断区间是否相交
                if (analysisDataDetails.get(i).getFailureEndTime() > analysisDataDetails.get(j).getLastInspectTime()) {
                    updatedList.add(EwbAnalysisDataDetail.builder().lastInspectTime(Math.max(analysisDataDetails.get(i).getLastInspectTime(), analysisDataDetails.get(j).getLastInspectTime())).failureEndTime(Math.min(analysisDataDetails.get(i).getFailureEndTime(), analysisDataDetails.get(j).getFailureEndTime())).build());
                    if (updatedList.size() >= 2) {
                        return updatedList;
                    }
                    hasIntersection = true;
                }
            }
            if (!hasIntersection) {
                updatedList.add(analysisDataDetails.get(i));
                if (updatedList.size() >= 2) {
                    return updatedList;
                }
            }
        }
        if (hasIntersection) {
            updatedList = seperatedInterval(updatedList);
        }
        return updatedList;
    }

    /**
     * @param lifeData     分析明细数据
     * @param groupingFlag 页面复选框"分组数据"选中标记
     * @param startDataNo  当前页面开始数据编号
     * @param rightFlag    页面复选框"包含右删失数据"选中标记标记
     * @param endDataNo    当前页结束数据编号
     * @param dataId       数据ID
     * @Title: validatorAnalysisDetailData
     * @Description: 分析明细数据输入校验
     * 消息处理辅助类
     */
    public void validatorAnalysisDetailData(List<String[]> lifeData, String groupingFlag, int startDataNo, String rightFlag, int endDataNo, String dataId, Integer languageKey) {
        // 当前页数据总数量
        int currentPageQuantity = 0;
        if (lifeData != null && lifeData.size() > 0) {
            // 判断页面明细数据列表是否有值
            int rowNum = 0;
            if (startDataNo > 0) {
                rowNum = startDataNo - 1;
            }
            rowNum = rowNum % 10;

            for (String[] arr : lifeData) {
                if (arr != null && ValidateUtil.isNotEmpty(arr[1])) {
                    rowNum++;
                    // 获取数量
                    String quantity = arr[0];
                    // 页面复选框"分组数据"选中
                    if (EwbConstants.FLAG_CHECKED.equals(groupingFlag)) {
                        // 数量校验
                        this.validatorQuantity(rowNum, quantity, languageKey);
                    } else {
                        quantity = String.valueOf(1);
                    }
                    if (ValidateUtil.isNotEmpty(quantity)
                            && ValidateUtil.isInteger(quantity)) {
                        currentPageQuantity = currentPageQuantity + Integer.parseInt(quantity);
                    }

                    // 取得F或者S的状态
                    String failureSuspension = arr[2];
                    // 页面复选框"包含右删失数据"选中
                    if (EwbConstants.FLAG_CHECKED.equals(rightFlag)) {
                        // F或者S的状态校验
                        this.validatorFs(rowNum, failureSuspension, languageKey);
//                        if (!messageSupport.hasErrors()) {
//                            analysisDataDetail.setFailureSuspension(dataDetailModel.getFailureSuspension());
//                        }
                    }
                }
            }
        }
        this.validateAnalysisTotalityQuantity(dataId, startDataNo, endDataNo, currentPageQuantity);
    }

    /**
     * @param lifeData     分析明细数据
     * @param groupingFlag 页面复选框"分组数据"选中标记
     * @param startDataNo  当前页面开始数据编号
     * @param leftFlag     页面复选框"包含区间删失数据或左删失数据"选中
     * @param rightFlag    页面复选框"包含右删失数据"选中标记标记
     * @param endDataNo    当前页结束数据编号
     * @param dataId       数据ID
     * @Title: validatorAnalysisDetailData
     * @Description: 分析明细数据输入校验
     * 消息处理辅助类
     */
    public void validatorAnalysisDetailData(String lifeData, String groupingFlag, int startDataNo, String
            leftFlag, String rightFlag, int endDataNo, String dataId, Integer languageKey) {
        // 将Json字符窜转换为Json数组
        // 将Json数组转为泛型List<LdAnalysisDataDetail>
        List<EwbAnalysisDataDetailVo> dataDetailsModel = JSONArray.parseArray(lifeData, EwbAnalysisDataDetailVo.class);
        // 当前页数据总数量
        int currentPageQuantity = 0;
        if (dataDetailsModel != null && dataDetailsModel.size() > 0) {
            // 删除倒数第一个空白行对象
            EwbAnalysisDataDetailVo dataDetail = dataDetailsModel.get(dataDetailsModel.size() - EwbConstants.NUM_ONE);
            if (ValidateUtil.isEmpty(dataDetail.getQuantity()) && ValidateUtil.isEmpty(dataDetail.getFailureEndTime()) && ValidateUtil.isEmpty(dataDetail.getLastInspectTime()) && ValidateUtil.isEmpty(dataDetail.getFailureSuspension())) {
                dataDetailsModel.remove(dataDetailsModel.size() - EwbConstants.NUM_ONE);
            }
            // 删除倒数第二个空白行对象
            EwbAnalysisDataDetailVo dataDetail2 = dataDetailsModel.get(dataDetailsModel.size() - EwbConstants.NUM_ONE);
            if (ValidateUtil.isEmpty(dataDetail2.getQuantity()) && ValidateUtil.isEmpty(dataDetail2.getFailureEndTime()) && ValidateUtil.isEmpty(dataDetail2.getLastInspectTime()) && ValidateUtil.isEmpty(dataDetail2.getFailureSuspension())) {
                dataDetailsModel.remove(dataDetailsModel.size() - EwbConstants.NUM_ONE);
            }
            //if (rowTotal <= EwbConstants.INTEGER_TEN) {
            // 判断页面明细数据列表是否有值
            if (dataDetailsModel.size() <= 0) {
                throw new DescribeException(ReliaqubeParamDataCheckEnum.暂未添加数据, languageKey);
            }
            // 行号
            int rowNum = 0;
            if (startDataNo > 0) {
                rowNum = startDataNo - 1;
            }
            rowNum = rowNum % 10;

            for (EwbAnalysisDataDetailVo dataDetailModel : dataDetailsModel) {
                if (dataDetailModel != null && ValidateUtil.isNotEmpty(dataDetailModel.getFailureEndTime())) {
                    EwbAnalysisDataDetail analysisDataDetail = new EwbAnalysisDataDetail();
                    rowNum++;
                    // 获取数量
                    String quantity = dataDetailModel.getQuantity();
                    // 页面复选框"分组数据"选中
                    if (EwbConstants.FLAG_CHECKED.equals(groupingFlag)) {
                        // 数量校验
                        this.validatorQuantity(rowNum, quantity, languageKey);
                    } else {
                        quantity = String.valueOf(1);
                    }
                    if (ValidateUtil.isNotEmpty(quantity)
                            && ValidateUtil.isInteger(quantity)) {
                        currentPageQuantity = currentPageQuantity + Integer.parseInt(quantity);
                    }
                    // 获取最近检查时间
                    String lastInspectTime = dataDetailModel.getLastInspectTime();
                    // 页面复选框"包含区间删失数据或左删失数据"选中
                    if (EwbConstants.FLAG_CHECKED.equals(leftFlag)) {
                        // 截取指定小数位数的小数
                        lastInspectTime = interceptLastInspectTimeForAnalysis(analysisDataDetail, lastInspectTime);
                        // 最近检查时间校验
                        this.validatorLastInspectTime(rowNum, lastInspectTime, languageKey);
//                        if (!messageSupport.hasErrors()) {
//                            analysisDataDetail.setLastInspectTime(Double.valueOf(lastInspectTime));
//                        }
                    }

                    // 取得F或者S的状态
                    String failureSuspension = dataDetailModel.getFailureSuspension();
                    // 页面复选框"包含右删失数据"选中
                    if (EwbConstants.FLAG_CHECKED.equals(rightFlag)) {
                        // F或者S的状态校验
                        this.validatorFs(rowNum, failureSuspension, languageKey);
//                        if (!messageSupport.hasErrors()) {
//                            analysisDataDetail.setFailureSuspension(dataDetailModel.getFailureSuspension());
//                        }
                    }

                    // 获取结束时间
                    String failureEndTime = dataDetailModel.getFailureEndTime();
                    // 截取指定小数位数的小数
                    failureEndTime = interceptLastInspectTimeForAnalysis(analysisDataDetail, failureEndTime);
                    // 结束时间校验
                    this.validatorFailureEndTime(rowNum, failureEndTime, lastInspectTime, leftFlag, languageKey);
//                    if (!messageSupport.hasErrors()) {
//                        // 给转化后的明细数据对象设置值
//                        analysisDataDetail.setDataType(dataDetailModel.getDataType());
//                        analysisDataDetail.setFailureEndTime(Double.valueOf(dataDetailModel.getFailureEndTime()));
//                        analysisDataDetail.setQuantity(Integer.valueOf(quantity));
//                        dataDetails.add(analysisDataDetail);
//                    }
                }
            }
        }
        this.validateAnalysisTotalityQuantity(dataId, startDataNo, endDataNo, currentPageQuantity);
    }

    /**
     * @param dataId              数据ID
     * @param startDataNo         当前页起始数据编号
     * @param endDataNo           当前页结束数据编号
     * @param currentPageQuantity 当前页数据总数量
     * @Title: validateAnalysisTotalityQuantity
     * @Description:校验寿命数据总数量 消息处理辅助类
     */
    public void validateAnalysisTotalityQuantity(String dataId, int startDataNo, int endDataNo,
                                                 int currentPageQuantity) throws DescribeException {
        // dataId不为空表示数据来源于编辑页面，否则为创建页面数据
        if (ValidateUtil.isNotEmpty(dataId)) {
            int sumQuantity = ldAnalysisDataDetailMapper.statSumQuantity(dataId, startDataNo, endDataNo);
            int totalityQuantity = sumQuantity + currentPageQuantity;
            if (totalityQuantity > EwbConstants.MAX_QUANTITY) {
                // throw new DescribeException("数据总数量必须小于等于1400000", ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
        } else {
            if (currentPageQuantity > EwbConstants.MAX_QUANTITY) {
                // throw new DescribeException("数据总数量必须小于等于1400000", ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
        }
    }

    /**
     * @param rowNum          行数
     * @param failureEndTime  结束时间
     * @param lastInspectTime 最近检查时间
     * @param leftFlag        页面复选框"包含区间删失数据或左删失数据"选中
     * @Title: validatorFailureEndTime
     * @Description: 结束时间校验
     * 消息处理辅助类
     */
    public void validatorFailureEndTime(int rowNum, String failureEndTime, String lastInspectTime, String leftFlag, Integer languageKey) throws
            DescribeException {
        // F或者S不能为空
        if (ValidateUtil.isEmpty(failureEndTime)) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.结束时间不可为空, languageKey);
        }
        // 去掉输入内容的前后空格
        if (ValidateUtil.isNotEmpty(lastInspectTime)) {
            lastInspectTime = lastInspectTime.trim();
        }
        if (ValidateUtil.isNotEmpty(failureEndTime)) {
            failureEndTime = failureEndTime.trim();
        }
        if (!ValidateUtil.isNumber(failureEndTime)) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.结束时间必须为数字, languageKey);
        }
        if (WeibullNumberUtil.parseDouble(failureEndTime) < 0.0001 || WeibullNumberUtil.parseDouble(failureEndTime) > 999999999.9999) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.结束时间范围必须是999999999, languageKey);
        }

        if (EwbConstants.FLAG_CHECKED.equals(leftFlag) && ValidateUtil.isNumber(lastInspectTime) && ValidateUtil.isNumber(failureEndTime) && ValidateUtil.isNotEmpty(failureEndTime) && ValidateUtil.isNotEmpty(lastInspectTime)) {
            BigDecimal valF = WeibullNumberUtil.parseBigDecimal(failureEndTime);
            BigDecimal valL = WeibullNumberUtil.parseBigDecimal(lastInspectTime);
            // 校验结束时间必须大于最近检查时间
            if (valF.compareTo(valL) < 0) {
                throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.结束时间必须大于等于最近检查时间, languageKey);
            }
        }
    }

    /**
     * @param rowNum            行数
     * @param failureSuspension F或者S的状态
     * @Title: validatorFs
     * @Description: F或者S的状态校验
     * 消息处理辅助类
     */
    private void validatorFs(int rowNum, String failureSuspension, Integer languageKey) throws DescribeException {
        // F或者S不能为空
        if (ValidateUtil.isEmpty(failureSuspension)) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.F或者S不可为空, languageKey);
        }
        // 输入必须是f或者s
        if (ValidateUtil.isNotEmpty(failureSuspension) && !(EwbConstants.STATE_SUSPENSION.equalsIgnoreCase(failureSuspension) || EwbConstants.STATE_FAILURE.equalsIgnoreCase(failureSuspension))) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.输入内容必须是F或者S, languageKey);
        }
    }

    /**
     * @param rowNum   行数
     * @param quantity 数量
     * @Title: validatorQuantity
     * @Description: 数量校验
     * 消息处理辅助类
     */
    private void validatorQuantity(int rowNum, String quantity, Integer languageKey) throws DescribeException {
        // 数量不能为空
        if (ValidateUtil.isEmpty(quantity)) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.数量不可为空, languageKey);
        } else if (!ValidateUtil.isInteger(quantity)) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.数量必须为整数, languageKey);
        } else if (WeibullNumberUtil.parseInteger(quantity) < 1 || WeibullNumberUtil.parseInteger(quantity) > 99999) {
//            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.数量范围必须是99999, languageKey);
        }
    }

    /**
     * @param rowNum          行数
     * @param lastInspectTime 最近检查时间
     * @Title: validatorLastInspectTime
     * @Description: 最近检查时间校验
     * 消息处理辅助类
     */
    private void validatorLastInspectTime(int rowNum, String lastInspectTime, Integer languageKey) throws DescribeException {
        if (ValidateUtil.isEmpty(lastInspectTime)) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.最近检查时间不可为空, languageKey);
        } else if (!ValidateUtil.isNumber(lastInspectTime)) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.最近检查时间必须为数字, languageKey);
        } else if (WeibullNumberUtil.parseInteger(lastInspectTime) < 0) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.最近检查时间范围必须大于0, languageKey);
        }
    }

    /**
     * @param analysisDataDetail 转换后的对象
     * @param lastInspectTime    最近检查时间
     * @return 截取之后的小数
     * @Title: interceptLastInspectTimeForAnalysis
     * @Description: 截取最近检查时间的小数位数
     */
    private String interceptLastInspectTimeForAnalysis(EwbAnalysisDataDetail analysisDataDetail, String
            lastInspectTime) {
        if (ValidateUtil.isNotEmpty(lastInspectTime)
                && ValidateUtil.isNumber(lastInspectTime)
                && !ValidateUtil.isMatch(lastInspectTime, "^-?(\\d*|\\d*\\.\\d{1," + EwbConstants.NUM_DIGIT_4 + "})$")) {
            // 截取指定小数位数
            double newLastInspectTime = WeibullNumberUtil.round(Double.valueOf(lastInspectTime), EwbConstants.NUM_DIGIT_4);
            // 赋值
            analysisDataDetail.setLastInspectTime(newLastInspectTime);
            lastInspectTime = String.valueOf(newLastInspectTime);
        }
        return lastInspectTime;
    }
}
