package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.bo.EwbCalcParamBo;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.enums.*;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDataDetailMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDataRankMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDisWizardMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisSettingMapper;
import com.nationrel.modules.weibull.util.ValidateUtil;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.vo.EwbAnalysisStartVo;
import com.nationrel.modules.weibull.vo.SolveDemoRelWeibullChiSquareRelVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.math3.special.Gamma;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author Aimee on 2020/9/22.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RgaApiService {

    private final EwbAnalysisDataDetailMapper ewbAnalysisDataDetailMapper;
    private final EwbAnalysisDataRankMapper ewbAnalysisDataRankMapper;
    private final EwbAnalysisDisWizardMapper ewbAnalysisDisWizardMapper;

    private final AllVariableConfig allVariableConfig;

    private int retry;

    public String getResultsCallApi(String requestData, String url, String contentType, String acceptType)
            throws Exception {
        PostMethod postMethod = null;
        String responseResult = "";
        try {
            HttpClient clientPlot = new HttpClient();
            postMethod = new PostMethod(url);
            System.err.println(requestData);
            System.err.println(url);
            RequestEntity requestEntityPlot = new StringRequestEntity(requestData, "text/xml", "UTF-8");
            postMethod.setRequestEntity(requestEntityPlot);
            postMethod.setRequestHeader("Content-Type", contentType);
            if (ValidateUtil.isNotEmpty(acceptType)) {
                postMethod.setRequestHeader("Accept", acceptType);
            }
            clientPlot.executeMethod(postMethod);
            // responseResult = new
            // String(postMethod.getResponseBodyAsString().getBytes("UTF-8"));
            InputStream inputStream = postMethod.getResponseBodyAsStream();
            if (EwbConstants.SERVER_SYSTEM_LINUX.equals(allVariableConfig.getServerSystem()) && inputStream == null) {
                return "null";
            }
            BufferedReader br = new BufferedReader(new InputStreamReader(inputStream));
            StringBuilder stringBuffer = new StringBuilder();
            String str;
            while ((str = br.readLine()) != null) {
                stringBuffer.append(str);
            }
            responseResult = stringBuffer.toString();
        } catch (Exception e) {
            if (retry < 3) {
                retry++;
                // 如果连接失效，重试
                getResultsCallApi(requestData, url, contentType, acceptType);
            } else {
                retry = 0;
                e.printStackTrace();
                throw new Exception("无法链接服务器");
            }
        } finally {
            Objects.requireNonNull(postMethod).releaseConnection();
        }
        System.err.println("responseResult => " + responseResult);
        return responseResult;
    }


    public String getResultsCallApi(JSONObject requestData, String url, String contentType, String acceptType,
                                    String body) throws Exception {
        System.err.println(body);
        System.err.println(url);
        PostMethod postMethod = null;
        String responseResult = "";
        try {
            HttpClient clientPlot = new HttpClient();
            postMethod = new PostMethod(url);
            RequestEntity requestEntityPlot = new StringRequestEntity(requestData.toString(), "text/xml", "UTF-8");
            postMethod.setRequestEntity(requestEntityPlot);
            postMethod.setRequestHeader("Content-Type", contentType);
            if (ValidateUtil.isNotEmpty(acceptType)) {
                postMethod.setRequestHeader("Accept", acceptType);
            }

            postMethod.setRequestBody(body);
            clientPlot.executeMethod(postMethod);
            // responseResult = new
            // String(postMethod.getResponseBodyAsString().getBytes("UTF-8"));
            InputStream inputStream = postMethod.getResponseBodyAsStream();
            BufferedReader br = new BufferedReader(new InputStreamReader(inputStream));
            StringBuilder stringBuffer = new StringBuilder();
            String str;
            while ((str = br.readLine()) != null) {
                stringBuffer.append(str);
            }
            responseResult = stringBuffer.toString();
        } catch (Exception e) {
            if (retry < 3) {
                retry++;
                // 如果连接失效，重试
                getResultsCallApi(requestData.toString(), url, contentType, acceptType);
            } else {
                retry = 0;
                e.printStackTrace();
                throw new Exception("无法链接服务器");
            }
        } finally {
            Objects.requireNonNull(postMethod).releaseConnection();
        }
        System.err.println("responseResult => " + responseResult);
        return responseResult;
    }

    /**
     * 组装内华达图计算beta/eta/lambda所需的参数
     *
     * @param nevadaDatas
     * @return
     */
    public JSONObject getRequestNevadaDataJson(List<EwbNevadaResultData> nevadaDatas, int increment) {
        String serverSystem = allVariableConfig.getServerSystem();
        // 调用api的输入参数
        JSONObject requestNevadaDataJson = new JSONObject();
        // 组装后的json样本数据
        JSONArray dataArray = getRequestNevadaFailureDataRow(nevadaDatas, increment);
        // 组转最终需要传入接口的数据
        requestNevadaDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureDataRow), dataArray);
        requestNevadaDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.startTime), 0);
        requestNevadaDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.endTime), nevadaDatas.get(nevadaDatas.size() - 1).getSort());
        requestNevadaDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lastFailureTime), nevadaDatas.get(nevadaDatas.size() - 1).getSort());
        requestNevadaDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.terminationType), "FailureTerminated");
        requestNevadaDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.sorted), true);
        requestNevadaDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.systemID), "string");
        return requestNevadaDataJson;
    }

    public JSONArray getRequestNevadaFailureDataRow(List<EwbNevadaResultData> nevadaDatas, int increment) {
        String serverSystem = allVariableConfig.getServerSystem();
        // 组装后的json样本数据
        JSONArray dataArray = new JSONArray();
        // 组装需要传入的数据
        for (int i = 0; i < nevadaDatas.size(); i++) {
            if (increment != 0) {
                nevadaDatas.get(i).setSort((i + 1) * increment);
            }
            JSONObject nodeData = new JSONObject();
            nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.systemID), "string");
            nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureMode), "string");
            nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureSeverity), "string");
            nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.time), increment != 0 ? (i + 1) * increment : nevadaDatas.get(i).getSort());
            nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.numFailures), nevadaDatas.get(i).getData());
            dataArray.add(nodeData);
        }
        return dataArray;
    }

    public static void deleteUselessData(String serverSystem, JSONObject requestPlotData) {
        requestPlotData.remove(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataRow));
        requestPlotData.remove("maxTime");
        requestPlotData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataRow), new String[]{});
    }

    public void addFailureRankAndRow(EwbAnalysisSettingMapper ldAnalysisSettingMapper, JSONObject requestPlotData, EwbAnalysisStartVo param) {
        String serverSystem = allVariableConfig.getServerSystem();
        // 计算似然比需要传入全部的FailureRank和DataRow数据
        if (param.getMethod().equals(EwbConfidenceMethodCodeEnum.likelihoodradio.getIndex()) || param.getPlotType().equals(EwbConstants.PLOT_TYPE_CONTOUR)) {
            EwbAnalysisSetting ldAnalysisSetting = ldAnalysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().select("FAILURE_RANK").lambda().eq(EwbAnalysisSetting::getSettingId, param.getSettingId()));
            requestPlotData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), JSONObject.parseArray(ldAnalysisSetting.getFailureRank()));
            // TimeForRank--
            requestPlotData.getJSONObject(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data)).put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataRow), getDataRowJson(param.getDataId(), new ArrayList<>(), false, new JSONArray()));
        }
    }

    /**
     * 获取失效的DataRow数据
     * isRank:新增的概率VS时间计算方式的判断依据
     *
     * @param dataId
     * @return
     */
    public JSONArray getDataRowJson(String dataId, List<EwbAnalysisDataDetail> analysisDataDetails, boolean isRank, JSONArray failureRank) {
        String serverSystem = allVariableConfig.getServerSystem();
        JSONArray dataArray = new JSONArray();
        if (!isRank) {
            List<EwbAnalysisDataDetail> dataDetails = !CollectionUtils.isEmpty(analysisDataDetails) ? analysisDataDetails : ewbAnalysisDataDetailMapper.selectList(new QueryWrapper<EwbAnalysisDataDetail>().select("quantity", "last_inspect_time", "F_S AS failureSuspension", "failure_end_time").lambda().eq(EwbAnalysisDataDetail::getDataId, dataId).orderByAsc(EwbAnalysisDataDetail::getLastInspectTime).orderByDesc(EwbAnalysisDataDetail::getFailureSuspension));
            for (EwbAnalysisDataDetail dataDetail : dataDetails) {
                JSONObject nodeData = new JSONObject();
                // 默认为失效状态
                int apiFsState = 0;
                // ==================
                // 判断明细数据的数据类型
                // ==================
                if (EwbConstants.STATE_S.equalsIgnoreCase(dataDetail.getFailureSuspension()) || EwbConstants.STATE_SUSPENSION.equalsIgnoreCase(dataDetail.getFailureSuspension())) {
                    // 若状态为s，则数据类型为右删失数据类型
                    apiFsState = 1;
                }
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.time), dataDetail.getFailureEndTime());
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lastInspectTime), dataDetail.getLastInspectTime());
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.fSstatus), apiFsState);
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.numofForS), dataDetail.getQuantity());
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureSeverity), "string");
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureMode), StringUtils.isBlank(dataDetail.getComment1()) ? "string" : dataDetail.getComment1());
                dataArray.add(nodeData);
            }
        } else {
            List<EwbAnalysisDataRank> dataRanks = ewbAnalysisDataRankMapper.selectList(new QueryWrapper<EwbAnalysisDataRank>().lambda().eq(EwbAnalysisDataRank::getDataId, dataId).orderByAsc(EwbAnalysisDataRank::getFailureTime));
            for (EwbAnalysisDataRank dataRank : dataRanks) {
                JSONObject nodeData = new JSONObject();
                // ==================
                // 判断明细数据的数据类型
                // ==================
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.time), dataRank.getFailureTime());
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lastInspectTime), dataRank.getFailureTime());
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.fSstatus), 0);
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.numofForS), 1);
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureSeverity), "string");
                nodeData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureMode), "string");
                dataArray.add(nodeData);
                failureRank.add(dataRank.getFailureRank() / 100);
            }

        }
        return dataArray;
    }

    public JSONObject getRequestLifeDataJson(String dataId, String apiDataType, SolveDemoRelWeibullChiSquareRelVo entity, boolean isRank) {
        String serverSystem = allVariableConfig.getServerSystem();
        // 调用api的输入参数
        JSONObject requestLifeDataJson = new JSONObject();
        // 原始样本数据
        JSONArray failureRank = new JSONArray();
        // 组装后的json样本数据
        JSONArray dataArray = getDataRowJson(dataId, entity.getAnalysisDataDetails(), isRank, failureRank);
        if (dataArray.size() == 0) {
            return null;
        }
        double endTime = dataArray.size() <= 0 ? 0.0 : JSONObject.parseObject(dataArray.get(dataArray.size() - 1).toString()).getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.time));
        Object[] newInputData = dataArray.toArray();
        int totalNumFailureAndSuspensions = isRank ? 0 : (StringUtils.isBlank(dataId) ? entity.getTotalNumFailureAndSuspensions() : ewbAnalysisDataDetailMapper.sumQuantity(dataId));
        int totalNumSuspensions = StringUtils.isBlank(dataId) ? entity.getTotalNumSuspensions() : ewbAnalysisDataDetailMapper.sumRightQuantity(dataId);
        int totalNumFailures = StringUtils.isBlank(dataId) ? entity.getTotalNumFailures() : totalNumFailureAndSuspensions - totalNumSuspensions;
        int totalNumofRows = StringUtils.isBlank(dataId) ? entity.getTotalNumofRows() : ewbAnalysisDataDetailMapper.countEwbDatas(dataId);
        int totalFailureRows = StringUtils.isBlank(dataId) ? entity.getTotalFailureRows() : ewbAnalysisDataDetailMapper.countFdata(dataId);
        int totalSuspensionRows = totalNumofRows - totalFailureRows;
        requestLifeDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataRow), newInputData);
        requestLifeDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.endTime), endTime);
        requestLifeDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.totalNumFailures), totalNumFailures);
        requestLifeDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.totalNumSuspensions), totalNumSuspensions);
        requestLifeDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.totalNumFailureandSuspensions), totalNumFailureAndSuspensions);
        requestLifeDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.totalFailureRows), totalFailureRows);
        requestLifeDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.totalSuspensionRows), totalSuspensionRows);
        requestLifeDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.totalNumofRows), totalNumofRows);
        requestLifeDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.dataType), isRank ? EwbConstants.RGA_API_DATATYPE_FREEFORM : apiDataType);
        requestLifeDataJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.failureRank), isRank ? failureRank : new JSONArray(0));
        return requestLifeDataJson;
    }

    public void excuteApiSaveAnalysisDisWizard(JSONObject requestLifeData, String apiLifeDataUrl,
                                               String distributingCode, String dataId, String parameterCode, HeaderUserInfo userInfo) throws Exception {
        String serverSystem = allVariableConfig.getServerSystem();
        String results = this.getResultsCallApi(requestLifeData.toString(), apiLifeDataUrl, EwbConstants.RGA_API_CONTENTTYPE_1,
                null);
        JSONObject jsonObj = JSONObject.parseObject(results);
        double beta = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.beta));
        double eta = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.eta));
        double gamma = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.gamma));
        double mu = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.mu));
        double sigma = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.sigma));
        double logMu = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logMu));
        double logSigma = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.logSigma));
        double lambda = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lambda));
        double lkv = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.lkv));
        double pho = jsonObj.getDouble(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.pho));
        double p1;
        double p2;
        double p3 = 0;
        double rao;
        double lk;
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode)) {
            p1 = WeibullNumberUtil.round(beta, EwbConstants.NUM_DIGIT_12);
            p2 = WeibullNumberUtil.round(eta, EwbConstants.NUM_DIGIT_6);
            p3 = WeibullNumberUtil.round(gamma, EwbConstants.NUM_DIGIT_6);
            rao = WeibullNumberUtil.round(pho, EwbConstants.NUM_DIGIT_6);
            lk = WeibullNumberUtil.round(lkv, EwbConstants.NUM_DIGIT_6);
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode)) {
            p1 = WeibullNumberUtil.round(logMu, EwbConstants.NUM_DIGIT_12);
            p2 = WeibullNumberUtil.round(logSigma, EwbConstants.NUM_DIGIT_6);
            rao = WeibullNumberUtil.round(pho, EwbConstants.NUM_DIGIT_6);
            lk = WeibullNumberUtil.round(lkv, EwbConstants.NUM_DIGIT_6);
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode)) {
            p1 = WeibullNumberUtil.round(mu, EwbConstants.NUM_DIGIT_12);
            p2 = WeibullNumberUtil.round(sigma, EwbConstants.NUM_DIGIT_6);
            rao = WeibullNumberUtil.round(pho, EwbConstants.NUM_DIGIT_6);
            lk = WeibullNumberUtil.round(lkv, EwbConstants.NUM_DIGIT_6);
        } else {
            p1 = WeibullNumberUtil.round(eta, EwbConstants.NUM_DIGIT_12);
            p2 = WeibullNumberUtil.round(lambda, EwbConstants.NUM_DIGIT_6);
            rao = WeibullNumberUtil.round(pho, EwbConstants.NUM_DIGIT_6);
            lk = WeibullNumberUtil.round(lkv, EwbConstants.NUM_DIGIT_6);
        }
        EwbAnalysisDisWizard ldAnalysisDisWizard = new EwbAnalysisDisWizard();
        ldAnalysisDisWizard.setDataId(dataId);
        ldAnalysisDisWizard.setDistributingCode(distributingCode);
        ldAnalysisDisWizard.setParameterCode(parameterCode);
        ldAnalysisDisWizard.setP1(p1);
        ldAnalysisDisWizard.setP2(p2);
        ldAnalysisDisWizard.setP3(p3);
        ldAnalysisDisWizard.setLk(lk);
        ldAnalysisDisWizard.setPho(rao);
        ldAnalysisDisWizard.setInsertUserId(userInfo.getUserId());
        ewbAnalysisDisWizardMapper.insert(ldAnalysisDisWizard);
    }

    /**
     * 判断RGA_API中寿命数据类型
     *
     * @param rightFlag
     * @param leftFlag
     * @return
     */
    public String getDataType(String rightFlag, String leftFlag) {
        leftFlag = !StringUtils.isBlank(leftFlag) ? leftFlag : EwbConstants.FLAG_NO_CHECKED;
        rightFlag = !StringUtils.isBlank(rightFlag) ? rightFlag : EwbConstants.FLAG_NO_CHECKED;
        String apiDataType = "";
        if (EwbConstants.FLAG_NO_CHECKED.equals(rightFlag) && EwbConstants.FLAG_NO_CHECKED.equals(leftFlag)) {
            apiDataType = EwbConstants.RGA_API_DATATYPE_EXACTFAILUREONLY;
        } else if (EwbConstants.FLAG_NO_CHECKED.equals(rightFlag) && EwbConstants.FLAG_CHECKED.equals(leftFlag)) {
            apiDataType = EwbConstants.RGA_API_DATATYPE_INTERVALFAILUREONLY;
        } else if (EwbConstants.FLAG_CHECKED.equals(rightFlag) && EwbConstants.FLAG_NO_CHECKED.equals(leftFlag)) {
            apiDataType = EwbConstants.RGA_API_DATATYPE_EXACTFAILUREANDSUSPENSION;
        } else if (EwbConstants.FLAG_CHECKED.equals(rightFlag) && EwbConstants.FLAG_CHECKED.equals(leftFlag)) {
            apiDataType = EwbConstants.RGA_API_DATATYPE_INTERVALFAILUREANDSUSPENSION;
        }
        return apiDataType;
    }

    public String getApiLifeDataUrl(String distributingCode, String estimationMethodCode, String parameterCode, String parameter, boolean zeroFailure, String cl) {
        String reliaQubeUrl = allVariableConfig.getReliaQubeUrl();
        String serverSystem = allVariableConfig.getServerSystem();
        String pathLink = EwbConstants.SERVER_SYSTEM_LINUX.equals(serverSystem) ? "&" : "?";
        // 接口调用路径
        String apiLifeDataUrl = "";

        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode)) {
            if (EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_ONEPARAMETER_RRY) + pathLink + "beta=" + parameter;
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_ONEPARAMETER_RRX) + pathLink + "beta=" + parameter;
                } else if (EwbEvaluateCodeEnum.mle.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_ONEPARAMETER_MLE) + pathLink + "beta=" + parameter;
                }
                if (zeroFailure) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_ONEPARAMETER_ZEROFAILURE) + "?" + "beta="
                            + parameter + "&CL=" + (Double.parseDouble(cl) > 1 ? Double.parseDouble(cl) / 100 : cl);
                }
            } else if (EwbParameterCodeEnum.二参数.getIndex().equals(parameterCode)) {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_TWOPARAMETER_RRY);
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_TWOPARAMETER_RRX);
                } else {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_TWOPARAMETER_MLE);
                }
            } else if (EwbParameterCodeEnum.三参数.getIndex().equals(parameterCode)) {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_THREEPARAMETER_RRY);
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_THREEPARAMETER_RRX);
                } else {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_THREEPARAMETER_MLE);
                }
            } else if (EwbParameterCodeEnum.混合2参数.getIndex().equals(parameterCode)) {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_TWOPOPULATIONMIXEDWEIBULLPARAMETER_RRY);
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_TWOPOPULATIONMIXEDWEIBULLPARAMETER_RRX);
                } else {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_TWOPOPULATIONMIXEDWEIBULLPARAMETER_MLE);
                }
            } else if (EwbParameterCodeEnum.混合3参数.getIndex().equals(parameterCode)) {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_THREEPOPULATIONMIXEDWEIBULLPARAMETER_RRY);
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_THREEPOPULATIONMIXEDWEIBULLPARAMETER_RRX);
                } else {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_THREEPOPULATIONMIXEDWEIBULLPARAMETER_MLE);
                }
            }
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode)) {
            if (EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_LOGNORMAL_ONEPARAMETER_RRY) + pathLink + "Logsigma="
                            + parameter;
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_LOGNORMAL_ONEPARAMETER_RRX) + pathLink + "Logsigma="
                            + parameter;
                } else {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_LOGNORMAL_ONEPARAMETER_MLE) + pathLink + "Logsigma="
                            + parameter;
                }
            } else {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_LOGNORMAL_TWOPARAMETER_RRY);
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_LOGNORMAL_TWOPARAMETER_RRX);
                } else {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_LOGNORMAL_TWOPARAMETER_MLE);
                }
            }
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode)) {
            if (EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_NORMAL_ONEPARAMETER_RRY) + pathLink + "sigma=" + parameter;
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_NORMAL_ONEPARAMETER_RRX) + pathLink + "sigma=" + parameter;
                } else {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_NORMAL_ONEPARAMETER_MLE) + pathLink + "sigma=" + parameter;
                }
            } else {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_NORMAL_TWOPARAMETER_RRY);
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_NORMAL_TWOPARAMETER_RRX);
                } else {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_NORMAL_TWOPARAMETER_MLE);
                }
            }
        } else if (EwbDistributingCodeEnum.Exponential.getIndex().equals(distributingCode)) {
            if (EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_EXPONENTIAL_ONEPARAMETER_RRY);
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_EXPONENTIAL_ONEPARAMETER_RRX);
                } else if (EwbEvaluateCodeEnum.mle.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_EXPONENTIAL_ONEPARAMETER_MLE);
                    if (zeroFailure) {
                        apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_WEIBULL_ONEPARAMETER_ZEROFAILURE) + "?" + "beta=1&CL=0.5";
                    }
                }
            } else {
                if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_EXPONENTIAL_TWOPARAMETER_RRY);
                } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_EXPONENTIAL_TWOPARAMETER_RRX);
                } else {
                    apiLifeDataUrl = reliaQubeUrl + EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.API_LIFEDATA_EXPONENTIAL_TWOPARAMETER_MLE);
                }
            }
        }
        return apiLifeDataUrl;
    }

    public String getApiPlotUrl(String distributingCode, String parameterCode, String plotType, String moreParameter) {
        String reliaQubeUrl = allVariableConfig.getReliaQubeUrl();
        String serverSystem = allVariableConfig.getServerSystem();
        // 接口调用路径
        String apiPlotUrl = reliaQubeUrl + EwbConstants.API_PLOT_URL;
        if (EwbConstants.PLOT_TYPE_PROBABLITY.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "ProbablityPlotData";
        } else if (EwbConstants.PLOT_TYPE_RELIABILITY.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "ReliabilityPlotData";
        } else if (EwbConstants.PLOT_TYPE_UNRELIABILITY.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "UnReliabilityPlotData";
        } else if (EwbConstants.PLOT_TYPE_FAILURERATE.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "FailureRatePlotData";
        } else if (EwbConstants.PLOT_TYPE_PDF.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "PdfPlotData";
        } else if (EwbConstants.PLOT_TYPE_CONTOUR.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "ContourPlotData";
        } else if (EwbConstants.PLOT_TYPE_HISTOGRAM.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "HistogramPlotData";
        } else if (EwbConstants.PLOT_TYPE_STRESSSTRENGTH.equals(plotType)) {
            apiPlotUrl = reliaQubeUrl + EwbConstants.API_STRESS_URL;
        } else if (EwbConstants.PLOT_TYPE_REPLACECOST.equals(plotType)) {
            apiPlotUrl = reliaQubeUrl + (EwbConstants.SERVER_SYSTEM_LINUX.equals(serverSystem) ? "OptimalMaintenance" : "/LifeData/Tools/OptimalMaintenancePlot") + "/OptimalReplacementCostPlotData";
        } else if (EwbConstants.PLOT_TYPE_INSPECTCOST.equals(plotType)) {
            apiPlotUrl = reliaQubeUrl + (EwbConstants.SERVER_SYSTEM_LINUX.equals(serverSystem) ? "OptimalMaintenance" : "/LifeData/Tools/OptimalMaintenancePlot") + "/OptimalInspectionCostPlotData";
        }

        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode)) {
            if (EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=Weibull1P";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Weibull1P";
            } else if (EwbParameterCodeEnum.二参数.getIndex().equals(parameterCode)) {
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=Weibull";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Weibull";
            } else if (EwbParameterCodeEnum.三参数.getIndex().equals(parameterCode)) {
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=Weibull3P";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Weibull3P";
            } else if (EwbParameterCodeEnum.混合2参数.getIndex().equals(parameterCode)) {
                // apiPlotUrl = EwbConstants.API_PLOT_URL1;
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=WeibullMixed2Pop";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=WeibullMixed2Pop";
            } else if (EwbParameterCodeEnum.混合3参数.getIndex().equals(parameterCode)) {
                // apiPlotUrl = EwbConstants.API_PLOT_URL1;
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=WeibullMixed3Pop";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=WeibullMixed3Pop";
            }
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode)) {
            if (EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=Lognormal1P";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Lognormal1P";
            } else {
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=Lognormal";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Lognormal";
            }
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode)) {
            if (EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=Normal1P";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Normal1P";
            } else {
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=Normal";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Normal";
            }
        } else if (EwbDistributingCodeEnum.Exponential.getIndex().equals(distributingCode)) {
            if (EwbParameterCodeEnum.一参数.getIndex().equals(parameterCode)) {
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=Exponential";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Exponential";
            } else {
                // apiPlotUrl = apiPlotUrl+"?"+"distribution=Exponential2P";
                apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Exponential2P";
            }
        }

        return apiPlotUrl;
    }

    /**
     * @return apiRadomUrl 接口路径
     * @Title: getApiRadomData
     * @Description:获取生成随机数的接口路径
     */
    public static String getApiRadomUrl(Integer failureQuantity, Integer produceQuantity) {
        String apiRadomUrl = EwbConstants.API_RANDOM_URL;
        if (failureQuantity != null && failureQuantity != 0) {
            produceQuantity = produceQuantity - failureQuantity;
            apiRadomUrl = apiRadomUrl + "?FailureN=" + failureQuantity + "&SuspN=" + produceQuantity;
        } else {
            apiRadomUrl = apiRadomUrl + "?FailureN=" + produceQuantity + "&SuspN=" + 0;
        }
        return apiRadomUrl;
    }

    public JSONObject getApiPlotArgs(String believeCode, String believeMethodCode, String believePercent, String responseResultsForLifeData, JSONObject finalResultsJsonForFailureRank) {
        String serverSystem = allVariableConfig.getServerSystem();
        JSONObject requestPlotData = new JSONObject();
        JSONObject boundsJson = new JSONObject();
        double confidenceLevelApi = EwbConfidenceTypeCodeEnum.无.getIndex().equals(believeCode) ? 50 : Double.parseDouble(believePercent);
        String sidedApi = EwbConfidenceTypeCodeEnum.getValue(believeCode);
        String methodApi = EwbConfidenceMethodCodeEnum.getValue(believeMethodCode);
        boundsJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.confidenceLevel), confidenceLevelApi / 100);
        boundsJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.method), methodApi);
        boundsJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.sided), sidedApi);
        boundsJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.on), "Function");
        JSONObject settingsJson = new JSONObject();
        settingsJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.bounds), boundsJson);

        requestPlotData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.data), finalResultsJsonForFailureRank);
        requestPlotData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.results), JSONObject.parseObject(responseResultsForLifeData));
        requestPlotData.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.settings), settingsJson);

        /*
         * requestPlotData.put("Settings", settingsJson.toString());
         * requestPlotData.put("results", responseResultsForLifeData);
         * requestPlotData.put("Data", finalResultsJsonForFailureRank.toString());
         */
        return requestPlotData;
    }

    public JSONObject getApiPlotBoundsArgs(String believeCode, String believeMethodCode, String believePercent) {
        String serverSystem = allVariableConfig.getServerSystem();
        JSONObject boundsJson = new JSONObject();
        boundsJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.confidenceLevel), WeibullNumberUtil.parseDouble(believePercent));
        boundsJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.method), believeMethodCode);
        boundsJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.sided), believeCode);
        boundsJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.on), "Function");
        JSONObject settingsJson = new JSONObject();
        settingsJson.put(EwbCalcParametersEnum.getParameter(serverSystem, EwbCalcParametersEnum.bounds), boundsJson);
        return settingsJson;
    }

    public String getApiCalcFunctionUrl(String calcType, String endTime, String missionTime, String distributingCode, String startTime, String failurePercent, String parameterCode) {
        // 接口调用路径
        String apiPlotUrl = allVariableConfig.getReliaQubeUrl();
        String apiParameter = EwbDistributingCodeEnum.getName(distributingCode) + EwbDistributionTypeEnum.getName(parameterCode);
        if (EwbConstants.CALC_TYPE_RELIABILITY.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_FUNCTIONS_RELIABILITY + "?" + "time=" + endTime + "&" + "distribution="
                    + apiParameter;
        } else if (EwbConstants.CALC_TYPE_UNRELIABILITY.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_FUNCTIONS_UNRELIABILITY + "?" + "time=" + endTime + "&" + "distribution="
                    + apiParameter;
        } else if (EwbConstants.CALC_TYPE_MTBF.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_FUNCTIONS_MEANLIFE + "?" + "distribution=" + apiParameter;
        } else if (EwbConstants.CALC_TYPE_FAILURERATE.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_FUNCTIONS_FAILURERATE + "?" + "time=" + endTime + "&" + "distribution="
                    + apiParameter;
        } else if (EwbConstants.CALC_TYPE_BLIFE.equals(calcType)) {
            double unReliability = Double.parseDouble(failurePercent) / 100;
            apiPlotUrl = apiPlotUrl + EwbConstants.API_FUNCTIONS_TIMEATUNRELIABILITY + "?" + "unreliability=" + unReliability + "&"
                    + "distribution=" + apiParameter;
        } else if (EwbConstants.CALC_TYPE_TIMEATRELIABILITY.equals(calcType)) {
            double reliability = Double.parseDouble(failurePercent) / 100;
            apiPlotUrl = apiPlotUrl + EwbConstants.API_FUNCTIONS_TIMEATRELIABILITY + "?" + "reliability=" + reliability + "&"
                    + "distribution=" + apiParameter;
        } else if (EwbConstants.CALC_TYPE_CONDITIONALPROBABILITY.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_FUNCTIONS_CONDITIONALRELIABILITY + "?" + "time=" + startTime + "&"
                    + "missionTime=" + missionTime + "&" + "distribution=" + apiParameter;
        } else if (EwbConstants.CALC_TYPE_CONDITIONALUNRELIABILITY.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_FUNCTIONS_CONDITIONALUNRELIABILITY + "?" + "time=" + startTime + "&"
                    + "missionTime=" + missionTime + "&" + "distribution=" + apiParameter;
        } else if (EwbConstants.CALC_TYPE_PDF.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_FUNCTIONS_PDF + "?" + "time=" + startTime + "&" + "distribution="
                    + apiParameter;
        }
        return apiPlotUrl;
    }

    public String getApiBoundsUrl(EwbCalcParamBo input) {
        String serverSystem = allVariableConfig.getServerSystem();
        String pathLink = EwbConstants.SERVER_SYSTEM_LINUX.equals(serverSystem) ? "&" : "?";
        // 接口调用路径
        String apiPlotUrl = allVariableConfig.getReliaQubeUrl();
        String calcType = input.getCalcType();
        String endTime = input.getEndTime();
        String missionTime = input.getAdditionalTime();
        String startTime = input.getStartTime();
        String distributingCode = input.getDistributingCode();
        String apiDistribution = EwbDistributingCodeEnum.getName(distributingCode);
        String apiConfidenceType = EwbConfidenceTypeCodeEnum.getValue(input.getConfidenceType());
        String apiMethod = EwbConfidenceMethodCodeEnum.getValue(input.getConfidenceMethod());
        double confidencePercent = Double.parseDouble(input.getConfidencePercent()) / 100;
        if (EwbConstants.CALC_TYPE_RELIABILITY.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONRELIABILITY + pathLink + "confidenceLevel=" + confidencePercent + "&"
                    + "time=" + endTime + "&" + "distribution=" + apiDistribution + "&" + "sided=" + apiConfidenceType
                    + "&" + "method=" + apiMethod;
        } else if (EwbConstants.CALC_TYPE_UNRELIABILITY.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONUNRELIABILITY + pathLink + "confidenceLevel=" + confidencePercent + "&"
                    + "time=" + endTime + "&" + "distribution=" + apiDistribution + "&" + "sided=" + apiConfidenceType
                    + "&" + "method=" + apiMethod;
        } else if (EwbConstants.CALC_TYPE_MTBF.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONMTTF + "?" + "confidenceLevel=" + confidencePercent + "&"
                    + "distribution=" + apiDistribution + "&" + "sided=" + apiConfidenceType + "&" + "method="
                    + apiMethod;
        } else if (EwbConstants.CALC_TYPE_FAILURERATE.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONFAILURERATE + "?" + "confidenceLevel=" + confidencePercent + "&"
                    + "time=" + endTime + "&" + "distribution=" + apiDistribution + "&" + "sided=" + apiConfidenceType
                    + "&" + "method=" + apiMethod;
        } else if (EwbConstants.CALC_TYPE_BLIFE.equals(calcType)) {
            double unReliability = Double.parseDouble(input.getFailurePercent()) / 100;
            apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONTIMEGIVENUNRELIABILITY + "?" + "confidenceLevel=" + confidencePercent
                    + "&" + "unreliability=" + unReliability + "&" + "distribution=" + apiDistribution + "&" + "sided="
                    + apiConfidenceType + "&" + "method=" + apiMethod;
        } else if (EwbConstants.CALC_TYPE_TIMEATRELIABILITY.equals(calcType)) {
            double unReliability = Double.parseDouble(input.getFailurePercent()) / 100;
            apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONTIMEGIVENRELIABILITY + "?" + "confidenceLevel=" + confidencePercent
                    + "&" + "reliability=" + unReliability + "&" + "distribution=" + apiDistribution + "&" + "sided="
                    + apiConfidenceType + "&" + "method=" + apiMethod;
        } else if (EwbConstants.CALC_TYPE_CONDITIONALPROBABILITY.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONCONDITIONALRELIABILITY + "?" + "confidenceLevel="
                    + confidencePercent + "&" + "currentTime=" + startTime + "&" + "missionTime="
                    + missionTime + "&" + "distribution=" + apiDistribution + "&" + "sided=" + apiConfidenceType + "&"
                    + "method=" + apiMethod;
        } else if (EwbConstants.CALC_TYPE_CONDITIONALUNRELIABILITY.equals(calcType)) {
            apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONCONDITIONALUNRELIABILITY + "?" + "confidenceLevel="
                    + confidencePercent + "&" + "currentTime=" + startTime + "&" + "missionTime="
                    + missionTime + "&" + "distribution=" + apiDistribution + "&" + "sided=" + apiConfidenceType + "&"
                    + "method=" + apiMethod;
        } else if (EwbConstants.CALC_TYPE_PARAMETER.equals(calcType)) {
            String parameter = input.getParameterType();
            if (EwbConstants.RGA_API_PARAMETER_TYPE_BETA.equals(parameter)) {
                apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONBETA;
            } else if (EwbConstants.RGA_API_PARAMETER_TYPE_BETAMIXEDWEIBULL.equals(parameter)) {
                apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONBETAMIXEDWEIBULL;
            } else if (EwbConstants.RGA_API_PARAMETER_TYPE_ETA.equals(parameter)) {
                apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONETA;
            } else if (EwbConstants.RGA_API_PARAMETER_TYPE_THETA.equals(parameter)) {
                apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONTHETA;
            } else if (EwbConstants.RGA_API_PARAMETER_TYPE_LAMBDA.equals(parameter)) {
                apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONLAMBDA;
            } else if (EwbConstants.RGA_API_PARAMETER_TYPE_ETAMIXEDWEIBULL.equals(parameter)) {
                apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONETAMIXEDWEIBULL;
            } else if (EwbConstants.RGA_API_PARAMETER_TYPE_PMIXEDWEIBULL.equals(parameter)) {
                apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONPMIXEDWEIBULL;
            } else if (EwbConstants.RGA_API_PARAMETER_TYPE_SIGMA.equals(parameter)) {
                apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONSIGMA;
            } else if (EwbConstants.RGA_API_PARAMETER_TYPE_MU.equals(parameter)) {
                apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_ONMU;
            } else if (EwbConstants.RGA_API_PARAMETER_TYPE_GAMMA.equals(parameter)) {
                apiPlotUrl = apiPlotUrl + EwbConstants.API_BOUNDS_GAMMA;// todo
            }
            apiPlotUrl = apiPlotUrl + "?" + "confidenceLevel=" + confidencePercent + "&" + "distribution="
                    + apiDistribution + "&" + "sided=" + apiConfidenceType + "&" + "method=" + apiMethod;
        }
        return apiPlotUrl;
    }

    public String getEstimationMethod(String estimationMethodCode) {
        String apiEstimationMethodCode;
        if (EwbEvaluateCodeEnum.rry.getIndex().equals(estimationMethodCode)) {
            apiEstimationMethodCode = "RRY";
        } else if (EwbEvaluateCodeEnum.rrx.getIndex().equals(estimationMethodCode)) {
            apiEstimationMethodCode = "RRX";
        } else {
            apiEstimationMethodCode = "MLE";
        }
        return apiEstimationMethodCode;
    }

    public String getParameter(String distributingCode, String beta, String sigma, String logsigma, EwbAnalysisSetting analysisSetting) {
        String parameter = "";
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode)) {
            parameter = StringUtils.isBlank(beta) ? analysisSetting.getBeta() == null ? null : analysisSetting.getBeta().toString() : beta;
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode)) {
            parameter = StringUtils.isBlank(sigma) ? analysisSetting.getSigma() == null ? null : analysisSetting.getSigma().toString() : sigma;
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode)) {
            parameter = StringUtils.isBlank(logsigma) ? analysisSetting.getLogsigma() == null ? null : analysisSetting.getLogsigma().toString() : logsigma;
        }
        return parameter;
    }

    /**
     * @param distributingCode 页面输入数据
     * @param p1               参数1
     * @param p2               参数2
     * @return Mttf
     * @Title: calcMttf
     * @Description: 计算Mttf
     */
    public double calcMttf(String distributingCode, double p1, double p2) {
        double mttf;
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode)) {
            double i = 1 + ((double) 1 / p1);
            mttf = p2 * Gamma.gamma(i);
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode)) {
            // 参数1
            // 参数2
            mttf = Math.exp(p1 + ((double) 1 / (double) 2) * Math.pow(p2, 2));
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode)) {
            mttf = p1;
        } else {
            mttf = 1d / p1 + p2;
        }
        if(Double.isNaN(mttf)){
            mttf = 0.0;
        }
        if (Double.isInfinite(mttf)) {
            mttf = 99999999999999.999999;
        }
        return mttf;
    }

    /**
     * @param p1 参数1
     * @param p2 参数2
     * @return calcB10
     * @Title: calcB10
     * @Description: 计算B10
     */
    public double calcB10(double p1, double p2) {
        double result;
        result = ((double) 1 / p1) * Math.log(-Math.log(EwbConstants.B10_PERCENT)) + Math.log(p2);
        if (Double.isInfinite(result) || Double.isInfinite(result * -1) || Double.isNaN(result)) {
            result = 999999999999.999999;
        }
        return result;
    }

    /**
     * @param p1 参数1
     * @param p2 参数2
     * @return calcB10
     * @Title: calcB10
     * @Description: 计算B10
     */
    public double calcB102(double p1, String p2) {
        double result;
        result = ((double) 1 / p1) * Math.log(-Math.log(EwbConstants.B10_PERCENT)) + Math.log(new BigDecimal(p2).doubleValue());
        if (Double.isInfinite(result) || Double.isInfinite(result * -1)) {
            result = 999999999999.999999;
        }
        return result;
    }
}
