package com.nationrel.modules.weibull.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.util.common.BasicDataUtil;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDataDetailMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDataMapper;
import com.nationrel.modules.weibull.mapper.EwbAnalysisDisWizardMapper;
import com.nationrel.modules.weibull.entity.*;
import com.nationrel.modules.weibull.vo.EwbAnalysisStartVo;
import com.nationrel.modules.weibull.util.JDBC2Util;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aimee on 2020/9/23.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SaveDataDetailService {

    @Value("${spring.datasource.dynamic.datasource.master.url}")
    private String mysqlUrl;
    @Value("${spring.datasource.dynamic.datasource.master.username}")
    private String mysqlUserName;
    @Value("${spring.datasource.dynamic.datasource.master.password}")
    private String mysqlPassword;
    private final EwbAnalysisDataMapper ewbAnalysisDataMapper;
    private final EwbAnalysisDataDetailMapper ewbAnalysisDataDetailMapper;
    private final EwbAnalysisDisWizardMapper ewbAnalysisDisWizardMapper;

    /**
     * @param dataId          数据ID
     * @param listDataDetails 转换后的明细数据集合
     * @param startDataNo     起始数据编号
     * @param leftFlag        页面复选框"包含区间删失数据或左删失数据"选中标记
     * @Title: saveAnalysisDataDetail
     * @Description:保存分析明细数据
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveAnalysisDataDetail(String dataId, List<EwbAnalysisDataDetail> listDataDetails, int startDataNo, String leftFlag) {
        // ==================
        // 1.删除当前页面分析明细数据表数据
        // ==================
        ewbAnalysisDisWizardMapper.delete(new QueryWrapper<EwbAnalysisDisWizard>().lambda().eq(EwbAnalysisDisWizard::getDataId, dataId));

        // ==================
        // 2.更新大于等于当前页起始数据编号的明细表数据
        // ==================
        // 插入起始数据编号
        int newStartDataNo = startDataNo;
        if (startDataNo > 0) {
            newStartDataNo = startDataNo - EwbConstants.NUM_ONE;
        }
        if (listDataDetails != null && listDataDetails.size() > 0) {
            // ==================
            // 3.保存寿命数据分析明细数据
            // ==================
            // 分析明细数据处理
            List<EwbAnalysisDataDetail> dataDetails = new ArrayList<>();
            for (EwbAnalysisDataDetail dataDetail : listDataDetails) {
                if (dataDetail != null && dataDetail.getFailureEndTime() != null) {
                    double lastInspectTime = dataDetail.getFailureEndTime();
                    if (EwbConstants.FLAG_CHECKED.equals(leftFlag)) {
                        lastInspectTime = dataDetail.getLastInspectTime();
                    }
                    String fs = dataDetail.getFailureSuspension();
                    if (null == dataDetail.getComment1()) {
                        dataDetail.setComment1(BasicDataUtil.EMPTY_STR);
                    }
                    // ==================
                    // 判断明细数据的数据类型
                    // ==================
                    if (EwbConstants.STATE_SUSPENSION.equalsIgnoreCase(fs)) {
                        // 若状态为s，则数据类型为右删失数据类型
                        dataDetail.setFailureSuspension(EwbConstants.STATE_S);
                        dataDetail.setFailureTime(dataDetail.getFailureEndTime());
                    } else {
                        dataDetail.setFailureSuspension(EwbConstants.STATE_F);
                        // 对于左删失，区间，完整数据，需要计算平均时间
                        dataDetail.setFailureTime((dataDetail.getFailureEndTime() + lastInspectTime) / 2);
                    }
                    newStartDataNo++;
                    // 数据编号
                    dataDetail.setDataNo(newStartDataNo);
                    // 数据ID
                    dataDetail.setDataId(dataId);
                    dataDetails.add(dataDetail);
                }
            }
            JDBC2Util.batchInsert(dataDetails, mysqlUrl, mysqlUserName, mysqlPassword);
        }
    }

    /**
     * 更新概率VS时间表格数据
     * @param dataId          数据ID
     * @param listDataRanks 转换后的明细数据集合
     * @param startDataNo     起始数据编号
     * @Title: saveAnalysisDataDetail
     * @Description:保存分析明细数据
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveAnalysisDataRank(String dataId, List<EwbAnalysisDataRank> listDataRanks, int startDataNo) {
        // ==================
        // 1.删除当前页面分析明细数据表数据
        // ==================
        ewbAnalysisDisWizardMapper.delete(new QueryWrapper<EwbAnalysisDisWizard>().lambda().eq(EwbAnalysisDisWizard::getDataId, dataId));
        // ==================
        // 2.更新大于等于当前页起始数据编号的明细表数据
        // ==================
        // 插入起始数据编号
        int newStartDataNo = startDataNo;
        if (startDataNo > 0) {
            newStartDataNo = startDataNo - EwbConstants.NUM_ONE;
        }
        if (listDataRanks != null && listDataRanks.size() > 0) {
            // ==================
            // 3.保存寿命数据分析明细数据
            // ==================
            // 分析明细数据处理
            List<EwbAnalysisDataRank> dataDetails = new ArrayList<>();
            for (EwbAnalysisDataRank dataRank : listDataRanks) {
                if (dataRank != null && dataRank.getFailureTime() != null && dataRank.getFailureRank() != null) {
                    newStartDataNo++;
                    // 数据编号
                    dataRank.setDataNo(newStartDataNo);
                    // 数据ID
                    dataRank.setDataId(dataId);
                    dataDetails.add(dataRank);
                }
            }
            JDBC2Util.batchInsertRank(dataDetails, mysqlUrl, mysqlUserName, mysqlPassword);
        }
    }

    /**
     * 更新寿命数据分析数据表的各种类型的数据量
     *
     * @param userInfo 当前登录的用户
     * @param dataId   数据ID
     * @Title: updateAnalysisData
     * @Description:更新寿命数据分析数据
     */
    public void updateAnalysisData(HeaderUserInfo userInfo, String dataId) {
        EwbAnalysisData upAnalysisData = ewbAnalysisDataMapper.selectById(dataId);
        // 完整数据量
        int completeQuantity = ewbAnalysisDataDetailMapper.countCquantity(dataId);
        upAnalysisData.setCQuantity(completeQuantity);
        // 右删失数据量
        int rightQuantity = ewbAnalysisDataDetailMapper.countRquantity(dataId);
        upAnalysisData.setRQuantity(rightQuantity);
        // 左删失数据量
        int leftQuantity = ewbAnalysisDataDetailMapper.countLquantity(dataId);
        upAnalysisData.setLQuantity(leftQuantity);
        // 区间数据量
        int intervalQuantity = ewbAnalysisDataDetailMapper.countIquantity(dataId);
        upAnalysisData.setIQuantity(intervalQuantity);
        // 数据量总数
        int quantity = completeQuantity + rightQuantity + leftQuantity + intervalQuantity;
        upAnalysisData.setQuantity(quantity);
        // 更新者
        upAnalysisData.setUpdateUserId(userInfo.getUserId());
        // 更新寿命数据分析数据明细表
        ewbAnalysisDataMapper.updateByPrimaryKey(upAnalysisData);
    }

    /**
     * 根据页面复选框的选中情况更新明细数据
     *
     * @param dataId       数据Id
     * @param leftFlag     页面复选框"包含区间删失数据或左删失数据"选中标记
     * @param rightFlag    页面复选框"包含右删失数据"选中标记
     * @param groupingFlag 页面复选框"分组数据"选中标记
     * @Title: updateAnalysisDataDetail
     * @Description:根据页面复选框的选中情况更新明细数据
     */
    public boolean updateAnalysisDataDetail(String dataId, String leftFlag, String rightFlag, String groupingFlag) {
        boolean changeFlag = false;
        // 页面复选框未选中"包含右删失数据"标记
        if (EwbConstants.FLAG_NO_CHECKED.equals(rightFlag)) {
            // 将明细数据"F或者S状态"修改为F
            int changeSum = ewbAnalysisDataDetailMapper.updateF(dataId);
            changeFlag = changeSum > 0;
            // 页面复选框未选中"包含区间删失数据或左删失数据"标记
        }
        if (EwbConstants.FLAG_NO_CHECKED.equals(leftFlag)) {
            // 将明细数据的最近检查时间更新为"最近检查时间=结束时间"数据
            int changeSum = ewbAnalysisDataDetailMapper.updateLastInspectTime(dataId);
            changeFlag = changeSum > 0 || changeFlag;
            // 页面复选框未选中"包含区间删失数据或左删失数据"标记和"包含右删失数据"标记
        }
        // 页面复选框未选中"分组数据"标记
        if (EwbConstants.FLAG_NO_CHECKED.equals(groupingFlag)) {
            // 将明细数据的数量更新为默认值等于1的
            int changeSum = ewbAnalysisDataDetailMapper.updateQuantityEqDefault(dataId);
            changeFlag = changeSum > 0 || changeFlag;
        }

        // 更新符合完整数据的数据类型为完整数据
        ewbAnalysisDataDetailMapper.updateDataTypeEqC(dataId);
        // 更新符合右删失数据的数据类型为右删失数据
        ewbAnalysisDataDetailMapper.updateDataTypeEqR(dataId);
        // 更新符合左删失数据的数据类型为左删失数据
        ewbAnalysisDataDetailMapper.updateDataTypeEqL(dataId);
        // 更新符合区间数据的数据类型为区间数据
        ewbAnalysisDataDetailMapper.updateDataTypeEqI(dataId);
        return changeFlag;
    }

    /**
     * 保存分析的详细数据
     *
     * @param userInfo           当前登录的用户
     * @param dataId             数据ID
     * @param analysisId         分析ID
     * @param analysisResultJson 分析结果json
     * @Title: saveApiAnalysisDataDetail
     * @Description:保存来自API的分布分析数据 (Store All distribution API data result)
     */
    public void saveEwbApiAnalysisData(HeaderUserInfo userInfo, String analysisId, String dataId, String analysisResultJson, String plotType, EwbAnalysisStartVo param) {
        EwbApiAnalysisResult apiAnalysisData = new EwbApiAnalysisResult();
        // 数据ID
        apiAnalysisData.setDataId(dataId);
        // 分析ID
        apiAnalysisData.setAnalysisId(analysisId);
        apiAnalysisData.setBoundSided(param.getBoundSided());
        apiAnalysisData.setConfidenceLevel(param.getConfidenceLevel());
        apiAnalysisData.setMethod(param.getMethod());
        // 分析结果json
        apiAnalysisData.setAnalysisResult(analysisResultJson);
        // 创建程序ID
        apiAnalysisData.setPlotType(plotType);
        // 寿命数据分析数据表插入
//        rldApiAnalysisResultMapper.insert(apiAnalysisData);
    }
}
